﻿using System.Collections;

namespace Butterfly.Concurrency
{
    class QueuedAction<T>
    {
        /// <summary>
        /// Queue holding all the actions to be executed
        /// </summary>
        private Queue queue;

        public delegate void HandleCycle(T context);

        private HandleCycle contextCallback;

        /// <summary>
        /// Prepares a new QueuedAction object
        /// </summary>
        /// <param name="contextCallbacl">Function context is called on upon HandleActions</param>
        public QueuedAction(HandleCycle contextCallback)
        {
            this.queue = new Queue();
            this.contextCallback = contextCallback;
        }

        /// <summary>
        /// Dequeues all the actions. MUST be called from the mother thread
        /// </summary>
        internal void HandleActions()
        {
            lock (queue.SyncRoot)
            {
                while (queue.Count > 0)
                {
                    T context = (T)queue.Dequeue();
                    contextCallback(context);
                }
            }
        }

        /// <summary>
        /// Enqueues an action to be handled upon HandleActions call
        /// </summary>
        /// <param name="context">User-defined value which performs the action</param>
        internal void RequestAction(T context)
        {
            lock (queue.SyncRoot)
            {
                queue.Enqueue(context);
            }
        }
    }
}
