﻿using System;
using System.Collections.Generic;
using System.IO;

namespace Butterfly.Core
{
    class ConfigurationData
    {
        internal Dictionary<string, string> data;
        internal bool fileHasBeenRead = false;
		private string path;

        internal ConfigurationData(string filePath, bool maynotexist = false)
        {
            this.data = new Dictionary<string, string>();
			this.path = filePath;

            if (!File.Exists(filePath))
            {
                if (!maynotexist)
                    throw new ArgumentException("Unable to locate configuration file at '" + filePath + "'.");
                else
                    return;
            }
            fileHasBeenRead = true;

            try
            {
                using (StreamReader stream = new StreamReader(filePath))
                {
                    string line = "";

                    while ((line = stream.ReadLine()) != null)
                    {
                        if (line.Length < 1 || line.StartsWith("#"))
                        {
                            continue;
                        }

                        int delimiterIndex = line.IndexOf('=');

                        if (delimiterIndex != -1)
                        {
                            string key = line.Substring(0, delimiterIndex);
                            string val = line.Substring(delimiterIndex + 1);

                            data.Add(key, val);
                        }
                    }

                    stream.Close();
                }
            }

            catch (Exception e)
            {
                throw new ArgumentException("Could not process configuration file: " + e.Message);
            }
        }

		internal int ReadInt(string key)
		{
			return TextHandling.ParseInt32(GetProperty(key));
		}

		internal uint ReadUInt(string key)
		{
			return (uint)TextHandling.Parse(GetProperty(key));
		}

		internal bool ReadBool(string key)
		{
			string value;
			if (data.TryGetValue(key, out value))
			{
				if (value.ToLower() == "true" || value == "1")
					return true;
			}
			return false;
		}

		internal string GetProperty(string key)
		{
			string value;
			if (data.TryGetValue(key, out value))
				return value;
			else
				throw new KeyNotFoundException(string.Format("Config file {0} has no entry {1}", this.path, key));
		}

		internal bool HasProperty(string key)
		{
			return data.ContainsKey(key);
		}
    }
}
