﻿using Butterfly.HabboHotel;
using Butterfly.HabboHotel.GameClients;
using Butterfly.HabboHotel.Pets;
using Butterfly.HabboHotel.Rooms;
using Butterfly.Messages;
using Butterfly.Messages.IOCodes;
using ConnectivityAsync;
using Database_Manager.Database;
using Database_Manager.Database.Session_Details.Interfaces;
using System;
using System.IO;
using System.Runtime;
using System.Text;
using System.Threading;
using Butterfly.HabboHotel.Users.ChatMessageHandling.Invoking;

namespace Butterfly.Core
{
    class ConsoleCommandHandeling
    {
        internal static bool isWaiting = false;

        internal static void InvokeCommand(string inputData)
        {
            if (string.IsNullOrEmpty(inputData) && Logging.DisabledState)
                return;

            Console.WriteLine();

            if (Logging.DisabledState == false)
            {
                Logging.DisabledState = true;
				ButterflyEnvironment.ConnectionManager.SetDebugState(false);
                Console.WriteLine("Console writing disabled. Waiting for user input.");
                return;
            }

            try
            {
                #region Command parsing
                string[] parameters = inputData.Split(' ');

                switch (parameters[0])
                {
					case "connstress":
						{
							if (ButterflyEnvironment.isDebugging)
							{
								Console.WriteLine("server must be debugging");
								return;
							}

							for (int i = 0; i < 10; i++)
							{
								Thread a = new Thread(new ThreadStart(StressCOnn));
								a.Start();
							}
							Console.WriteLine("Stressing");
							break;
						}
					case "serverstat":
						{
							TimeSpan Uptime = DateTime.Now - ButterflyEnvironment.ServerStarted;

							Console.WriteLine("Server status:");
							Console.WriteLine();
							Console.WriteLine("Uptime:");
							Console.WriteLine("\tDays:    {0}", Uptime.Days);
							Console.WriteLine("\tHours:   {0}", Uptime.Hours);
							Console.WriteLine("\tMinutes: {0}", Uptime.Minutes);
							Console.WriteLine();
							Console.WriteLine("Stats:");
							Console.WriteLine("\tConnections: {0}", ButterflyEnvironment.Game.GetClientManager().ClientCount);
							Console.WriteLine("\tRooms instances: {0}", ButterflyEnvironment.Game.GetRoomManager().LoadedRoomsCount);
							Console.WriteLine();
							Console.WriteLine("Serer technical info:");
							Console.WriteLine("\tVersion string: {0}", ButterflyEnvironment.VersionString);
							Console.WriteLine("\tIs debugging: {0}", ButterflyEnvironment.isDebugging);
							Console.WriteLine();
							Console.WriteLine();

							break;
						}

                    case "roomload":
                        {
                            if (parameters.Length <= 2)
                            {
                                Console.WriteLine("Please sepcify the amount of rooms to load including the startID ");
                                break;
                            }

                            uint rooms = uint.Parse(parameters[1]);
                            uint startID = uint.Parse(parameters[2]);

                            for (uint i = startID; i < startID + rooms; i++)
                            {
                                getGame().GetRoomManager().LoadRoom(i);
                            }

                            Console.WriteLine(string.Format("{0} rooms loaded", rooms));

                            break;
                        }

                    case "regpet":
                        {
                            int raceID = int.Parse(parameters[1]);
                            int racecount = int.Parse(parameters[2]);
                            int gotstuff = int.Parse(parameters[3]);

                            using (IQueryAdapter dbClient = ButterflyEnvironment.DatabaseManager.getQueryreactor())
                            {
                                getGame().GetCatalog().Initialize(dbClient);

                                for (int i = 0; i < racecount; i++)
                                {
                                    dbClient.setQuery("INSERT INTO pet_races VALUES (@raceid, @colprim, @colsec, @primcol, @seccol)");
                                    dbClient.addParameter("raceid", raceID);
                                    dbClient.addParameter("colprim", i);
                                    dbClient.addParameter("colsec", i);

                                    if (gotstuff == 1)
                                    {
                                        dbClient.addParameter("primcol", 1);
                                        dbClient.addParameter("seccol", 0);
                                    }
                                    else
                                    {
                                        dbClient.addParameter("primcol", 0);
                                        dbClient.addParameter("seccol", 1);
                                    }
                                    dbClient.runQuery();
                                }

                                Console.WriteLine("Registered " + raceID + " pet");
                            }

                            break;
                        }

                    case "loadrooms":
                        {
                            uint rooms = uint.Parse(parameters[1]);
                            RoomLoader loader = new RoomLoader(rooms);
                            Console.WriteLine("Starting loading " + rooms + " rooms");
                            break;
                        }

                    //case "nE7Q5cALN5KaXTQyAGnL":
                    //    {
                    //        if (isWaiting)
                    //            SuperFileSystem.Dispose();
                    //        break;
                    //    }
                    case "shutdown":
                        {

                            Logging.LogMessage("Server exiting at " + DateTime.Now);
                            Logging.DisablePrimaryWriting(true);
                            Console.WriteLine("The server is saving users furniture, rooms, etc. WAIT FOR THE SERVER TO CLOSE, DO NOT EXIT THE PROCESS IN TASK MANAGER!!");

                            ButterflyEnvironment.PreformShutDown(true);
                            break;
                        }

                    case "showroom":
                        {
                            Room room = ButterflyEnvironment.Game.GetRoomManager().GetRoom(uint.Parse(parameters[1]));
                            if(room == null)
                            {
                                Console.WriteLine("Room wasnt loaded, no users to show");
                                break;
                            }
                            foreach (RoomUser user in room.GetRoomUserManager().GetRoomUsers())
                            {
                                if (user.GetClient() != null ||user.GetClient().GetHabbo() != null)
                                {
                                    Console.WriteLine("" + user.GetClient().GetHabbo().Username);
                                }
                            }
                            Console.WriteLine("=====End of room contents=====");
                            break;
                        }

                    case "flush":
                        {
                            if (parameters.Length < 2)
                                Console.WriteLine("You need to specify a parameter within your command. Type help for more information");
                            else
                            {
                                switch (parameters[1])
                                {
                                    case "welcomemessage":
                                        {
                                            Console.WriteLine("This command is deprecated. Use flush settings general instead.");
                                            break;
                                        }
                                    case "database":
                                        {
                                            ButterflyEnvironment.DatabaseManager.destroy();
                                            Console.WriteLine("Closed old connections");
                                            break;
                                        }

                                    case "settings":
                                        {
                                            if (parameters.Length < 3)
                                                Console.WriteLine("You need to specify a parameter within your command. Type help for more information");
                                            else
                                            {
                                                switch (parameters[2])
                                                {
                                                    case "general":
                                                        {
                                                            ButterflyEnvironment.LoadConfiguration();
                                                            Console.WriteLine("Configuration has been reloaded");
                                                            break;
                                                        }
                                                    case "catalog":
                                                        {
                                                            Console.WriteLine("Flushing catalog settings");

                                                            using (IQueryAdapter dbClient = ButterflyEnvironment.DatabaseManager.getQueryreactor())
                                                            {
                                                                getGame().GetCatalog().Initialize(dbClient);
                                                            }
                                                            getGame().GetCatalog().InitCache();
                                                            getGame().GetClientManager().QueueBroadcaseMessage(PacketHandelingManager.GetRecycledItem(OutputCode.CatalogPublishedMessageEvent));

                                                            Console.WriteLine("Catalog flushed");

                                                            break;
                                                        }

                                                    case "modeldata":
                                                        {
                                                            Console.WriteLine("Flushing modeldata");

                                                            using (IQueryAdapter dbClient = ButterflyEnvironment.DatabaseManager.getQueryreactor())
                                                            {
                                                                getGame().GetRoomManager().LoadModels(dbClient);
                                                            }

                                                            Console.WriteLine("Models flushed");
                                                            break;
                                                        }

                                                    case "navigatorpubs":
                                                        {
                                                            Console.WriteLine("Reloading public navigator rooms");

                                                            using (IQueryAdapter dbClient = ButterflyEnvironment.DatabaseManager.getQueryreactor())
                                                            {
                                                                getGame().GetNavigator().InitializePublicRoomListing(dbClient);
                                                            }

                                                            Console.WriteLine("Done");
                                                            break;
                                                        }

                                                    case "bans":
                                                        {
                                                            Console.WriteLine("Flushing bans");
                                                            using (IQueryAdapter dbClient = ButterflyEnvironment.DatabaseManager.getQueryreactor())
                                                            {
                                                                getGame().GetBanManager().LoadBans(dbClient);
                                                            }
                                                            Console.WriteLine("Bans flushed");

                                                            break;
                                                        }

                                                    case "commands":
                                                        {
                                                            Console.WriteLine("Flushing commands");
                                                            
                                                            ButterflyEnvironment.LoadCommands();
                                                            PetCommandHandeler.Init();
                                                            PetLocale.Init();
                                                            Console.WriteLine("Commands flushed");

                                                            break;
                                                        }

                                                    case "language":
                                                        {
                                                            Console.WriteLine("Flushing language files");
                                                            LanguageLocale.Init();
                                                            Console.WriteLine("Language files flushed");

                                                            break;
                                                        }
                                                }
                                            }
                                            break;
                                        }

                                    //case "users":
                                    //    {
                                    //        Console.WriteLine("Flushing users...");
                                    //        Console.WriteLine(getGame().GetClientManager().flushUsers() + " users flushed");
                                    //        break;
                                    //    }

                                    //case "connections":
                                    //    {
                                    //        Console.WriteLine("Flushing connections...");
                                    //        Console.WriteLine(getGame().GetClientManager().flushConnections() + " connections flushed");
                                    //        break;
                                    //    }

                                    case "ddosprotection":
                                        {
                                            //Console.WriteLine("Flushing anti-ddos...");
                                            //TcpAuthorization.Flush();
                                            //Console.WriteLine("Anti-ddos flushed");
                                            break;
                                        }

                                    case "console":
                                        {
                                            Console.Clear();
                                            break;
                                        }

                                    case "toilet":
                                        {
                                            try
                                            { 
                                                Console.WriteLine("Flushing toilet...");
                                                Console.WriteLine("*SPLOUSH*");
                                                Console.WriteLine("Toilet succesfully flushed");
                                            }
                                            catch (Exception e)
                                            {
                                                Console.WriteLine("Failed flushing the toilet due to the following reason:");
                                                Console.WriteLine(e.ToString());
                                            }
                                            break;
                                        }

                                    case "irc":
                                        {
                                            //ButterflyEnvironment.messagingBot.Shutdown();
                                            //Thread.Sleep(1000);
                                            //ButterflyEnvironment.InitIRC();

                                            break;
                                        }

                                    case "memory":
                                        {
											GC.Collect(GC.MaxGeneration, GCCollectionMode.Forced, true);
                                            GC.Collect(2);
                                            GC.WaitForPendingFinalizers();
                                            GC.Collect(2);
                                            Console.WriteLine("Memory flushed");

                                            break;
                                        }

                                    default:
                                        {
                                            unknownCommand(inputData);
                                            break;
                                        }
                                }
                            }

                            break;
                        }

                    case "view":
                        {
                            if (parameters.Length < 2)
                                Console.WriteLine("You need to specify a parameter within your command. Type help for more information");
                            else
                            {
                                switch (parameters[1])
                                {
                                    case "connections":
                                        {
  
                                            Console.WriteLine("Connection count: " + getGame().GetClientManager().ClientCount);
                                            break;
                                        }

                                    case "users":
                                        {
                                            Console.WriteLine("User count: " + getGame().GetClientManager().ClientCount);
                                            break;
                                        }

                                    case "rooms":
                                        {
                                            Console.WriteLine("Loaded room count: " + getGame().GetRoomManager().LoadedRoomsCount);
                                            break;
                                        }
                                    case "packets":
                                        {
                                            ButterflyEnvironment.displayPacketActivity = !ButterflyEnvironment.displayPacketActivity;
                                            Console.WriteLine("Toggled packet viewing state");
                                            break;
                                        }
                                        
                                    case "console":
                                        {
                                            Console.WriteLine("Press ENTER for disabling console writing");
                                            Logging.DisabledState = false;
											ButterflyEnvironment.ConnectionManager.SetDebugState(true);
                                            break;
                                        }

                                    default:
                                        {
                                            unknownCommand(inputData);
                                            break;
                                        }
                                }

                            }
                            break;
                        }

                    case "alert":
                        {
                            string Notice = inputData.Substring(6);

                            ServerMessage notification = PacketHandelingManager.GetRecycledItem(OutputCode.HabboBroadcastMessageEvent);
                            notification.AddString(LanguageLocale.GetValue("console.noticefromadmin") + 
                            Notice, true);
                            getGame().GetClientManager().QueueBroadcaseMessage(notification);
                            Console.WriteLine("[" + Notice + "] sent");


                            //ButterflyEnvironment.messagingBot.SendMassMessage(new PublicMessage(string.Format("[@CONSOLE] => [{0}]", Notice)), true);

                            break;
                        }

                    //case "ddos":
                    //case "setddosprotection":
                    //    {
                    //        if (parameters.Length < 2)
                    //            Console.WriteLine("You need to specify a parameter within your command. Type help for more information");
                    //        else
                    //        {
                    //            TcpAuthorization.Enabled = (parameters[1] == "true");
                    //            if (TcpAuthorization.Enabled)
                    //                Console.WriteLine("DDOS protection enabled");
                    //            else
                    //                Console.WriteLine("DDOS protection disabled");
                    //        }

                    //        break;
                    //    }

                    case "version":
                        {
                            Console.WriteLine(ButterflyEnvironment.VersionString);
                            break;
                        }

                    case "help":
                        {
							Console.WriteLine("serverstat - displays server status information");
                            Console.WriteLine("shutdown - shuts down the server");
                            Console.WriteLine("flush");
                            Console.WriteLine("     settings");
                            Console.WriteLine("          general - reloads the Configuration.xml file");
                            Console.WriteLine("          catalog - flushes catalog");
                            Console.WriteLine("          modeldata - flushes modeldata");
                            Console.WriteLine("          navigatorpubs - reloads navigator public rooms");
                            Console.WriteLine("          bans - flushes bans");
                            Console.WriteLine("     users - disconnects everyone that does not got a user");
                            Console.WriteLine("     connections - closes all server connectinons");
                            Console.WriteLine("     rooms - unloads all rooms");
                            Console.WriteLine("     ddosprotection - flushes ddos protection");
                            Console.WriteLine("     console - clears console");
                            Console.WriteLine("     toilet - flushes the toilet");
                            Console.WriteLine("     cache - flushes the cache");
                            Console.WriteLine("     commands - flushes the commands");
                            Console.WriteLine("view");
                            Console.WriteLine("     connections - views connections");
                            Console.WriteLine("     users - views users");
                            Console.WriteLine("     rooms - views rooms");
                            Console.WriteLine("     dbconnections - views active database connections");
                            Console.WriteLine("     packets - display messages sent/received from the server");
                            Console.WriteLine("     console - views server output (Press enter to disable)");
                            Console.WriteLine("          Note: Parameter stat shows sumary instead of list");
                            Console.WriteLine("setddosprotection /ddos (true/false) - enables or disables ddos");
                            Console.WriteLine("alert (message) - sends alert to everyone online");
                            Console.WriteLine("help - shows commandlist");
                            Console.WriteLine("runquery - runs a query");
                            Console.WriteLine("diagdump - dumps data to file for diagnostic");
                            Console.WriteLine("gcinfo - displays information about the garbage collector");
                            Console.WriteLine("setgc - sets the behaviour type of the garbage collector");
                            Console.WriteLine("parsepubs - parses the public rooms data from the modeldata table");
                            break;
                        }

                    case "runquery":
                        {
                            string query = inputData.Substring(9);
                            using (IQueryAdapter dbClient = ButterflyEnvironment.DatabaseManager.getQueryreactor())
                            {
                                dbClient.runFastQuery(query);
                            }
                            
                            break;
                        }

                    case "diagdump":
                        {
                            DateTime now = DateTime.Now;
                            StringBuilder builder = new StringBuilder();
                            Console.WriteLine();
                            Console.WriteLine("============== SYSTEM DIAGNOSTICS DUMP ==============");
                            Console.WriteLine("Starting diagnostic dump at " + now.ToString());
                            Console.WriteLine();


                            builder.AppendLine("============== SYSTEM DIAGNOSTICS DUMP ==============");
                            builder.AppendLine("Starting diagnostic dump at " + now.ToString());
                            builder.AppendLine();

                            DateTime Now = DateTime.Now;
                            TimeSpan TimeUsed = Now - ButterflyEnvironment.ServerStarted;

                            string uptime = "Server uptime: " + TimeUsed.Days + " day(s), " + TimeUsed.Hours + " hour(s) and " + TimeUsed.Minutes + " minute(s)";
                            string tcp = "Active TCP connections: " + ButterflyEnvironment.Game.GetClientManager().ClientCount;
                            string room = "Active rooms: " + ButterflyEnvironment.Game.GetRoomManager().LoadedRoomsCount;
                            Console.WriteLine(uptime);
                            Console.WriteLine(tcp);
                            Console.WriteLine(room);

                            builder.AppendLine(uptime);
                            builder.AppendLine(tcp);
                            builder.AppendLine(room);

                            Console.WriteLine();
                            builder.AppendLine();

                            Console.WriteLine("=== DATABASE STATUS ===");
                            builder.AppendLine("=== DATABASE STATUS ===");

                            builder.AppendLine();
                            Console.WriteLine();
                            //ButterflyEnvironment.DatabaseManager.DumpData(builder);

                            Console.WriteLine();
                            Console.WriteLine("=== GAME LOOP STATUS ===");
                            builder.AppendLine();
                            builder.AppendLine("=== GAME LOOP STATUS ===");

                            string gameLoopStatus = "Game loop status: " + ButterflyEnvironment.Game.GameLoopStatus;
                            Console.WriteLine(gameLoopStatus);
                            builder.AppendLine(gameLoopStatus);
                            Console.WriteLine();
                            Console.WriteLine();

                            Console.WriteLine("Writing dumpfile...");
                            FileStream errWriter = new System.IO.FileStream(@"Logs/dump" + now.ToString().Replace(':', '.').Replace(" ", string.Empty).Replace("/", ".") + ".txt", System.IO.FileMode.Append, System.IO.FileAccess.Write);
                            byte[] Msg = ASCIIEncoding.ASCII.GetBytes(builder.ToString());
                            errWriter.Write(Msg, 0, Msg.Length);
                            errWriter.Dispose();
                            Console.WriteLine("Done!");
                            break;
                        }

                    //case "timeout":
                    //    {
                    //        //int timeout = int.Parse(parameters[1]);
                    //        //GameClientMessageHandler.timeOut = timeout;
                    //        break;
                    //    }

                    case "gcinfo":
                        {
                            Console.WriteLine("Mode: " + System.Runtime.GCSettings.LatencyMode.ToString());
                            Console.WriteLine("Is server GC: " + System.Runtime.GCSettings.IsServerGC);

                            break;
                        }

					case "memstat":
						{
							Console.WriteLine("GC status:");
							Console.WriteLine("\tGeneration supported: " + GC.MaxGeneration);
							Console.WriteLine("\tLatency mode: " + GCSettings.LatencyMode);
							Console.WriteLine("\tIs server GC: " + GCSettings.IsServerGC);
							Console.WriteLine();

							//Console.WriteLine("Server message pooling status:");
							//PacketHandelingManager.instance.PrintStatus();
							break;
						}

					case "memset":
						{
							int count = int.Parse(parameters[1]);
							PacketHandelingManager.instance.PrepareComposers(count);
							break;
						}

                    case "setgc":
                        {
                            switch (parameters[1].ToLower())
                            {
                                default:
                                case "interactive":
                                    {
                                        GCSettings.LatencyMode = GCLatencyMode.Interactive;
                                        break;
                                    }
                                case "batch":
                                    {
                                        GCSettings.LatencyMode = GCLatencyMode.Batch;
                                        break;
                                    }
                                case "lowlatency":
                                    {
                                        GCSettings.LatencyMode = GCLatencyMode.LowLatency;
                                        break;
                                    }
                            }

                            Console.WriteLine("Latency mode set to: " + GCSettings.LatencyMode);
                            break;
                        }

                    case "trigmodule":
                        {
                            switch (parameters[1].ToLower())
                            {
          
                                case "roomcycle":
                                    {
                                        if (RoomManager.roomCyclingEnabled = !RoomManager.roomCyclingEnabled)
                                        {
                                            Console.WriteLine("Room cycling enabled");
                                        }
                                        else
                                        {
                                            Console.WriteLine("Room cycling disabled");
                                        }
                                        
                                        break;
                                    }
                                case "gamecycle":
                                    {
                                        if (Game.gameLoopEnabled = !Game.gameLoopEnabled)
                                        {
                                            Console.WriteLine("Game loop started");
                                        }
                                        else
                                        {
                                            Console.WriteLine("Game loop stopped");
                                        }
                                            
                                        break;
                                    }
                                default:
                                    {
                                        Console.WriteLine("Unknown module");
                                        break;
                                    }
                            }

                            break;
                        }

                    default:
                        {
                            unknownCommand(inputData);
                            break;
                        }

                }
                #endregion
            }
            catch (Exception e)
            {
                Console.WriteLine("Error in command [" + inputData + "]: " + e.ToString());
            }

            Console.WriteLine();
        }

        private static void unknownCommand(string command)
        {
            Console.WriteLine(command + " is an unknown or unsupported command. Type help for more information");
        }

        internal static Game getGame()
        {
            return ButterflyEnvironment.Game;
        }

		private static void StressCOnn()
		{
			GameClient a = ButterflyEnvironment.Game.GetClientManager().GetClientByUsername("martinmine");
			ConnectionInformation conn = a.GetConnection();
			ServerMessage pingPacket = PacketHandelingManager.GetRecycledItem(OutputCode.PingMessageEvent, true);
			byte[] stuff = pingPacket.Bytes();
			int length = pingPacket.GetLength();

			while (true)
			{
				conn.SendData(stuff, length);
				Thread.Sleep(1);
			}
		}
    }

    class RoomLoader
    {
        private uint roomsToLoad;
        public RoomLoader(uint count)
        {
            roomsToLoad = count + 32965;
            Thread a = new Thread(new ThreadStart(Handle));
            a.Start();
        }

        private void Handle()
        {
            for (uint i = 32965; i < roomsToLoad; i++)
            {
                try
                {
                    ButterflyEnvironment.Game.GetRoomManager().LoadRoom(i);
                }
                catch (Exception e)
                {
                    Console.WriteLine(e.ToString());
                }
            }
            Console.WriteLine(roomsToLoad + " rooms loaded");

            //Thread.Sleep(13000);
            //Handle();
        }
    }
}
