﻿using Butterfly.Exceptions;
using System.Collections.Generic;
using System.ComponentModel;
using System.IO;

namespace Butterfly.Core
{
    class IniReader<T, V>
    {
        internal static Dictionary<T, V> ReadFile(string path)
        {
            if (!File.Exists(path))
                throw new ModuleInitializationException(string.Format("The file at the following location is required, but it does not exist: {0}", path));
            
            Dictionary<T, V> values = new Dictionary<T, V>();
            TypeConverter tConverter = TypeDescriptor.GetConverter(typeof(T));
            TypeConverter vConverter = TypeDescriptor.GetConverter(typeof(V));
            string line;

            using (StreamReader stream = new StreamReader(path))
            {
                while ((line = stream.ReadLine()) != null)
                {
                    if (line.Length != 0 && line.Contains("=") && !line.StartsWith("#") && !line.StartsWith("["))
                    {
                        string[] keyValuePair = line.Split('=');
                        values.Add((T)tConverter.ConvertFromString(keyValuePair[0]), (V)vConverter.ConvertFromString(keyValuePair[1]));
                    }
                }
             }

             return values;
        }
    }
}
