﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using Butterfly.Core;
using Butterfly.HabboHotel.GameClients;
using Butterfly.HabboHotel.Items;
using Butterfly.HabboHotel.Pets;
using Butterfly.HabboHotel.Users.Inventory;
using Butterfly.Messages;
using Database_Manager.Database.Session_Details.Interfaces;
using Butterfly.Messages.IOCodes;
using Butterfly.Messages.UnitTesting;
using Butterfly.HabboHotel.Users.Subscriptions;
using Butterfly.HabboHotel.RoomBots;
using Butterfly.HabboHotel.Group;

namespace Butterfly.HabboHotel.Catalogs
{
    class Catalog
    {
        internal Dictionary<int, CatalogPage> Pages;
        private Hashtable gifts;
        private Hashtable giftregister;

        private Marketplace Marketplace;

        private ServerMessage[] mCataIndexCache;
        //private Task mFurniIDCYcler;

        internal Catalog()
        {
            Marketplace = new Marketplace();
        }

        internal void Initialize(IQueryAdapter dbClient)
        {
            Pages = new Dictionary<int, CatalogPage>();
            gifts = new Hashtable();
            giftregister = new Hashtable();

            dbClient.setQuery("SELECT * FROM catalog_pages ORDER BY order_num");
            DataTable Data = dbClient.getTable();

            //dbClient.setQuery("SELECT * FROM ecotron_rewards ORDER BY item_id");

            Hashtable CataItems = new Hashtable();
            dbClient.setQuery("SELECT * FROM catalog_items");
            DataTable CatalogueItems = dbClient.getTable();

            Dictionary<int, Hashtable> itemRegister = new Dictionary<int, Hashtable>();

            if (CatalogueItems != null)
            {
                //uint id;
                foreach (DataRow Row in CatalogueItems.Rows)
                {
                    if (string.IsNullOrEmpty(Row["item_ids"].ToString()) || (int)Row["amount"] <= 0)
                        continue;

                    CatalogItem item = new CatalogItem(Row);
                    CataItems.Add((uint)Row["id"], item);
                    //Items.Add(new CatalogItem((uint)Row["id"], (string)Row["catalog_name"], (string)Row["item_ids"], (int)Row["cost_credits"], (int)Row["cost_pixels"], (int)Row["amount"]));

                    if (!itemRegister.ContainsKey(item.pageID))
                        itemRegister.Add(item.pageID, new Hashtable());
                    itemRegister[item.pageID].Add(item.ID, item);
                }
            }


            if (Data != null)
            {
                foreach (DataRow Row in Data.Rows)
                {
                    Boolean Visible = false;
                    Boolean Enabled = false;
                    Boolean ComingSoon = false;

                    if (Row["visible"].ToString() == "1")
                    {
                        Visible = true;
                    }

                    if (Row["enabled"].ToString() == "1")
                    {
                        Enabled = true;
                    }

                    if (Row["coming_soon"].ToString() == "1")
                    {
                        ComingSoon = true;
                    }

                    Pages.Add((int)Row["id"], new CatalogPage((int)Row["id"], (int)Row["parent_id"],
                        (string)Row["caption"], Visible, Enabled, ComingSoon, Convert.ToUInt32(Row["min_rank"]),
                        TextHandling.EnumToBool(Row["club_only"].ToString()), (int)Row["icon_color"],
                        (int)Row["icon_image"], (string)Row["page_layout"], (string)Row["page_headline"],
                        (string)Row["page_teaser"], (string)Row["page_special"], (string)Row["page_text1"],
                        (string)Row["page_text2"], (string)Row["page_text_details"], (string)Row["page_text_teaser"], itemRegister));
                }
            }


            PetRaceHandler.Initialize(dbClient);

            RestackByFrontpage();
        }

        internal void RestackByFrontpage()
        {
            CatalogPage fronpage = Pages[1];
            Dictionary<int, CatalogPage> restOfCata = new Dictionary<int, CatalogPage>(Pages);

            restOfCata.Remove(1);
            Pages.Clear();

            Pages.Add(fronpage.PageId, fronpage);

            foreach (KeyValuePair<int, CatalogPage> pair in restOfCata)
                Pages.Add(pair.Key, pair.Value);
        }

        internal void InitCache()
        {
            mCataIndexCache = new ServerMessage[8]; //Max 7 ranks

            for (int i = 1; i < 8; i++)
            {
                mCataIndexCache[i] = SerializeIndexForCache(i);
            }

            foreach (CatalogPage Page in Pages.Values)
            {
                Page.InitMsg();
            }

            using (IQueryAdapter dbClient = ButterflyEnvironment.DatabaseManager.getQueryreactor())
            {
                dbClient.setQuery("SELECT * FROM items_base_gifts");
                DataTable dgifts = dbClient.getTable();
                foreach (DataRow dRow in dgifts.Rows)
                {
                    uint ID = Convert.ToUInt32(dRow[0]);
                    Item baseItem = ButterflyEnvironment.Game.GetItemManager().GetBaseItem((int)ID);

                    if (baseItem == null)
                    {
                        Console.WriteLine("WARNING: Unknown gift ID: " + ID);
                        continue;
                    }

                    baseItem.IsGift = true;
                    gifts.Add(ID, baseItem);
                    giftregister.Add((int)ID, baseItem);
                }

                if (giftregister.Count == 0)
                    Console.WriteLine("Warning: You have no packets defined. Packets are disabled.");
            }
        }

        internal bool IsGift(uint baseItem)
        {
            return gifts.ContainsKey(baseItem);
        }

        internal CatalogItem FindItem(uint ItemId)
        {
            foreach (CatalogPage Page in Pages.Values)
            {
                if (Page.Items.ContainsKey(ItemId))
                    return (CatalogItem)Page.Items[ItemId];
            }

            return null;
        }

        //internal Boolean IsItemInCatalog(uint BaseId)
        //{
        //    DataRow Row = null;

        //    using (DatabaseClient dbClient = ButterflyEnvironment.GetDatabase().GetClient())
        //    {
        //        Row = dbClient.getRow("SELECT id FROM catalog_items WHERE item_ids = '" + BaseId + "' LIMIT 1");
        //    }

        //    if (Row != null)
        //    {
        //        return true;
        //    }

        //    return false;
        //}

        internal int GetTreeSize(int rank, int TreeId)
        {
            int i = 0;

            foreach (CatalogPage Page in Pages.Values)
            {
                if (Page.MinRank > rank)
                {
                    continue;
                }

                if (Page.ParentId == TreeId)
                {
                    i++;
                }
            }


            return i;
        }

        internal CatalogPage GetPage(int Page)
        {
            if (!Pages.ContainsKey(Page))
            {
                return null;
            }

            return Pages[Page];
        }

        internal void HandlePurchase(GameClient Session, int PageId, uint ItemId, string ExtraData, int amount, GiftData giftData)
        {
            bool IsGift = giftData != null;

            int discountCount = 0;
            if (amount >= 6)
                discountCount += 1;
            if (amount >= 12)
                discountCount += 2;
            if (amount >= 18)
                discountCount += 2;
            if (amount >= 24)
                discountCount += 2;
            if (amount >= 30)
                discountCount += 2;
            if (amount >= 36)
                discountCount += 2;
            if (amount >= 42)
                discountCount += 2;
            if (amount >= 48)
                discountCount += 2;
            if (amount >= 54)
                discountCount += 2;
            if (amount >= 60)
                discountCount += 2;
            if (amount >= 66)
                discountCount += 2;
            if (amount >= 72)
                discountCount += 2;
            if (amount >= 78)
                discountCount += 2;
            if (amount >= 84)
                discountCount += 2;
            if (amount >= 90)
                discountCount += 2;
            if (amount >= 96)
                discountCount += 2;
            if (amount >= 99)
                discountCount += 2;

            CatalogPage Page;
            if (!Pages.TryGetValue(PageId, out Page))
                return;

            if (Page == null || Page.ComingSoon || !Page.Enabled || !Page.Visible || Session == null || Session.GetHabbo() == null)
            {
                return;
            }

            if (Page.ClubOnly && !Session.GetHabboDataContainer().GetSubscriptionManager().HasSubscription(SubscriptionManager.SUBSCRIPTION_TYPE_VIP))
            {
                return;
            }
            if (Page.MinRank > Session.GetHabbo().Rank)
            {
                return;
            }
            CatalogItem Item = Page.GetItem(ItemId);

            if (Item == null)
            {
                return;
            }

            uint GiftUserId = 0;

            if (Item.GetBaseItem().Type == 'i' && Session.GetHabboDataContainer().GetInventoryComponent().WallItemCount() > InventoryComponent.MAX_ITEMS_COUNT)
            {
                Session.SendNotif(LanguageLocale.GetValue("inventory.max_reached"));
                return;
            }
            else if (Item.GetBaseItem().Type != 'i' && Session.GetHabboDataContainer().GetInventoryComponent().FloorItemCount() > InventoryComponent.MAX_ITEMS_COUNT)
            {
                Session.SendNotif(LanguageLocale.GetValue("inventory.max_reached"));
                return;
            }
            
            if (Item.isLimited)
            {
                amount = 1;
                discountCount = 0;
                if (Item.limitedInStock <= Item.limitedSold++)
                    return;

                using (IQueryAdapter dbClient = ButterflyEnvironment.DatabaseManager.getQueryreactor())
                {
                    dbClient.runFastQuery("UPDATE catalog_items SET sold_amount = " + Item.limitedSold + " WHERE id = " + Item.ID);
                }
                Page.InitMsg();
            }

			if (ButterflyEnvironment.Configuration.SyncCrystalsOnPurchase)
				Session.GetHabboDataContainer().SynchronizeCrystalBalance();

            int total_cost_credits = Item.creditCost * (amount - discountCount);
			int total_cost_pixels = Item.pixelCost * (amount - discountCount);
            int total_cost_crystals = Item.crystalCost * (amount - discountCount);
            int total_cost_belcredits = Item.belcreditsCost * (amount - discountCount);
            if (total_cost_crystals > 0)
            {
                Session.GetHabbo().SynchronizeCrystalBalance();
            }
            if (IsGift)
            {
                if (!Item.GetBaseItem().AllowGift)
                {
                    return;
                }

                DataRow dRow;
                using (IQueryAdapter dbClient = ButterflyEnvironment.DatabaseManager.getQueryreactor())
                {
                    dbClient.setQuery("SELECT id FROM users WHERE username = @gift_user");
                    dbClient.addParameter("gift_user", giftData.sendingUser);
                    dRow = dbClient.getRow();
                }

                if (dRow == null)
                {
                    ServerMessage response = PacketHandelingManager.GetRecycledItem(OutputCode.GiftReceiverNotFoundEvent);
                    response.AddBoolean(true);
                    response.AddString(giftData.sendingUser);
                    Session.SendMessage(response);

                    return;
                }

                GiftUserId = Convert.ToUInt32(dRow[0]);

                if (GiftUserId == 0)
                {
                    ServerMessage response = PacketHandelingManager.GetRecycledItem(OutputCode.GiftReceiverNotFoundEvent);
                    response.AddBoolean(true);
                    response.AddString(giftData.sendingUser);
                    Session.SendMessage(response);
                    return;
                }
            }

            if (Session.GetHabbo().ActivityPoints < total_cost_pixels || Session.GetHabbo().Credits < total_cost_credits 
				|| Session.GetHabbo().Crystals < total_cost_crystals)
            {
                //ServerMessage response = new ServerMessage(OutputCode.NotEnoughBalanceMessageEvent);
                //response.AddBoolean(CreditsError);
                //response.AddBoolean(PixelError);
                //response.AddBoolean(false);
                //Session.SendMessage(response);

                return;
            }

            if (IsGift && Item.GetBaseItem().Type == 'e')
            {
                Session.SendNotif(LanguageLocale.GetValue("catalog.gift.send.error"));
                return;
            }

            if (total_cost_belcredits > 0)
            {
                int oudeCredits = 0;
                using (IQueryAdapter dbClient = ButterflyEnvironment.DatabaseManager.getQueryreactor())
                {
                    dbClient.setQuery("SELECT belcredits FROM users WHERE id = " + Session.GetHabbo().Id);
                    oudeCredits = dbClient.getInteger();
                }

                if (total_cost_belcredits > oudeCredits)
                {
                    Session.SendNotif(LanguageLocale.GetValue("catalog.oudebelcreditserror") + total_cost_belcredits);
                    return;
                }

                oudeCredits = oudeCredits - total_cost_belcredits;
                using (IQueryAdapter dbClient = ButterflyEnvironment.DatabaseManager.getQueryreactor())
                {
                    dbClient.runFastQuery("UPDATE users SET belcredits = " + oudeCredits + " WHERE id = " + Session.GetHabbo().Id);
                }

                Session.SendNotif(LanguageLocale.GetValue("catalog.oudebelcreditsok") + oudeCredits);
            }

            if (Item.catalogName.Contains("HABBO_CLUB_VIP") || Item.catalogName.Equals("deal_vip_1_year_and_badge") || Item.catalogName.ToUpper().Contains("DEAL_HC"))
            {
                if (Item.creditCost > Session.GetHabbo().Credits)
                    return;

                int Days = 0;

                //DEAL_HC_3

                if (Item.catalogName.ToUpper().Contains("DEAL_HC_"))
                {
                    int k = int.Parse(Item.catalogName.Split('_')[2]);
                    switch (k)
                    {
                        case 1:
                            Days = 1 * 31;
                            break;
                        case 2:
                            Days = 3 * 31;
                            break;
                        case 3:
                            Days = 6 * 31;
                            break;
                    }
                }
                else if (Item.catalogName.Contains("HABBO_CLUB_VIP_"))
                {
                    if (Item.catalogName.Contains("_DAY"))
                    {
                        Days = int.Parse(Item.catalogName.Split('_')[3]);
                    }
                    else if (Item.catalogName.Contains("_MONTH"))
                    {
                        Days = 31 * int.Parse(Item.catalogName.Split('_')[3]);
                    }
                }
                else if (Item.catalogName.Equals("deal_vip_1_year_and_badge"))
                {
                    Days = 31 * 12;
                }
                else if (Item.catalogName.Equals("HABBO_CLUB_VIP_5_YEAR"))
                {
                    Days = 31 * 5 * 12;
                }

                if (Item.creditCost > 0)
                {
                    Session.GetHabbo().Credits -= total_cost_credits;
                    Session.GetHabboDataContainer().UpdateCreditsBalance();
                }

                if (Item.pixelCost > 0 || Item.crystalCost > 0)
                {
                    Session.GetHabbo().Crystals -= total_cost_crystals;
                    Session.GetHabbo().ActivityPoints -= total_cost_pixels;
                    Session.GetHabboDataContainer().UpdateActivityPointsBalance();
                    if (ButterflyEnvironment.Configuration.SyncCrystalsOnPurchase)
                        Session.GetHabboDataContainer().StoreCrystalsNow();
                }

                Session.GetHabboDataContainer().GetSubscriptionManager().AddOrExtendSubscription(Days * 24 * 3600, SubscriptionManager.SUBSCRIPTION_TYPE_VIP);
                
                //Session.GetHabboDataContainer().SerializeClub();
                new GetSubscriptionDataMessageEventHandler().HandleEvent(Session, null);
                

                ServerMessage ok = PacketHandelingManager.GetRecycledItem(OutputCode.PurchaseOKMessageEvent);

                ok.AddInt32((int)Item.GetBaseItem().ItemId);
                ok.AddString(Item.GetBaseItem().Name);
                ok.AddInt32(total_cost_credits);
                ok.AddInt32(Item.pixelCost);
                ok.AddInt32(0);
                ok.AddBoolean(true);
                ok.AddInt32(1);
                ok.AddString(Item.GetBaseItem().Type.ToString().ToLower());
                ok.AddInt32(Item.GetBaseItem().SpriteId);
                ok.AddString("");
                ok.AddInt32(1);
                ok.AddInt32(0);
                ok.AddString("");
                ok.AddInt32(1);
                Session.SendMessage(ok);
                Session.GetHabboDataContainer().SerializeClub();
                Session.SendNotifWithScroll(LanguageLocale.GetValue("club.onpurchase"));
                return;
            }
            //Console.WriteLine(Item.GetBaseItem().ItemId);
            //Console.WriteLine(Item.GetBaseItem().InteractionType.ToLower());
            // Extra Data is _NOT_ filtered at this point and MUST BE VERIFIED BELOW:
            switch (Item.GetBaseItem().InteractionType)
            {
                case InteractionType.groupClickable:
                case InteractionType.groupGate:
                case InteractionType.none:
                    if (Item.GetBaseItem().IsGroupItem)
                    {
                        uint groupID = TextHandling.ParseUInt32(ExtraData);
                        GroupData group = ButterflyEnvironment.Game.GetGroupManager().GetGroup(groupID);
                        if (group == null || !group.IsMember(Session.GetHabbo().Id))
                            ExtraData = string.Empty;
                    }
                    else
                    {
                        ExtraData = "";
                    }
                    break;

                case InteractionType.musicdisc:
                    ExtraData = Item.songID.ToString();
                    break;

                #region Pet handling
                case InteractionType.pet0:
                case InteractionType.pet1:
                case InteractionType.pet2:
                case InteractionType.pet3:
                case InteractionType.pet4:
                case InteractionType.pet5:
                case InteractionType.pet6:
                case InteractionType.pet7:
                case InteractionType.pet8:
                case InteractionType.pet9:
                case InteractionType.pet10:
                case InteractionType.pet11:
                case InteractionType.pet12:
                case InteractionType.pet14:
                case InteractionType.pet15:
                case InteractionType.pet16:
                case InteractionType.pet17:
                case InteractionType.pet18:
                case InteractionType.pet19:
                case InteractionType.pet20:
                case InteractionType.pet21:
                case InteractionType.pet22:
                    try
                    {

                        //uint count = 0;
                        //using (IQueryAdapter dbClient = ButterflyEnvironment.DatabaseManager.getQueryreactor())
                        //{
                        //    dbClient.setQuery("SELECT COUNT(*) FROM user_pets WHERE user_id = " + Session.GetHabbo().Id);
                        //    count = uint.Parse(dbClient.getString());
                        //}

                        //if (count > 5)
                        //{
                        //    Session.SendNotif(LanguageLocale.GetValue("catalog.pets.maxpets"));
                        //    return;
                        //}

                        string[] Bits = ExtraData.Split('\n');
                        string PetName = Bits[0];
                        string Race = Bits[1];
                        string Color = Bits[2];

                        int.Parse(Race); // to trigger any possible errors

                        if (!CheckPetName(PetName))
                            return;

                        if (Race.Length != 1 && Race.Length != 2)
                            return;

                        if (Color.Length != 6)
                            return;
                    }
                    catch (Exception e) { 
                        //Logging.WriteLine(e.ToString()); 
                        Logging.HandleException(e, "Catalog.HandlePurchase");
                        return; 
                    }

                    break;

                #endregion

                case InteractionType.roomeffect:

                    Double Number = 0;

                    try
                    {
                        if (string.IsNullOrEmpty(ExtraData))
                            Number = 0;
                        else
                            Number = Double.Parse(ExtraData, TextHandling.cultureInfo);
                    }
                    catch (Exception e) { Logging.HandleException(e, "Catalog.HandlePurchase: " + ExtraData); }

                    ExtraData = Number.ToString().Replace(',', '.');
                    break; // maintain extra data // todo: validate

                case InteractionType.postit:
                    ExtraData = "FFFF33";
                    break;

                case InteractionType.dimmer:
                    ExtraData = "1,1,1,#000000,255";
                    break;

                case InteractionType.trophy:
                    ExtraData = Session.GetHabbo().Username + Convert.ToChar(9) + DateTime.Now.Day + "-" + DateTime.Now.Month + "-" + DateTime.Now.Year + Convert.ToChar(9) + TextHandling.FilterInjectionChars(ExtraData, true);
                    break;
                case InteractionType.bot:
                    ExtraData = Item.specialText;
                    break;
                default:
                    ExtraData = "";
                    break;
            }

            if (Item.creditCost > 0)
            {
                Session.GetHabbo().Credits -= total_cost_credits;
                Session.GetHabboDataContainer().UpdateCreditsBalance();
            }

            if (Item.pixelCost > 0 || Item.crystalCost > 0)
            {
               Session.GetHabbo().Crystals -= total_cost_crystals;
                Session.GetHabbo().ActivityPoints -= total_cost_pixels;
                Session.GetHabboDataContainer().UpdateActivityPointsBalance();
                if (ButterflyEnvironment.Configuration.SyncCrystalsOnPurchase)
                    Session.GetHabboDataContainer().StoreCrystalsNow();
            }

            ServerMessage resp = PacketHandelingManager.GetRecycledItem(OutputCode.FurniListUpdateEvent);
            Session.SendMessage(resp);

            resp = PacketHandelingManager.GetRecycledItem(OutputCode.PurchaseOKMessageEvent);

            resp.AddInt32((int)Item.GetBaseItem().ItemId);
            resp.AddString(Item.GetBaseItem().Name);
            resp.AddInt32(total_cost_credits);
            resp.AddInt32(total_cost_pixels);
            resp.AddInt32(0);
            resp.AddBoolean(true);
            resp.AddInt32(1);
            resp.AddString(Item.GetBaseItem().Type.ToString().ToLower());
            resp.AddInt32(Item.GetBaseItem().SpriteId);
            resp.AddString("");
            resp.AddInt32(1);
            resp.AddInt32(0);
            resp.AddString("");
            resp.AddInt32(1);
            Session.SendMessage(resp);

            if (IsGift)
            {
                uint itemID;
                Item Present = GetPresent(giftData.spriteID);

                if (Present == null)
                {
                    //using (IQueryAdapter dbClient = ButterflyEnvironment.DatabaseManager.getQueryreactor())
                    //    dbClient.runFastQuery("INSERT INTO items_base_gifts VALUES (" + giftData.spriteID + ")");
                    //Console.WriteLine("BaseID registered for gift");
                    return;
                }

                ExtraData = giftData.SerializeExtraData(Session.GetHabbo().Id);

                using (IQueryAdapter dbClient = ButterflyEnvironment.DatabaseManager.getQueryreactor())
                {
					dbClient.setQuery("INSERT INTO items (base_id) VALUES (" + Present.ItemId + ")");
                    itemID = (uint)dbClient.insertQuery();

                    dbClient.runFastQuery("INSERT INTO items_users VALUES (" + itemID + "," + GiftUserId + ")");

                    dbClient.setQuery("INSERT INTO items_extradata (item_id, data) VALUES (@id,@data)");
                    dbClient.addParameter("id", itemID);
                    dbClient.addParameter("data", ExtraData);
                    dbClient.runQuery();

                    dbClient.setQuery("INSERT INTO user_presents (item_id,base_id,amount) VALUES " + 
                        "(" + itemID + "," + Item.GetBaseItem().ItemId + "," + Item.amount + ")");
                    dbClient.runQuery();
                }

                GameClient Receiver = ButterflyEnvironment.Game.GetClientManager().GetClientByUserID(GiftUserId);

                if (Receiver != null)
                {
                    Receiver.SendNotif(LanguageLocale.GetValue("catalog.gift.received") + Session.GetHabbo().Username);
                    Receiver.GetHabboDataContainer().GetInventoryComponent().AddNewItem(itemID, Present.ItemId, ExtraData, false, false, 0);
                    Receiver.GetHabboDataContainer().GetInventoryComponent().SendFloorInventoryUpdate();

                    InventoryComponent targetInventory = Receiver.GetHabboDataContainer().GetInventoryComponent();
                    if (targetInventory != null)
                        targetInventory.RunDBUpdate();
                }

                Session.SendNotif(LanguageLocale.GetValue("catalog.gift.sent"));
            }
            else
            {
                DeliverItems(Session, Item.GetBaseItem(), Item.amount * amount, ExtraData, Item.songID);
            }
        }

        internal static bool CheckPetName(string PetName)
        {
            if (PetName.Length < 1 || PetName.Length > 16)
            {
                return false;
            }

            if (!TextHandling.IsValidAlphaNumeric(PetName))
            {
                return false;
            }

            return true;
        }

        internal void DeliverItems(GameClient Session, Item Item, int Amount, String ExtraData, uint songID = 0)
        {
            switch (Item.Type.ToString())
            {
                case "i":
                case "s":
                    for (int i = 0; i < Amount; i++)
                    {
                        //uint GeneratedId = GenerateItemId();
                        switch (Item.InteractionType)
                        {
                            case InteractionType.pet0:


                                string[] PetData = ExtraData.Split('\n');

                                Pet GeneratedPet = CreatePet(Session.GetHabbo().Id, PetData[0], 0, PetData[1], PetData[2]);

                                Session.GetHabboDataContainer().GetInventoryComponent().AddPet(GeneratedPet);
                                Session.GetHabboDataContainer().GetInventoryComponent().AddNewItem(0, 320, "0", true, false, 0);

                                break;

                            case InteractionType.pet1:

                                string[] PetData1 = ExtraData.Split('\n');

                                Pet GeneratedPet1 = CreatePet(Session.GetHabbo().Id, PetData1[0], 1, PetData1[1], PetData1[2]);

                                Session.GetHabboDataContainer().GetInventoryComponent().AddPet(GeneratedPet1);
                                Session.GetHabboDataContainer().GetInventoryComponent().AddNewItem(0, 320, "0", true, false, 0);

                                break;

                            case InteractionType.pet2:

                                string[] PetData5 = ExtraData.Split('\n');

                                Pet GeneratedPet5 = CreatePet(Session.GetHabbo().Id, PetData5[0], 2, PetData5[1], PetData5[2]);

                                Session.GetHabboDataContainer().GetInventoryComponent().AddPet(GeneratedPet5);
                                Session.GetHabboDataContainer().GetInventoryComponent().AddNewItem(0, 320, "0", true, false, 0);

                                break;

                            case InteractionType.pet3:

                                string[] PetData2 = ExtraData.Split('\n');

                                Pet GeneratedPet2 = CreatePet(Session.GetHabbo().Id, PetData2[0], 3, PetData2[1], PetData2[2]);

                                Session.GetHabboDataContainer().GetInventoryComponent().AddPet(GeneratedPet2);
                                Session.GetHabboDataContainer().GetInventoryComponent().AddNewItem(0, 320, "0", true, false, 0);

                                break;

                            case InteractionType.pet4:

                                string[] PetData3 = ExtraData.Split('\n');

                                Pet GeneratedPet3 = CreatePet(Session.GetHabbo().Id, PetData3[0], 4, PetData3[1], PetData3[2]);

                                Session.GetHabboDataContainer().GetInventoryComponent().AddPet(GeneratedPet3);
                                Session.GetHabboDataContainer().GetInventoryComponent().AddNewItem(0, 320, "0", true, false, 0);

                                break;

                            case InteractionType.pet5:

                                string[] PetData7 = ExtraData.Split('\n');

                                Pet GeneratedPet7 = CreatePet(Session.GetHabbo().Id, PetData7[0], 5, PetData7[1], PetData7[2]);

                                Session.GetHabboDataContainer().GetInventoryComponent().AddPet(GeneratedPet7);
                                Session.GetHabboDataContainer().GetInventoryComponent().AddNewItem(0, 320, "0", true, false, 0);

                                break;

                            case InteractionType.pet6:
                                string[] PetData4 = ExtraData.Split('\n');

                                Pet GeneratedPet4 = CreatePet(Session.GetHabbo().Id, PetData4[0], 6, PetData4[1], PetData4[2]);

                                Session.GetHabboDataContainer().GetInventoryComponent().AddPet(GeneratedPet4);
                                Session.GetHabboDataContainer().GetInventoryComponent().AddNewItem(0, 320, "0", true, false, 0);

                                break;

                            case InteractionType.pet7:

                                string[] PetData6 = ExtraData.Split('\n');

                                Pet GeneratedPet6 = CreatePet(Session.GetHabbo().Id, PetData6[0], 7, PetData6[1], PetData6[2]);

                                Session.GetHabboDataContainer().GetInventoryComponent().AddPet(GeneratedPet6);
                                Session.GetHabboDataContainer().GetInventoryComponent().AddNewItem(0, 320, "0", true, false, 0);

                                break;

                            case InteractionType.pet8:

                                string[] PetData8 = ExtraData.Split('\n');

                                Pet GeneratedPet8 = CreatePet(Session.GetHabbo().Id, PetData8[0], 8, PetData8[1], PetData8[2]);

                                Session.GetHabboDataContainer().GetInventoryComponent().AddPet(GeneratedPet8);
                                Session.GetHabboDataContainer().GetInventoryComponent().AddNewItem(0, 320, "0", true, false, 0);

                                break;

                            case InteractionType.pet9:

                                string[] PetData9 = ExtraData.Split('\n');

                                Pet GeneratedPet9 = CreatePet(Session.GetHabbo().Id, PetData9[0], 9, PetData9[1], PetData9[2]);

                                Session.GetHabboDataContainer().GetInventoryComponent().AddPet(GeneratedPet9);
                                Session.GetHabboDataContainer().GetInventoryComponent().AddNewItem(0, 320, "0", true, false, 0);

                                break;
                            case InteractionType.pet10:

                                string[] PetData10 = ExtraData.Split('\n');

                                Pet GeneratedPet10 = CreatePet(Session.GetHabbo().Id, PetData10[0], 10, PetData10[1], PetData10[2]);

                                Session.GetHabboDataContainer().GetInventoryComponent().AddPet(GeneratedPet10);
                                Session.GetHabboDataContainer().GetInventoryComponent().AddNewItem(0, 320, "0", true, false, 0);

                                break;
                            case InteractionType.pet11:

                                string[] PetData11 = ExtraData.Split('\n');

                                Pet GeneratedPet11 = CreatePet(Session.GetHabbo().Id, PetData11[0], 11, PetData11[1], PetData11[2]);

                                Session.GetHabboDataContainer().GetInventoryComponent().AddPet(GeneratedPet11);
                                Session.GetHabboDataContainer().GetInventoryComponent().AddNewItem(0, 320, "0", true, false, 0);

                                break;
                            case InteractionType.pet12:
                                string[] PetData12 = ExtraData.Split('\n');

                                Pet GeneratedPet12 = CreatePet(Session.GetHabbo().Id, PetData12[0], 12, PetData12[1], PetData12[2]);

                                Session.GetHabboDataContainer().GetInventoryComponent().AddPet(GeneratedPet12);
                                Session.GetHabboDataContainer().GetInventoryComponent().AddNewItem(0, 320, "0", true, false, 0);

                                break;
                            case InteractionType.pet14:
                                string[] PetData14 = ExtraData.Split('\n');

                                Pet GeneratedPet14 = CreatePet(Session.GetHabbo().Id, PetData14[0], 14, PetData14[1], PetData14[2]);

                                Session.GetHabboDataContainer().GetInventoryComponent().AddPet(GeneratedPet14);
                                Session.GetHabboDataContainer().GetInventoryComponent().AddNewItem(0, 320, "0", true, false, 0);

                                break;
                            case InteractionType.pet15:
                                string[] PetData15 = ExtraData.Split('\n');

                                Pet GeneratedPet15 = CreatePet(Session.GetHabbo().Id, PetData15[0], 15, PetData15[1], PetData15[2]);

                                Session.GetHabboDataContainer().GetInventoryComponent().AddPet(GeneratedPet15);
                                Session.GetHabboDataContainer().GetInventoryComponent().AddNewItem(0, 320, "0", true, false, 0);

                                break;
                            case InteractionType.teleport:

                                uint idOne = Session.GetHabboDataContainer().GetInventoryComponent().AddNewItem(0, Item.ItemId, "0", true, false, 0).itemID;
                                uint idTwo = Session.GetHabboDataContainer().GetInventoryComponent().AddNewItem(0, Item.ItemId, "0", true, false, 0).itemID;

                                using (IQueryAdapter dbClient = ButterflyEnvironment.DatabaseManager.getQueryreactor())
                                {
                                    dbClient.runFastQuery("INSERT INTO items_tele_links (tele_one_id,tele_two_id) VALUES (" + idOne + "," + idTwo + ")");
                                    dbClient.runFastQuery("INSERT INTO items_tele_links (tele_one_id,tele_two_id) VALUES (" + idTwo + "," + idOne + ")");
                                }

                                break;

                            case InteractionType.dimmer:

                                uint id = Session.GetHabboDataContainer().GetInventoryComponent().AddNewItem(0, Item.ItemId, ExtraData, true, false, 0).itemID;
                                using (IQueryAdapter dbClient = ButterflyEnvironment.DatabaseManager.getQueryreactor())
                                {
                                    dbClient.runFastQuery("INSERT INTO items_moodlight (item_id,enabled,current_preset,preset_one,preset_two,preset_three) VALUES (" + id + ",0,1,'#000000,255,0','#000000,255,0','#000000,255,0')");
                                }


                                break;

                            case InteractionType.musicdisc:
                                {
                                    Session.GetHabboDataContainer().GetInventoryComponent().AddNewItem(0, Item.ItemId, songID.ToString(), true, false, songID);
                                    break;
                                }

                            default:

                                Session.GetHabboDataContainer().GetInventoryComponent().AddNewItem(0, Item.ItemId, ExtraData, true, false, songID);
                                break;
                        }
                    }

                    Session.GetHabboDataContainer().GetInventoryComponent().UpdateItems(false);
                    break;
                case "r":
                    {
                        switch (Item.InteractionType)
                        {
                            case InteractionType.bot:
                                {
                                    CreateBotForUser(Session, Item, ExtraData);
                                }
                                break;
                        }
                    }
                    break;
                case "e":

                    for (int i = 0; i < Amount; i++)
                    {
                        Session.GetHabboDataContainer().GetAvatarEffectsInventoryComponent().AddEffect(Item.SpriteId, 3600);
                    }

                    break;

                //case "h":

                //    for (int i = 0; i < Amount; i++)
                //    {
                //        Session.GetHabboDataContainer().GetSubscriptionManager().AddOrExtendSubscription("habbo_club", 2678400);
                //    }

                //    if (!Session.GetHabboDataContainer().GetBadgeComponent().HasBadge("HC1"))
                //    {
                //        Session.GetHabboDataContainer().GetBadgeComponent().GiveBadge("HC1", true);
                //    }

                //    ServerMessage message = new ServerMessage(OutputCode.ScrSendUserInfoEvent);
                //    message.AddString("habbo_club");

                //    if (Session.GetHabboDataContainer().GetSubscriptionManager().HasSubscription("habbo_club"))
                //    {
                //        Double Expire = Session.GetHabboDataContainer().GetSubscriptionManager().GetSubscription("habbo_club").ExpireTime;
                //        Double TimeLeft = Expire - TextHandling.GetUnixTimestamp();
                //        int TotalDaysLeft = (int)Math.Ceiling(TimeLeft / 86400);
                //        int MonthsLeft = TotalDaysLeft / 31;

                //        if (MonthsLeft >= 1) MonthsLeft--;

                //        message.AddInt32(TotalDaysLeft - (MonthsLeft * 31));
                //        message.AddBoolean(true);
                //        message.AddInt32(MonthsLeft);
                //        message.AddInt32(2);
                //        message.AddBoolean(true);
                //        message.AddBoolean(false);
                //        message.AddInt32((int)TimeLeft);
                //        message.AddInt32(0);
                //        message.AddBoolean(false);
                //        message.AddInt32(100);
                //        message.AddInt32(50);
                //    }
                //    else
                //    {
                //        for (int i = 0; i < 11; i++)
                //        {
                //            message.AddInt32(0);
                //        }
                //    }

                //    Session.SendMessage(message);

                //    //List<string> Rights = ButterflyEnvironment.Game.GetRoleManager().GetRightsForHabbo(Session.GetHabbo());

                //    //ServerMessage message = new ServerMessage(OutputCode.UserRightsMessageEvent);
                //    //message.AddInt32(Rights.Count);

                //    //foreach (string Right in Rights)
                //    //{
                //    //    message.AddString(Right);
                //    //}

                //    //Session.SendMessage(message);
                //    ButterflyEnvironment.Game.GetAchievementManager().ProgressUserAchievement(Session, "ACH_BasicClub", 1); //ACH_VipClub
                //    break;

                default:

                    Session.SendNotif(LanguageLocale.GetValue("catalog.buyerror"));
                    break;
            }
        }
        private void CreateBotForUser(GameClient Session, Item item, string looks)
        {
            DataRow botRow; 
            uint botID;
            using (IQueryAdapter dbClient = ButterflyEnvironment.DatabaseManager.getQueryreactor())
            {
                dbClient.setQuery("INSERT INTO `bots` (`ai_type`, `look`, `owner_id`,rotation,walk_mode) VALUES ('generic', @looks, @uid,4,'freeroam')");
                dbClient.addParameter("uid", Session.GetHabboDataContainer().data.Id);
                dbClient.addParameter("looks", looks);
                botID = (uint)dbClient.insertQuery();
                dbClient.setQuery("SELECT * FROM bots WHERE id = " + botID);
                botRow = dbClient.getRow();
            }

            RoomBot bot = ButterflyEnvironment.Game.GetBotManager().FetchBot(botRow);
            Session.GetHabboDataContainer().GetInventoryComponent().AddBot(bot);
        }

        internal Item GetPresent(int spriteID)
        {
            if (giftregister.ContainsKey(spriteID))
                return (Item)giftregister[spriteID];
            else return null;
        }

        internal static Pet CreatePet(uint UserId, string Name, int Type, string Race, string Color)
        {
            Pet pet = new Pet(404, UserId, 0, Name, (uint)Type, Race, Color, 0, 100, 100, 0, TextHandling.GetUnixTimestamp(), 0, 0, 0.0, 0, 0);
            pet.databaseState = DatabaseUpdateState.PendingUpdate;

            using (IQueryAdapter dbClient = ButterflyEnvironment.DatabaseManager.getQueryreactor())
            {
				//if (dbClient.dbType == Database_Manager.Database.DatabaseType.MSSQL)
				//	dbClient.setQuery("INSERT INTO user_pets (user_id,name,type,race,color,expirience,energy,createstamp,nutrition,respect,x,y,z) OUTPUT INSERTED.* VALUES (" + pet.ownerID + ",@" + pet.petID + "name," + pet.petType + ",@" + pet.petID + "race,@" + pet.petID + "color,0,100,'" + pet.creationStamp + "',0,0,0,0,0)");
				//else
                 dbClient.setQuery("INSERT INTO user_pets (user_id,name,type,race,color,expirience,energy,createstamp) VALUES (" + pet.ownerID + ",@" + pet.petID + "name," + pet.petType + ",@" + pet.petID + "race,@" + pet.petID + "color,0,100,'" + pet.creationStamp + "')");
                dbClient.addParameter(pet.petID + "name", pet.name);
                dbClient.addParameter(pet.petID + "race", pet.race);
                dbClient.addParameter(pet.petID + "color", pet.color);
                pet.petID = (uint)dbClient.insertQuery();
            }
            return pet;
        }

        internal static Pet GeneratePetFromRow(DataRow Row)
        {
            if (Row == null)
            {
                return null;
            }

            return new Pet(Convert.ToUInt32(Row["id"]), Convert.ToUInt32(Row["user_id"]), Convert.ToUInt32(Row["room_id"]), (string)Row["name"], Convert.ToUInt32(Row["type"]), (string)Row["race"], (string)Row["color"], (int)Row["expirience"], (int)Row["energy"], (int)Row["nutrition"], (int)Row["respect"], (double)Row["createstamp"], (int)Row["x"], (int)Row["y"], (double)Row["z"], (int)Row["hair_type"], (int)Row["hair_dye"]);
        }

        //internal Pet GeneratePetFromRow(DataRow Row, uint PetID)
        //{
        //    if (Row == null)
        //        return null;

        //    return new Pet(PetID, (uint)Row["user_id"], (uint)Row["room_id"], (string)Row["name"], (uint)Row["type"], (string)Row["race"], (string)Row["color"], (int)Row["expirience"], (int)Row["energy"], (int)Row["nutrition"], (int)Row["respect"], (double)Row["createstamp"], (int)Row["x"], (int)Row["y"], (double)Row["z"]);
        //}

        //internal uint GenerateItemId()
        //{
        //    //uint i = 0;

        //    //using (DatabaseClient dbClient = ButterflyEnvironment.GetDatabase().GetClient())
        //    //{
        //    //    i = mCacheID++;
        //    //    dbClient.runFastQuery("UPDATE item_id_generator SET id_generator = '" + mCacheID + "' LIMIT 1");
        //    //}

        //    return mCacheID++;
        //}

  
        internal ServerMessage SerializeIndexForCache(int rank)
        {
            //ServerMessage Index = new ServerMessage(126);
            //Index.AddBoolean(false);
            //Index.AddInt32(0);
            //Index.AddInt32(0);
            //Index.AddInt32(-1);
            //Index.AddString("");
            //Index.AddBoolean(false);
            ServerMessage Index = PacketHandelingManager.GetRecycledItem(OutputCode.CatalogIndexMessageEvent, true); //Fix for r61
            Index.AddBoolean(true);
            Index.AddInt32(0);
            Index.AddInt32(0);
            Index.AddInt32(-1);
            Index.AddString("root");
            Index.AddString(string.Empty);

            int checksumSecond = GetTreeSize(rank, -1);
            int _checksumSecond = 0;
            Index.AddInt32(checksumSecond);

            foreach (CatalogPage Page in Pages.Values)
            {
                if (Page.ParentId != -1 || Page.MinRank > rank)
                    continue;

                int count = Page.Serialize(rank, Index);
                int checksum = 0;

                if (count > 0)
                {
                    foreach (CatalogPage _Page in Pages.Values)
                    {
                        if (_Page.ParentId != Page.PageId || _Page.MinRank > rank)
                        {
                            continue;
                        }
                        else
                        {
                            if (_Page.Serialize(rank, Index) > 0)
                                throw new Exception("Invalid tree size count: larger than 0");
                            checksum++;
                        }
                    }

                    if (count != checksum)
                        throw new Exception("Count != checksum");
                }
                _checksumSecond++;
            }

            if (_checksumSecond != checksumSecond)
                throw new Exception("AD");

            Index.AddBoolean(false);

            //CatalogUnitTest.Parse(Index.GetUnitTest());

            return Index;
        }

        internal ServerMessage GetIndexMessageForRank(uint Rank)
        {
            if (Rank < 1)
                Rank = 1;
            if (Rank > 7)
                Rank = 7;

            return mCataIndexCache[Rank];
        }

        internal static ServerMessage SerializePage(CatalogPage Page)
        {
            ServerMessage PageData = PacketHandelingManager.GetRecycledItem(OutputCode.CatalogPageMessageEvent,true);
            PageData.AddInt32(Page.PageId);

            switch (Page.Layout)
            {
                case "frontpage":

                    PageData.AddString("frontpage3");
                    PageData.AddInt32(3);
                    PageData.AddString("catalog_frontpage_headline_shop_GENERAL");
                    PageData.AddString("Bundles_ts");
                    PageData.AddString("");
                    PageData.AddInt32(11);
                    PageData.AddString("");
                    PageData.AddString("");
                    PageData.AddString("");
                    PageData.AddString("How to get Habbo Credits");
                    PageData.AddString("You can get Habbo Credits via Prepaid Cards, Home Phone, Credit Card, Mobile, completing offers and more! " + Convert.ToChar(13) + Convert.ToChar(10) + Convert.ToChar(13) + Convert.ToChar(10) + "To redeem your Habbo Credits, enter your voucher code below.");
                    PageData.AddString(Page.TextDetails);
                    PageData.AddString("");
                    PageData.AddString("#FEFEFE");
                    PageData.AddString("#FEFEFE");
                    PageData.AddString(LanguageLocale.GetValue("catalog.waystogetcredits"));
                    PageData.AddString("credits");
                    break;

				case "guilds":
					PageData.AddString("guild_frontpage");
					PageData.AddInt32(2);
					PageData.AddString(Page.LayoutHeadline);
					PageData.AddString(Page.LayoutTeaser);
					PageData.AddInt32(3);
					PageData.AddString(Page.Text1);
					PageData.AddString(Page.TextDetails.Replace("[13]", Convert.ToChar(13).ToString()).Replace("[10]", Convert.ToChar(10).ToString()));
					PageData.AddString(Page.TextTeaser);
					break;

				case "guild_furni":
					PageData.AddString("guild_custom_furni");
					PageData.AddInt32(3);
					PageData.AddString(Page.LayoutHeadline);
					PageData.AddString(Page.LayoutTeaser);
					PageData.AddString(Page.LayoutSpecial);
					PageData.AddInt32(3);
					PageData.AddString(Page.Text1);
					PageData.AddString(Page.TextDetails);
					PageData.AddString(Page.TextTeaser);
					break;


				case "guild_fontpage":

                    PageData.AddString(Page.Layout);
                    PageData.AddInt32(2);
                    PageData.AddString(Page.LayoutHeadline);
                    PageData.AddString(Page.LayoutTeaser);
                    PageData.AddInt32(3);
                    PageData.AddString(Page.Text1);
                    PageData.AddString(Page.Text2);
                    PageData.AddString(Page.TextDetails);

                    break;

                case "recycler_prizes":

                    PageData.AddString("recycler_prizes");
                    PageData.AddInt32(1);
                    PageData.AddString("catalog_recycler_headline3");
                    PageData.AddInt32(1);
                    PageData.AddString(Page.Text1);

                    break;

                case "spaces_new":

                    PageData.AddString(Page.Layout);
                    PageData.AddInt32(1);
                    PageData.AddString(Page.LayoutHeadline);
                    PageData.AddInt32(1);
                    PageData.AddString(Page.Text1);

                    break;

                case "trophies":

                    PageData.AddString("trophies");
                    PageData.AddInt32(1);
                    PageData.AddString(Page.LayoutHeadline);
                    PageData.AddInt32(2);
                    PageData.AddString(Page.Text1);
                    PageData.AddString(Page.TextDetails);

                    break;

                case "pets":

                    PageData.AddString("pets");
                    PageData.AddInt32(2);
                    PageData.AddString(Page.LayoutHeadline);
                    PageData.AddString(Page.LayoutTeaser);
                    PageData.AddInt32(4);
                    PageData.AddString(Page.Text1);
                    PageData.AddString(LanguageLocale.GetValue("catalog.pickname"));
                    PageData.AddString(LanguageLocale.GetValue("catalog.pickcolor"));
                    PageData.AddString(LanguageLocale.GetValue("catalog.pickrace"));

                    break;

                case "soundmachine":

                    PageData.AddString(Page.Layout);
                    PageData.AddInt32(2);
                    PageData.AddString(Page.LayoutHeadline);
                    PageData.AddString(Page.LayoutTeaser);
                    PageData.AddInt32(2);
                    PageData.AddString(Page.Text1);
                    PageData.AddString(Page.TextDetails);
                    break;

                case "club_buy":

                    PageData.AddString("vip_buy"); // layout
                    PageData.AddInt32(2);
                    PageData.AddString("ctlg_buy_vip_header");
                    PageData.AddString("ctlg_gift_vip_teaser");
                    PageData.AddInt32(0);
                    break;

                case "bots":

                    PageData.AddString(Page.Layout);
                    PageData.AddInt32(3);
                    PageData.AddString(Page.LayoutHeadline);
                    PageData.AddString(Page.LayoutTeaser);
                    PageData.AddString(Page.LayoutSpecial);
                    PageData.AddInt32(3);
                    PageData.AddString(Page.Text1);
                    PageData.AddString(Page.TextDetails);
                    PageData.AddString(Page.TextTeaser);
                    break;

                default:

                    PageData.AddString(Page.Layout);
                    PageData.AddInt32(3);
                    PageData.AddString(Page.LayoutHeadline);
                    PageData.AddString(Page.LayoutTeaser);
                    PageData.AddString(Page.LayoutSpecial);
                    PageData.AddInt32(3);
                    PageData.AddString(Page.Text1);
                    PageData.AddString(Page.TextDetails);
                    PageData.AddString(Page.TextTeaser);

                    break;
            }

            if (!Page.Layout.Equals("frontpage") && !Page.Layout.Equals("club_buy"))
            {
                PageData.AddInt32(Page.Items.Values.Count);
                foreach (CatalogItem Item in Page.Items.Values)
                {
                    Item.Serialize(PageData);
                }
            }
            else
            {
                PageData.AddInt32(0);
            }

            PageData.AddInt32(-1);
            PageData.AddBoolean(false);

            return PageData;
        }

        //internal ServerMessage SerializeTestIndex()
        //{
        //    ServerMessage Message = new ServerMessage(126);

        //    Message.AddInt32(0);
        //    Message.AddInt32(0);
        //    Message.AddInt32(0);
        //    Message.AddInt32(-1);
        //    Message.AddString("");
        //    Message.AddInt32(0);
        //    Message.AddInt32(100);

        //    for (int i = 1; i <= 150; i++)
        //    {
        //        Message.AddInt32(1);
        //        Message.AddInt32(i);
        //        Message.AddInt32(i);
        //        Message.AddInt32(i);
        //        Message.AddString("#" + i);
        //        Message.AddInt32(0);
        //        Message.AddInt32(0);
        //    }

        //    return Message;

        //   
        //}

        //internal VoucherHandler GetVoucherHandler()
        //{
        //    return VoucherHandler;
        //}

        internal Marketplace GetMarketplace()
        {
            return Marketplace;
        }

        internal Hashtable GetGiftPresets()
        {
            return gifts;
        }
    }
}
