﻿using Butterfly.Exceptions;
using Butterfly.HabboHotel.Items;
using Butterfly.Messages;
using System;
using System.Data;

namespace Butterfly.HabboHotel.Catalogs
{
    class CatalogItem
    {
        internal readonly uint ID;
        internal readonly uint baseID;
        internal readonly string catalogName;
        internal readonly int creditCost;
        internal readonly int pixelCost;
        internal readonly int amount;
        internal readonly int pageID;
        internal readonly int crystalCost;
        internal readonly int belcreditsCost;
        internal readonly uint songID;
        internal readonly bool allowMassBuy;
        internal readonly bool isLimited;
        internal readonly int limitedInStock;
        internal int limitedSold;
        internal const int CURRENCY_USED = 105; //105 = crystal, 103 = pixel
        internal string specialText;

        internal CatalogItem(DataRow Row) 
        {
            this.ID = Convert.ToUInt32(Row["id"]);
            this.catalogName = (string)Row["catalog_name"];
            this.baseID = Convert.ToUInt32(Row["item_ids"]);
            this.pageID = (int)Row["page_id"];
            this.creditCost = (int)Row["cost_credits"];
            this.pixelCost = (int)Row["cost_pixels"];
            this.amount = (int)Row["amount"];
            this.crystalCost = (int)Row["cost_crystal"];
            this.belcreditsCost = (int)Row["cost_oude_belcredits"];
            this.songID = 0;// Convert.ToUInt32(Row["song_id"]);
            this.allowMassBuy = (bool)Row["allow_massbuy"];
            this.isLimited = (bool)Row["is_limited"];
            this.limitedInStock = (int)Row["in_stock"];
            this.limitedSold = (int)Row["sold_amount"];
            this.specialText = string.Empty;

            if (isLimited)
                allowMassBuy = false;

            if (GetBaseItem() == null && baseID > 0)
                throw new ModuleInitializationException(string.Format("The base item {0} for item with ID {1} was not found. The catalog module will refuse to load.", baseID, ID));

            if (catalogName.Contains("wallpaper_single") || catalogName.Contains("floor_single") || catalogName.Contains("landscape_single"))
            {
                string[] SplittedName = catalogName.Split('_');
                specialText = SplittedName[2];
            }
            else if (DBNull.Value != Row["data"])
            {
                
                this.specialText = (string)Row["data"];
            }
			//else if (this.songID > 0 && GetBaseItem().InteractionType == InteractionType.musicdisc)
			//{
			//	specialText = songID.ToString();
			//}
        }

        internal Item GetBaseItem()
        {
            Item baseItem = ButterflyEnvironment.Game.GetItemManager().GetItem(baseID);

            if (baseItem == null)
                Console.WriteLine("Missing base item: " + baseID);

            return baseItem;
        }

        internal void Serialize(ServerMessage Message)
        {
            Message.AddInt32((int)this.ID);
            Message.AddString(this.catalogName);
            Message.AddInt32(this.creditCost);
			Message.AddInt32(crystalCost > 0 ? this.crystalCost : this.pixelCost);
            Message.AddInt32(crystalCost > 0 ? CURRENCY_USED : 0); // Pixel or crystal?
            Message.AddBoolean(true); // AllowBuy

            Message.AddInt32(baseID > 0 ? 1 : 0); // items on pack

			if (baseID > 0)
			{
				// and serialize it
				Message.AddString(GetBaseItem().Type.ToString());
				Message.AddInt32(GetBaseItem().SpriteId);
				// extradata
				if (catalogName.Contains("wallpaper_single") || catalogName.Contains("floor_single") || catalogName.Contains("landscape_single"))
				{
					string[] Analyze = catalogName.Split('_');
					Message.AddString(Analyze[2]);
				}
				else if (this.songID > 0 && GetBaseItem().InteractionType == InteractionType.musicdisc)
				{
					Message.AddString(songID.ToString());
				}
				else
				{
					Message.AddString(this.specialText);
				}
				Message.AddInt32(amount);
                if (!this.isLimited)
				    Message.AddInt32(0); // getItemDuration
			}

            Message.AddBoolean(isLimited); // IsLimited

            if (isLimited)
            {
                Message.AddInt32(this.limitedInStock);
                Message.AddInt32(this.limitedInStock - this.limitedSold);
                Message.AddInt32(0); // club_level
            }

            
            Message.AddBoolean(allowMassBuy); // IsOffer
        }

        internal void SerializeClub(ServerMessage Message, GameClients.GameClient Session)
        {
            try
            {
                Message.AddInt32((int)this.ID);
                Message.AddString(catalogName);
                Message.AddInt32(this.creditCost);
                Message.AddBoolean(true); // don't know
                int Days = 0;
                int Months = 0;
                if (catalogName.Contains("HABBO_CLUB_VIP_"))
                {
                    if (catalogName.Contains("_DAY"))
                    {
                        Days = int.Parse(catalogName.Split('_')[3]);
                    }
                    else if (catalogName.Contains("_MONTH"))
                    {
                        Months = int.Parse(catalogName.Split('_')[3]);
                        Days = 31 * Months;
                    }
                }
                else if (catalogName.Equals("deal_vip_1_year_and_badge"))
                {
                    Months = 12;
                    Days = 31 * Months;
                }
                else if (catalogName.Equals("HABBO_CLUB_VIP_5_YEAR"))
                {
                    Months = 5 * 12;
                    Days = 31 * Months;
                }
                DateTime future = DateTime.Now;
                if (Session.GetHabboDataContainer().GetSubscriptionManager().HasSubscription())
                {
                    Double Expire = Session.GetHabboDataContainer().GetSubscriptionManager().GetSubscription().ExpireTime;
                    Double TimeLeft = Expire - TextHandling.GetUnixTimestamp();
                    int TotalDaysLeft = (int)Math.Ceiling(TimeLeft / 86400);
                    future = DateTime.Now.AddDays(TotalDaysLeft);
                }
                future = future.AddDays(Days);
                Message.AddInt32(Months); // months
                Message.AddInt32(Days); // days
                Message.AddInt32(Days); // wtf
                Message.AddInt32(future.Year); // year
                Message.AddInt32(future.Month); // month
                Message.AddInt32(future.Day); // day
            }
            catch
            {
                //Console.WriteLine("Unable to load club item " + Id + ": " + Name);
            }
        }
    }
}
