﻿using Butterfly.HabboHotel.GameClients;
using Butterfly.HabboHotel.Items;
using Butterfly.Messages;
using Butterfly.Messages.IOCodes;
using Database_Manager.Database.Session_Details.Interfaces;
using System;
using System.Data;
using System.Text;

namespace Butterfly.HabboHotel.Catalogs
{
    class Marketplace
    {
        internal static Boolean CanSellItem(UserItem Item)
        {
            return (Item.GetBaseItem().AllowTrade && Item.GetBaseItem().AllowMarketplaceSell);
        }

        internal static int CalculateComissionPrice(float SellingPrice)
        {
            return (int)Math.Ceiling((float)(SellingPrice / 100));
        }

        internal static Double FormatTimestamp()
        {
            return TextHandling.GetUnixTimestamp() - 172800;
        }

        internal static string FormatTimestampString()
        {
            return TextHandling.GetFirstSiffer(FormatTimestamp()).ToString();
        }

        internal static void SellItem(GameClient Session, uint ItemId, int SellingPrice)
        {
            UserItem item = Session.GetHabboDataContainer().GetInventoryComponent().GetItem(ItemId);

            if (item == null || SellingPrice > 10000 || !CanSellItem(item))
            {
                ServerMessage message = PacketHandelingManager.GetRecycledItem(OutputCode.MarketplaceMakeOfferResult);
                message.AddInt32(0);
                Session.SendMessage(message);
            }
            else
            {
                int comissionPrice = CalculateComissionPrice(SellingPrice);
                int totalPrice = SellingPrice + comissionPrice;
                int itemType = (item.GetBaseItem().Type == 'i') ? 2 : 1;

                using (IQueryAdapter dbClient = ButterflyEnvironment.DatabaseManager.getQueryreactor())
                {
                    dbClient.setQuery("INSERT INTO catalog_marketplace_offers (item_id,user_id,asking_price,total_price,public_name,sprite_id,item_type,timestamp,extra_data) VALUES (" + item.BaseItemID + "," + Session.GetHabbo().Id + "," + SellingPrice + "," + totalPrice + ",@public_name," + item.GetBaseItem().SpriteId + "," + itemType + "," + TextHandling.GetUnixTimestamp() + ",@extra_data)");
                    dbClient.addParameter("public_name", item.GetBaseItem().PublicName);
                    dbClient.addParameter("extra_data", item.extraData);
                    dbClient.runQuery();
                }

                Session.GetHabboDataContainer().GetInventoryComponent().RemoveItem(ItemId, true);
                Session.GetHabboDataContainer().GetInventoryComponent().RunDBUpdate();

                ServerMessage message = PacketHandelingManager.GetRecycledItem(OutputCode.MarketplaceMakeOfferResult);
                message.AddInt32(1);
                Session.SendMessage(message);
            }
        }

        internal static ServerMessage SerializeOffers(int MinCost, int MaxCost, String SearchQuery, int FilterMode)
        {
            DataTable dataRow = new DataTable();
            StringBuilder whereClause = new StringBuilder();
            string orderMode = "";

            whereClause.Append("WHERE state = '1' AND timestamp >= " + FormatTimestampString());

            if (MinCost >= 0)
                whereClause.Append(" AND total_price > " + MinCost);

            if (MaxCost >= 0)
                whereClause.Append(" AND total_price < " + MaxCost);

            switch (FilterMode)
            {
                case 1:
                default:
                    orderMode = "ORDER BY asking_price DESC";
                    break;

                case 2:
                    orderMode = "ORDER BY asking_price ASC";
                    break;
            }

            using (IQueryAdapter dbClient = ButterflyEnvironment.DatabaseManager.getQueryreactor())
            {
                if (SearchQuery.Length >= 1)
                    whereClause.Append(" AND public_name LIKE @search_query");

                    dbClient.setQuery("SELECT offer_id, item_type, sprite_id, total_price FROM catalog_marketplace_offers " + whereClause.ToString() + " " + orderMode + " LIMIT 100");
                dbClient.addParameter("search_query", SearchQuery + "%");

                dataRow = dbClient.getTable();
            }

            ServerMessage Message = PacketHandelingManager.GetRecycledItem(OutputCode.MarketPlaceOffersEvent);
            Message.AddInt32(dataRow.Rows.Count);

            foreach (DataRow Row in dataRow.Rows)
            {
                Message.AddUInt32(Convert.ToUInt32(Row["offer_id"]));
                Message.AddInt32(1);
                Message.AddInt32(1);
                Message.AddInt32((int)Row["sprite_id"]); // Sprite ID
                Message.AddInt32(0);
                Message.AddString(string.Empty); // Extra Chr (R52)
                Message.AddInt32((int)Row["total_price"]); // Price
                Message.AddInt32((int)Row["sprite_id"]); // ??
                Message.AddInt32((int)Row["total_price"]); // Avg
                Message.AddInt32(0); // Offers
            }

            Message.AddInt32(0);

            return Message;
        }

        internal static ServerMessage SerializeOwnOffers(uint HabboId)
        {
            decimal profit = 0;
            DataTable dTable;

            using (IQueryAdapter dbClient = ButterflyEnvironment.DatabaseManager.getQueryreactor())
            {
                dbClient.setQuery("SELECT timestamp, state, offer_id, item_type, sprite_id, total_price FROM catalog_marketplace_offers WHERE user_id = " + HabboId);
                dTable = dbClient.getTable();

                dbClient.setQuery("SELECT SUM(asking_price) FROM catalog_marketplace_offers WHERE state = '2' AND user_id = " + HabboId);
                DataRow dRow = dbClient.getRow();

                profit = (dRow[0] == DBNull.Value ? 0 : (decimal)dRow[0]);
            }

            ServerMessage message = PacketHandelingManager.GetRecycledItem(OutputCode.MarketPlaceOwnOffersEvent);
            message.AddInt32((int)profit);
            message.AddInt32(dTable.Rows.Count);

            foreach (DataRow Row in dTable.Rows)
            {
                int minutesLeft = (int)Math.Floor((((Double)Row["timestamp"] + 172800) - TextHandling.GetUnixTimestamp()) / 60);
                int state = int.Parse(Row["state"].ToString());

                if (minutesLeft <= 0)
                {
                    state = 3;
                    minutesLeft = 0;
                }

                message.AddUInt32(Convert.ToUInt32(Row["offer_id"]));
                message.AddInt32(state); // 1 = active, 2 = sold, 3 = expired
                message.AddInt32(int.Parse(Row["item_type"].ToString())); // always 1 (??)
                message.AddInt32((int)Row["sprite_id"]);
                message.AddInt32(0);
                message.AddString(string.Empty); // Extra Chr (R52)
                message.AddInt32((int)Row["total_price"]); // ??
                message.AddInt32(minutesLeft);
                message.AddInt32((int)Row["sprite_id"]);
            }

            return message;
        }
    }
}
