﻿using System.Collections;
using System.Collections.Generic;
using System.Linq;
using Butterfly.HabboHotel.Rooms;
using System;
using Collections;

namespace Butterfly.HabboHotel.Events
{
    class EventCategory
    {
        private readonly int categoryID;

        private Dictionary<RoomData, int> events;
        private OrderedArray<RoomData> orderedEventRooms;

        private Queue addQueue;
        private Queue removeQueue;
        private Queue updateQueue;

        internal ISortable<RoomData>[] GetActiveRooms(out int count)
        {
            count = orderedEventRooms.WritePosition;
            return orderedEventRooms.StorageArray;
        }

        internal EventCategory(int categoryID)
        {
            this.categoryID = categoryID;

            this.events = new Dictionary<RoomData, int>();
            this.orderedEventRooms = new OrderedArray<RoomData>(40);
            this.addQueue = new Queue();
            this.removeQueue = new Queue();
            this.updateQueue = new Queue();
        }

        internal void onCycle()
        {
            workRemoveQueue();
            workAddQueue();
            workUpdate();
            orderedEventRooms.Defrag();
        }

        private void workAddQueue()
        {
            if (addQueue.Count > 0)
            {
                lock (addQueue.SyncRoot)
                {
                    while (addQueue.Count > 0)
                    {
                        RoomData data = (RoomData)addQueue.Dequeue();
                        if (!events.ContainsKey(data))
                            events.Add(data, data.UsersNow);
                        orderedEventRooms.Insert(new ArrayElement<RoomData>(data, data.UsersNow, data.Id));
                    }
                }
            }
        }

        private void workRemoveQueue()
        {
            if (removeQueue.Count > 0)
            {
                lock (removeQueue.SyncRoot)
                {
                    while (removeQueue.Count > 0)
                    {
                        RoomData data = (RoomData)removeQueue.Dequeue();
                        events.Remove(data);
                        orderedEventRooms.RemoveItem(data.Id);
                    }
                }
            }
        }

        private void workUpdate()
        {
			if (updateQueue.Count > 0)
            {
				lock (updateQueue.SyncRoot)
                {
					while (updateQueue.Count > 0)
                    {
                        RoomData data = (RoomData)updateQueue.Dequeue();

                        if (!events.ContainsKey(data))
                            events.Add(data, data.UsersNow);
                        else
                            events[data] = data.UsersNow;
                        orderedEventRooms.Insert(new ArrayElement<RoomData>(data, data.UsersNow, data.Id));
                    }
                }
            }
        }

        internal void QueueAddEvent(RoomData data)
        {
            lock (addQueue.SyncRoot)
            {
                addQueue.Enqueue(data);
            }
        }

        internal void QueueRemoveEvent(RoomData data)
        {
            lock (removeQueue.SyncRoot)
            {
                removeQueue.Enqueue(data);
            }
        }

        internal void QueueUpdateEvent(RoomData data)
        {
            lock (updateQueue.SyncRoot)
            {
                updateQueue.Enqueue(data);
            }
        }
    }
}
