﻿using System.Collections.Generic;
using Butterfly.HabboHotel.Rooms;
using System.Collections;
using System.Linq;
using System;
using Collections;

namespace Butterfly.HabboHotel.Events
{
    class EventManager
    {
        #region Fields
        private Dictionary<RoomData, int> events;
        //private IOrderedEnumerable<KeyValuePair<RoomData, int>> orderedEventRooms;
        private OrderedArray<RoomData> orderedEventRooms;

        private Queue addQueue;
        private Queue removeQueue;
        private Queue updateQueue;

        private Dictionary<int, EventCategory> eventCategories;

        #endregion
        #region Return values
        internal ISortable<RoomData>[] GetRoomsForCategory(int categoryID, out int count)
        {
            if (categoryID > 0)
            {
                return eventCategories[categoryID].GetActiveRooms(out count);
            }
            else
            {
                count = 0;
                return new ISortable<RoomData>[] { };
            }
        }
        #endregion

        #region Constructor
        public EventManager()
        {
            this.eventCategories = new Dictionary<int, EventCategory>();
            this.events = new Dictionary<RoomData, int>();
            this.orderedEventRooms = new OrderedArray<RoomData>(40);
            this.addQueue = new Queue();
            this.removeQueue = new Queue();
            this.updateQueue = new Queue();

            for (int i = 0; i < 30; i++)
            {
                eventCategories.Add(i, new EventCategory(i));
            }
        }
        #endregion

        #region Methods
        #region Threading

        internal void onCycle()
        {
            workRemoveQueue();
            workAddQueue();
            workUpdate();
            orderedEventRooms.Defrag();
            
            foreach (EventCategory category in eventCategories.Values)
            {
                category.onCycle();
            }
        }

        private void workAddQueue()
        {
            if (addQueue.Count > 0)
            {
                lock (addQueue.SyncRoot)
                {
                    while (addQueue.Count > 0)
                    {
                        RoomData data = (RoomData)addQueue.Dequeue();
                        if (!events.ContainsKey(data))
                        {
                            events.Add(data, data.UsersNow);
                            orderedEventRooms.Insert(new ArrayElement<RoomData>(data, data.UsersNow, data.Id));
                        }
                    }
                }
            }
        }

        private void workRemoveQueue()
        {
            if (removeQueue.Count > 0)
            {
                lock (removeQueue.SyncRoot)
                {
                    while (removeQueue.Count > 0)
                    {
                        RoomData data = (RoomData)removeQueue.Dequeue();
                        events.Remove(data);
                        orderedEventRooms.RemoveItem(data.Id);
                    }
                }
            }
        }

        private void workUpdate()
        {
			if (updateQueue.Count > 0)
            {
				lock (updateQueue.SyncRoot)
                {
					while (updateQueue.Count > 0)
                    {
                        RoomData data = (RoomData)updateQueue.Dequeue();

                        if (events.ContainsKey(data))
                            events[data] = data.UsersNow;
                        orderedEventRooms.RemoveItem(data.Id);
                    }
                }
            }
        }
        #endregion

        internal void QueueAddEvent(RoomData data, int roomEventCategory)
        {
            lock (addQueue.SyncRoot)
            {
                addQueue.Enqueue(data);
            }

            eventCategories[roomEventCategory].QueueAddEvent(data);
        }

        internal void QueueRemoveEvent(RoomData data, int roomEventCategory)
        {
            lock (removeQueue.SyncRoot)
            {
                removeQueue.Enqueue(data);
            }
            eventCategories[roomEventCategory].QueueRemoveEvent(data);
        }

        internal void QueueUpdateEvent(RoomData data, int roomEventCategory)
        {
            lock (updateQueue.SyncRoot)
            {
                updateQueue.Enqueue(data);
            }
            eventCategories[roomEventCategory].QueueUpdateEvent(data);
        }
        #endregion
    }
}
