﻿using Butterfly.Core;
using Butterfly.HabboHotel.Misc;
using Butterfly.HabboHotel.Rooms;
using Butterfly.HabboHotel.Users;
using Butterfly.HabboHotel.Users.Subscriptions;
using Butterfly.HabboHotel.Users.UserDataManagement;
using Butterfly.Messages;
using Butterfly.Messages.IOCodes;
using Butterfly.Messages.Parsing;
using Butterfly.Messages.StaticMessageHandlers;
using ConnectivityAsync;
using System;

namespace Butterfly.HabboHotel.GameClients
{
    internal class GameClient : ISendSubject
    {
        private uint Id;
        
        private ConnectionInformation Connection;
        private SessionDataProvider sessionData;
        private HabboDataContainer Habbo;
        internal int ignoredPingCount;
        
        internal bool SetDoorPos;
        internal Point newDoorPos;
        private GamePacketParser packetParser;
        internal bool modeldataSent = false;


        internal int FloodCount;
        internal DateTime FloodTime;

        internal uint ConnectionID
        {
            get
            {
                return Id;
            }
        }

        internal string machineID;

        internal GameClient(uint ClientId, ConnectionInformation pConnection)
        {
            this.Id = ClientId;
			this.ignoredPingCount = 0;
            this.modeldataSent = false;
			this.Connection = pConnection;
			this.SetDoorPos = false;
			this.packetParser = new GamePacketParser();
            this.sessionData = new SessionDataProvider();
        }

        void SwitchParserRequest(byte[] data, int amountOfBytes)
        {
            packetParser.SetConnection(Connection);
            packetParser.onNewPacket += new GamePacketParser.HandlePacket(OnNewParsedPacket);
            Connection.Parser.Dispose();
			Connection.Parser = packetParser;
			Connection.Parser.handlePacketData(data, amountOfBytes);
        }

        private void OnNewParsedPacket(ClientMessage message, int messageID)
        {
            try
            {
                IMessageEventHandler handler = StaticClientMessageHandler.GetEventHandler(messageID);

                if (handler != null)
                {
                    Logging.WriteLine(string.Format("Handling ID -> {0}", messageID));
                    handler.HandleEvent(this, message);
                }
                else
                {
                    if (ButterflyEnvironment.displayPacketActivity)
                        Logging.WriteLine(string.Format("Unknown ID -> {0}", messageID));
                }
            }
            catch (Exception e) 
            { 
                Logging.LogPacketException("Packet delegator", e.ToString()); 
            }
        }

        internal void PolicyRequest()
        {
            byte[] data = ButterflyEnvironment.DefaultEncoding.GetBytes(CrossdomainPolicy.GetXmlPolicy());

			Connection.SendData(data, data.Length);
        }

        internal ConnectionInformation GetConnection()
        {
            return Connection;
        }


        internal HabboDataContainer GetHabboDataContainer()
        {
            return Habbo;
        }

        internal Habbo GetHabbo()
        {
            if (Habbo == null)
                return null;
            return Habbo.data;
        }

        internal void StartConnection()
        {
            if (Connection == null)
            {
                return;
            }

            //TimePingedReceived = DateTime.Now;
			this.ignoredPingCount = 0;

			(Connection.Parser as InitialPacketParser).PolicyRequest += PolicyRequest;
			(Connection.Parser as InitialPacketParser).SwitchParserRequest += SwitchParserRequest;

            Connection.StartPacketProcessing();
        }

        internal bool TryLogin(string AuthTicket, bool useSSO = true)
        {
            try
            {
                string ip = GetConnection().GetIP();
                byte errorCode = 0;
                UserData userData = UserDataFactory.GetUserData(AuthTicket, ip, out errorCode, useSSO);

                if (errorCode == 1)
                {
                    SendNotifWithScroll(LanguageLocale.GetValue("login.invalidsso"));
                    return false;
                }
                else if (errorCode == 2)
                {
                    SendNotifWithScroll("Your user is already signed in.");
                    return false;
                }

                ButterflyEnvironment.Game.GetClientManager().LogClonesOut(userData.userID);
                ButterflyEnvironment.Game.GetClientManager().RegisterClient(this, userData.userID, userData.user.data.Username);
                this.Habbo = userData.user;
                userData.user.LoadData(userData);

                if (userData.user.data.Username == null)
                {
                    SendBanMessage("You have no username.");
                    return false;
                }
                string banReason = ButterflyEnvironment.Game.GetBanManager().GetBanReason(userData.user.data.Username, userData.user.getIP());

                if (!string.IsNullOrEmpty(banReason))
                {
                    SendBanMessage(banReason);
                    return false;
                }
                userData.user.Init(this, userData);

                ServerMessage message;

                message = PacketHandelingManager.GetRecycledItem(OutputCode.UniqueIDEvent);
                message.AddString(this.machineID);
                SendMessage(message);

                message = PacketHandelingManager.GetRecycledItem(OutputCode.AuthenticationOKEvent);
                SendMessage(message);


                if (userData.user.HasFuse("fuse_mod"))
                {
                    ServerMessage ModTool = ButterflyEnvironment.Game.GetModerationTool().SerializeTool();
                    SendMessage(ModTool);


                    ButterflyEnvironment.Game.GetModerationTool().SerializeOpenTickets(this, userData.userID);
                }

                message = userData.user.GetAvatarEffectsInventoryComponent().Serialize();
        
                SendMessage(message);

                message = PacketHandelingManager.GetRecycledItem(OutputCode.AvailabilityStatusMessageEvent);
                message.AddBoolean(true);
                message.AddBoolean(false);

                SendMessage(message);

                //message = PacketHandelingManager.GetRecycledItem(OutputCode.InfoFeedEnableMessageEvent);
                //message.AddBoolean(true);
                //SendMessage(message); Dunno

                if (ButterflyEnvironment.Configuration.PixelLoginAmount > 0)
                {
                    PixelManager.GivePixels(this, ButterflyEnvironment.Configuration.PixelLoginAmount);
                }

                if (ButterflyEnvironment.Configuration.CreditsLoginAmount > 0)
                {
                    userData.user.data.Credits += ButterflyEnvironment.Configuration.CreditsLoginAmount;
                    userData.user.UpdateCreditsBalance();
                }

                if (userData.user.data.HomeRoom > 0)
                {
                    message = PacketHandelingManager.GetRecycledItem(OutputCode.NavigatorSettingsEvent);
                    message.AddUInt32(userData.user.data.HomeRoom);
                    message.AddUInt32(userData.user.data.HomeRoom);
                    SendMessage(message);
                }

                message = PacketHandelingManager.GetRecycledItem(OutputCode.FavouritesEvent);
                message.AddInt32(30);
                message.AddInt32(userData.user.data.FavoriteRooms.Count);

                foreach (uint Id in userData.user.data.FavoriteRooms.ToArray())
                {
                    message.AddUInt32(Id);
                }

                SendMessage(message);

                userData.user.UpdateCreditsBalance();
                userData.user.UpdateActivityPointsBalance();


                if (userData.user.GetSubscriptionManager().HasSubscription(SubscriptionManager.SUBSCRIPTION_TYPE_CLUB) && !userData.user.GetBadgeComponent().HasBadge("ACH_BasicClub1"))
                {
                    userData.user.GetBadgeComponent().GiveBadge("ACH_BasicClub1", true);
                }
                else if (!userData.user.GetSubscriptionManager().HasSubscription(SubscriptionManager.SUBSCRIPTION_TYPE_CLUB) && userData.user.GetBadgeComponent().HasBadge("ACH_BasicClub1"))
                {
                    userData.user.GetBadgeComponent().RemoveBadge("ACH_BasicClub1");
                }

                
                ServerMessage fuserights = PacketHandelingManager.GetRecycledItem(OutputCode.FuserightEvent);
                if (GetHabboDataContainer().GetSubscriptionManager().HasSubscription(SubscriptionManager.SUBSCRIPTION_TYPE_CLUB)) // VIP 
                    fuserights.AddInt32(1);
                else
                    fuserights.AddInt32(0);
                fuserights.AddInt32((int)GetHabbo().Rank);

                SendMessage(fuserights);

                GetHabboDataContainer().SerializeClub();

                //ServerMessage setRanking = new ServerMessage(OutputCode.SerializeCompetitionWinners);
                //setRanking.AddString("hlatCompetitions"); // competition type
                //setRanking.AppendInt32(Ranking.getCompetitionForInfo("hlatCompetitions").Count);
                //int i = 0;
                //foreach (Ranking r in Ranking.getCompetitionForInfo("hlatCompetitions"))
                //{
                //    i++;
                //    setRanking.AppendUInt(r.UserId);
                //    Habbo data = ButterflyEnvironment.getHabboForId(r.UserId);
                //    setRanking.AddString((data != null) ? data.Username : "Desconocido");
                //    setRanking.AddString((data != null) ? data.Look : "sh-907-96.hd-3096-3.he-3082-91.lg-3018-81.ch-660-95.hr-9534-34");
                //    setRanking.AppendInt32(i); // position
                //    setRanking.AppendInt32(r.Score);
                //}
                //response.appendResponse(setRanking);
                //response.sendResponse();

                //appendingServerMessage message = new ServerMessage(OutputCode.UserRightsMessageEvent);
                //message.AddInt32(Rights.Count);

                //foreach (string Right in Rights)
                //{
                //    message.AddString(Right);
                //}

                //SendMessage(message);

                if (ButterflyEnvironment.Configuration.WelcomeMessageEnabled)
                {
                    this.SendNotif(ButterflyEnvironment.Configuration.WelcomeMessage);
                }

                GetHabboDataContainer().InitMessenger();
                GetHabboDataContainer().InitGroups();

                SendMessage(GetHabboDataContainer().GetMessenger().SerializeFriends() as ServerMessage);
                SendMessage(GetHabboDataContainer().GetMessenger().SerializeRequests() as ServerMessage);
                
                Logging.WriteLine("[" + GetHabbo().Username + "] logged in");

                return true;
            }
            catch (UserDataNotFoundException)
            {
                SendNotif(LanguageLocale.GetValue("login.invalidsso"));
            }
            catch (Exception e)
            {
                Logging.LogCriticalException("Invalid Dario bug duing user login: " + e.ToString());
                SendNotifWithScroll("An error occurred while logging in. Error has been logged.");
            }
            return false;
        }

        internal void SendNotifWithScroll(string message)
        {
            ServerMessage notification = PacketHandelingManager.GetRecycledItem(OutputCode.ScrollNotification);
            notification.AddInt32(1);
            notification.AddString(message, true);
            SendMessage(notification);
        }

        internal void SendBanMessage(string Message)
        {
            //ServerMessage BanMessage = new ServerMessage(OutputCode.UserBannedMessageEvent);
            //BanMessage.AddString(LanguageLocale.GetValue("moderation.banmessage"));
            //BanMessage.AddString(Message);
            //GetConnection().SendData(BanMessage.Bytes());
            SendNotif(Message, true);
        }

        internal void SendNotif(string Message)
        {
            SendNotif(Message, false);
        }

        internal void SendNotif(string Message, Boolean FromHotelManager)
        {
            //ServerMessage nMessage;

            //if (FromHotelManager)
            //{
            //    nMessage = new ServerMessage(OutputCode.HabboBroadcastMessageEvent);
            //}
            //else
            //{
            //    nMessage = new ServerMessage(OutputCode.ModMessageEvent);
            //}
            SendNotifWithScroll(Message);
            /*nMessage = PacketHandelingManager.GetRecycledItem(OutputCode.MOTDNotificationEvent);
            nMessage.AddString(Message);
            nMessage.AddString(string.Empty);
            SendMessage(nMessage);*/
        }

        internal void Stop()
        {
            if (GetHabboDataContainer() != null)
            {
                GetHabboDataContainer().Destroy();
            }

            this.Habbo = null;
            this.Connection = null;
        }

        private bool Disconnected = false;
        internal bool GroupColorsSent = false;
        
        internal void Disconnect()
        {
            if (GetHabboDataContainer() != null && GetHabboDataContainer().GetInventoryComponent() != null)
                GetHabboDataContainer().GetInventoryComponent().RunDBUpdate();
            if (!Disconnected)
            {
                if (Connection != null)
                    Connection.Destroy();
                Disconnected = true;
            }

            ButterflyEnvironment.Game.GetClientManager().CloseConnection(this.ConnectionID);
        }

        internal void SendMessageWithoutReturning(ServerMessage message)
        {
            if (message == null)
                return;
            if (GetConnection() == null)
                return;
				GetConnection().SendData(message.Bytes(), message.GetLength());

            if (ButterflyEnvironment.displayPacketActivity)
                Logging.WriteLine(string.Format("Sending ID -> {0}", message.GetMessageID()));
        }
        internal void SendMessage(ServerMessage message)
        {
            if (message == null) 
                return;
            SendMessageWithoutReturning(message);
            message.ReturnToOwner();
        }

        public void SendDataToConnection(byte[] buffer, int bufferCutOff)
        {
            if (GetConnection() == null)
                return;
			GetConnection().SendData(buffer, bufferCutOff);
        }

        internal SessionDataProvider GetSessionData()
        {
            return this.sessionData;
        }

        internal void PrepareRoomForUser(uint Id, string Password, bool override_doorbell = false)
        {
            GetHabbo().ClearRoomLoading();
            if (ButterflyEnvironment.ShutdownInitiated)
            {
                SendNotif(LanguageLocale.GetValue("shutdown.alert"));
                return;
            }

            if (GetHabbo().InRoom)
            {
                Room OldRoom = ButterflyEnvironment.Game.GetRoomManager().GetRoom(GetHabbo().CurrentRoomId);

                if (OldRoom != null)
                {
                    OldRoom.GetRoomUserManager().RemoveUserFromRoom(this, false, false);
                }
            }

            Room Room = ButterflyEnvironment.Game.GetRoomManager().LoadRoom(Id);

            if (Room == null || GetHabbo() == null)
                return;

            if (GetHabbo().IsTeleporting && GetHabbo().TeleportingRoomID != Id)
            {
                GetHabbo().TeleportingRoomID = 0;
                GetHabbo().IsTeleporting = false;
                GetHabbo().TeleporterId = 0;
                return;
            }

            GetHabbo().LoadingRoom = Id;


            if (!GetHabboDataContainer().HasFuse("fuse_enter_any_room") && Room.UserIsBanned(GetHabbo().Id))
            {
                if (Room.HasBanExpired(GetHabbo().Id))
                {
                    Room.RemoveBan(GetHabbo().Id);
                }
                else
                {
                    ServerMessage message = PacketHandelingManager.GetRecycledItem(OutputCode.CantConnectMessageEvent);
                    message.AddInt32(4);
                    SendMessage(message);

                    message = PacketHandelingManager.GetRecycledItem(OutputCode.CloseConnectionMessageEvent);
                    SendMessage(message);
                    return;
                }
            }

            if (Room.UsersNow >= Room.UsersMax && !GetHabboDataContainer().HasFuse("fuse_enter_full_rooms"))
            {
                if (!ButterflyEnvironment.Game.GetRoleManager().RankHasRight(GetHabbo().Rank, "fuse_enter_full_rooms"))
                {
                    ServerMessage _message = PacketHandelingManager.GetRecycledItem(OutputCode.CantConnectMessageEvent);
                    _message.AddInt32(1);
                    SendMessage(_message);

                    _message = PacketHandelingManager.GetRecycledItem(OutputCode.CloseConnectionMessageEvent);
                    SendMessage(_message);
                    return;
                }
            }



            if (!GetHabboDataContainer().HasFuse("fuse_enter_any_room") && !Room.CheckRights(this, true) && !(GetHabbo().IsTeleporting && GetHabbo().TeleportingRoomID == Room.RoomId))
            {
                Room currentRoom = ButterflyEnvironment.Game.GetRoomManager().GetRoom(GetHabbo().CurrentRoomId);
                if (Room.State == 1 && !override_doorbell)
                {
                    if (Room.UserCount == 0)
                    {
                        ServerMessage message = PacketHandelingManager.GetRecycledItem(OutputCode.FlatAccessDeniedMessageEvent);
                        SendMessage(message);
                    }
                    else
                    {
                        ServerMessage message = PacketHandelingManager.GetRecycledItem(OutputCode.DoorbellMessageEvent);

                        message.AddString("");
                        SendMessage(message);

                        ServerMessage RingMessage = PacketHandelingManager.GetRecycledItem(OutputCode.DoorbellMessageEvent);
                        RingMessage.AddString(GetHabbo().Username);
                        Room.SendMessageToUsersWithRights(RingMessage);
                    }

                    return;
                }
                else if (Room.State == 2)
                {
                    if (Password.ToLower() != Room.Password.ToLower())
                    {
                        ServerMessage message = PacketHandelingManager.GetRecycledItem(OutputCode.GenericErrorEvent);
                        message.AddInt32(-100002);
                        SendMessage(message);

                        message = PacketHandelingManager.GetRecycledItem(OutputCode.CloseConnectionMessageEvent);
                        SendMessage(message);

                        return;
                    }
                }
            }
            GetHabbo().LoadingChecksPassed = true;

            ButterflyEnvironment.Game.GetRoomManager().RequestRoomLoad(new Butterfly.HabboHotel.Rooms.RoomManager.OnRoomLoad(LoadRoomForUser));
        }

        private void LoadRoomForUser()
        {
            Room room = ButterflyEnvironment.Game.GetRoomManager().GetRoom(GetHabbo().LoadingRoom);
            ServerMessage message;


            if (room == null) //Room unloaded or used disconnected
                return;

            message = PacketHandelingManager.GetRecycledItem(OutputCode.RoomReadyMessageEvent);
            message.AddString(room.ModelName);
            message.AddInt32((int)room.RoomId);
            SendMessage(message);

            room.SerializeRoomLayout(this);
            room.SerializeRights(this);
            room.SerializeRoomRating(this);
        }
    }
}
