﻿using Butterfly.Core;
using Butterfly.Messages;
using Butterfly.Messages.IOCodes;
using System;
using System.Collections;
using System.Diagnostics;
using System.Threading;

namespace Butterfly.HabboHotel.GameClients
{
    class PingHandler
    {
        private Timer handleTimeoutTimer;
        private Stopwatch pingClock;
        private Queue timedOutconnections;
        private Queue pingQueue;
        private readonly ServerMessage pingMessage;

        public PingHandler()
        {
            this.timedOutconnections = new Queue();
            this.pingQueue = new Queue();
            this.handleTimeoutTimer = new Timer(new TimerCallback(HandleTimeouts), null, 0, 5000);
            this.pingMessage = PacketHandelingManager.GetRecycledItem(OutputCode.PingMessageEvent, true);
            this.pingClock = new Stopwatch();
            this.pingClock.Start();
        }

        private void HandleTimeouts(object state)
        {
            try
            {
                while (timedOutconnections.Count > 0)
                {
                    GameClient client = null;
                    lock (timedOutconnections.SyncRoot)
                    {
                        if (timedOutconnections.Count > 0)
                            client = (GameClient)timedOutconnections.Dequeue();
                    }

                    if (client != null)
                        client.Disconnect();
                }
            }
            catch (Exception e)
            {
                Logging.LogThreadException(e.ToString(), "HandleTimeouts");
            }
        }        

        internal void CheckClientState(GameClient client)
        {
            if (client.ignoredPingCount < 3)
            {
                client.ignoredPingCount++;

                lock (pingQueue.SyncRoot)
                {
                    pingQueue.Enqueue(client);
                }
            }
            else
            {
                Logging.WriteLine(client.ConnectionID + " => Connection timed out");

                lock (timedOutconnections.SyncRoot)
                {
                    timedOutconnections.Enqueue(client);
                }
            }
        }

        internal void SendPings()
        {
            DateTime start = DateTime.Now;

            while (pingQueue.Count > 0)
            {
                lock (pingQueue.SyncRoot)
                {
                    GameClient client = (GameClient)pingQueue.Dequeue();
                    client.SendMessage(this.pingMessage);
                }
            }

            TimeSpan spent = DateTime.Now - start;
            if (spent.TotalSeconds > 3)
            {
                Console.WriteLine("High latency in send ping task: " + (int)spent.TotalSeconds);
            }
        }

        internal bool IsTimeForPing()
        {
            ButterflyConfiguration config = ButterflyEnvironment.Configuration;

            if (config.Networking.ClientPingEnabled && pingClock.ElapsedMilliseconds >= config.Networking.ClientPingInterval)
            {
                pingClock.Restart();
                return true;

                // TODO: Redirect all users to this: PING \o\ PONG /o/
                //       https://www.youtube.com/watch?v=Z8V5GfFeUDs
            }

            return false;
        }

        internal void Destroy()
        {
            this.handleTimeoutTimer.Dispose();
        }
    }
}
