﻿using Butterfly.HabboHotel.Rooms;
using Butterfly.HabboHotel.Users;
using Butterfly.HabboHotel.Users.Group;
using Butterfly.Messages;
using Butterfly.Messages.IOCodes;
using Database_Manager.Database.Session_Details.Interfaces;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Text;

namespace Butterfly.HabboHotel.Group
{
    class GroupData : IUpdateListener<Habbo>
    {
        internal readonly uint ID;
		internal string Name;
        internal string Description;
        internal string Badge;

        private List<int> states;
        internal GroupType type;
		internal string Created;

		internal Habbo ownerID;
		internal uint homeRoom;

        internal string image;
        internal int customColorPrimary;
        internal int customColorSecondary;
        internal int baseBadge;
        internal int baseColor;
        internal string htmlColorPrimary;
        internal string htmlColorSecondary;
        internal string petitions;
        internal int rights;
        

		internal Hashtable members;
		private Hashtable membershipRequests;

		/// <summary>
		/// Total count of members in the group
		/// </summary>
		internal int MemberCount
		{
			get
			{
				return members.Count;
			}
		}

		//Called when loaded from DB
        public GroupData(DataRow dRow_groupInfo, DataTable dTable_members, DataTable dTable_membershipRequests)
        {
			this.ID = (uint)dRow_groupInfo["id"];
			this.Name = (string)dRow_groupInfo["name"];
			this.Description = (string)dRow_groupInfo["description"];
            this.Badge = (string)dRow_groupInfo["image"];
            this.Created = (string)dRow_groupInfo["date_create"];
			this.homeRoom = (uint)dRow_groupInfo["room_id"];
			this.type = (GroupType)((int)dRow_groupInfo["type"]);
            this.image = (string)dRow_groupInfo["image"];
            this.customColorPrimary = (int)dRow_groupInfo["custom_color_primary"];
            this.customColorSecondary = (int)dRow_groupInfo["custom_color_secondary"];
            this.baseBadge = (int)dRow_groupInfo["base"];
            this.baseColor = (int)dRow_groupInfo["base_color"];
            this.htmlColorPrimary = (string)dRow_groupInfo["html_color_primary"];
            this.htmlColorSecondary = (string)dRow_groupInfo["html_color_secondary"];
            this.petitions = (string)dRow_groupInfo["petitions"];
            this.rights = (int)dRow_groupInfo["rights_type"];

            this.ownerID = ButterflyEnvironment.Game.GetHabboDataManager().GetHabboData((uint)dRow_groupInfo["owner_id"]);//what to do if owner does not exist? this will be a problem for the whole group!
            if (ownerID == null) return;
            this.ownerID.RegisterUpdater(this);

			this.members = new Hashtable();
			this.membershipRequests = new Hashtable();

			foreach (DataRow dRow in dTable_members.Rows)
			{
                members.Add((uint)dRow[0], new GroupMember(ButterflyEnvironment.Game.GetHabboDataManager().GetHabboData((uint)dRow[0]), this, MembershipStatus.Registered, Convert.ToInt32(dRow[1])));
			}

			foreach (DataRow dRow in dTable_membershipRequests.Rows)
			{
                membershipRequests.Add((uint)dRow[0], new GroupMember(ButterflyEnvironment.Game.GetHabboDataManager().GetHabboData((uint)dRow[0]), this, MembershipStatus.RegistrationPending, 3));
			}
        }
        
        public override bool Equals(object obj)
        {
            return (obj is GroupData && ((GroupData)obj).ID == this.ID);
        }

        public override int GetHashCode()
        {
            return (int)this.ID;
        }

        internal void CancelGroupJoin(uint userID)
        {
           GroupMember m = GetPendingGroupUser(userID);
           if (m != null)
           {
               this.DoDeleteUserEntry(userID);
               this.membershipRequests.Remove(userID);
               this.GetMembership(this.ownerID.Id).OnUserRemovedRequest(m);
           }
        }

        internal List<int> GetStates()
        {
            if (this.states == null)
            {
                this.states = new List<int>();

                DataTable dTable;
                using (IQueryAdapter dbClient = ButterflyEnvironment.DatabaseManager.getQueryreactor())
                {
                    dbClient.setQuery("SELECT state FROM groups_states WHERE group_id = " + this.ID);
                    dTable = dbClient.getTable();
                }

                foreach (DataRow dRow in dTable.Rows)
                {
                    states.Add((int)dRow[0]);
                }
            }

            return this.states;
        }

		/// <summary>
		/// Gets membership status for a userID
		/// </summary>
		/// <param name="userID">ID of the user we want the membership status of</param>
		/// <returns>Membership status</returns>
		internal MembershipStatus GetMembershipStatus(uint userID)
		{
			if (members.ContainsKey(userID))
				return MembershipStatus.Registered;
			else if (membershipRequests.Contains(userID))
				return MembershipStatus.RegistrationPending;
			else
				return MembershipStatus.Unregistered;
		}

		/// <summary>
		/// Requests creation of a group membership
		/// </summary>
		/// <param name="userID">The ID of the user requesting membership</param>
		/// <returns>The type of the created membership</returns>
        internal GroupMember RequestMembership(Habbo data, UserGroupManager manager)
		{
            if (members.ContainsKey(data.Id))
                return null;
            if (membershipRequests.ContainsKey(data.Id))
                return null;

            using (IQueryAdapter dbClient = ButterflyEnvironment.DatabaseManager.getQueryreactor())
            {
                dbClient.setQuery("INSERT INTO `groups_memberships` (userid, groupid, member_rank, is_current, is_pending) VALUES " +
                    "(@userid, @groupid, 3, 0, 1)");
                dbClient.addParameter("groupid", ID);
                dbClient.addParameter("userid", data.Id);
                dbClient.runQuery();
            }

            if (type == GroupType.Open)
            {
                GroupMember toReturn = new GroupMember(data, this, MembershipStatus.Registered, 2);
                toReturn.SetNewClient(manager);
                return ApproveMembership(data.Id, toReturn);
            }
            else
            {
                GroupMember toReturn = new GroupMember(data, this, MembershipStatus.RegistrationPending, 3);
                membershipRequests.Add(data.Id, toReturn);
                toReturn.SetNewClient(manager);
                GetMembership(ownerID.Id).OnNewUserJoinRequest(toReturn);
                return toReturn;
            }
		}

        internal void RegisterOwner(Habbo data, UserGroupManager manager)
        {
            GroupMember owner = new GroupMember(data, this, MembershipStatus.Registered, 0);
            using (IQueryAdapter dbClient = ButterflyEnvironment.DatabaseManager.getQueryreactor())
            {
                dbClient.setQuery("INSERT INTO `groups_memberships` (userid, groupid, member_rank, is_current, is_pending) VALUES " +
                    "(@userid, @groupid, " + owner.GetRank() + ", " + owner.GetRank() + ", 0)");
                dbClient.addParameter("groupid", ID);
                dbClient.addParameter("userid", data.Id);
                dbClient.runQuery();
            }

            AcceptMember(owner);
            owner.SetRank(0);
        }

		/// <summary>
		/// Approves a membership
		/// </summary>
		/// <param name="pendingMemberID">ID of the pending member</param>
		/// <returns>The type of the created membership</returns>
        public GroupMember ApproveMembership(uint pendingMemberID, GroupMember data )
		{
            if (type == GroupType.Open && data != null)
            {
                AcceptMember(data);
            }
            else
            {
                data = GetPendingGroupUser(pendingMemberID);
                AcceptMember(data);
            }
            return data;
		}

        /// <summary>
        /// Accepts a member to the group
        /// </summary>
        /// <param name="member"></param>
        private void AcceptMember(GroupMember member)
        {
            member.GroupJoinAccepted();
            this.BroadcastMemberJoin(member);
            this.members.Add(member.memberID, member);
            this.membershipRequests.Remove(member.memberID);
            member.MembershipApproved();

            if (member.GetRank() != 0)
            {
                using (IQueryAdapter dbClient = ButterflyEnvironment.DatabaseManager.getQueryreactor())
                {
                    dbClient.setQuery("UPDATE `groups_memberships` SET `is_pending`='0', `member_rank`='2' WHERE (`userid`=@userid) AND (`groupid`=@groupid)");
                    dbClient.addParameter("groupid", ID);
                    dbClient.addParameter("userid", member.memberID);
                    dbClient.runQuery();
                }
            }
        }


		/// <summary>
		/// Denies a requesting user to become a member of the group
		/// </summary>
		internal void DisapproveMembership(uint pendingMemberID)
		{
            GroupMember m = GetPendingGroupUser(pendingMemberID);
            if (m != null)
            {
                m.GroupJoinDeclined();
                DoDeleteUserEntry(pendingMemberID);
                membershipRequests.Remove(pendingMemberID);
            }
		}

        /// <summary>
        /// Gets a pending group member
        /// </summary>
        /// <param name="userID"></param>
        /// <returns></returns>
        internal GroupMember GetPendingGroupUser(uint userID)
        {
            return (GroupMember)this.membershipRequests[userID];
        }

        internal int GetPendingUserCount()
        {
            return membershipRequests.Count;
        }

        internal bool IsPendingGroupUser(uint userID)
        {
            return membershipRequests.ContainsKey(userID);
        }

		/// <summary>
		/// Removes a member from the group
		/// </summary>
		/// <param name="memberID">ID of the member</param>
        internal void RemoveMember(uint memberID, GroupLeaveType type)
        {
            GroupMember member = GetMembership(memberID);
            if (member == null)
                return;
            members.Remove(memberID);

            if (ownerID.Id == memberID) //destroy group
            {
                DestroyGroup();
            }
            else
            {
                DoDeleteUserEntry(memberID);
                member.LeftGroup(type);
                member.Clean();
            }

            using (IQueryAdapter dbClient = ButterflyEnvironment.DatabaseManager.getQueryreactor())
            {
                dbClient.runFastQuery("UPDATE users SET group_favorite = 0 WHERE id = " + memberID + " AND group_favorite = " + this.ID);
            }
        }

        internal GroupMember GetMembership(uint memberID)
        {
            return this.members[memberID] as GroupMember;
        }

        private void DoDeleteUserEntry(uint userID)
        {
            using (IQueryAdapter dbClient = ButterflyEnvironment.DatabaseManager.getQueryreactor())
            {
                dbClient.setQuery("DELETE FROM groups_memberships WHERE groupid = @groupid AND userid = @userid");
                dbClient.addParameter("groupid", ID);
                dbClient.addParameter("userid", userID);
                dbClient.runQuery();
            }
        }
		/// <summary>
		/// Removes the group form DB and memory
		/// </summary>
		internal void DestroyGroup()
		{
			using (IQueryAdapter dbClient = ButterflyEnvironment.DatabaseManager.getQueryreactor())
			{
				dbClient.setQuery("DELETE FROM groups_details WHERE id = @groupid");
				dbClient.addParameter("groupid", ID);
				dbClient.runQuery();

				dbClient.setQuery("DELETE FROM groups_memberships WHERE groupid = @groupid");
				dbClient.addParameter("groupid", ID);
				dbClient.runQuery();
			}

            Disbanned();
			ButterflyEnvironment.Game.GetGroupManager().UnloadGroup(ID);
		}

        private void Disbanned()
        {
			if (ownerID != null)
			{
				this.ownerID.RemoveUpdater(this);
				this.ownerID = null;
			}
            lock (membershipRequests.SyncRoot)
            {
                foreach (DictionaryEntry entry in this.members)
                {
                    GroupMember member = entry.Value as GroupMember;
                    member.GroupJoinDeclined();
                    member.Clean();
                }
            }
            this.membershipRequests.Clear();

            lock (members.SyncRoot)
            {
                foreach (DictionaryEntry entry in this.members)
                {
                    GroupMember member = entry.Value as GroupMember;
                    member.LeftGroup(GroupLeaveType.GROUP_DISBANNED);
                    member.Clean();
                }
            }
            this.members.Clear();
        }


        private void BroadcastMemberJoin(GroupMember member)
        {
            lock (this.members.SyncRoot)
            {
                foreach (DictionaryEntry entry in this.members)
                {
                    GroupMember itemnewMember = entry.Value as GroupMember;
                    itemnewMember.GroupUserJoined(member);
                }
            }
        }

        internal override void ItemUpdated(Habbo updatable)
        {
        }

        internal List<GroupMember> GetMembers(string searchPattern)
        {
            List<GroupMember> members = new List<GroupMember>();

            lock (this.members.SyncRoot)
            {
                foreach (DictionaryEntry entry in this.members)
                {
                    GroupMember member = entry.Value as GroupMember;
                    if (member.data.Username.Contains(searchPattern))
                        members.Add(member);
                }
            }

            return members;
        }

        internal List<GroupMember> GetPendingMembers(string searchPattern)
        {
            List<GroupMember> members = new List<GroupMember>();

            lock (this.membershipRequests.SyncRoot)
            {
                foreach (DictionaryEntry entry in this.membershipRequests)
                {
                    GroupMember member = entry.Value as GroupMember;
                    if (member.data.Username.Contains(searchPattern))
                        members.Add(member);
                }
            }

            return members;            
        }

        internal void SetNameAndDescription(string name, string desc)
        {
            this.Name = name;
            this.Description = desc;
            using (IQueryAdapter dbClient = ButterflyEnvironment.DatabaseManager.getQueryreactor())
            {
                dbClient.setQuery("UPDATE groups SET name = @name, description = @desc WHERE id = @id");
                dbClient.addParameter("id", ID);
                dbClient.addParameter("name", name);
                dbClient.addParameter("desc", desc);
                dbClient.runQuery();
            }

            UpdateSettings();
        }

        internal void SetTypeAndRightType(int type, int rightType)
        {
            this.type = (GroupType)type;
            this.rights = rightType;

            using (IQueryAdapter dbClient = ButterflyEnvironment.DatabaseManager.getQueryreactor())
            {
                dbClient.setQuery("UPDATE groups SET type = @type AND rights_type = @rights WHERE id = @id");
                dbClient.addParameter("id", ID);
                dbClient.addParameter("type", type);
                dbClient.addParameter("rights", rightType);
                dbClient.runQuery();

                dbClient.runFastQuery("DELETE FROM groups_memberships WHERE is_pending = '1' AND groupid = " + ID);
            }

            UpdateSettings();
        }

        private void UpdateSettings()
        {
            Room room = ButterflyEnvironment.Game.GetRoomManager().GetRoom(homeRoom);
            
            if (room != null)
            {
                ServerMessage message = PacketHandelingManager.GetRecycledItem(OutputCode.GetGuestRoomResultEvent);
                room.RoomData.SerializeGuestRoomResults(message);
                room.SendMessage(message);
            }
        }

        internal void SetImage(int groupBase, int groupBaseColor, List<int> states)
        {
            this.baseBadge = groupBase;
            this.baseColor = groupBaseColor;
            this.states = states;
            this.image = ButterflyEnvironment.Game.GetGroupManager().GenerateGuildImage(groupBase, groupBaseColor, states);

            using (IQueryAdapter dbClient = ButterflyEnvironment.DatabaseManager.getQueryreactor())
            {
                dbClient.runFastQuery("DELETE FROM groups_states WHERE group_id = " + this.ID);

                foreach (int state in states)
                    dbClient.runFastQuery("INSERT INTO groups_states VALUES (" + ID + ", " + state + ")");

                dbClient.setQuery("UPDATE groups SET base = @baseBadge, base_color = @baseColor, image = @image WHERE id = @id");
                dbClient.addParameter("id", ID);
                dbClient.addParameter("baseBadge", baseBadge);
                dbClient.addParameter("baseColor", baseColor);
                dbClient.addParameter("image", image);
                dbClient.runQuery();
            }

            UpdateSettings();
        }

        internal void SetColors(int colorPrimary, int colorSecondary)
        {
            this.customColorPrimary = colorPrimary;
            this.customColorSecondary = colorSecondary;
            this.htmlColorPrimary = ButterflyEnvironment.Game.GetGroupManager().GetHtmlColor(colorPrimary);
            this.htmlColorSecondary = ButterflyEnvironment.Game.GetGroupManager().GetHtmlColor(colorSecondary);

            using (IQueryAdapter dbClient = ButterflyEnvironment.DatabaseManager.getQueryreactor())
            {
                dbClient.setQuery("UPDATE groups SET html_color_primary = @htmlColorPrimary, html_color_primary = @customColorSecondary, " + 
                    "custom_color_primary = @customColorPrimary, custom_color_secondary = @customColorSecondary WHERE id = @id");
                dbClient.addParameter("id", ID);
                dbClient.addParameter("customColorPrimary", customColorPrimary);
                dbClient.addParameter("customColorSecondary", customColorSecondary);
                dbClient.addParameter("htmlColorPrimary", htmlColorPrimary);
                dbClient.addParameter("htmlColorSecondary", htmlColorSecondary);
                dbClient.runQuery();
            }

            UpdateSettings();
        }

        internal bool IsMember(uint p)
        {
            return members.ContainsKey(p);
        }
    }
}
