﻿using Butterfly.HabboHotel.Users;
using Butterfly.HabboHotel.Users.Group;
using Butterfly.Messages;
using Butterfly.Messages.IOCodes;
using Database_Manager.Database.Session_Details.Interfaces;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Butterfly.HabboHotel.Group
{
	class GroupManager
	{
		private Hashtable groups;
        
        private List<GuildsPartsData> baseBadges;
        private List<GuildsPartsData> colorBadges1;
        private List<GuildsPartsData> colorBadges2;
        private List<GuildsPartsData> colorBadges3;
        private List<GuildsPartsData> symbolBadges;
        private Hashtable htmlBadges;

		public GroupManager()
		{
			this.groups = new Hashtable();
            this.baseBadges = new List<GuildsPartsData>();
            this.symbolBadges = new List<GuildsPartsData>();
            this.colorBadges1 = new List<GuildsPartsData>();
            this.colorBadges2 = new List<GuildsPartsData>();
            this.colorBadges3 = new List<GuildsPartsData>();
            this.htmlBadges = new Hashtable();
		}

        internal void InitGroups(IQueryAdapter dbClient)
        {
            dbClient.setQuery("SELECT * FROM groups_elements");
            DataTable table = dbClient.getTable();
            string type;

            foreach (DataRow row in table.Rows)
            {
                type = (string)row["Type"];
                GuildsPartsData data = new GuildsPartsData((int)row["Id"], (string)row["ExtraData1"], (string)row["ExtraData2"]);

                if (type == "Base")
                {
                    baseBadges.Add(data);
                }
                else if (data.ExtraData1.StartsWith("symbol_"))
                {
                    symbolBadges.Add(data);
                }
                else if (type == "Color1")
                {
                    colorBadges1.Add(data);
                }
                else if (type == "Color2")
                {
                    colorBadges2.Add(data);
                }
                else if (type == "Color3")
                {
                    colorBadges3.Add(data);
                    htmlBadges.Add(data.Id, data.ExtraData1);
                }
            }
        }

		/// <summary>
		/// Gets group data
		/// </summary>
		/// <param name="groupID">ID of the group</param>
		internal GroupData GetGroup(uint groupID)
		{
            if (groupID == 0)
                return null;
			GroupData group;

			if (!groups.ContainsKey(groupID))
			{
				group = LoadGroup(groupID);
				if (group == null)
					return null;
				groups.Add(groupID, group);
			}
			else
				group = (GroupData)groups[groupID];

			return group;
		}

        internal bool GroupExists(uint groupID)
        {
            if (groups.ContainsKey(groupID))
                return true;

            bool found = false;
            using (IQueryAdapter dbClient = ButterflyEnvironment.DatabaseManager.getQueryreactor())
            {
                dbClient.setQuery("SELECT id FROM groups WHERE id = " + groupID);
                found = dbClient.findsResult();
            }

            return found;
        }

        internal ServerMessage SerializeGuildPage()
        {
            ServerMessage message = PacketHandelingManager.GetRecycledItem(Outgoing.SendGuildElements);

            message.AddInt32(baseBadges.Count);
            foreach (GuildsPartsData data in baseBadges)
            {
                message.AddInt32(data.Id);
                message.AddString(data.ExtraData1);
                message.AddString(data.ExtraData2);
            }

            message.AddInt32(symbolBadges.Count);
            foreach (GuildsPartsData data in symbolBadges)
            {
                message.AddInt32(data.Id);
                message.AddString(data.ExtraData1);
                message.AddString(data.ExtraData2);
            }

            message.AddInt32(colorBadges1.Count);
            foreach (GuildsPartsData data in colorBadges1)
            {
                message.AddInt32(data.Id);
                message.AddString(data.ExtraData1);
            }

            message.AddInt32(colorBadges2.Count);
            foreach (GuildsPartsData data in colorBadges2)
            {
                message.AddInt32(data.Id);
                message.AddString(data.ExtraData1);
            }

            message.AddInt32(colorBadges3.Count);
            foreach (GuildsPartsData data in colorBadges3)
            {
                message.AddInt32(data.Id);
                message.AddString(data.ExtraData1);
            }

            return message;
        }

		/// <summary>
		/// Loads the group form the db
		/// </summary>
		/// <param name="groupID">ID of the group to load</param>
		private GroupData LoadGroup(uint groupID)
		{
			DataRow dRow_info;
			DataTable dTable_members;
			DataTable dTable_members_pending;

			using (IQueryAdapter dbClient = ButterflyEnvironment.DatabaseManager.getQueryreactor())
			{

				dbClient.setQuery("SELECT groups.*, users.username FROM groups " +
								"LEFT JOIN users ON (groups.owner_id = users.id) " +
								"WHERE groups.id = @groupid");
				dbClient.addParameter("groupid", groupID);
				dRow_info = dbClient.getRow();

				dbClient.setQuery("SELECT userid FROM groups_memberships WHERE groupid = @groupid AND is_pending = '1'");
				dbClient.addParameter("groupid", groupID);
				dTable_members_pending = dbClient.getTable();

                dbClient.setQuery("SELECT userid, member_rank FROM groups_memberships WHERE groupid = @groupid AND is_pending = '0'");
				dbClient.addParameter("groupid", groupID);
				dTable_members = dbClient.getTable();
			}

            if (dRow_info == null) 
                return null;

			GroupData data = new GroupData(dRow_info, dTable_members, dTable_members_pending);
			if(data.ownerID == null)//group is not OK
				//TODO delete group by using data.disbanned and return null or something
				//USER will still crash but the faulty group is than deleted., should fix
			{
				data.DestroyGroup();
			}

			return data;
		}

		/// <summary>
		/// Removes a group instance
		/// </summary>
		/// <param name="groupID">ID of the group to be removed</param>
		internal void UnloadGroup(uint groupID)
		{
			groups.Remove(groupID);
		}


        internal string GenerateGuildImage(int GuildBase, int GuildBaseColor, List<int> states)
        {
            StringBuilder image = new StringBuilder(String.Format("b{0:00}{1:00}", GuildBase, GuildBaseColor));

            for (int i = 0; i < 3 * 4; i += 3)
            {
                if (i >= states.Count)
                    image.Append("s");
                else
                    image.Append(String.Format("s{0:00}{1:00}{2}", states[i] - 20, states[i + 1], states[i + 2]));
            }

            return image.ToString();
        }

        internal string GetHtmlColor(int color)
        {
            string val = htmlBadges[color] as string;
            return string.IsNullOrEmpty(val) ? string.Empty : val;
        }

        internal GroupData RegisterGroup(string name, uint ownerid, string description, int roomid, string image, int customcolor1, 
            int customcolor2, int guildbase, int guildbasecolor, string htmlcolor1, string htmlcolor2, List<int> states, Habbo owner, UserGroupManager manager)
        {
            uint groupID;
            using (IQueryAdapter dbClient = ButterflyEnvironment.DatabaseManager.getQueryreactor())
            {

                dbClient.setQuery("INSERT INTO groups (name, owner_id, description, room_id, image, custom_color_primary, " + 
                    "custom_color_secondary, base, base_color, html_color_primary, html_color_secondary, date_create) " + 
                    "VALUES (@name, @ownerid, @description, @roomid, @image, @customprimaryhtml, @customsecondaryhtml, " +
                    "@base, @basecolor, @htmlcolorprimary, @htmlcolorsecondary, @created)");
                
                dbClient.addParameter("name", name);
                dbClient.addParameter("description", description);
                dbClient.addParameter("ownerid", owner.Id);
                dbClient.addParameter("roomid", roomid);
                dbClient.addParameter("created", DateTime.Now.ToShortDateString());
                dbClient.addParameter("image", image);
                dbClient.addParameter("customprimaryhtml", customcolor1);
                dbClient.addParameter("customsecondaryhtml", customcolor2);
                dbClient.addParameter("base", guildbase);
                dbClient.addParameter("basecolor", guildbasecolor);
                dbClient.addParameter("htmlcolorprimary", htmlcolor1);
                dbClient.addParameter("htmlcolorsecondary", htmlcolor2);

                groupID = (uint)dbClient.insertQuery();

                dbClient.runFastQuery("UPDATE rooms SET group_id = " + groupID + " WHERE id = " + roomid);
                dbClient.runFastQuery("UPDATE users SET group_favorite = " + groupID + " WHERE id = " + owner.Id);

                foreach (int state in states)
                    dbClient.runFastQuery("INSERT INTO groups_states (group_id, state) VALUES (" + groupID + "," + state + ")");
            }

            owner.EquippedGroup = groupID;
            GroupData group = GetGroup(groupID);
            group.RegisterOwner(owner, manager);

            return group;
        }
    }
}
