﻿using Butterfly.HabboHotel.GameClients;
using Butterfly.HabboHotel.Users;
using Butterfly.HabboHotel.Users.Group;
using Butterfly.Messages;
using Butterfly.Messages.IOCodes;
using Database_Manager.Database.Session_Details.Interfaces;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Butterfly.HabboHotel.Group
{
    class GroupMember : IUpdateListener<Habbo>
    {
        public MembershipStatus status { get; private set; }
        internal Habbo data { get; private set; }
        private UserGroupManager mClient;
        private GroupData theGroup;
        public GroupData Group { get { return theGroup; } }
        public uint memberID { get { return data.Id; } }
        private int rank;

        internal int GetRank()
        {
            return rank;
        }

        internal void SetRank(int rank)
        {
            this.rank = rank;

            using (IQueryAdapter dbClient = ButterflyEnvironment.DatabaseManager.getQueryreactor())
            {
                dbClient.runFastQuery("UPDATE groups_memberships SET member_rank = " + rank + " WHERE userid = " + data.Id + " AND groupid = " + Group.ID);
            }
        }

        internal void MembershipApproved()
        {
            this.rank = 2;
        }

        public GroupMember(Habbo data, GroupData theGroup, MembershipStatus status, int rank)
        {
            this.data = data;
            this.data.RegisterUpdater(this);
            this.status = status;
            this.theGroup = theGroup;
            this.rank = rank;
        }

        internal void Clean()
        {
            if (data != null)
            {
                this.data.RemoveUpdater(this);
                this.data = null;
                this.mClient = null;
                this.theGroup = null;
            }
        }

        internal override void ItemUpdated(Habbo updatable)
        {
            //PLEASE IGNORE
        }


        /// <summary>
        /// Indication that a new member has joined the group
        /// </summary>
        /// <param name="member">The member that has joined</param>
        internal void GroupUserJoined(GroupMember member)
        {
            //TODO send the new user maybe
        }

        #region group joining/leaving
        /// <summary>
        /// Only called when the user is the owner of the group
        /// </summary>
        /// <param name="m">The member that has cancelled his own Group join request</param>
        internal void OnUserRemovedRequest(GroupMember m)
        {
            //TODO send new group join request packet? or remove user from join request list maybe
        }

        /// <summary>
        /// Indication that this user has left the group
        /// </summary>
        /// <param name="leaveType"></param>
        internal void LeftGroup(GroupLeaveType leaveType)
        {
            //TODO create left user message
            switch (leaveType)
            {
                case GroupLeaveType.GROUP_DISBANNED:
                case GroupLeaveType.USER_DELETE:
                    {
                        if (mClient != null)
                        {
                            ServerMessage message = PacketHandelingManager.GetRecycledItem(Outgoing.SendGroup);
                            message.AddInt32(0);
                            mClient.SendMessage(message);
                        }
                    }
                    break;
            }
        }

        /// <summary>
        /// Indication the the group join was declined by the owner
        /// </summary>
        internal void GroupJoinDeclined()
        {
            this.data.RemovePendingGroupJoin(this);
            //TODO send group join declined or something
            if (this.mClient != null)
            { this.mClient.RemovePendingGroupJoin(this); }
            this.Clean();
        }

        /// <summary>
        /// Indication that the group join has been accepted by the server/owner
        /// </summary>
        internal void GroupJoinAccepted()
        {
            this.data.RemovePendingGroupJoin(this);
            this.data.AddNewMembership(this);
            if (this.mClient != null)
            {
                this.mClient.RemovePendingGroupJoin(this);
                this.mClient.AddNewMembership(this);
            }
            //TODO send group accepted to user
        }

        /// <summary>
        /// Indication that the group join request has been created
        /// </summary>
        /// <param name="client">the client that requested the join</param>
        internal void GroupJoinRequestCreated(UserGroupManager client)
        {
            this.SetNewClient(client);
            this.data.AddPendingGroupJoin(this);
            if (this.mClient != null)
            {
                this.mClient.AddPendingGroupJoin(this);
            }
        }

        /// <summary>
        /// Owner receives new member request
        /// </summary>
        /// <param name="joinedMember">The member that has joined</param>
        internal void OnNewUserJoinRequest(GroupMember joinedMember)
        {
            //TODO send packet to client
        }

        #endregion


        #region client call locations
        internal void LeaveGroup()
        {
            this.theGroup.RemoveMember(data.Id, GroupLeaveType.USER_DELETE);
        }

        internal void CancelGroupJoin()
        {
            this.theGroup.CancelGroupJoin(this.data.Id);
           
            this.Clean();
        }
        #endregion

        /// <summary>
        /// Sends a message to the client of the attatched client, if any
        /// </summary>
        /// <param name="message">The server message to send</param>
        private void SendMessage(ServerMessage message)
        {
            if (this.mClient != null)
                mClient.SendMessage(message);
        }

        public void SetNewClient(UserGroupManager client)
        {
            this.mClient = client;
        }

        internal void Serialize(ServerMessage message)
        {
            message.AddInt32(this.rank);
            message.AddUInt32(data.Id);
            message.AddString(data.Username);
            message.AddString(data.Look);
            message.AddString(string.Empty); //todo
        }
    }

}
