﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Butterfly.HabboHotel.Rooms;

namespace Butterfly.HabboHotel.Items.FurniInteractors.Interactors
{
    [InteractorAttribute(InteractionType.teleport)]
    class TeleporterInteractor : IInteractable
    {
        public void ExecuteAction(RoomItem theItem)
        {
            RoomUser User = null;
            RoomUser User2 = null;

            bool keepDoorOpen = false;
            bool showTeleEffect = false;

            // Do we have a primary user that wants to go somewhere?
            if (theItem.InteractingUser > 0)
            {
                User = theItem.GetRoom().GetRoomUserManager().GetRoomUserByHabbo(theItem.InteractingUser);

                // Is theItem user okay?
                if (User != null)
                {
                    // Is he in the tele?
                    if (User.Coordinate == theItem.Coordinate)
                    {
                        //Remove the user from the square
                        User.AllowOverride = false;

                        if (TeleHandler.IsTeleLinked(theItem.Id, theItem.Room))
                        {
                            showTeleEffect = true;

                            if (true)
                            {
                                // Woop! No more delay.
                                uint TeleId = TeleHandler.GetLinkedTele(theItem.Id, theItem.Room);
                                uint RoomId = TeleHandler.GetTeleRoomId(TeleId, theItem.Room);

                                // Do we need to tele to the same room or gtf to another?
                                if (RoomId == theItem.RoomId)
                                {
                                    RoomItem Item = theItem.GetRoom().GetRoomItemHandler().GetItem(TeleId);

                                    if (Item == null)
                                    {
                                        User.UnlockWalking();
                                    }
                                    else
                                    {


                                        // Set pos
                                        User.SetPos(Item.GetX, Item.GetY, Item.GetZ);
                                        User.SetRot(Item.Rot, false);

                                        // Force tele effect update (dirty)
                                        Item.ExtraData = "2";
                                        Item.UpdateState(false, true);

                                        // Set secondary interacting user
                                        Item.InteractingUser2 = theItem.InteractingUser;
                                    }
                                }
                                else
                                {
                                    // Let's run the teleport delegate to take futher care of theItem.. WHY DARIO?!
                                    if (!User.IsBot && User != null && User.GetClient() != null && User.GetClient().GetHabbo() != null)
                                    {
                                        User.GetClient().GetHabbo().IsTeleporting = true;
                                        User.GetClient().GetHabbo().TeleportingRoomID = RoomId;
                                        User.GetClient().GetHabbo().TeleporterId = TeleId;
                                        User.GetClient().PrepareRoomForUser(RoomId, "");
                                    }
                                    //ButterflyEnvironment.Game.GetRoomManager().AddTeleAction(new TeleUserData(User.GetClient().GetMessageHandler(), User.GetClient().GetHabbo(), RoomId, TeleId));
                                }

                                // We're done with theItem tele. We have another one to bother.
                                theItem.InteractingUser = 0;
                            }
                            else
                            {
                                // We're linked, but there's a delay, so decrease the delay and wait it out.
                                //User.TeleDelay--;
                            }
                        }
                        else
                        {
                            // theItem tele is not linked, so let's gtfo.
                            User.UnlockWalking();
                            theItem.InteractingUser = 0;
                        }
                    }
                    // Is he in front of the tele?
                    else if (User.Coordinate == theItem.SquareInFront)
                    {
                        User.AllowOverride = true;
                        // Open the door
                        keepDoorOpen = true;

                        // Lock his walking. We're taking control over him. Allow overriding so he can get in the tele.
                        if (User.IsWalking && (User.GoalX != theItem.GetX || User.GoalY != theItem.GetY))
                        {
                            User.ClearMovement(true);
                        }

                        User.CanWalk = false;
                        User.AllowOverride = true;

                        // Move into the tele
                        User.MoveTo(theItem.Coordinate.X, theItem.Coordinate.Y, true);
                    }
                    // Not even near, do nothing and move on for the next user.
                    else
                    {
                        theItem.InteractingUser = 0;
                    }
                }
                else
                {
                    // Invalid user, do nothing and move on for the next user. 
                    theItem.InteractingUser = 0;
                }
            }

            // Do we have a secondary user that wants to get out of the tele?
            if (theItem.InteractingUser2 > 0)
            {
                User2 = theItem.GetRoom().GetRoomUserManager().GetRoomUserByHabbo(theItem.InteractingUser2);

                // Is theItem user okay?
                if (User2 != null)
                {
                    // If so, open the door, unlock the user's walking, and try to push him out in the right direction. We're done with him!
                    keepDoorOpen = true;
                    User2.UnlockWalking();
                    User2.MoveTo(theItem.SquareInFront);
                }

                // theItem is a one time thing, whether the user's valid or not.
                theItem.InteractingUser2 = 0;
            }

            // Set the new item state, by priority
            if (keepDoorOpen)
            {
                if (theItem.ExtraData != "1")
                {
                    theItem.ExtraData = "1";
                    theItem.UpdateState(false, true);
                }
            }
            else if (showTeleEffect)
            {
                if (theItem.ExtraData != "2")
                {
                    theItem.ExtraData = "2";
                    theItem.UpdateState(false, true);
                }
            }
            else
            {
                if (theItem.ExtraData != "0")
                {
                    theItem.ExtraData = "0";
                    theItem.UpdateState(false, true);
                }
            }

            // We're constantly going!
            theItem.ReqUpdate(1, false);
        }
    }
}
