﻿using Butterfly.HabboHotel.GameClients;
using Butterfly.HabboHotel.Group;
using Butterfly.HabboHotel.Items.FurniInteractors;
using Butterfly.HabboHotel.Rooms;

namespace Butterfly.HabboHotel.Items.FurniTriggers.Triggers
{
    [TriggerAttribute(InteractionType.groupGate)]
    class GroupGate : IFurniTrigger
    {private void SetExtraData(string data, RoomItem item)
        {
            string[] extraData = item.ExtraData.Split(':');
            item.ExtraData = extraData[0] + ':' + data;
        }
        

        public bool CanBeTriggered(GameClient Session, RoomItem Item, int Request, bool UserHasRights)
        {
            string[] extraData = Item.ExtraData.Split(':');
            uint groupID = TextHandling.ParseUInt32(extraData[0]);
            GroupData group = ButterflyEnvironment.Game.GetGroupManager().GetGroup(groupID);

            return (group != null && group.IsMember(Session.GetHabbo().Id) && groupID == Session.GetHabbo().EquippedGroup);
        }
        void IFurniTrigger.OnPlace(GameClient Session, RoomItem Item)
        {
            SetExtraData("0", Item);

            if (Item.InteractingUser != 0)
            {
                RoomUser User = Item.GetRoom().GetRoomUserManager().GetRoomUserByHabbo(Item.InteractingUser);

                if (User != null)
                {
                    User.ClearMovement(true);
                    User.UnlockWalking();
                }

                Item.InteractingUser = 0;
            }
        }

        void IFurniTrigger.OnRemove(GameClient Session, RoomItem Item)
        {
            SetExtraData("0", Item);

            if (Item.InteractingUser != 0)
            {
                RoomUser User = Item.GetRoom().GetRoomUserManager().GetRoomUserByHabbo(Item.InteractingUser);

                if (User != null)
                {
                    User.ClearMovement(true);
                    User.UnlockWalking();
                }

                Item.InteractingUser = 0;
            }
        }

        void IFurniTrigger.OnTrigger(GameClient Session, RoomItem Item, int Request, bool UserHasRights)
        {
            if (Session == null)
                return;
            RoomUser User = Item.GetRoom().GetRoomUserManager().GetRoomUserByHabbo(Session.GetHabbo().Id);

            if (User == null)
            {
                return;
            }

            if (User.Coordinate != Item.SquareInFront && User.CanWalk)
            {
                User.MoveTo(Item.SquareInFront);
                return;
            }

            if (!Item.GetRoom().GetGameMap().CanWalk(Item.SquareBehind.X, Item.SquareBehind.Y, User.AllowOverride))
            {
                return;
            }

            string[] extraData = Item.ExtraData.Split(':');
            uint groupID = TextHandling.ParseUInt32(extraData[0]);
            GroupData group = ButterflyEnvironment.Game.GetGroupManager().GetGroup(groupID);

            if (group == null || !group.IsMember(Session.GetHabbo().Id))
                return;

            if (Item.InteractingUser == 0)
            {
                Item.InteractingUser = User.HabboId;

                User.CanWalk = false;

                if (User.IsWalking && (User.GoalX != Item.SquareInFront.X || User.GoalY != Item.SquareInFront.Y))
                {
                    User.ClearMovement(true);
                }

                SetExtraData("1", Item);
                Item.UpdateState();
                User.AllowOverride = true;
                User.MoveTo(Item.Coordinate);

                Item.ReqUpdate(4, true);
            }
        }
    }
}
