﻿using System;
using System.Collections.Generic;
using System.Data;
using Butterfly.Core;
using Butterfly.HabboHotel.Items;
using Butterfly.HabboHotel.Items.FurniInteractors;
using Database_Manager.Database.Session_Details.Interfaces;
using System.Reflection;
using System.Collections;

namespace Butterfly.HabboHotel.Items
{
    class ItemManager
    {
		private Hashtable items;

        internal ItemManager()
        {
            InitializeHandlers();
        }

        private static void InitializeHandlers()
        {
            //foreach (var assembly in AppDomain.CurrentDomain.GetAssemblies())
            //{
            //    if (assembly.FullName.Contains("Butterfly Emulator"))
            //    {
            try
            {
                foreach (var type in Assembly.GetExecutingAssembly().GetTypes())
                {
                    FurniInteractorFactory.LoadType(type);
                    FurniIInitializerFactory.LoadType(type);
                    FurniTriggerFactory.LoadType(type);
                }
            }
            catch (ReflectionTypeLoadException ex)
            {
                foreach (Exception exs in ex.LoaderExceptions)
                {
                    Console.WriteLine("==================");
                    Console.WriteLine(exs);
                    Console.WriteLine("==================");
                }
            }
            //    }
            //}
        }

        internal void LoadItems(IQueryAdapter dbClient)
        {

            dbClient.setQuery("SELECT * FROM items_base");
            DataTable ItemData = dbClient.getTable();
			dbClient.setQuery("SELECT * FROM items_base_height");
			DataTable itemHeights = dbClient.getTable();

			Hashtable heights = new Hashtable();
			this.items = new Hashtable(ItemData.Rows.Count);

			uint baseID;
			uint heightID;
			double height;

			foreach (DataRow dRow in itemHeights.Rows)
			{
				baseID = (uint)dRow[0];
				heightID = (uint)dRow[1];
				height = (double)dRow[2];

				if (heights.ContainsKey(baseID))
				{
					((SortedDictionary<uint, double>)heights[baseID]).Add(heightID, height);
				}
				else
				{
					SortedDictionary<uint, double> _heights = new SortedDictionary<uint, double>();
					_heights.Add(heightID, height);
					heights.Add(baseID, _heights);
				}
			}

            if (ItemData != null)
            {
                ushort id;
                int spriteID;
                string publicName;
                string itemName;
                string type;
                int width;
                int length;
                bool allowStack;
                bool allowWalk;
                bool allowSit;
                bool allowRecycle;
                bool allowTrade;
                bool allowMarketplace;
                bool allowGift;
                bool allowInventoryStack;
                InteractionType interactionType;
                int cycleCount;
                string vendingIDS;
                bool isGroupItem;

                foreach (DataRow dRow in ItemData.Rows)
                {
                    try
                    {
                        id = Convert.ToUInt16(dRow[0]);
                        spriteID = (int)dRow[1];
                        publicName = (string)dRow[2];
                        itemName = (string)dRow[3];
                        type = (string)dRow[4];
                        width = (int)dRow[5];
                        length = (int)dRow[6];
                        height = Convert.ToDouble(dRow[7]);
                        allowStack = Convert.ToInt32(dRow[8]) == 1;
                        allowWalk = Convert.ToInt32(dRow[9]) == 1;
                        allowSit = Convert.ToInt32(dRow[10]) == 1;
                        allowRecycle = Convert.ToInt32(dRow[11]) == 1;
                        allowTrade = Convert.ToInt32(dRow[12]) == 1;
                        allowMarketplace = Convert.ToInt32(dRow[13]) == 1;
                        allowGift = Convert.ToInt32(dRow[14]) == 1;
                        allowInventoryStack = Convert.ToInt32(dRow[15]) == 1;
                        interactionType = InterractionTypes.GetTypeFromString((string)dRow[16]);
                        cycleCount = (int)dRow[17];
                        vendingIDS = (string)dRow[18];
                        isGroupItem = (bool)dRow["is_groupitem"];

                        Item item = new Item(id, spriteID, publicName, itemName, type, width, length, height, allowStack, allowWalk, allowSit, allowRecycle, allowTrade, allowMarketplace, allowGift, allowInventoryStack, interactionType, cycleCount, vendingIDS, (SortedDictionary<uint, double>)heights[id], isGroupItem);
                        items.Add(id, item);
                    }
                    catch (Exception e)
                    {
                        Console.WriteLine(e.ToString());
                        Console.ReadKey();
                        Logging.WriteLine("Could not load item #" + Convert.ToUInt32(dRow[0]) + ", please verify the data is okay.");
                    }
                }
            }
        }

		internal Item GetItem(uint id)
		{
			return GetItem((ushort)id);
		}

        internal Item GetBaseItem(int baseID)
        {
            foreach (DictionaryEntry entry in items)
            {
                Item item = (Item)entry.Value;
                if (item.SpriteId == baseID)
                    return item;
            }
            return null;
        }

		internal Item GetItem(ushort id)
        {
			return (Item)items[id];
        }
    }
}
