﻿using System;
using System.Collections.Generic;
using System.Drawing;
using Butterfly.Core;
using Butterfly.HabboHotel.Pathfinding;
using Butterfly.HabboHotel.Rooms;
using Butterfly.HabboHotel.Rooms.Games;
using Butterfly.HabboHotel.Rooms.Wired;
using Butterfly.Messages;
using Butterfly.HabboHotel.Rooms.Wired.WiredHandlers.Interfaces;
using Butterfly.Messages.IOCodes;
using Database_Manager.Database.Session_Details.Interfaces;
using Butterfly.HabboHotel.Items.FurniTriggers;
using Butterfly.HabboHotel.Items.FurniTriggers.Triggers;
using Butterfly.HabboHotel.Users.Inventory;
using Butterfly.HabboHotel.Users;
using Butterfly.HabboHotel.Group;
using System.Diagnostics;

namespace Butterfly.HabboHotel.Items
{
    internal delegate void OnItemTrigger(object sender, ItemTriggeredArgs e);
	internal delegate void OnItemWalk(RoomUser user, bool cyclegameitems);

    internal class RoomItem : IEquatable<RoomItem>
    {
        internal const int MAX_ITEM_ALLOWED = 2500;
        internal UInt32 Id;
        internal UInt32 RoomId;
        internal UInt32 BaseItem;
        internal string ExtraData;
        internal string Figure;
        internal string Gender;
        internal uint interactingBallUser;
        internal Team team;
        internal byte interactionCountHelper;
        internal byte interactionCount;
        internal int value;
        internal FreezePowerUp freezePowerUp;
        internal IWiredCondition wiredCondition;
        internal IWiredTrigger wiredHandler;
        internal event OnItemTrigger itemRemovedEventHandler;
        internal event OnItemTrigger itemMovedEventHandler;
        internal event OnItemTrigger itemTriggerEventHandler;
        internal event UserWalksFurniDelegate OnUserWalksOffFurni;
        internal event UserWalksFurniDelegate OnUserWalksOnFurni;

        private Dictionary<int, ThreeDCoord> mAffectedPoints;
        private GiftData giftData;
        
        
        internal Dictionary<int, ThreeDCoord> GetAffectedTiles
        {
            get
            {
                return mAffectedPoints;
            }
        }

        private int mX; //byte
        internal int GetX
        {
            get
            {
                return mX;
            }
        }

        private int mY;//byte

        internal int GetY
        {
            get
            {
                return mY;
            }
        }

        private double mZ;

        internal Double GetZ
        {
            get
            {
                return mZ;
            }
        }
     

        internal void SetState(int pX, int pY, Double pZ, Dictionary<int, ThreeDCoord> Tiles)
        {
            this.mX = pX;
            this.mY = pY;
            if (!double.IsInfinity(pZ))
            {
                this.mZ = pZ;
            }
            this.mAffectedPoints = Tiles;
        }

        internal int Rot;//byte

        internal WallCoordinate wallCoord;

        private bool updateNeeded;
        internal bool UpdateNeeded
        {
            get
            {
                return updateNeeded;
            }
            set
            {
                if (value == true)
                    GetRoom().GetRoomItemHandler().QueueRoomItemUpdate(this);
                updateNeeded = value;
            }
        }
        internal int UpdateCounter; //byte

        internal UInt32 InteractingUser;
        internal UInt32 InteractingUser2;

        public Room Room {
            get { return mRoom; }
        }

        private Item mBaseItem;
        private Room mRoom;
        private bool mIsWallItem;
        private bool mIsFloorItem;

        private bool mIsRoller;
        internal bool IsTrans;
        internal bool pendingReset = false;
        private uint userid;

        internal bool IsRoller
        {
            get
            {
                return mIsRoller;
            }
        }

        internal Point Coordinate
        {
            get
            {
                return new Point(mX, mY);
            }
        }

		internal double Height
		{
			get
			{
				if (mBaseItem.IsMultiHeight)
				{
					uint mode = (uint)TextHandling.Parse(ExtraData);

					double extraHeight;
					if (mBaseItem.HeightModes.TryGetValue(mode, out extraHeight))
						return mBaseItem.Height + extraHeight;
				}

				return mBaseItem.Height;
			}
		}

		internal int GetDoubleCoordinate()
		{
			return TextHandling.CombineXYCoord(mX, mY);
		}

        internal List<Point> GetCoords
        {
            get
            {
                List<Point> toReturn = new List<Point>();
                foreach (ThreeDCoord tile in mAffectedPoints.Values)
                {
                    toReturn.Add(new Point(tile.X, tile.Y));
                }

                return toReturn;
            }
        }

		internal List<int> GetDoubleCoordinates()
		{
			List<int> toReturn = new List<int>();
			
			foreach (ThreeDCoord tile in mAffectedPoints.Values)
			{
				toReturn.Add(TextHandling.CombineXYCoord(tile.X, tile.Y));
			}

			return toReturn;
		}

        internal double TotalHeight
        {
            get
            {
                return mZ + Height;
            }
        }

        internal bool IsWallItem
        {
            get
            {
                return mIsWallItem;
            }
        }

        internal bool IsFloorItem
        {
            get
            {
                return mIsFloorItem;
            }
        }

        internal Point SquareInFront
        {
            get
            {
                Point Sq = new Point(mX, mY);

                if (Rot == 0)
                {
                    Sq.Y--;
                }
                else if (Rot == 2)
                {
                    Sq.X++;
                }
                else if (Rot == 4)
                {
                    Sq.Y++;
                }
                else if (Rot == 6)
                {
                    Sq.X--;
                }

                return Sq;
            }
        }

		internal int GetSquareInFront(out int X, out int Y)
		{
			X = mX;
			Y = mY;

			if (Rot == 0)
				Y--;
			else if (Rot == 2)
				X++;
			else if (Rot == 4)
				Y++;
			else if (Rot == 6)
				X--;

			return TextHandling.CombineXYCoord(X, Y);
		}

        internal Point SquareBehind
        {
            get
            {
                Point Sq = new Point(mX, mY);

                if (Rot == 0)
                {
                    Sq.Y++;
                }
                else if (Rot == 2)
                {
                    Sq.X--;
                }
                else if (Rot == 4)
                {
                    Sq.Y--;
                }
                else if (Rot == 6)
                {
                    Sq.X++;
                }

                return Sq;
            }
        }

        internal IFurniTrigger Interactor
        {
            get
            {
                if (GetBaseItem().FurniTrigger != null)
                    return GetBaseItem().FurniTrigger;
                else
                    return GenericTriggerWithModes.defaultTrigger;
            }
        }

        internal void OnTrigger(RoomUser user)
        {
            if (itemTriggerEventHandler != null)
                itemTriggerEventHandler(null, new ItemTriggeredArgs(user, this));
        }

        internal void OnItemMoved(RoomUser user)
        {
            if (itemMovedEventHandler != null)
                itemMovedEventHandler(null, new ItemTriggeredArgs(user, this));
        }

        internal void OnItemRemoved(RoomUser user)
        {
            if (itemRemovedEventHandler != null)
                itemRemovedEventHandler(null, new ItemTriggeredArgs(user, this));
        }

        internal RoomItem(UInt32 Id, UInt32 RoomId, UInt32 BaseItem, UInt32 userid, string ExtraData, int X, int Y, Double Z, int Rot, Room pRoom)
        {
            this.Id = Id;
            this.RoomId = RoomId;
            this.BaseItem = BaseItem;
            this.ExtraData = ExtraData;
            this.userid = userid;
            this.mX = X;
            this.mY = Y;
            if(!double.IsInfinity(Z))
                this.mZ = Z;
            this.Rot = Rot;
            this.UpdateNeeded = false;
            this.UpdateCounter = 0;
            this.InteractingUser = 0;
            this.InteractingUser2 = 0;
            this.IsTrans = false;
            this.interactingBallUser = 0;
            this.interactionCount = 0;
            this.value = 0;

            mBaseItem = ButterflyEnvironment.Game.GetItemManager().GetItem(BaseItem);
            mRoom = pRoom; //Todo: rub my penis

            if (GetBaseItem() == null)
                Logging.LogException("Unknown baseID: " + BaseItem);

            if (GetBaseItem().FurniInitializer != null)
            {
                GetBaseItem().FurniInitializer.InitializeItem(this);
            }

            mIsWallItem = (GetBaseItem().Type == 'i');
            mIsFloorItem = (GetBaseItem().Type == 's');
            mAffectedPoints = Gamemap.GetAffectedTiles(GetBaseItem().Length, GetBaseItem().Width, mX, mY, Rot);
            LoadGift();
        }

        internal RoomItem(UInt32 Id, UInt32 RoomId, UInt32 BaseItem, string ExtraData, WallCoordinate wallCoord, Room pRoom)
        {
            this.Id = Id;
            this.RoomId = RoomId;
            this.BaseItem = BaseItem;
            this.ExtraData = ExtraData;
            this.mX = 0;
            this.mY = 0;
            this.mZ = 0.0;
            this.UpdateNeeded = false;
            this.UpdateCounter = 0;
            this.InteractingUser = 0;
            this.InteractingUser2 = 0;
            this.IsTrans = false;
            this.interactingBallUser = 0;
            this.interactionCount = 0;
            this.value = 0;
            this.wallCoord = wallCoord;

            mBaseItem = ButterflyEnvironment.Game.GetItemManager().GetItem(BaseItem);
            mRoom = pRoom;

            if (GetBaseItem() == null)
                Logging.LogException("Unknown baseID: " + BaseItem);

            mIsWallItem = true;
            mIsFloorItem = false;
            mAffectedPoints = new Dictionary<int, ThreeDCoord>();
            LoadGift();
        }

        private void LoadGift()
        {
            if (GetBaseItem().IsGift && ExtraData.Length >= 12)
            {
                GiftData giftData = new GiftData(ExtraData);

                if (giftData.wrappingPaper < 20 && giftData.decorationType < 20)
                    this.giftData = giftData;
            }
        }

        internal void Destroy()
        {
            mRoom = null;
            mAffectedPoints.Clear();

            if (wiredHandler != null)
                wiredHandler.Dispose();
            wiredHandler = null;
            itemTriggerEventHandler = null;
            OnUserWalksOffFurni = null;
            OnUserWalksOnFurni = null;
        }

        public bool Equals(RoomItem comparedItem)
        {
            return (comparedItem.Id == this.Id);
        }

        internal void ProcessUpdates()
        {
            this.UpdateCounter--;

            if (this.UpdateCounter <= 0 || IsTrans)
            {
                this.UpdateNeeded = false;
                this.UpdateCounter = 0;

                if (GetBaseItem().FurniInteractor != null)
                {
                    GetBaseItem().FurniInteractor.ExecuteAction(this);
                }
            }
        }

        internal void ReqUpdate(int Cycles, bool setUpdate)
        {
            this.UpdateCounter = Cycles;
            if (setUpdate)
                this.UpdateNeeded = true;
        }

        internal void UpdateState()
        {
            UpdateState(true, true);
        }

        internal void UpdateState(bool inDb, bool inRoom)
        {
            if (GetRoom() == null)
                return;

            if (inDb)
            {
                GetRoom().GetRoomItemHandler().UpdateItem(this);
            }

            if (inRoom)
            {
                ServerMessage Message;

                if (IsFloorItem)
                {
                    Message = PacketHandelingManager.GetRecycledItem(OutputCode.ObjectDataUpdateMessageEvent);
                    Message.AddString(Id.ToString());
                    Message.AddInt32(0);

                    if (GetBaseItem().IsGroupItem)
                    {
                        string[] extra = ExtraData.Split(':');
                        int pos = this.ExtraData.IndexOf(':');
                        string waroi = this.ExtraData.Substring(pos + 1);
                        Message.AddString(waroi);
                    }
                    else
                        Message.AddString(ExtraData);
                }
                else
                {
                    Message = PacketHandelingManager.GetRecycledItem(OutputCode.ItemUpdateMessageEvent);
                    Serialize(Message);
                }

                GetRoom().SendMessage(Message);
            }
        }

        internal void Serialize(ServerMessage Message)
        {
            if (IsFloorItem)
            {
                Message.AddUInt32(Id);
                Message.AddInt32(GetBaseItem().SpriteId);
                Message.AddInt32(mX);
                Message.AddInt32(mY);
                Message.AddInt32(Rot);
                Message.AddString(String.Format("{0:0.00}", TextHandling.GetString(mZ)));

                if (this.GetBaseItem().Name == "ads_mpu_720" || this.GetBaseItem().Name == "ads_background" || this.GetBaseItem().Name == "ads_mpu_300" || this.GetBaseItem().Name == "ads_mpu_160")
                {
                    Message.AddInt32(0);
                    Message.AddInt32(1);
                    string[] parsedData = ExtraData.Split(Convert.ToChar(9));
                    if (ExtraData != "" && parsedData.Length > 1)
                    {

                        Message.AddInt32(parsedData.Length / 2);

                        for (int i = 0; i <= parsedData.Length - 1; i++)
                        {
                            Message.AddString(parsedData[i]);
                        }
                    }
                    else
                    {
                        Message.AddInt32(0);
                    }
                }
                else if (this.GetBaseItem().Name == "boutique_mannequin1")
                {
                    Message.AddInt32(0);
                    Message.AddInt32(1);
                    Message.AddInt32(3);
                    Message.AddString("GENDER");
                    Message.AddString("m");
                    Message.AddString("FIGURE");
                    Message.AddString(this.ExtraData);
                    Message.AddString("OUTFIT_NAME");
                    Message.AddString("");
                }
                else if (this.GetBaseItem().IsGift)
                {
                    string message = "";
                    string sender = "";
                    string figure = "";
                    int spriteCode = 0;

                    if (giftData != null)
                    {
                        message = giftData.message;
                        spriteCode = (giftData.decorationType * 1000) + giftData.wrappingPaper;
                        if (giftData.showUsername)
                        {
                            sender = giftData.sendingUser;
                            figure = giftData.GetUserLook();
                        }
                    }

                    Message.AddInt32(spriteCode);
                    Message.AddInt32(1);
                    Message.AddInt32(6);

                    Message.AddString("EXTRA_PARAM");
                    Message.AddString("");
                    Message.AddString("MESSAGE");
                    Message.AddString(message);
                    Message.AddString("PURCHASER_NAME");
                    Message.AddString(sender);
                    Message.AddString("PURCHASER_FIGURE");
                    Message.AddString(figure); // LOOK
                    Message.AddString("PRODUCT_CODE");
                    Message.AddString("");
                    Message.AddString("state");
                    Message.AddString("0");
                }
                else
                {
                    Message.AddInt32(0);

                    if (GetBaseItem().IsGroupItem)
                    {
                        string[] extra = ExtraData.Split(':');
                        uint groupID = TextHandling.ParseUInt32(extra[0]);
                        GroupData groupData = ButterflyEnvironment.Game.GetGroupManager().GetGroup(groupID);

                        if (groupData != null)
                        {
                            Message.AddInt32(2);
                            Message.AddInt32(5);
                            int pos = this.ExtraData.IndexOf(':');
                            string waroi = this.ExtraData.Substring(pos + 1);
                            Message.AddString(extra.Length > 1 ? waroi : ExtraData);
                            Message.AddString(groupID.ToString());
                            Message.AddString(groupData.image);
                            Message.AddString(groupData.htmlColorPrimary);
                            Message.AddString(groupData.htmlColorSecondary);
                        }
                        else
                        {
                            Message.AddInt32(0);
                            Message.AddString(this.ExtraData);
                        }
                    }
                    else
                    {
                        Message.AddInt32(0);
                        if (GetBaseItem().InteractionType != InteractionType.fbgate)
                            Message.AddString(ExtraData);
                        else
                            Message.AddString(string.Empty);
                    }
                }
                
                Message.AddInt32(0);
            
                Message.AddInt32(0);
                Message.AddUInt32(this.userid); //userid
                //Message.AddInt32(0);
            }
            else if (IsWallItem)
            {
                Message.AddString(Id + String.Empty);
                Message.AddInt32(GetBaseItem().SpriteId);
                Message.AddString(wallCoord == null ? string.Empty : wallCoord.ToString());


                if (GetBaseItem().InteractionType == InteractionType.postit)
                    Message.AddString(ExtraData.Split(' ')[0]);
                else
                    Message.AddString(ExtraData);

                Message.AddInt32(1); // If the "use" option should be available
                Message.AddInt32(-1);
                Message.AddUInt32(this.userid); //UserID
            }
        }

        internal Item GetBaseItem()
        {
            if (mBaseItem == null)
                mBaseItem = ButterflyEnvironment.Game.GetItemManager().GetItem(BaseItem);

            return mBaseItem;
        }

        internal Room GetRoom()
        {
            if (mRoom == null)
                mRoom = ButterflyEnvironment.Game.GetRoomManager().GetRoom(RoomId); //Todo: rub my penis

            return mRoom;
        }

        internal void UserWalksOnFurni(RoomUser user)
        {
            if (OnUserWalksOnFurni != null)
                OnUserWalksOnFurni(this, new UserWalksOnArgs(user));
        }

        internal void UserWalksOffFurni(RoomUser user)
        {
            if (OnUserWalksOffFurni != null)
                OnUserWalksOffFurni(this, new UserWalksOnArgs(user));
        }

        internal void SetIsRoller()
        {
            this.mIsRoller = true;
        }

        internal void OnUserWalk(RoomUser User, bool cyclegameitems)
        {
            RoomItem Item = this;
            #region loooooop
            if (cyclegameitems)
            {
                Item.UserWalksOnFurni(User);
            }

            if (Item.GetBaseItem().IsSeat)
            {
                if (!User.Statusses.ContainsKey("sit"))
                {
                    User.Statusses.Add("sit", TextHandling.GetString(Item.Height));
                }

                User.Z = Item.GetZ;
                if (User.isFlying)
                    User.Z += 4 + (0.5 * Math.Sin(0.7 * User.flyk));
                User.RotHead = Item.Rot;
                User.RotBody = Item.Rot;

                User.UpdateNeeded = true;
            }


			switch (Item.GetBaseItem().InteractionType)
			{
				case InteractionType.bed:
					{
                        if (User.Z != Item.mZ)
                            break;
						if (!User.Statusses.ContainsKey("lay"))
						{
							User.Statusses.Add("lay", TextHandling.GetString(Item.Height) + " null");
						}

                        User.Z = Item.GetZ;
                        if (User.isFlying)
                            User.Z += 4 + (0.2 * 0.5 * Math.Sin(0.7 * User.flyk));
                        User.RotHead = Item.Rot;
                        User.RotBody = Item.Rot;

                        User.UpdateNeeded = true;
                        break;
                    }

                case InteractionType.fbgate:
                    {
                        if (cyclegameitems)
                        {
                            if (User.team != Item.team)
                                User.team = Item.team;

                            else if (User.team == Item.team)
                                User.team = Team.none;

                            if (!string.IsNullOrEmpty(Item.Figure))
                            {
                                //User = GetUserForSquare(Item.Coordinate.X, Item.Coordinate.Y);
                                if (User != null && !User.IsBot)
                                {
                                    if (User.Coordinate == Item.Coordinate)
                                    {
                                        if (User.GetClient().GetHabbo().Gender != Item.Gender && User.GetClient().GetHabbo().Look != Item.Figure
                                            && Item.Gender != null && Item.Figure != null)
                                        {

                                            User.GetClient().GetHabbo().tempGender = User.GetClient().GetHabbo().Gender;
                                            User.GetClient().GetHabbo().tempLook = User.GetClient().GetHabbo().Look;

                                            User.GetClient().GetHabbo().Gender = Item.Gender;
                                            User.GetClient().GetHabbo().Look = Item.Figure;
                                        }
                                        else if (User.GetClient().GetHabbo().tempGender != null && User.GetClient().GetHabbo().tempLook != null)
                                        {
                                            User.GetClient().GetHabbo().Gender = User.GetClient().GetHabbo().tempGender;
                                            User.GetClient().GetHabbo().Look = User.GetClient().GetHabbo().tempLook;
                                        }

                                        ServerMessage RoomUpdate = PacketHandelingManager.GetRecycledItem(OutputCode.UserChangeMessageEvent);
                                        RoomUpdate.AddInt32(User.VirtualId);
                                        RoomUpdate.AddString(User.GetClient().GetHabbo().Look);
                                        RoomUpdate.AddString(User.GetClient().GetHabbo().Gender.ToLower());
                                        RoomUpdate.AddString(User.GetClient().GetHabbo().Motto);
                                        RoomUpdate.AddInt32(User.GetClient().GetHabbo().AchievementPoints);
                                        GetRoom().SendMessage(RoomUpdate);
                                    }
                                }
                            }
                        }

                        break;
                    }

                //33: Red
                //34: Green
                //35: Blue
                //36: Yellow

                case InteractionType.banzaigategreen:
                case InteractionType.banzaigateblue:
                case InteractionType.banzaigatered:
                case InteractionType.banzaigateyellow:
                    {
                        if (cyclegameitems)
                        {
                            int effectID = (int)Item.team + 32;
                            AvatarEffectsInventoryComponent efectmanager = User.GetClient().GetHabboDataContainer().GetAvatarEffectsInventoryComponent();

                            if (User.team != Item.team)
                            {
                                User.team = Item.team;

                                if (efectmanager.CurrentEffect != effectID)
                                    efectmanager.ApplyCustomEffect(effectID);
                            }
                            else
                            {
                                if (efectmanager.CurrentEffect == effectID)
                                    efectmanager.ApplyCustomEffect(0);
                                User.team = Team.none;
                            }
                        }
                        break;
                    }

                case InteractionType.freezeyellowgate:
                case InteractionType.freezeredgate:
                case InteractionType.freezegreengate:
                case InteractionType.freezebluegate:
                    {
                        if (cyclegameitems)
                        {
                            int effectID = (int)Item.team + 39;
                            AvatarEffectsInventoryComponent efectmanager = User.GetClient().GetHabboDataContainer().GetAvatarEffectsInventoryComponent();

                            if (User.team != Item.team)
                            {
                                User.team = Item.team;

                                if (efectmanager.CurrentEffect != effectID)
                                    efectmanager.ApplyCustomEffect(effectID);
                            }
                            else
                            {
                                if (efectmanager.CurrentEffect == effectID)
                                    efectmanager.ApplyCustomEffect(0);
                                User.team = Team.none;
                            }

                            //ServerMessage message = new ServerMessage(OutputCode.YouArePlayingGameMessageEvent);
                            //message.AddBoolean((User.team != Team.none));

                            //User.GetClient().SendMessage(message);
                        }
                        break;
                    }

                case InteractionType.banzaitele:
                    {
                        GetRoom().GetGameItemHandler().onTeleportRoomUserEnter(User, Item);
                        break;
                    }
            }
            #endregion
        }

        internal bool CanBeTriggered(GameClients.GameClient Session, int Request, Boolean UserHasRights)
        {
            if (this.Interactor == null || this.Interactor.CanBeTriggered(Session, this, Request, UserHasRights))
            {
                return true;
            }
            return false;
        }

        internal byte GetSquareState()
        {
            if (GetBaseItem().Walkable || (GetBaseItem().InteractionType == Butterfly.HabboHotel.Items.InteractionType.gate && ExtraData == "1"))
                return 1;
            if (GetBaseItem().IsSeat || GetBaseItem().InteractionType == Butterfly.HabboHotel.Items.InteractionType.bed)
                return 3;

            return 0;
        }
    }
}
