﻿using Butterfly.Core;
using Butterfly.HabboHotel.Catalogs;
using Butterfly.HabboHotel.Group;
using Butterfly.Messages;
using System;

namespace Butterfly.HabboHotel.Items
{
    class UserItem
    {
        internal uint itemID;
		private string _extraData;
		private ushort itemBaseID;
        private GiftData giftData;

		private Item itemBase
		{
			get
			{
				return ButterflyEnvironment.Game.GetItemManager().GetItem(itemBaseID);
			}
		}

		internal string extraData
		{
			get
			{
                 return (_extraData == null ? string.Empty : _extraData);
			}
		}

		internal ushort BaseItemID
		{
			get
			{
				return itemBaseID;
			}
		}

		internal UserItem(uint itemID, ushort baseID, string extraData)
        {
            this.itemID = itemID;
			this.itemBaseID = baseID;
			if (extraData.Length > 0)
				this._extraData = extraData;

			if (itemBase == null)
            {
                Console.WriteLine("Unknown baseItem ID: " + baseID);
                Logging.LogException("Unknown baseItem ID: " + baseID);
            }
        }


        internal void SerializeWall(ServerMessage Message, Boolean Inventory)
        {
            Message.AddUInt32(itemID);
			Message.AddChar(char.ToUpper(itemBase.Type));
            Message.AddUInt32(itemID);
            Message.AddInt32(GetBaseItem().SpriteId);

            if (GetBaseItem().Name.Contains("a2"))
                Message.AddInt32(3);
            else if (GetBaseItem().Name.Contains("wallpaper"))
                Message.AddInt32(2);
            else if (GetBaseItem().Name.Contains("landscape"))
                Message.AddInt32(4);
            else
                Message.AddInt32(1);

            Message.AddInt32(0);
            Message.AddString(extraData);
            Message.AddBoolean(GetBaseItem().AllowRecycle);
            Message.AddBoolean(GetBaseItem().AllowTrade);
            Message.AddBoolean(GetBaseItem().AllowInventoryStack);
            Message.AddBoolean(Marketplace.CanSellItem(this));
            Message.AddInt32(-1);
            Message.AddBoolean(false);
            Message.AddInt32(0);
        }

        internal void SerializeFloor(ServerMessage Message, Boolean Inventory)
        {
            if (GetBaseItem().IsGift)
            {
                if (giftData == null) LoadGift();

                string message = string.Empty;
                int spriteCode = 0;
                string figure = string.Empty;

                if (giftData != null)
                {
                    message = giftData.message;
                    spriteCode = (giftData.decorationType * 1000) + giftData.wrappingPaper;
                }

                Message.AddInt32((-1) * (int)this.itemID);
                Message.AddString(this.GetBaseItem().Type.ToString().ToUpper());
                Message.AddUInt32(this.itemID);
                Message.AddInt32(this.GetBaseItem().SpriteId);
                Message.AddInt32(9);
                Message.AddInt32(0);
                Message.AddString(message);
                Message.AddBoolean(this.GetBaseItem().AllowRecycle);
                Message.AddBoolean(this.GetBaseItem().AllowTrade);
                Message.AddBoolean(this.GetBaseItem().AllowInventoryStack);
                Message.AddBoolean(Marketplace.CanSellItem(this));
                Message.AddInt32(-1);
                Message.AddBoolean(false);
                Message.AddInt32(-1);
                Message.AddString("");
                Message.AddInt32(spriteCode);
            }
            else
            {
                Message.AddUInt32(this.itemID);
                Message.AddChar(GetBaseItem().Type);
                Message.AddUInt32(this.itemID);
                Message.AddInt32(this.GetBaseItem().SpriteId);

                if (GetBaseItem().IsGroupItem && _extraData != null)
                {
                    uint groupID = TextHandling.ParseUInt32(_extraData.Split(':')[0]);
                    GroupData groupData = ButterflyEnvironment.Game.GetGroupManager().GetGroup(groupID);
                    if (groupData != null)
                    {
                        Message.AddInt32(0x11);

                        Message.AddInt32(2);
                        Message.AddInt32(5);
                        Message.AddString("0");
                        Message.AddString(groupID.ToString());
                        Message.AddString(groupData.image);
                        Message.AddString(groupData.htmlColorPrimary);
                        Message.AddString(groupData.htmlColorSecondary);
                    }
                    else
                    {
                        Message.AddInt32(1);
                        Message.AddInt32(0);
                        Message.AddString(extraData);
                    }
                }
                else
                {
                    Message.AddInt32(1);
                    Message.AddInt32(0);
                    Message.AddString(extraData);
                }
                
                Message.AddBoolean(this.GetBaseItem().AllowRecycle);
                Message.AddBoolean(this.GetBaseItem().AllowTrade);
                Message.AddBoolean(this.GetBaseItem().AllowInventoryStack);
                Message.AddBoolean(Marketplace.CanSellItem(this));
                Message.AddInt32(-1);
                Message.AddBoolean(false);
                Message.AddInt32(0);

            }
            //Message.AddUInt32(itemID);
            //Message.AddChar(char.ToUpper(itemBase.Type));
            //Message.AddUInt32(itemID);
            //Message.AddInt32(GetBaseItem().SpriteId);
            //Message.AddInt32(1); // Disc = 8, allow group item = 0x11
            //Message.AddInt32(0);
            //Message.AddInt32(0);
            //Message.AddString(extraData);

            //Message.AddBoolean(GetBaseItem().AllowRecycle);
            //Message.AddBoolean(GetBaseItem().AllowTrade);
            //Message.AddBoolean(GetBaseItem().AllowInventoryStack);
            //Message.AddBoolean(Marketplace.CanSellItem(this));
            //Message.AddInt32(-1);
            //Message.AddBoolean(true);
            //Message.AddInt32(-1);

            //if (GetBaseItem().Type == 'S')
            //{
            //    Message.AddString("");
            //    Message.AddInt32(-1);
            //}
        }

        private void LoadGift()
        {
            if (GetBaseItem().IsGift && extraData.Length >= 12)
            {
                GiftData giftData = new GiftData(extraData);

                if (giftData.wrappingPaper < 20 && giftData.decorationType < 20)
                    this.giftData = giftData;
            }
        }

        internal Item GetBaseItem()
        {
			return itemBase;
        }

		internal bool IsWallItem()
		{
            return (itemBase != null && itemBase.Type == 'i');
		}
    }
}
