﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections;
using Butterfly.HabboHotel.Items;
using System.Drawing;
using Butterfly.HabboHotel.Rooms.Wired.WiredHandlers.Interfaces;

namespace Butterfly.HabboHotel.Rooms.Wired
{
    class ConditionHandler
    {
        private Room room;
        private Hashtable roomMatrix; //doube<coord> | List<IWiredCondition>
        private Hashtable itemToOldCoordinate; //doube<coord> | List<IWiredCondition>
        private Queue addQueue;
        private Queue removeQueue;
        private Queue removeItemQueue;

        public ConditionHandler(Room room)
        {
            this.room = room;
            this.roomMatrix = new Hashtable();
            this.itemToOldCoordinate = new Hashtable();
            this.addQueue = new Queue();
            this.removeQueue = new Queue();
            this.removeItemQueue = new Queue();
        }

        internal void OnCycle()
        {
            WorkAddQueue();
            WorkRemoveQueue();
        }

        private void WorkAddQueue()
        {
            lock (addQueue.SyncRoot)
            {
                while (addQueue.Count > 0)
                {
                    RoomItem item = (RoomItem)addQueue.Dequeue();
                    AddOrUpdateRefferance(item.Coordinate, item.wiredCondition);
                }
            }
        }

		private void AddOrUpdateRefferance(int coordinate, IWiredCondition item)
		{
            if (itemToOldCoordinate.ContainsKey(item.GetOriginalWiredItem().Id))
            {
                int oldLocation =(int) itemToOldCoordinate[item.GetOriginalWiredItem().Id];
                List<IWiredCondition> list = (List<IWiredCondition>)roomMatrix[oldLocation];
                IWiredCondition toRemove = null;
                foreach (IWiredCondition condition in list)
                {
                    if (condition.GetOriginalWiredItem().Id == item.GetOriginalWiredItem().Id)
                    {
                        toRemove = condition;
                        break;
                    }
                }
                list.Remove(toRemove);
                itemToOldCoordinate.Remove(item.GetOriginalWiredItem().Id);

            }
			if (roomMatrix.ContainsKey(coordinate))
			{
				List<IWiredCondition> items = (List<IWiredCondition>)roomMatrix[coordinate];
				if (!items.Contains(item))
					items.Add(item);
			}
			else
			{
				List<IWiredCondition> items = new List<IWiredCondition>(1);
				items.Add(item);
				roomMatrix.Add(coordinate, items);
			}
            itemToOldCoordinate.Add(item.GetOriginalWiredItem().Id, coordinate);
		}

        private void AddOrUpdateRefferance(Point coordinate, IWiredCondition item)
        {
			AddOrUpdateRefferance(TextHandling.CombineXYCoord(coordinate.X, coordinate.Y), item);
        }

        private void WorkRemoveQueue()
        {
            lock (removeQueue.SyncRoot)
            {
                int coordinateToClear;
                RoomItem itemToRemove;
                while (removeQueue.Count > 0)
                {
                    coordinateToClear = (int)removeQueue.Dequeue();
                    itemToRemove = (RoomItem)removeItemQueue.Dequeue();
                    itemToOldCoordinate.Remove(itemToRemove.Id);
                    roomMatrix.Remove(coordinateToClear);
                }
            }
        }

        internal void AddOrIgnoreRefferance(RoomItem item)
        {
            lock (addQueue.SyncRoot)
            {
                addQueue.Enqueue(item);
            }
        }

		internal void ClearTile(int coordinate, RoomItem item)
		{
			lock (removeQueue.SyncRoot)
			{
				removeQueue.Enqueue(coordinate);
                removeItemQueue.Enqueue(item);
			}
		}

        internal void ClearTile(Point coordinate, RoomItem item)
        {
			ClearTile(TextHandling.CombineXYCoord(coordinate.X, coordinate.Y), item);
        }

        internal bool AllowsHandling(int x, int y, RoomUser user, Room theRoom)
        {
            int coordinate = TextHandling.CombineXYCoord(x, y);
			List<IWiredCondition> conditions = (List<IWiredCondition>)roomMatrix[coordinate];

			if (conditions == null)
                return true;
            
            foreach (IWiredCondition condition in conditions)
            {
                if (!condition.AllowsExecution(user, theRoom))
                    return false;
            }

            return true;
        }
    }
}
