﻿using Butterfly.HabboHotel.Items;
using Butterfly.HabboHotel.Pathfinding;
using Butterfly.HabboHotel.Rooms.Wired.WiredHandlers.Interfaces;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Butterfly.HabboHotel.Items.FurniTriggers.Triggers;

namespace Butterfly.HabboHotel.Rooms.Wired.WiredHandlers.Conditions
{
    class FurniHasFurni : IWiredCondition, IWiredSettingsTrigger
    {
        #region init wired
        private RoomItem item;
        private List<RoomItem> items;
        private bool isDisposed;
        private WiredHandler handler;
        private bool allItems;
        public FurniHasFurni(RoomItem item, List<RoomItem> items, WiredHandler handler, bool allItems)
        {
            this.item = item;
            this.items = items;
            this.isDisposed = false;
            this.handler = handler;
            this.allItems = allItems;
            AttatchListenersToItems();
        }

        void AttatchListenersToItems()
        {
            foreach (RoomItem item in items)
            {
                item.itemRemovedEventHandler += item_itemRemovedEventHandler;
            }
        }

        void DetatchFromListeners()
        {
            if (items == null)
                return;
            foreach (RoomItem item in items)
            {
                item.itemRemovedEventHandler -= item_itemRemovedEventHandler;
            }
        }

        void item_itemRemovedEventHandler(object sender, ItemTriggeredArgs e)
        {
            e.TriggeringItem.itemRemovedEventHandler -= item_itemRemovedEventHandler;
            this.onItemRemove(e.TriggeringItem);
        }

        #endregion

        public bool AllowsExecution(RoomUser user, Room theRoom)
        {
            GenericTriggerWithModes.DoAnimation(this.item);
            int coord = 0;
            if (items.Count == 0)
                return true;
            Gamemap map = theRoom.GetGameMap();
            Dictionary<int, ThreeDCoord> points;
            List<RoomItem> itemsOnSquare;
            bool itemFoundOnLoop;
            bool foundOneFurni = false;
            foreach (RoomItem item in items)
            {
                itemFoundOnLoop = false;
                points = Gamemap.GetAffectedTiles(item.GetBaseItem().Length, item.GetBaseItem().Width, item.GetX, item.GetY, item.Rot);
                foreach (ThreeDCoord coor in points.Values)
                {
                    coord = TextHandling.CombineXYCoord(coor.X, coor.Y);
                    itemsOnSquare = map.GetCoordinatedItems(ref coord);
                    foreach (RoomItem itemOntile in itemsOnSquare)
                    {
                        if (itemOntile.Height > item.Height)
                        {
                            if (!allItems)
                                return true;
                            foundOneFurni = true;
                            itemFoundOnLoop = true;
                            break;
                        }
                    }
                    if (itemFoundOnLoop)
                        break;
                }
                if (!itemFoundOnLoop && allItems)
                {
                    return false;
                }

            }
            return foundOneFurni;
        }

        #region wired related
        public void Dispose()
        {
            isDisposed = true;
            item = null;
            handler = null;
            DetatchFromListeners();
            if (items != null)
                items.Clear();
            items = null;
        }

        public bool Disposed()
        {
            return isDisposed;
        }


        public List<RoomItem> getAffectedItemList()
        {
            return items;
        }
        public bool WiredConditionsMet(RoomUser user, Room theRoom)
        {
            if (handler.conditionHandler != null)
            {
                return handler.conditionHandler.AllowsHandling(item.GetX, item.GetY, user, theRoom);
            }
            return true;
        }

        public bool onItemRemove(RoomItem item)
        {
            return this.items.Remove(item);
        }


        public void LoadFromWiredObject(WiredLoader.WiredLoaderObject loadedData)
        {
            this.items = loadedData.items;
            allItems = loadedData.GetBooleanWithIndex(0);
            AttatchListenersToItems();
        }

        public RoomItem GetOriginalWiredItem()
        {
            return this.item;
        }

        public string[] GetSettings()
        {
            return new string[] { this.allItems.ToString() };
        }
        #endregion

    }
}
