﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Butterfly.HabboHotel.Rooms.Wired.WiredHandlers.Interfaces;
using Butterfly.HabboHotel.Items;
using Database_Manager.Database.Session_Details.Interfaces;
using System.Data;
using Butterfly.HabboHotel.Rooms.Wired.WiredHandlers.WiredLoader;
using Butterfly.HabboHotel.Items.FurniTriggers.Triggers;

namespace Butterfly.HabboHotel.Rooms.Wired.WiredHandlers.Conditions
{
    class FurniStatePosMatch : IWiredCondition, IWiredOriginalItemTrigger, IWiredSettingsTrigger
    {
        #region init wired
        private RoomItem item;
        private List<RoomItem> items;
        private Dictionary<uint, OriginalItemLocation> originalItemLocation;
        
        bool checkState = false;
        bool checkDirection = false;
        bool checkPosition = false;

        private bool isDisposed;
        private WiredHandler handler;
        public FurniStatePosMatch(RoomItem item, List<RoomItem> items, WiredHandler handler, bool checkState, bool checkDirection, bool checkPosition)
        {
            originalItemLocation = new Dictionary<uint, OriginalItemLocation>(items.Count);

            foreach (RoomItem nItem in items)
            {
                this.originalItemLocation.Add(nItem.Id, new OriginalItemLocation(nItem.Id, nItem.GetX, nItem.GetY, nItem.Height, nItem.Rot, nItem.ExtraData));
            }
            this.item = item;
            this.items = items;
            this.isDisposed = false;
            this.handler = handler;
            this.checkPosition = checkPosition;
            this.checkDirection = checkDirection;
            this.checkState = checkState;
            AttatchListenersToItems();
        }

        void AttatchListenersToItems()
        {
            foreach (RoomItem item in items)
            {
                item.itemRemovedEventHandler += item_itemRemovedEventHandler;
            }
        }

        void DetatchFromListeners()
        {
            if (items == null)
                return;
            foreach (RoomItem item in items)
            {
                item.itemRemovedEventHandler -= item_itemRemovedEventHandler;
            }
        }

        void item_itemRemovedEventHandler(object sender, ItemTriggeredArgs e)
        {
            e.TriggeringItem.itemRemovedEventHandler -= item_itemRemovedEventHandler;
            this.onItemRemove(e.TriggeringItem);
        }
        #endregion

        public bool AllowsExecution(RoomUser user, Room theRoom)
        {
            GenericTriggerWithModes.DoAnimation(this.item);
            OriginalItemLocation location;
            foreach (RoomItem item in items)
            {
                location = this.originalItemLocation[item.Id];

                if (checkPosition && !(location.X == item.GetX && item.GetY == location.Y && item.Height == location.Height))
                {
                    return false;
                }
                if (checkDirection && item.Rot != location.Rot)
                {
                    return false;
                }
                if (checkState && item.ExtraData != location.ExtraData)
                    return false;
            }

            return true;
        }

        #region wired related 
        public void Dispose()
        {
            isDisposed = true;
            item = null; 
            handler = null;
            DetatchFromListeners();
            if (items != null)
                items.Clear();
            items = null;
        }

        public bool Disposed()
        {
            return isDisposed;
        }


        public List<RoomItem> getAffectedItemList()
        {
            return items;
        }

        public bool WiredConditionsMet(RoomUser user, Room theRoom)
        {
            if (handler.conditionHandler != null)
            {
                return handler.conditionHandler.AllowsHandling(item.GetX, item.GetY, user, theRoom);
            }
            return true;
        }
        
        public bool onItemRemove(RoomItem item)
        {
            return this.items.Remove(item);
        }

        public Dictionary<uint, WiredLoader.OriginalItemLocation> GetOriginalItemLocation()
        {
            return this.originalItemLocation;
        }


        public void LoadFromWiredObject(WiredLoader.WiredLoaderObject loadedData)
        {
            this.originalItemLocation = loadedData.originalPositionList;
            this.items = loadedData.items;
            checkState = loadedData.GetBooleanWithIndex(0);
            checkDirection = loadedData.GetBooleanWithIndex(1);
            checkPosition = loadedData.GetBooleanWithIndex(2);

            AttatchListenersToItems();
        }

        public RoomItem GetOriginalWiredItem()
        {
            return this.item;
        }
        #endregion

        public string[] GetSettings()
        {
            return new string[] { checkState.ToString(), checkDirection.ToString(), checkPosition.ToString() };
        }
    }
}
