﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Butterfly.HabboHotel.Rooms.Wired.WiredHandlers.Interfaces;
using Database_Manager.Database.Session_Details.Interfaces;
using System.Data;
using Butterfly.HabboHotel.Items;
using Butterfly.HabboHotel.Items.FurniTriggers.Triggers;

namespace Butterfly.HabboHotel.Rooms.Wired.WiredHandlers.Conditions
{
    class MoreThanTimer : IWiredCondition, IWiredSettingsTrigger
    {
        #region wired init
        private int timeout;
        private Room room;
        private RoomItem item;
        private bool isDisposed = false;
        private WiredHandler handler;

        public MoreThanTimer(int timeout, Room room, RoomItem item, WiredHandler handler)
        {
            this.timeout = timeout;
            this.room = room;
            this.isDisposed = false;
            this.item = item;
            this.handler = handler;
        }

        public bool AllowsExecution(RoomUser user, Room theRoom)
        {
            GenericTriggerWithModes.DoAnimation(this.item);
            if (room.lastTimerReset == null)
                return false;

            TimeSpan sinceTimerReset = DateTime.Now - room.lastTimerReset;
            return (sinceTimerReset.TotalSeconds > timeout);
        }

        public void Dispose()
        {
            isDisposed = true;
            item = null;
            room = null;
            handler = null;
        }

        public bool Disposed()
        {
            return isDisposed;
        }


        public List<RoomItem> getAffectedItemList()
        {
            return new List<RoomItem>(0);
        }

        public bool onItemRemove(RoomItem item)
        {
            return false;
        }
        #endregion

        public bool WiredConditionsMet(RoomUser user, Room theRoom)
        {
            if (handler.conditionHandler != null)
            {
                return handler.conditionHandler.AllowsHandling(item.GetX, item.GetY, user, theRoom);
            }
            return true;
        }

        #region wired loading/handling
        public void LoadFromWiredObject(WiredLoader.WiredLoaderObject loadedData)
        {
            this.timeout = loadedData.GetIntWithIndex(1, 20);
            
        }

        public RoomItem GetOriginalWiredItem()
        {
            return this.item;
        }

        public string[] GetSettings()
        {
            return new string[] { timeout.ToString() };
        }
        #endregion
    }
}
