﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Butterfly.HabboHotel.Rooms.Wired.WiredHandlers.Interfaces;
using Butterfly.HabboHotel.Items;
using Database_Manager.Database.Session_Details.Interfaces;
using System.Data;
using Butterfly.HabboHotel.Pathfinding;
using Butterfly.HabboHotel.Items.FurniTriggers.Triggers;

namespace Butterfly.HabboHotel.Rooms.Wired.WiredHandlers.Conditions
{

    class TriggerUserIsOnFurni : IWiredCondition
    {
        #region init wired
        private RoomItem item;
        private WiredHandler handler;
        private List<RoomItem> items;
        private bool isDisposed;

        public TriggerUserIsOnFurni(RoomItem item, List<RoomItem> items, WiredHandler handler)
        {
            this.item = item;
            this.items = items;
            this.isDisposed = false;
            this.handler = handler;
            AttatchListenersToItems();
        }

        void AttatchListenersToItems()
        {
            foreach (RoomItem item in items)
            {
                item.itemRemovedEventHandler += item_itemRemovedEventHandler;
            }
        }

        void DetatchFromListeners()
        {
            if (items == null)
                return;
            foreach (RoomItem item in items)
            {
                item.itemRemovedEventHandler -= item_itemRemovedEventHandler;
            }
        }

        void item_itemRemovedEventHandler(object sender, ItemTriggeredArgs e)
        {
            e.TriggeringItem.itemRemovedEventHandler -= item_itemRemovedEventHandler;
            this.onItemRemove(e.TriggeringItem);
        }
        #endregion

        public bool AllowsExecution(RoomUser user, Room theRoom)
        {
            GenericTriggerWithModes.DoAnimation(this.item);
            if (user == null)
                return false;
            Gamemap map = theRoom.GetGameMap();
            Dictionary<int, ThreeDCoord> points;
            foreach (RoomItem item in items)
            {
                item.GetDoubleCoordinates();

                points = Gamemap.GetAffectedTiles(item.GetBaseItem().Length, item.GetBaseItem().Width, item.GetX, item.GetY, item.Rot);
                
                foreach (ThreeDCoord coor in points.Values)
                {   
                    if(map.GetRoomUsers(Formatting.PointToInt(new Point(coor.X, coor.Y))).Count >= 1)
                        return true;
                }
            }
            return false;
        }

        #region wired handling
        public void Dispose()
        {
            isDisposed = true;
            item = null;
            handler = null;
            DetatchFromListeners();
            if (items != null)
                items.Clear();
            items = null;
        }

        public bool Disposed()
        {
            return isDisposed;
        }


        public List<RoomItem> getAffectedItemList()
        {
            return this.items;
        }

        public bool WiredConditionsMet(RoomUser user, Room theRoom)
        {
            if (handler.conditionHandler != null)
            {
                return handler.conditionHandler.AllowsHandling(item.GetX, item.GetY, user, theRoom);
            }
            return true;
        }

        public bool onItemRemove(RoomItem item)
        {
            return this.items.Remove(item);
        }


        public void LoadFromWiredObject(WiredLoader.WiredLoaderObject loadedData)
        {
            this.items = loadedData.items;
            AttatchListenersToItems();
        }

        public RoomItem GetOriginalWiredItem()
        {
            return item;
        }
        #endregion
    }
}
