﻿using Butterfly.HabboHotel.Rooms.Wired.WiredHandlers.Interfaces;
using Butterfly.HabboHotel.Rooms.Games;
using Butterfly.HabboHotel.Items;
using Database_Manager.Database.Session_Details.Interfaces;
using System.Data;
using System;
using Butterfly.HabboHotel.Items.FurniTriggers.Triggers;

namespace Butterfly.HabboHotel.Rooms.Wired.WiredHandlers.Effects
{
    class GiveScore : IWiredEffect, IWiredTrigger, IWiredSettingsTrigger
    {
        internal int maxCountPerGame;
        private int currentGameCount;
        internal int scoreToGive;
        private GameManager gameManager;
        private RoomEventDelegate delegateFunction;
        private RoomItem item;
        private WiredHandler handler;
        
        public GiveScore(int maxCountPerGame, int scoreToGive, GameManager gameManager, RoomItem item, WiredHandler handler)
        {
            this.maxCountPerGame = maxCountPerGame;
            this.currentGameCount = 0;
            this.scoreToGive = scoreToGive;
            this.delegateFunction = new RoomEventDelegate(gameManager_OnGameStart);
            this.gameManager = gameManager;
            this.item = item;
            this.handler = handler;

            gameManager.OnGameStart += delegateFunction;
        }

        private void gameManager_OnGameStart(object sender, System.EventArgs e)
        {
            currentGameCount = 0;
        }

        public bool Handle(RoomUser user, Team team, RoomItem item)
        {
            GenericTriggerWithModes.DoAnimation(this.item);
            if (team != Team.none && maxCountPerGame > currentGameCount)
            {
                currentGameCount++;
                gameManager.AddPointToTeam(team, scoreToGive, user);
                gameManager.GetRoom().GetWiredHandler().OnEvent(item.Id);
                return true;
            }

            return false;
        }

        public void Dispose()
        {
            gameManager.OnGameStart -= delegateFunction;
            gameManager = null;
            delegateFunction = null;
            handler = null;
        }

        public bool IsSpecial(out SpecialEffects function)
        {
            function = SpecialEffects.None;
            return false;
        }

        public string[] GetSettings()
        {
            return new string[] { this.maxCountPerGame.ToString(), this.scoreToGive.ToString() };
        }


        public void LoadFromWiredObject(WiredLoader.WiredLoaderObject loadedData)
        {
            this.maxCountPerGame = loadedData.GetIntWithIndex(0);
            this.scoreToGive = loadedData.GetIntWithIndex(1);
            if (maxCountPerGame == -1)
                maxCountPerGame = 0;
            if (scoreToGive == -1)
                scoreToGive = 0;
        }

        public RoomItem GetOriginalWiredItem()
        {
            return item;
        }

        public bool WiredConditionsMet(RoomUser userRef, Room theRoom)
        {
            return true;
        }
    }
}
