﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Drawing;
using Butterfly.HabboHotel.Items;
using Butterfly.HabboHotel.Items.FurniTriggers.Triggers;
using Butterfly.HabboHotel.Rooms.Games;
using Butterfly.HabboHotel.Rooms.Wired.WiredHandlers.Interfaces;
using Butterfly.HabboHotel.Rooms.Wired.WiredHandlers.WiredLoader;
using Database_Manager.Database.Session_Details.Interfaces;

namespace Butterfly.HabboHotel.Rooms.Wired.WiredHandlers.Effects
{
    class MoveRotate : MovementManagement, IWiredEffect, IWiredFurniTrigger, IWiredCycleable, IWiredSettingsTrigger
    {
        private Room room;
        private WiredHandler handler;
        private RoomItem itemID;

        internal MovementState movement;
        internal RotationState rotation;
        internal List<RoomItem> items;
        
        internal int delay;
        internal int cycles;

        private bool isDisposed;

        public MoveRotate(MovementState movement, RotationState rotation, List<RoomItem> items, int delay, Room room, WiredHandler handler, RoomItem itemID)
        {
            this.movement = movement;
            this.rotation = rotation;
            this.items = items;
            this.delay = delay;
            this.room = room;
            this.handler = handler;
            this.cycles = 0;
            this.itemID = itemID;
            this.isDisposed = false;
            AttatchListenersToItems();
        }

        void AttatchListenersToItems()
        { 
            foreach(RoomItem item in items)
            {
                item.itemRemovedEventHandler += item_itemRemovedEventHandler;
            }
        }

        void DetatchFromListeners()
        {
            if (items == null)
                return;
            foreach (RoomItem item in items)
            {
                item.itemRemovedEventHandler -= item_itemRemovedEventHandler;
            }
        }

        void item_itemRemovedEventHandler(object sender, ItemTriggeredArgs e)
        {
            e.TriggeringItem.itemRemovedEventHandler -= item_itemRemovedEventHandler;
            this.onItemRemove(e.TriggeringItem);
        }

        public bool Handle(RoomUser user, Team team, RoomItem item)
        {
            cycles = 0;

            if (delay == 0)
            {
                HandleItems();
            }
            else
            {
                handler.RequestCycle(this);
            }
            return false;
        }

        public bool OnCycle()
        {
            if (room == null)
                return false;

            cycles++;
            if (cycles > delay)
            {
                cycles = 0;
                HandleItems();
                return false;
            }

            return true;
        }

        public void Dispose()
        {
            isDisposed = true;
            room = null;
            handler = null;
            DetatchFromListeners();
            if (items != null)
                items.Clear();

            items = null;

        }

        private bool HandleItems()
        {
            GenericTriggerWithModes.DoAnimation(this.itemID);
            handler.OnEvent(itemID.Id);
            bool itemHandled = false;
            foreach (RoomItem item in items)
            {
                if (HandleMovement(item))
                    itemHandled = true;
            }

            return itemHandled;
        }

        private bool HandleMovement(RoomItem item)
        {
            if (room.GetRoomItemHandler().GetItem(item.Id) == null)
                return false;

            Point newPoint = base.HandleMovement(item.Coordinate, movement, item.Rot);
            int newRotation = base.HandleRotation(item.Rot, rotation);

            if (newPoint != item.Coordinate || newRotation != item.Rot)
            {
                if (room.GetGameMap().SquareIsOpen(newPoint.X, newPoint.Y, false))
                    return room.GetRoomItemHandler().SetFloorItem(null, item, newPoint.X, newPoint.Y, newRotation, false, false, true);
                else if (newRotation != item.Rot)
                {
                    return room.GetRoomItemHandler().SetFloorItem(null, item, item.GetX, item.GetY, newRotation, false, false, true);
                }
            }

            return false;
        }

        public bool IsSpecial(out SpecialEffects function)
        {
            function = SpecialEffects.None;
            return false;
        }

        public bool Disposed()
        {
            return isDisposed;
        }

        public bool WiredConditionsMet(RoomUser user, Room theRoom)
        {
            if (handler.conditionHandler != null)
            {
                return handler.conditionHandler.AllowsHandling(itemID.GetX, itemID.GetY, user, theRoom);
            }
            return true;
        }

        public List<RoomItem> getAffectedItemList()
        {
            return this.items;
        }

        public bool onItemRemove(RoomItem item)
        {
            return this.items.Remove(item);
        }

        public string[] GetSettings()
        {
            return new string[] { ((int)rotation).ToString(), ((int)movement).ToString(), this.delay.ToString() };
        }


        public void LoadFromWiredObject(WiredLoader.WiredLoaderObject loadedData)
        {
            this.rotation = (RotationState)loadedData.GetIntWithIndex(0, 0);
            this.movement = (MovementState)loadedData.GetIntWithIndex(1, 0);
            this.delay = loadedData.GetIntWithIndex(3);
            this.items = loadedData.items;
            AttatchListenersToItems();
        }

        public RoomItem GetOriginalWiredItem()
        {
            return this.itemID;
        }
    }
}
