﻿using System.Collections.Generic;
using System.Drawing;
using Butterfly.HabboHotel.Items;
using Butterfly.HabboHotel.Rooms.Games;
using Butterfly.HabboHotel.Rooms.Wired.WiredHandlers.Interfaces;
using Database_Manager.Database.Session_Details.Interfaces;
using System.Data;
using System;
using Butterfly.HabboHotel.Rooms.Wired.WiredHandlers.WiredLoader;
using Butterfly.HabboHotel.Items.FurniTriggers.Triggers;

namespace Butterfly.HabboHotel.Rooms.Wired.WiredHandlers.Effects
{
    class PositionReset : IWiredOriginalItemTrigger, IWiredCycleable, IWiredEffect, IWiredSettingsTrigger
    {
        private RoomItemHandling roomItemHandler;
        private WiredHandler handler;
        private uint itemID
        {
            get { return item.Id; }
        }
        private RoomItem item;
        internal List<RoomItem> items;
        internal Dictionary<uint, OriginalItemLocation> originalItemLocation;
        internal int delay;
        private int cycles;

        private bool disposed;

        public PositionReset(List<RoomItem> items, int delay, RoomItemHandling roomItemHandler, WiredHandler handler, RoomItem item)
        {
            this.items = items;
            originalItemLocation = new Dictionary<uint, OriginalItemLocation>(items.Count);

            foreach (RoomItem nItem in items)
            {
                this.originalItemLocation.Add(nItem.Id, new OriginalItemLocation(nItem.Id, nItem.GetX, nItem.GetY, nItem.Height, nItem.Rot, nItem.ExtraData));
            }
            this.delay = delay;
            this.roomItemHandler = roomItemHandler;
            this.cycles = 0;
            this.item = item;
            this.handler = handler;
            this.disposed = false;
            AttatchListenersToItems();
        }

        void AttatchListenersToItems()
        {
            foreach (RoomItem item in items)
            {
                item.itemRemovedEventHandler += item_itemRemovedEventHandler;
            }
        }

        void DetatchFromListeners()
        {
            if (items == null)
                return;
            foreach (RoomItem item in items)
            {
                item.itemRemovedEventHandler -= item_itemRemovedEventHandler;
            }
        }

        void item_itemRemovedEventHandler(object sender, ItemTriggeredArgs e)
        {
            e.TriggeringItem.itemRemovedEventHandler -= item_itemRemovedEventHandler;
            this.onItemRemove(e.TriggeringItem);
        }

        public bool OnCycle()
        {
            cycles++;
            if (cycles > delay)
            {
                HandleItems();
                return false;
            }
            return true;
        }

        public bool Handle(RoomUser user, Team team, RoomItem item)
        {
            cycles = 0;
            if (delay == 0)
            {
                return HandleItems();
            }
            else
            {
                handler.RequestCycle(this);
            }
            return false;
        }

        private bool HandleItems()
        {
            GenericTriggerWithModes.DoAnimation(this.item);
            handler.OnEvent(itemID);
            bool itemIsMoved = false;
            foreach (RoomItem item in items)
            {
                OriginalItemLocation oldCoordinate = this.GetOriginalItemLocation(item);
                item.ExtraData = oldCoordinate.ExtraData;
                if (roomItemHandler.SetFloorItem(null, item, oldCoordinate.X, oldCoordinate.Y, oldCoordinate.Rot, false, false, true))
                    itemIsMoved = true;
            }

            return itemIsMoved;
        }

        private OriginalItemLocation GetOriginalItemLocation(RoomItem item)
        {
            return this.originalItemLocation[item.Id];
        }

        public void Dispose()
        {
            disposed = true;
            roomItemHandler = null;
            handler = null;
            DetatchFromListeners();
            if (items != null)
                items.Clear();
            items = null;
        }
        
        public bool IsSpecial(out SpecialEffects function)
        {
            function = SpecialEffects.None;
            return false;
        }

        public bool Disposed()
        {
            return disposed;
        }
        public bool WiredConditionsMet(RoomUser user, Room theRoom)
        {
            if (handler.conditionHandler != null)
            {
                return handler.conditionHandler.AllowsHandling(item.GetX, item.GetY, user, theRoom);
            }
            return true;
        }


        public List<RoomItem> getAffectedItemList()
        {
            return this.items;
        }

        public bool onItemRemove(RoomItem item)
        {
            this.originalItemLocation.Remove(item.Id);
            return this.items.Remove(item);
        }

        public Dictionary<uint, WiredLoader.OriginalItemLocation> GetOriginalItemLocation()
        {
            return this.originalItemLocation;
        }


        public void LoadFromWiredObject(WiredLoader.WiredLoaderObject loadedData)
        {
            this.originalItemLocation = loadedData.originalPositionList;
            this.items = loadedData.items;
            this.delay = loadedData.GetIntWithIndex(0, 20); 
            AttatchListenersToItems();
        }

        public RoomItem GetOriginalWiredItem()
        {
            return item;
        }

        public string[] GetSettings()
        {
            return new string[] { delay.ToString() };
        }
    }
}
