﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Butterfly.HabboHotel.Rooms.Wired.WiredHandlers.Interfaces;
using Butterfly.HabboHotel.Items;
using Butterfly.HabboHotel.Rooms.Games;
using System.Collections;
using Database_Manager.Database.Session_Details.Interfaces;
using System.Data;
using Butterfly.HabboHotel.Items.FurniTriggers.Triggers;


namespace Butterfly.HabboHotel.Rooms.Wired.WiredHandlers.Effects
{
    class TeleportToItem : IWiredFurniTrigger, IWiredCycleable, IWiredEffect, IWiredSettingsTrigger
    {
        private Gamemap gamemap;
        private WiredHandler handler;

        internal List<RoomItem> items;
        internal int delay;
        private int cycles;
        private Queue delayedUsers;
        private Random rnd;
        private uint itemID { get { return item.Id; } }
        private RoomItem item;

        private bool disposed;

        public TeleportToItem(Gamemap gamemap, WiredHandler handler, List<RoomItem> items, int delay, RoomItem item)
        {
            this.gamemap = gamemap;
            this.handler = handler;
            this.items = items;
            this.delay = delay;
            this.item = item;
            this.cycles = 0;
            this.delayedUsers = new Queue();
            this.rnd = new Random();
            this.disposed = false;
            AttatchListenersToItems();
        }

        void AttatchListenersToItems()
        {
            foreach (RoomItem item in items)
            {
                item.itemRemovedEventHandler += item_itemRemovedEventHandler;
            }
        }

        void DetatchFromListeners()
        {
            if (items == null)
                return;
            foreach (RoomItem item in items)
            {
                item.itemRemovedEventHandler -= item_itemRemovedEventHandler;
            }
        }

        void item_itemRemovedEventHandler(object sender, ItemTriggeredArgs e)
        {
            e.TriggeringItem.itemRemovedEventHandler -= item_itemRemovedEventHandler;
            this.onItemRemove(e.TriggeringItem);
        }

        public bool OnCycle()
        {
            cycles++;
            if (cycles > delay-1)
            {
                if (delayedUsers.Count > 0)
                {
                    lock (delayedUsers.SyncRoot)
                    {
                        if (cycles == delay)
                        {
                            foreach (RoomUser user in delayedUsers)
                            {
                                DoAnimation(user);
                            }
                            return true;

                        }
                        else
                        {
                            while (delayedUsers.Count > 0)
                            {
                                RoomUser user = (RoomUser)delayedUsers.Dequeue();
                                TeleportUser(user);
                            }
                        }
                    }
                }
                return false;
            }

            return true;
        }

        public bool Handle(RoomUser user, Team team, RoomItem item)
        {
            cycles = 0;
            if (delay == 0 && user != null)
            {
                DoAnimation(user);
            }
            lock (delayedUsers.SyncRoot)
            {
                delayedUsers.Enqueue(user);
            }
            handler.RequestCycle(this);

            return false;
        }

        private void DoAnimation(RoomUser user)
        {
            if (user == null || user.GetClient() == null || user.GetClient().GetHabboDataContainer() == null || user.GetClient().GetHabboDataContainer().GetAvatarEffectsInventoryComponent() == null)
                return;
            user.GetClient().GetHabboDataContainer().GetAvatarEffectsInventoryComponent().ApplyCustomEffect(4);
        }
        private void ResetAnimation(RoomUser user)
        {
            if (user == null || user.GetClient() == null || user.GetClient().GetHabboDataContainer() == null || user.GetClient().GetHabboDataContainer().GetAvatarEffectsInventoryComponent() == null)
                return;
            user.GetClient().GetHabboDataContainer().GetAvatarEffectsInventoryComponent().ApplyCustomEffect(0);
        }

        private bool TeleportUser(RoomUser user)
        {
            ResetAnimation(user);
            if (user == null || disposed)
                return false;
            GenericTriggerWithModes.DoAnimation(this.item);
            if (items.Count > 1)
            {
                int toTest = 0;
                RoomItem item;
                for (int i = 0; i < items.Count; i++)
                {
                    toTest = rnd.Next(0, items.Count);
                    item = items[toTest];

                    if (item.Coordinate != user.Coordinate)
                    {
                        gamemap.TeleportToItem(user, item);
                        return true;
                    }
                }
            }
            else if (items.Count == 1)
            {
                gamemap.TeleportToItem(user, items.First());
                return true;
            }

            return false;
        }

        public void Dispose()
        {
            disposed = true;
            gamemap = null;
            handler = null;
            DetatchFromListeners();
            if (items != null)
                items.Clear();
            items = null;
            if (delayedUsers != null)
                delayedUsers.Clear();
        }

        public bool IsSpecial(out SpecialEffects function)
        {
            function = SpecialEffects.None;
            return false;
        }

        public bool Disposed()
        {
            return disposed;
        }
        public bool WiredConditionsMet(RoomUser user, Room theRoom)
        {
            if (handler.conditionHandler != null)
            {
                return handler.conditionHandler.AllowsHandling(item.GetX, item.GetY, user, theRoom);
            }
            return true;
        }
        public List<RoomItem> getAffectedItemList()
        {
            return this.items;
        }

        public bool onItemRemove(RoomItem item)
        {
            return this.items.Remove(item);
        }

        public string[] GetSettings()
        {
            return new string[] { this.delay.ToString() };
        }


        public void LoadFromWiredObject(WiredLoader.WiredLoaderObject loadedData)
        {
            this.delay = loadedData.GetIntWithIndex(0, 0);
            this.items = loadedData.items;
            AttatchListenersToItems();
        }

        public RoomItem GetOriginalWiredItem()
        {
            return item;
        }
    }
}
