﻿using System;
using System.Collections.Generic;
using System.Data;
using Butterfly.HabboHotel.Items;
using Butterfly.HabboHotel.Items.FurniTriggers.Triggers;
using Butterfly.HabboHotel.Rooms.Games;
using Butterfly.HabboHotel.Rooms.Wired.WiredHandlers.Interfaces;
using Database_Manager.Database.Session_Details.Interfaces;

namespace Butterfly.HabboHotel.Rooms.Wired.WiredHandlers.Effects
{
    class TimerReset : IWiredEffect, IWiredFurniTrigger, IWiredCycleable, IWiredSettingsTrigger
    {
        private Room room;
        private uint itemID { get { return item.Id; } }
        private RoomItem item;
        private WiredHandler handler;
        internal List<RoomItem> items;

        internal int delay;
        private int cycles;

        private bool disposed;

        public TimerReset(Room room, WiredHandler handler, List<RoomItem> items, int delay, RoomItem item)
        {
            this.room = room;
            this.handler = handler;
            this.items = items;
            this.delay = delay;
            this.cycles = 0;
            this.disposed = false;
            this.item = item;
            AttatchListenersToItems();
        }

        void AttatchListenersToItems()
        {
            foreach (RoomItem item in items)
            {
                item.itemRemovedEventHandler += item_itemRemovedEventHandler;
            }
        }

        void DetatchFromListeners()
        {
            if (items == null)
                return;
            foreach (RoomItem item in items)
            {
                item.itemRemovedEventHandler -= item_itemRemovedEventHandler;
            }
        }

        void item_itemRemovedEventHandler(object sender, ItemTriggeredArgs e)
        {
            e.TriggeringItem.itemRemovedEventHandler -= item_itemRemovedEventHandler;
            this.onItemRemove(e.TriggeringItem);
        }


        public bool Handle(RoomUser user, Team team, RoomItem item)
        {
            if (delay > 0)
            {
                cycles = 0;
                handler.RequestCycle(this);
            }
            else
            {
                return ResetTimers();
            }

            return false;
        }

        public bool OnCycle()
        {
            if (cycles > delay)
            {
                ResetTimers();
                return false;
            }
            else
            {
                cycles++;
            }
            return true;
        }

        public void Dispose()
        {
            disposed = true;
            room = null;
            handler = null;
            if (items != null)  
                items.Clear();
            items = null;
        }

        private bool ResetTimers()
        {
            GenericTriggerWithModes.DoAnimation(this.item);
            handler.OnEvent(itemID);
            bool itemReset = false;
            foreach (RoomItem item in items)
            {
                if (item.wiredHandler != null)
                {
                    IWiredTimer timer = item.wiredHandler as IWiredTimer;
                    if (timer == null)
                        continue;

                    timer.ResetTimer();
                    itemReset = true;
                }
            }

            return itemReset;
        }

        public bool IsSpecial(out SpecialEffects function)
        {
            function = SpecialEffects.None;
            return false;
        }

        public bool Disposed()
        {
            return disposed;
        }

        public bool WiredConditionsMet(RoomUser user, Room theRoom)
        {
            if (handler.conditionHandler != null)
            {
                return handler.conditionHandler.AllowsHandling(item.GetX, item.GetY, user, theRoom);
            }
            return true;
        }

        public string[] GetSettings()
        {
            return new string[] { this.delay.ToString() };
        }


        public void LoadFromWiredObject(WiredLoader.WiredLoaderObject loadedData)
        {
            this.delay = loadedData.GetIntWithIndex(0);
            if (delay == -1)
            {
                this.delay = 20;
            }
            this.items = loadedData.items;
            this.AttatchListenersToItems();
        }

        public RoomItem GetOriginalWiredItem()
        {
            return item;
        }


        public List<RoomItem> getAffectedItemList()
        {
            return this.items;
        }

        public bool onItemRemove(RoomItem item)
        {
            return this.items.Remove(item);
        }
    }
}
