﻿using System.Collections;
using System.Collections.Generic;
using Butterfly.HabboHotel.Items;
using Butterfly.HabboHotel.Rooms.Games;
using Butterfly.HabboHotel.Rooms.Wired.WiredHandlers.Interfaces;
using Database_Manager.Database.Session_Details.Interfaces;
using System.Data;
using System;
using Butterfly.HabboHotel.Items.FurniTriggers.Triggers;

namespace Butterfly.HabboHotel.Rooms.Wired.WiredHandlers.Effects
{
    class ToggleItemState: IWiredFurniTrigger, IWiredCycleable, IWiredEffect, IWiredSettingsTrigger
    {
        private uint itemID { get { return item.Id; } }
        private RoomItem item;
        private Gamemap gamemap;
        private WiredHandler handler;

        internal List<RoomItem> items;
        internal int delay;
        private int cycles;
        private Queue delayedTriggeringUsers;

        private bool disposed;

        public ToggleItemState(Gamemap gamemap, WiredHandler handler, List<RoomItem> items, int delay, RoomItem item)
        {
            this.gamemap = gamemap;
            this.handler = handler;
            this.items = items;
            this.delay = delay;
            this.cycles = 0;
            this.delayedTriggeringUsers = new Queue();
            this.disposed = false;
            this.item = item;
            AttatchListenersToItems();
        }

        void AttatchListenersToItems()
        {
            foreach (RoomItem item in items)
            {
                item.itemRemovedEventHandler += item_itemRemovedEventHandler;
            }
        }

        void DetatchFromListeners()
        {
            if (items == null)
                return;
            foreach (RoomItem item in items)
            {
                item.itemRemovedEventHandler -= item_itemRemovedEventHandler;
            }
        }

        void item_itemRemovedEventHandler(object sender, ItemTriggeredArgs e)
        {
            e.TriggeringItem.itemRemovedEventHandler -= item_itemRemovedEventHandler;
            this.onItemRemove(e.TriggeringItem);
        }

        public bool OnCycle()
        {
            if (disposed)
                return false;

            cycles++;
            if (cycles > delay)
            {
                if (delayedTriggeringUsers.Count > 0)
                {
                    lock (delayedTriggeringUsers.SyncRoot)
                    {
                        while (delayedTriggeringUsers.Count > 0)
                        {
                            RoomUser user = (RoomUser)delayedTriggeringUsers.Dequeue();
                            ToggleItems(user);
                        }
                    }
                }
                return false;
            }

            return true;
        }

        public bool Handle(RoomUser user, Team team, RoomItem item)
        {
            if (disposed)
                return false;
            cycles = 0;
            if (delay == 0 && user != null)
            {
                return ToggleItems(user);
            }
            else
            {
                lock (delayedTriggeringUsers.SyncRoot)
                {
                    delayedTriggeringUsers.Enqueue(user);
                }
                handler.RequestCycle(this);
            }
            return false;
        }

        private bool ToggleItems(RoomUser user)
        {
            if (disposed)
                return false;
            GenericTriggerWithModes.DoAnimation(this.item);
            handler.OnEvent(itemID);
            bool itemTriggered = false;

            foreach (RoomItem item in items)
            {
                if (item == null)
                    continue;
                if (user != null && user.GetClient() != null)
                    item.Interactor.OnTrigger(user.GetClient(), item, 0, true);
                else
                    item.Interactor.OnTrigger(null, item, 0, true);
                itemTriggered = true;
            }
            return itemTriggered;
        }

        public void Dispose()
        {
            disposed = true;
            gamemap = null;
            handler = null;
            DetatchFromListeners();
            if (items != null)
                items.Clear();
            delayedTriggeringUsers.Clear();
        }

        public bool IsSpecial(out SpecialEffects function)
        {
            function = SpecialEffects.None;
            return false;
        }

        public bool Disposed()
        {
            return disposed;
        }

        public bool WiredConditionsMet(RoomUser user, Room theRoom)
        {
            if (handler.conditionHandler != null)
            {
                return handler.conditionHandler.AllowsHandling(item.GetX, item.GetY, user, theRoom);
            }
            return true;
        }


        public List<RoomItem> getAffectedItemList()
        {
            return this.items;
        }

        public bool onItemRemove(RoomItem item)
        {
            return this.items.Remove(item);
        }

        public string[] GetSettings()
        {
            return new string[] { delay.ToString() };
        }


        public void LoadFromWiredObject(WiredLoader.WiredLoaderObject loadedData)
        {
            this.items = loadedData.items;
            this.delay = loadedData.GetIntWithIndex(0, 20);
            AttatchListenersToItems();
        }

        public RoomItem GetOriginalWiredItem()
        {
            return item;
        }
    }
}
