﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Butterfly.HabboHotel.Rooms.Wired.WiredHandlers.Interfaces;
using Butterfly.HabboHotel.Items;
using Butterfly.HabboHotel.Rooms.Games;
using Database_Manager.Database.Session_Details.Interfaces;
using System.Data;
using Butterfly.HabboHotel.Items.FurniTriggers.Triggers;

namespace Butterfly.HabboHotel.Rooms.Wired.WiredHandlers.Triggers
{
    class EntersRoom : IWiredTrigger, IWiredSettingsTrigger
    {
        private RoomItem item;
        private WiredHandler handler;
        private bool isOneUser;
        internal string userName;
        private RoomEventDelegate delegateFunction;

        public EntersRoom(RoomItem item, WiredHandler handler, RoomUserManager roomUserManager, bool isOneUser, string userName)
        {
            this.item = item;
            this.handler = handler;
            this.isOneUser = isOneUser;
            this.userName = userName;
            this.delegateFunction = new RoomEventDelegate(roomUserManager_OnUserEnter);

            roomUserManager.OnUserEnter += delegateFunction;
        }

        private void roomUserManager_OnUserEnter(object sender, EventArgs e)
        {
            GenericTriggerWithModes.DoAnimation(this.item);
            RoomUser user = (RoomUser)sender;
            if ((!user.IsBot && isOneUser && !string.IsNullOrEmpty(userName) && user.GetUsername() == userName) || !isOneUser)
            {
                handler.OnEvent(item.Id);
                handler.RequestStackHandle(item.GetX, item.GetY, null, user, Team.none, item.Room);
            }
        }

        public void Dispose()
        {
            handler = null;
            userName = null;
            if (item != null && item.GetRoom() != null)
                item.GetRoom().GetRoomUserManager().OnUserEnter -= delegateFunction;
            item = null;
        }

        public bool WiredConditionsMet(RoomUser user, Room theRoom)
        {
            if (handler.conditionHandler != null)
            {
                return handler.conditionHandler.AllowsHandling(item.GetX, item.GetY, user, theRoom);
            }
            return true;
        }

        public string[] GetSettings()
        {
            return new string[] { this.isOneUser.ToString(), this.userName };
        }


        public void LoadFromWiredObject(WiredLoader.WiredLoaderObject loadedData)
        {
            this.isOneUser = loadedData.GetBooleanWithIndex(0);
            this.userName = loadedData.StringSettings[1];
        }

        public RoomItem GetOriginalWiredItem()
        {
            return this.item;
        }
    }
}
