﻿using Butterfly.HabboHotel.Items;
using Butterfly.HabboHotel.Rooms.Wired.WiredHandlers.Interfaces;
using Database_Manager.Database.Session_Details.Interfaces;
using System.Data;
using System;

namespace Butterfly.HabboHotel.Rooms.Wired.WiredHandlers
{
    internal class Repeater : IWiredTrigger, IWiredCycleable, IWiredTimer , IWiredSettingsTrigger
    {
        internal int cyclesRequired;
        private int cycleCount;
        private WiredHandler handler;
        private RoomItem item;
        private bool disposed;

        public Repeater(WiredHandler handler, RoomItem item, int cyclesRequired)
        {
            this.handler = handler;
            this.cyclesRequired = cyclesRequired;
            this.item = item;

            handler.RequestCycle(this);
            this.disposed = false;
        }
        
        public bool OnCycle()
        {
            cycleCount++;

            if (cycleCount > cyclesRequired)
            {
                handler.RequestStackHandle(item.GetX, item.GetY, this.item, null, Games.Team.none, item.Room);
                handler.OnEvent(item.Id);
                cycleCount = 0;
            }
            return true;
        }

        public void Dispose()
        {
            disposed = true;
            handler = null;
            item = null;
        }

        public void ResetTimer()
        {
            cycleCount = 0;
        }


        public bool Disposed()
        {
            return disposed;
        }

        public bool WiredConditionsMet(RoomUser user, Room theRoom)
        {
            if (handler.conditionHandler != null)
            {
                return handler.conditionHandler.AllowsHandling(item.GetX, item.GetY, user, theRoom);
            }
            return true;
        }

        public string[] GetSettings()
        {
            return new string[] { this.cyclesRequired.ToString() };
        }


        public void LoadFromWiredObject(WiredLoader.WiredLoaderObject loadedData)
        {
            this.cyclesRequired = loadedData.GetIntWithIndex(0);
        }

        public RoomItem GetOriginalWiredItem()
        {
            return this.item;
        }
    }
}
