﻿using System;
using Butterfly.HabboHotel.Items;
using Butterfly.HabboHotel.Rooms.Games;
using Butterfly.HabboHotel.Rooms.Wired.WiredHandlers.Interfaces;
using Database_Manager.Database.Session_Details.Interfaces;
using System.Data;
using Butterfly.HabboHotel.Items.FurniTriggers.Triggers;

namespace Butterfly.HabboHotel.Rooms.Wired.WiredHandlers.Triggers
{
    class ScoreAchieved : IWiredTrigger , IWiredSettingsTrigger
    {
        private RoomItem item;
        private WiredHandler handler;
        internal int scoreLevel;
        private bool used;
        private TeamScoreChangedDelegate scoreChangedDelegate;
        private RoomEventDelegate gameEndDeletgate;

        public ScoreAchieved(RoomItem item, WiredHandler handler, int scoreLevel, GameManager gameManager)
        {
            this.item = item;
            this.handler = handler;
            this.scoreLevel = scoreLevel;
            this.used = false;
            this.scoreChangedDelegate = new TeamScoreChangedDelegate(gameManager_OnScoreChanged);
            this.gameEndDeletgate = new RoomEventDelegate(gameManager_OnGameEnd);


            gameManager.OnScoreChanged += scoreChangedDelegate;
            gameManager.OnGameEnd += gameEndDeletgate;
        }

        private void gameManager_OnGameEnd(object sender, EventArgs e)
        {
            this.used = false;
        }

        private void gameManager_OnScoreChanged(object sender, TeamScoreChangedArgs e)
        {
            if (e.Points > scoreLevel && !used)
            {
                GenericTriggerWithModes.DoAnimation(this.item);
                used = true;
                handler.RequestStackHandle(item.GetX, item.GetY, null, e.user, e.Team, item.Room);
                handler.OnEvent(item.Id);
            }
        }

        public void Dispose()
        {
            handler.GetRoom().GetGameManager().OnScoreChanged -= scoreChangedDelegate;
            handler.GetRoom().GetGameManager().OnGameEnd -= gameEndDeletgate;
            this.item = null;
            this.handler = null;
        }

        public void SaveToDatabase(IQueryAdapter dbClient)
        {
            WiredUtillity.SaveTriggerItem(dbClient, (int)item.Id, "integer", string.Empty, scoreLevel.ToString(), false);
        }

        public void LoadFromDatabase(IQueryAdapter dbClient, Room insideRoom)
        {
            dbClient.setQuery("SELECT trigger_data FROM trigger_item WHERE trigger_id = @id ");
            dbClient.addParameter("id", (int)this.item.Id);
            DataRow dRow = dbClient.getRow();
            if (dRow != null)
                this.scoreLevel = Convert.ToInt32(dRow[0].ToString());
            else
                this.scoreLevel = 200;
        }

        public void DeleteFromDatabase(IQueryAdapter dbClient)
        {
            dbClient.runFastQuery("DELETE FROM trigger_item WHERE trigger_id = '" + this.item.Id + "'");
        }

        public bool WiredConditionsMet(RoomUser user, Room theRoom)
        {
            if (handler.conditionHandler != null)
            {
                return handler.conditionHandler.AllowsHandling(item.GetX, item.GetY, user, theRoom);
            }
            return true;
        }

        public string[] GetSettings()
        {
            return new string[] { this.scoreLevel.ToString() };
        }


        public void LoadFromWiredObject(WiredLoader.WiredLoaderObject loadedData)
        {
                this.scoreLevel = loadedData.GetIntWithIndex(0);
            if(scoreLevel == -1)
                this.scoreLevel = 200;
        }

        public RoomItem GetOriginalWiredItem()
        {
            return item;
        }
    }
}
