﻿using System.Collections.Generic;
using Butterfly.HabboHotel.Items;
using Butterfly.HabboHotel.Rooms.Wired.WiredHandlers.Interfaces;
using System.Collections;
using Database_Manager.Database.Session_Details.Interfaces;
using System.Data;
using System;
using Butterfly.HabboHotel.Items.FurniTriggers.Triggers;

namespace Butterfly.HabboHotel.Rooms.Wired.WiredHandlers
{
    internal class StateChanged : IWiredFurniTrigger, IWiredTimer, IWiredCycleable, IWiredSettingsTrigger
    {
        private WiredHandler handler;
        internal List<RoomItem> items;
        private RoomItem item;
        private OnItemTrigger delegateFunction;
        private Queue triggeringQueue;
        internal int delay;
        private int cycleCount;
        private bool disposed;

        public StateChanged(WiredHandler handler, RoomItem item, List<RoomItem> items, int delay)
        {
            this.handler = handler;
            this.items = items;
            this.item = item;
            this.delay = delay;
            this.delegateFunction = new OnItemTrigger(itemTriggered);
            this.cycleCount = 0;
            this.triggeringQueue = new Queue();

            foreach (RoomItem _item in items)
            {
                _item.itemTriggerEventHandler += delegateFunction;
            }
            this.disposed = true;
            AttatchListenersToItems();
        }

        void AttatchListenersToItems()
        {
            foreach (RoomItem item in items)
            {
                item.itemRemovedEventHandler += item_itemRemovedEventHandler;
            }
        }

        void DetatchFromListeners()
        {
            if (items == null)
                return;
            foreach (RoomItem item in items)
            {
                item.itemRemovedEventHandler -= item_itemRemovedEventHandler;
            }
        }

        void item_itemRemovedEventHandler(object sender, ItemTriggeredArgs e)
        {
            e.TriggeringItem.itemRemovedEventHandler -= item_itemRemovedEventHandler;
            this.onItemRemove(e.TriggeringItem);
        }

        public bool OnCycle()
        {
            if (cycleCount > delay)
            {
                if (triggeringQueue.Count > 0)
                {
                    lock (triggeringQueue.SyncRoot)
                    {
                        while (triggeringQueue.Count > 0)
                        {
                            ItemTriggeredArgs e = (ItemTriggeredArgs)triggeringQueue.Dequeue();
                            onTrigger(e);
                        }
                    }
                }
                return false;
            }
            else
            {
                cycleCount++;
                return true;
            }
        }

        private void itemTriggered(object sender, ItemTriggeredArgs e)
        {
            GenericTriggerWithModes.DoAnimation(this.item);
            if (delay > 0)
            {
                triggeringQueue.Enqueue(e);
                handler.RequestCycle(this);
            }
            else
            {
                onTrigger(e);
            }
        }

        private void onTrigger(ItemTriggeredArgs e)
        {
            handler.RequestStackHandle(item.GetX, item.GetY, e.TriggeringItem, e.TriggeringUser, Games.Team.none, item.Room);
            handler.OnEvent(item.Id);
        }

        public void Dispose()
        {
            disposed = true;
            handler = null;

            if (items != null)
            {
                foreach (RoomItem _item in items)
                {
                    _item.itemTriggerEventHandler -= delegateFunction;
                }

                items.Clear();
            }
            items = null;
        }

        public void ResetTimer()
        {
            cycleCount = delay;
        }

        public bool Disposed()
        {
            return disposed;
        }
        public bool WiredConditionsMet(RoomUser user, Room theRoom)
        {
            if (handler.conditionHandler != null)
            {
                return handler.conditionHandler.AllowsHandling(item.GetX, item.GetY, user, theRoom);
            }
            return true;
        }

        public string[] GetSettings()
        {
            return new string[] { this.delay.ToString() };
        }


        public void LoadFromWiredObject(WiredLoader.WiredLoaderObject loadedData)
        {
            this.delay = loadedData.GetIntWithIndex(0);
            this.items = loadedData.items;
            foreach (RoomItem _item in items)
            {
                _item.itemTriggerEventHandler += delegateFunction;
            }
            this.AttatchListenersToItems();
        }

        public RoomItem GetOriginalWiredItem()
        {
            return item;
        }

        public List<RoomItem> getAffectedItemList()
        {
            return this.items;
        }

        public bool onItemRemove(RoomItem item)
        {
            if (items != null)
            {
                return this.items.Remove(item);
            }
            return false;
        }
    }
}
