﻿using System;
using Butterfly.HabboHotel.Items;
using Butterfly.HabboHotel.Rooms.Games;
using Butterfly.HabboHotel.Rooms.Wired.WiredHandlers.Interfaces;
using Database_Manager.Database.Session_Details.Interfaces;
using System.Data;
using Butterfly.HabboHotel.Items.FurniTriggers.Triggers;

namespace Butterfly.HabboHotel.Rooms.Wired.WiredHandlers.Triggers
{
    class Timer : IWiredTrigger, IWiredCycleable, IWiredTimer , IWiredSettingsTrigger
    {
        private RoomItem item;
        private WiredHandler handler;
        internal int requiredCycles;
        private int currentCycle;
        private RoomEventDelegate delegateFunction;
        private bool disposed;

        public Timer(RoomItem item, WiredHandler handler, int cycleCount, GameManager gameManager)
        {
            this.item = item;
            this.handler = handler;
            this.requiredCycles = cycleCount;
            this.currentCycle = 0;
            this.delegateFunction = new RoomEventDelegate(gameManager_OnGameEnd);

            gameManager.OnGameEnd += delegateFunction;
            this.disposed = false;
        }

        private void gameManager_OnGameEnd(object sender, EventArgs e)
        {
            continueTimer();   
        }

        private void continueTimer()
        {
            handler.RequestCycle(this);
        }

        private void resetTimer()
        {
            currentCycle = 0;
        }

        public bool OnCycle()
        {
            if (requiredCycles > currentCycle)
            {
                GenericTriggerWithModes.DoAnimation(this.item);
                handler.RequestStackHandle(item.GetX, item.GetY, null, null, Team.none, item.Room);
                handler.OnEvent(item.Id);
                resetTimer();
                return false;
            }
            else
            {
                currentCycle++;
                return true;
            }
        }

        public void Dispose()
        {
            disposed = true;
            this.handler.GetRoom().GetGameManager().OnGameEnd -= delegateFunction;
            this.item = null;
            this.handler = null;
        }

        public void ResetTimer()
        {
            currentCycle = requiredCycles;
        }

        public bool Disposed()
        {
            return disposed;
        }

        public bool WiredConditionsMet(RoomUser user, Room theRoom)
        {
            if (handler.conditionHandler != null)
            {
                return handler.conditionHandler.AllowsHandling(item.GetX, item.GetY, user, theRoom);
            }
            return true;
        }

        public string[] GetSettings()
        {
            return new string[] { this.requiredCycles.ToString() };
        }


        public void LoadFromWiredObject(WiredLoader.WiredLoaderObject loadedData)
        {
            this.requiredCycles = loadedData.GetIntWithIndex(0);
            if (requiredCycles == -1)
                requiredCycles = 20;
        }

        public RoomItem GetOriginalWiredItem()
        {
            return item;
        }
    }
}
