﻿using Butterfly.HabboHotel.Items;
using Butterfly.HabboHotel.Rooms.Wired.WiredHandlers.Interfaces;
using Butterfly.Messages;
using Butterfly.Messages.IOCodes;
using Database_Manager.Database.Session_Details.Interfaces;
using System.Data;
using System.Collections.Generic;
using Butterfly.HabboHotel.Items.FurniTriggers.Triggers;

namespace Butterfly.HabboHotel.Rooms.Wired.WiredHandlers.Triggers
{
    class UserSays : IWiredTrigger, IWiredSettingsTrigger
    {
        private RoomItem item;
        private WiredHandler handler;
        public bool IsOwnerOnly { get; private set; }
        internal string triggerMessage;
        private RoomUserSaysDelegate delegateFunction;

        public UserSays(RoomItem item, WiredHandler handler, bool isOwnerOnly, string triggerMessage, Room room)
        {
            this.item = item;
            this.handler = handler;
            this.IsOwnerOnly = isOwnerOnly;
            this.triggerMessage = triggerMessage;
            this.delegateFunction = new RoomUserSaysDelegate(roomUserManager_OnUserSays);

            room.OnUserSays += delegateFunction;
        }

        private void roomUserManager_OnUserSays(object sender, UserSaysArgs e, ref bool messageHandled)
        {
            RoomUser userSaying = e.user;
            string message = e.message;
            
            if ((!IsOwnerOnly && canBeTriggered(message, userSaying)) || (IsOwnerOnly && userSaying.IsOwner() && canBeTriggered(message,userSaying)))
            {
                GenericTriggerWithModes.DoAnimation(this.item);
                handler.RequestStackHandle(item.GetX, item.GetY, null, userSaying, Games.Team.none,item.Room);
                handler.OnEvent(item.Id);

                ServerMessage TellMsg = PacketHandelingManager.GetRecycledItem(OutputCode.WhisperMessageEvent);
                TellMsg.AddInt32(userSaying.VirtualId);
                TellMsg.AddString(message);
                TellMsg.AddInt32(0);
                TellMsg.AddInt32(0);
                TellMsg.AddInt32(0);
                TellMsg.AddInt32(-1);

                userSaying.GetClient().SendMessage(TellMsg);

                messageHandled = true;
            }
        }

        private bool canBeTriggered(string message, RoomUser userSaying)
        {
            if (WiredConditionsMet(userSaying, item.Room))
            {
                return message.ToLower() == triggerMessage.ToLower();
            }
            return false;
        }

        public void Dispose()
        {
            handler.GetRoom().OnUserSays -= delegateFunction;
            item = null;
            handler = null;
            triggerMessage = null;
        }

        public bool WiredConditionsMet(RoomUser user, Room theRoom)
        {
            if (handler.conditionHandler != null)
            {
                return handler.conditionHandler.AllowsHandling(item.GetX, item.GetY, user, theRoom);
            }
            return true;
        }

        string[] IWiredSettingsTrigger.GetSettings()
        {
            return new string[] { this.triggerMessage, this.IsOwnerOnly.ToString() };
        }


        public void LoadFromWiredObject(WiredLoader.WiredLoaderObject loadedData)
        {
            this.triggerMessage = loadedData.StringSettings[0];
            this.IsOwnerOnly = loadedData.GetBooleanWithIndex(1);
        }

        public RoomItem GetOriginalWiredItem()
        {
            return item;
        }
    }
}
