﻿using System.Collections.Generic;
using Butterfly.HabboHotel.Items;
using Butterfly.HabboHotel.Rooms.Wired.WiredHandlers.Interfaces;
using System.Collections;
using Database_Manager.Database.Session_Details.Interfaces;
using System.Data;
using System;
using Butterfly.HabboHotel.Items.FurniTriggers.Triggers;

namespace Butterfly.HabboHotel.Rooms.Wired.WiredHandlers.Triggers
{
    class WalksOnFurni : IWiredFurniTrigger, IWiredCycleable, IWiredTimer, IWiredSettingsTrigger
    {
        private RoomItem item;
        private WiredHandler handler;
        internal List<RoomItem> items;
        private UserWalksFurniDelegate delegateFunction;

        private int currentCycle;
        internal int requiredCycles;
        private Queue requestQueue;

        private bool disposed;

        public WalksOnFurni(RoomItem item, WiredHandler handler, List<RoomItem> targetItems, int requiredCycles)
        {
            this.item = item;
            this.handler = handler;
            this.items = targetItems;
            this.delegateFunction = new UserWalksFurniDelegate(targetItem_OnUserWalksOnFurni);

            this.currentCycle = 0;
            this.requiredCycles = requiredCycles;
            this.requestQueue = new Queue();
            
            foreach (RoomItem targetItem in targetItems)
            {
                targetItem.OnUserWalksOnFurni += delegateFunction;
            }

            this.disposed = false;
            AttatchListenersToItems();
        }

        void AttatchListenersToItems()
        {
            foreach (RoomItem item in items)
            {
                item.itemRemovedEventHandler += item_itemRemovedEventHandler;
            }
        }

        void DetatchFromListeners()
        {
            if (items == null)
                return;
            foreach (RoomItem item in items)
            {
                item.itemRemovedEventHandler -= item_itemRemovedEventHandler;
            }
        }

        void item_itemRemovedEventHandler(object sender, ItemTriggeredArgs e)
        {
            e.TriggeringItem.itemRemovedEventHandler -= item_itemRemovedEventHandler;
            this.onItemRemove(e.TriggeringItem);
        }

        public bool OnCycle()
        {
            if (currentCycle > requiredCycles)
            {
                if (requestQueue.Count > 0)
                {
                    lock (requestQueue.SyncRoot)
                    {
                        while (requestQueue.Count > 0)
                        {
                            GenericTriggerWithModes.DoAnimation(this.item);
                            UserWalksFurniValue obj = (UserWalksFurniValue)requestQueue.Dequeue();
                            handler.RequestStackHandle(item.GetX, item.GetY, obj.item, obj.user, Games.Team.none, item.Room);
                            handler.OnEvent(item.Id);
                        }
                    }
                }
                return false;
            }
            else
            {
                currentCycle++;
                return true;
            }
        }

        private void targetItem_OnUserWalksOnFurni(object sender, UserWalksOnArgs e)
        {
            currentCycle = 0;
            if (requiredCycles > 0)
            {
                UserWalksFurniValue obj = new UserWalksFurniValue(e.user, (RoomItem)sender);
                lock (requestQueue.SyncRoot)
                {
                    requestQueue.Enqueue(obj);
                }

                handler.RequestCycle(this);
            }
            else
            {
                GenericTriggerWithModes.DoAnimation(this.item);
                handler.RequestStackHandle(item.GetX, item.GetY, (RoomItem)sender, e.user, Games.Team.none, item.Room);
                handler.OnEvent(item.Id);
            }
        }

        public void Dispose()
        {
            disposed = true;
            if (items != null)
            {
                DetatchFromListeners();
                foreach (RoomItem targetItem in items)
                {
                    targetItem.OnUserWalksOnFurni -= delegateFunction;
                }
                items.Clear();
            }
            items = null;
            this.item = null;
            this.handler = null;

        }

        public void ResetTimer()
        {
            currentCycle = requiredCycles;
        }

        public bool Disposed()
        {
            return disposed;
        }

        public bool WiredConditionsMet(RoomUser user, Room theRoom)
        {
            if (handler.conditionHandler != null)
            {
                return handler.conditionHandler.AllowsHandling(item.GetX, item.GetY, user, theRoom);
            }
            return true;
        }
        public List<RoomItem> getAffectedItemList()
        {
            return this.items;
        }

        public bool onItemRemove(RoomItem item)
        {
            item.OnUserWalksOnFurni -= delegateFunction;
            return this.items.Remove(item);
        }

        public string[] GetSettings()
        {
            return new string[] { this.requiredCycles.ToString() };
        }


        public void LoadFromWiredObject(WiredLoader.WiredLoaderObject loadedData)
        {
            this.items = loadedData.items;
            this.requiredCycles = loadedData.GetIntWithIndex(0,0);
            foreach (RoomItem targetItem in items)
            {
                targetItem.OnUserWalksOnFurni += delegateFunction;
            }
            AttatchListenersToItems();
        }

        public RoomItem GetOriginalWiredItem()
        {
            return this.item;
        }
    }
}
