﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Butterfly.HabboHotel.Items;
using Database_Manager.Database.Session_Details.Interfaces;

namespace Butterfly.HabboHotel.Rooms.Wired.WiredHandlers.WiredLoader
{
    internal class WiredLoaderObject
    {
        internal static WiredLoaderObject LoadItem(RoomItem item, Room room, IQueryAdapter dbClient)
        {
            InteractionType type = item.GetBaseItem().InteractionType;
            List<RoomItem> items = null;
            if (NeedsFurnitures(type))
            {
                dbClient.setQuery("SELECT trigger_item FROM wired_to_item WHERE wired_id = " + item.Id);
                DataTable dTable = dbClient.getTable();
                RoomItem targetItem;
                items = new List<RoomItem>(5);
                foreach (DataRow dRows in dTable.Rows)
                {
                    targetItem = room.GetRoomItemHandler().GetItem(Convert.ToUInt32(dRows[0]));
                    if (targetItem == null || items.Contains(targetItem))
                    {
                        continue;
                    }

                    items.Add(targetItem);
                }
            }

            string[] settings = null;
            
            if(HasSettings(type))
            {
                settings = new string[4];
                dbClient.setQuery("SELECT data_one, data_two,data_three, data_four FROM wired_data WHERE wired_id = " + item.Id);

                DataRow dRow = dbClient.getRow();
                if (dRow != null)
                {
                    for (int i = 0; i < 4; i++)
                    {
                        settings[i] = dRow[i].ToString();
                    }
                }
                else
                {
                    settings = new string[] { "", "", "", "" };
                }
            }

            Dictionary<uint, OriginalItemLocation> originalPositionList = null;
            if (HasOriginalItemLocations(type))
            {
                originalPositionList = new Dictionary<uint, OriginalItemLocation>(5);
                dbClient.setQuery("SELECT item_id, original_x, original_y, rotation, height, extradata FROM wired_original_item_locations WHERE wired_id = " + item.Id);
                DataTable dTable = dbClient.getTable();
                foreach (DataRow dRows in dTable.Rows)
                {
                    originalPositionList.Add(uint.Parse(dRows[0].ToString()), new OriginalItemLocation(uint.Parse(dRows[0].ToString()), int.Parse(dRows[1].ToString()), int.Parse(dRows[2].ToString()), (double)Convert.ToDecimal(dRows[4]), int.Parse(dRows[3].ToString()), dRows[5].ToString()));
                }
            }
            return new WiredLoaderObject(items, originalPositionList, settings);
        }

        public static bool NeedsFurnitures(InteractionType type)
        {
            switch (type)
            {
                case InteractionType.triggerstatechanged:
                case InteractionType.triggerwalkonfurni:
                case InteractionType.triggerwalkofffurni:
                case InteractionType.actionposreset:
                case InteractionType.actionmoverotate:
                case InteractionType.actionresettimer:
                case InteractionType.actionteleportto:
                case InteractionType.actiontogglestate:
                case InteractionType.conditionfurnishaveusers:
                case InteractionType.conditionstatepos:
                case InteractionType.conditiontriggeronfurni:
                case InteractionType.conditionhasfurnionfurni:
                    return true;
            }
            return false;
        }

        public static bool HasSettings(InteractionType type)
        {
            switch (type)
            {
                case InteractionType.triggertimer:
                case InteractionType.triggerroomenter:
                case InteractionType.triggerrepeater:
                case InteractionType.triggeronusersay:
                case InteractionType.triggerscoreachieved:
                case InteractionType.triggerstatechanged:
                case InteractionType.triggerwalkonfurni:
                case InteractionType.triggerwalkofffurni:
                case InteractionType.actiongivescore:
                case InteractionType.actionposreset:
                case InteractionType.actionmoverotate:
                case InteractionType.actionresettimer:
                case InteractionType.actionshowmessage:
                case InteractionType.actionteleportto:
                case InteractionType.actiontogglestate:
                case InteractionType.conditiontimelessthan:
                case InteractionType.conditiontimemorethan:
                case InteractionType.conditionstatepos:
                case InteractionType.conditionhasfurnionfurni:
                    return true;
            }
            return false;
        }


        public static bool HasOriginalItemLocations(InteractionType type)
        {
            switch (type)
            {
                case InteractionType.actionposreset:
                case InteractionType.conditionstatepos:
                    return true;
            }
            return false;
        }


        public List<RoomItem> items { get; private set; }

        public Dictionary<uint, OriginalItemLocation> originalPositionList { get; private set; }

        public string[] StringSettings { get; private set; }

        private WiredLoaderObject(List<RoomItem> item, Dictionary<uint, OriginalItemLocation> originalPositionList, string[] customSettings)
        {
            this.items = item;
            this.originalPositionList = originalPositionList;
            this.StringSettings = customSettings;
        }

        public bool GetBooleanWithIndex(int index, bool def=false)
        {
            if (index < StringSettings.Length)
            {
                bool toReturn = false;
                if (bool.TryParse(StringSettings[index],out toReturn))
                    return toReturn;
            }
            return def;
        }

        public int GetIntWithIndex(int index, int def = -1)
        {
            if (index < StringSettings.Length)
            {
                int toReturn = 0;
                if (int.TryParse(StringSettings[index], out toReturn))
                    return toReturn;
            }
            return def;
        }
    }
}
