﻿using Butterfly.HabboHotel.GameClients;
using Butterfly.HabboHotel.Rooms;
using Butterfly.Messages;
using Butterfly.Messages.IOCodes;
using Collections;
using Database_Manager.Database.Session_Details.Interfaces;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using System.Linq;

namespace Butterfly.HabboHotel.Navigators
{
    class Navigator
    {
        //private Dictionary<int, string> PublicCategories;
        private Hashtable PrivateCategories;

        private Dictionary<Int32, PublicItem> PublicItems;

        internal Navigator()
        {
            //PublicCategories = new Dictionary<int, string>();
            PrivateCategories = new Hashtable();
            PublicItems = new Dictionary<int, PublicItem>();
            //PublicRecommended = new Dictionary<int, PublicItem>();
        }

        internal void Initialize(IQueryAdapter dbClient)
        {
            dbClient.setQuery("SELECT id,caption,min_rank FROM navigator_flatcats WHERE enabled = '1'");
            DataTable dPrivCats = dbClient.getTable();

            foreach (DataRow Row in dPrivCats.Rows)
            {
                PrivateCategories.Add((int)Row["id"], new FlatCat((int)Row["id"], (string)Row["caption"], (int)Row["min_rank"]));
            }

            InitializePublicRoomListing(dbClient);
        }

        internal void InitializePublicRoomListing(IQueryAdapter dbClient)
        {
            Dictionary<int, PublicItem> items = new Dictionary<int, PublicItem>();
            dbClient.setQuery("SELECT * FROM navigator_publics_new ORDER BY ordernum ASC");
            DataTable dPubItems = dbClient.getTable();

            foreach (DataRow Row in dPubItems.Rows)
            {
                items.Add((int)Row["id"], new PublicItem((int)Row["id"], int.Parse(Row["bannertype"].ToString()), (string)Row["caption"], (string)Row["description"],
                    (string)Row["image"], ((Row["image_type"].ToString().ToLower() == "internal") ? PublicImageType.INTERNAL : PublicImageType.EXTERNAL),
                    Convert.ToUInt32(Row["room_id"]), (int)Row["category_id"], (int)Row["category_parent_id"], TextHandling.EnumToBool(Row["recommended"].ToString()), (int)Row["typeofdata"], (string)Row["tag"]));
            }

            this.PublicItems = items;
        }

        internal Int32 GetCountForParent(Int32 ParentId)
        {
            int i = 0;

            foreach (PublicItem Item in PublicItems.Values)
            {
                if (Item.ParentId == ParentId || ParentId == -1)
                {
                    i++;
                }
            }

            return i;
        }

        internal FlatCat GetFlatCat(Int32 Id)
        {
            if (PrivateCategories.ContainsKey(Id))
                return (FlatCat)PrivateCategories[Id];

            return null;
        }

        internal ServerMessage SerializeFlatCategories()
        {
            ServerMessage Cats = PacketHandelingManager.GetRecycledItem(OutputCode.UserFlatCatsEvent);
            Cats.AddInt32(PrivateCategories.Count);
            foreach (FlatCat FlatCat in PrivateCategories.Values)
            {

                Cats.AddInt32(FlatCat.Id);
                Cats.AddString(FlatCat.Caption);
                Cats.AddBoolean(true);
            }

            Cats.AddBoolean(true);

            return Cats;
        }

        internal ServerMessage SerializePublicRooms()
        {
            ServerMessage Frontpage = PacketHandelingManager.GetRecycledItem(OutputCode.OfficialRoomsEvent);
            Frontpage.AddInt32(this.PublicItems.Count);
            foreach (PublicItem Pub in PublicItems.Values)
            {
                if (Pub.ParentId > 0)
                    continue;

                Pub.Serialize(Frontpage);

                if (Pub.itemType != PublicItemType.CATEGORY)
                    continue;

                foreach (PublicItem pub in PublicItems.Values)
                {
                    if (Pub.Id != pub.ParentId)
                        continue;

                    pub.Serialize(Frontpage);
                }
            }
            Frontpage.AddInt32(0);
            Frontpage.AddInt32(0);
            return Frontpage;
        }

        private void SerializeItemsFromCata(int Id, ServerMessage Frontpage)
        {
            foreach (PublicItem Item in this.PublicItems.Values)
            {
                if (Item.ParentId == Id && Item.itemType != PublicItemType.CATEGORY)
                {
                    Item.Serialize(Frontpage);
                }
            }
        }

        internal ServerMessage SerializeFavoriteRooms(GameClient Session)
        {
            ServerMessage Rooms = PacketHandelingManager.GetRecycledItem(OutputCode.GuestRoomSearchResultEvent);
            Rooms.AddInt32(0);
            Rooms.AddString("1");
            Rooms.AddInt32(Session.GetHabbo().FavoriteRooms.Count);

            RoomData room;
            foreach (uint Id in Session.GetHabbo().FavoriteRooms.ToArray())
            {
                room = ButterflyEnvironment.Game.GetRoomManager().GenerateRoomData(Id);
                if (room != null)
                    room.Serialize(Rooms, false);
            }

            Rooms.AddBoolean(false);

            return Rooms;
        }

        internal ServerMessage SerializeRecentRooms(GameClient Session)
        {
            //using (IQueryAdapter dbClient = ButterflyEnvironment.DatabaseManager.getQueryreactor())
            //{
            //    dbClient.setQuery("SELECT rooms.id, rooms.caption, rooms.description, rooms.roomtype, rooms.owner, rooms.state, rooms.category, rooms.users_now, rooms.users_max, rooms.model_name, rooms.public_ccts, rooms.score, rooms.allow_pets, rooms.allow_pets_eat, rooms.allow_walkthrough, rooms.allow_hidewall, rooms.password, rooms.wallpaper, rooms.floor, rooms.landscape, rooms.icon_items, rooms.icon_bg, rooms.icon_fg, rooms.tags, rooms.allow_rightsoverride FROM rooms JOIN user_roomvisits ON rooms.id = user_roomvisits.room_id WHERE user_roomvisits.user_id = " + Session.GetHabbo().Id + " ORDER BY entry_timestamp DESC LIMIT 50;");
            //    DataTable Data = dbClient.getTable();

            //    List<RoomData> ValidRecentRooms = new List<RoomData>();
            //    List<uint> RoomsListed = new List<uint>();

            //    if (Data != null)
            //    {
            //        foreach (DataRow Row in Data.Rows)
            //        {

            //            RoomData tData = ButterflyEnvironment.Game.GetRoomManager().FetchRoomData((uint)Row["id"], Row);
            //            tData.Fill(Row);
            //            ValidRecentRooms.Add(tData);
            //            RoomsListed.Add(tData.Id);
            //        }
            //    }

            ServerMessage Rooms = PacketHandelingManager.GetRecycledItem(OutputCode.GuestRoomSearchResultEvent);
            //Rooms.AddInt32(0);
            //Rooms.AddInt32(7);
            Rooms.AddInt32(0);
            Rooms.AddString("1");
            //Rooms.AddInt32(ValidRecentRooms.Count);
            Rooms.AddInt32(0);
            Rooms.AddBoolean(false);

            //    foreach (RoomData _Data in ValidRecentRooms)
            //    {
            //        _Data.Serialize(Rooms, false);
            //    }

            return Rooms;
            //}
        }

        internal ServerMessage SerializeEventListing(int Category)
        {
            ServerMessage Message = PacketHandelingManager.GetRecycledItem(OutputCode.GuestRoomSearchResultEvent);
            Message.AddInt32(Category);
            Message.AddString("12");

            int count = 0;
            ISortable<RoomData>[] eventRooms = ButterflyEnvironment.Game.GetRoomManager().GetEventRoomsForCategory(Category, out count);
            Message.AddInt32(count);

            for (int i = 0; i < count; i++)
            {
                eventRooms[i].GetValue().Serialize(Message, true);
            }

            Message.AddBoolean(false);

            return Message;
        }

        internal ServerMessage SerializePopularRoomTags()
        {
            Dictionary<string, int> Tags = new Dictionary<string, int>();

            using (IQueryAdapter dbClient = ButterflyEnvironment.DatabaseManager.getQueryreactor())
            {
                //if (dbClient.dbType == Database_Manager.Database.DatabaseType.MySQL)
                    dbClient.setQuery("SELECT rooms.tags, room_active.active_users FROM rooms LEFT JOIN room_active ON (room_active.roomid = rooms.id) WHERE roomtype = 'private' AND active_users > 0 ORDER BY active_users DESC LIMIT 50");
				//else
				//	dbClient.setQuery("SELECT TOP 50 rooms.tags, room_active.active_users FROM rooms LEFT JOIN room_active ON (room_active.roomid = rooms.id) WHERE roomtype = 'private' AND active_users > 0 ORDER BY active_users DESC");
                DataTable Data = dbClient.getTable();


                if (Data != null)
                {
                    int activeUsers;
                    foreach (DataRow Row in Data.Rows)
                    {
                        if (!string.IsNullOrEmpty(Row["active_users"].ToString()))
                            activeUsers = (int)Row["active_users"];
                        else
                            activeUsers = 0;

                        List<string> RoomTags = new List<string>();

                        foreach (string Tag in Row["tags"].ToString().Split(','))
                        {
                            RoomTags.Add(Tag);
                        }

                        foreach (string Tag in RoomTags)
                        {
                            if (Tags.ContainsKey(Tag))
                            {
                                Tags[Tag] += activeUsers;
                            }
                            else
                            {
                                Tags.Add(Tag, activeUsers);
                            }
                        }
                    }
                }

                List<KeyValuePair<string, int>> SortedTags = new List<KeyValuePair<string, int>>(Tags);

                SortedTags.Sort(

                    delegate(KeyValuePair<string, int> firstPair,

                    KeyValuePair<string, int> nextPair)
                    {
                        return firstPair.Value.CompareTo(nextPair.Value);
                    }

                );

                ServerMessage Message = PacketHandelingManager.GetRecycledItem(OutputCode.PopularRoomTagsResultEvent);
                Message.AddInt32(SortedTags.Count);

                foreach (KeyValuePair<string, int> TagData in SortedTags)
                {
                    Message.AddString(TagData.Key);
                    Message.AddInt32(TagData.Value);
                }

                return Message;
            }
        }

        internal ServerMessage SerializeSearchResults(string SearchQuery)
        {
            DataTable Data = new DataTable();
            if (SearchQuery.StartsWith("owner:"))
            {
                using (IQueryAdapter dbClient = ButterflyEnvironment.DatabaseManager.getQueryreactor())
                {
                    dbClient.setQuery("SELECT rooms.*, users.id AS ownerid FROM rooms JOIN users ON (users.username = rooms.owner) WHERE owner = @query AND roomtype = 'private'");
                    dbClient.addParameter("query", SearchQuery.Substring(6));
                    Data = dbClient.getTable();
                }
            }
            else
            {
                using (IQueryAdapter dbClient = ButterflyEnvironment.DatabaseManager.getQueryreactor())
                {
                    if (SearchQuery.Length > 0)
                    {
                        //if (dbClient.dbType == Database_Manager.Database.DatabaseType.MySQL)
                        dbClient.setQuery("SELECT rooms.*, users.id AS ownerid FROM rooms LEFT JOIN room_active ON (room_active.roomid = rooms.id) JOIN users ON (users.username = rooms.owner) " + 
                             "WHERE owner = @query OR caption = @query " +
                                        "ORDER BY active_users DESC LIMIT 50");
						//else
						//	dbClient.setQuery("SELECT TOP 50 rooms.*, room_active.active_users FROM rooms LEFT JOIN room_active ON (room_active.roomid = rooms.id) WHERE owner = @query AND roomtype = 'private' " +
						//				"UNION ALL " +
						//				"SELECT rooms.*, room_active.active_users FROM rooms LEFT JOIN room_active ON (room_active.roomid = rooms.id) WHERE caption = @query AND roomtype = 'private' " +
						//				"ORDER BY active_users DESC");
                        dbClient.addParameter("query", SearchQuery);
                        Data = dbClient.getTable();
                    }
                }
            }

            List<RoomData> Results = new List<RoomData>();

            if (Data != null)
            {
                foreach (DataRow Row in Data.Rows)
                {
                    RoomData RData = ButterflyEnvironment.Game.GetRoomManager().FetchRoomData(Convert.ToUInt32(Row["id"]), Row);
                    Results.Add(RData);
                }
            }

            ServerMessage Message = PacketHandelingManager.GetRecycledItem(OutputCode.GuestRoomSearchResultEvent);
            Message.AddInt32(1);
            Message.AddString("9");
           /* Message.AddBoolean(false);*/
           // Message.AddString(SearchQuery);
            Message.AddInt32(Results.Count);
            //Message.AddInt32(0); //@temporary fix
            foreach (RoomData Room in Results)
            {
           
                Room.Serialize(Message, false);
            }

            Message.AddBoolean(false);

            return Message;
        }

        private ServerMessage SerializeActiveRooms(int category)
        {
            ServerMessage reply = PacketHandelingManager.GetRecycledItem(OutputCode.GuestRoomSearchResultEvent);
            reply.AddInt32(category);
            //reply.AddString("1");
            int count = 0;
            ISortable<RoomData>[] rooms = ButterflyEnvironment.Game.GetRoomManager().GetRooms(category, out count);
            SerializeNavigatorRooms(reply, rooms, count);

            rooms = null;
            reply.AddBoolean(false);
            reply.AddBoolean(false);
            return reply;
        }

        internal ServerMessage SerializeNavigator(GameClient session, int mode)
        {
            if (mode > -1)
                return SerializeActiveRooms(mode);

            ServerMessage reply = PacketHandelingManager.GetRecycledItem(OutputCode.GuestRoomSearchResultEvent);

            switch (mode)
            {
                case -5:
                case -4:
                    {
                        reply.AddInt32(0);
                        reply.AddString("4");
                        
                        List<RoomData> activeFriends = session.GetHabboDataContainer().GetChatMessenger().GetActiveFriendsRooms().OrderByDescending(p => p.UsersNow).ToList();
                        SerializeNavigatorRooms(ref reply, activeFriends);

                        return reply;
                    }

                case -3:
                    {
                        reply.AddInt32(0);
                        reply.AddString("5");

                        SerializeNavigatorRooms(ref reply, session.GetHabbo().UsersRooms);

                        return reply;
                    }

                case -2:
                    {
                        reply.AddInt32(0);

                        int count = 0;
                        ISortable<RoomData>[] rooms = ButterflyEnvironment.Game.GetRoomManager().GetVotedRooms(out count);
                        SerializeNavigatorRooms(reply, rooms, count);

                        rooms = null;
                        reply.AddBoolean(false);
                        reply.AddBoolean(false);
                        return reply;
                    }

                case -1:
                    {
                        reply.AddInt32(-1);

                        int count = 0;
                        ISortable<RoomData>[] rooms = ButterflyEnvironment.Game.GetRoomManager().GetActiveRooms(out count);

                        SerializeNavigatorRooms(reply, rooms, count);

                        rooms = null;
                        reply.AddBoolean(false);
                        reply.AddBoolean(false);

                        return reply;
                    }
            }

            return reply;
        }

        private void SerializeNavigatorRooms(ref ServerMessage reply, List<RoomData> rooms)
        {
            //reply.AddInt32(1);
            //reply.AddString(string.Empty);
            reply.AddInt32(rooms.Count);

            foreach (RoomData data in rooms)
            {
                data.Serialize(reply, false);
            }
            reply.AddBoolean(false);

            //if (PublicRecommended.Count > 0)
            //{
            //    reply.AddString("");

            //    foreach (PublicItem Pub in PublicRecommended.Values)
            //    {
            //        Pub.Serialize(reply);
            //    }
            //}
        }

        private void SerializeNavigatorRooms(ServerMessage reply, ISortable<RoomData>[] rooms, int count)
        {
            reply.AddString(string.Empty);
            reply.AddInt32(count);

            for (int i = 0; i < count; i++)
            {
                rooms[i].GetValue().Serialize(reply, false);
            }

            reply.AddBoolean(false);
        }
    }
}
