﻿using System;

using Butterfly.Messages;
using Butterfly.HabboHotel.Rooms;
using Butterfly;

namespace Butterfly.HabboHotel.Navigators
{
    internal enum PublicImageType
    {
        INTERNAL = 0,
        EXTERNAL = 1
    }

    internal enum PublicItemType
    {
        //TAG(1), FLAT(2), PUBLIC_FLAT(3), CATEGORY(4);
        NONE = 0,
        TAG = 1,
        FLAT = 2,
        PUBLIC_FLAT = 3,
        CATEGORY = 4
    }

    internal class PublicItem
    {
        private readonly Int32 BannerId;

        internal int Type;

        internal string Caption;
        internal string Image;
        internal PublicImageType ImageType;

        internal UInt32 RoomId;

        internal Int32 ParentId;

        internal Int32 Id
        {
            get { return BannerId; }
        }

        internal RoomData RoomData
        {
            get
            {
                if (RoomId == 0)
                {
                    return new RoomData();
                }

                if (ButterflyEnvironment.Game == null)
                    throw new NullReferenceException();

                if (ButterflyEnvironment.Game.GetRoomManager() == null)
                    throw new NullReferenceException();

                if (ButterflyEnvironment.Game.GetRoomManager().GenerateRoomData(RoomId) == null)
                    throw new NullReferenceException();

                return ButterflyEnvironment.Game.GetRoomManager().GenerateRoomData(RoomId);
            }
        }

        internal string Description;
        internal Boolean Recommended;
        internal int CategoryId;
        internal PublicItemType itemType;
        internal string TagsToSearch;

        internal PublicItem(int mId, int mType, string mCaption, string mDescription, string mImage, PublicImageType mImageType, uint mRoomId, int mCategoryId, int mParentId, Boolean mRecommand, int mTypeOfData, string mTags)
        {
            BannerId = mId;
            Type = mType;
            Caption = mCaption;
            Description = mDescription;
            Image = mImage;
            ImageType = mImageType;
            RoomId = mRoomId;
            ParentId = mParentId;
            CategoryId = mCategoryId;
            Recommended = mRecommand;
            if (mTypeOfData == 1)
                itemType = PublicItemType.TAG;
            else if (mTypeOfData == 2)
                itemType = PublicItemType.FLAT;
            else if (mTypeOfData == 3)
                itemType = PublicItemType.PUBLIC_FLAT;
            else if (mTypeOfData == 4)
                itemType = PublicItemType.CATEGORY;
            else
                itemType = PublicItemType.NONE;
			this.TagsToSearch = string.Empty;
        }

        internal RoomData RoomInfo
        {
            get
            {
                try
                {
                    if (RoomId > 0)
                        return ButterflyEnvironment.Game.GetRoomManager().GenerateRoomData(RoomId);
                    else
                        return null;
                }
                catch
                {
                    return null;
                }
            }
        }

        internal void Serialize(ServerMessage Message)
        {
            Message.AddInt32(Id);
            Message.AddString(this.itemType != PublicItemType.PUBLIC_FLAT ? Caption : String.Empty);
            Message.AddString(Description); // description
            Message.AddInt32(Type); // image size
            Message.AddString(this.itemType == PublicItemType.PUBLIC_FLAT ? Caption : String.Empty);
            Message.AddString(Image);
            Message.AddInt32(ParentId > 0 ? ParentId : 0);
            Message.AddInt32(RoomInfo != null ? RoomInfo.UsersNow : 0);
            Message.AddInt32(itemType == PublicItemType.NONE ? 0 : (itemType == PublicItemType.TAG ? 1 : (itemType == PublicItemType.FLAT ? 2 : (itemType == PublicItemType.PUBLIC_FLAT ? 2 : (itemType == PublicItemType.CATEGORY ? 4 : 0)))));

            if (this.itemType == PublicItemType.TAG)
            {
                Message.AddString(TagsToSearch); return;
            }
            else if (this.itemType == PublicItemType.CATEGORY)
            {
                Message.AddBoolean(false); return;
            }
            else if (this.itemType == PublicItemType.FLAT)
            {
                this.RoomInfo.Serialize(Message, false);
            }
            else if (this.itemType == PublicItemType.PUBLIC_FLAT)
            {
                this.RoomInfo.Serialize(Message, false);
            }

            if ((this.itemType == PublicItemType.FLAT || this.itemType == PublicItemType.PUBLIC_FLAT) && Message == null)
            {
                Console.WriteLine("ID: " + Id + " is pointing to a room not existing");
            }
        }
    }
}