﻿using Butterfly.HabboHotel.Pathfinding.MapManagement;
using Butterfly.HabboHotel.Rooms;
using System;

namespace Butterfly.HabboHotel.Pathfinding
{
    static class DreamPathfinder
    {
        internal static SquarePoint GetNextStep(int pUserX, int pUserY, double pUserZ,
            int pUserTargetX, int pUserTargetY,
            byte[,] pGameMap, Stackmap pHeight, short[,] pHeightmap,
            int MaxX, int MaxY,
            bool pUserOverride, bool pDiagonal, Gamemap gamemap)
        {
            ModelInfo MapInfo = new ModelInfo(MaxX, MaxY, pGameMap);
            SquarePoint TargetPoint = new SquarePoint(pUserTargetX, pUserTargetY, pUserZ, pUserTargetX, pUserTargetY, MapInfo.GetState(pUserTargetX, pUserTargetY), pUserOverride, pHeight);
            if (pUserX == pUserTargetX && pUserY == pUserTargetY) //User is allready standing on its target
                return TargetPoint;

            SquareInformation SquareOnUser = new SquareInformation(pUserX, pUserY, pUserZ, TargetPoint, MapInfo, pUserOverride, pDiagonal, pHeight);

            //if (!TargetPoint.CanWalk)
            //    return SquareOnUser.Point;

            return GetClosetSqare(SquareOnUser, new HeightInfo(MaxX, MaxY, pHeight, pHeightmap), pUserZ, gamemap, pUserOverride);
        }

        private static SquarePoint GetClosetSqare(SquareInformation pInfo, HeightInfo Height, double currentHeight, Gamemap map, bool pOverride)
        {
            double Closest = pInfo.Point.GetDistance; //Initialized

            SquarePoint ClosestPoint = pInfo.Point;
            double InfoOnSqare = Height.GetState(pInfo.Point.X, pInfo.Point.Y, currentHeight);

            for (int i = 0; i < 8; i++)
            {
                SquarePoint Position = pInfo.Pos(i);
                if (!Position.InUse)
                    continue;

                double h = Height.GetState(Position.X, Position.Y, currentHeight);
                if (Position.CanWalk(h) && (!map.SquareOccupied(Position.X, Position.Y, h) || pOverride))
                {
                    if (h - InfoOnSqare < 3) //<3
                    {
                        double Distance = Position.GetDistance;
                        if (Closest > Distance)
                        {
                            Closest = Distance;
                            ClosestPoint = Position;
                        }
                    }
                }
            }
            return ClosestPoint;
        }

        internal static double GetDistance(int x1, int y1, int x2, int y2)
        {
            return Math.Sqrt(Math.Pow(x1 - x2, 2) + Math.Pow(y1 - y2, 2));
        }
    }

    struct ModelInfo
    {
        private byte[,] mMap;
        private int mMaxX;
        private int mMaxY;

        internal ModelInfo(int MaxX, int MaxY, byte[,] Map)
        {
            mMap = Map;
            mMaxX = MaxX;
            mMaxY = MaxY;
        }

        internal byte GetState(int x, int y)
        {
            if (x >= mMaxX || x < 0)
                return 0;
            if (y >= mMaxY || y < 0)
                return 0;
            return mMap[x, y];
        }
    }

    struct HeightInfo
    {
        private Stackmap mMap;
        private short[,] mRoomHeightmap;
        private int mMaxX;
        private int mMaxY;

        internal HeightInfo(int MaxX, int MaxY, Stackmap Map, short[,] heightmap)
        {
            mMap = Map;
            mMaxX = MaxX;
            mMaxY = MaxY;
            this.mRoomHeightmap = heightmap;
        }

        internal double GetState(int x, int y, double currentHeight)
        {
            if (x >= mMaxX || x < 0)
                return 0;
            if (y >= mMaxY || y < 0)
                return 0;

            return mMap.GetNewHeight(currentHeight, mRoomHeightmap[x,y], x, y);
        }
    }

    struct SquareInformation
    {
        #region Fields
        private int mX;
        private int mY;
        private double mZ;

        private SquarePoint[] mPos;

        private SquarePoint mTarget;
        private SquarePoint mPoint;
        #endregion

        #region Constructor
        internal SquareInformation(int pX, int pY, double pZ, SquarePoint pTarget, ModelInfo pMap, bool pUserOverride, bool CalculateDiagonal, Stackmap map)
        {
            mX = pX;
            mY = pY;
            mZ = pZ;
            mTarget = pTarget;

            mPoint = new SquarePoint(pX, pY, pZ, pTarget.X, pTarget.Y, pMap.GetState(pX, pY), pUserOverride, map);

            //Analyze all the squares arround the user
            mPos = new SquarePoint[8];

            if (CalculateDiagonal)
            {
                mPos[1] = new SquarePoint(pX - 1, pY - 1, pZ, pTarget.X, pTarget.Y, pMap.GetState(pX - 1, pY - 1), pUserOverride, map);
                mPos[3] = new SquarePoint(pX - 1, pY + 1, pZ, pTarget.X, pTarget.Y, pMap.GetState(pX - 1, pY + 1), pUserOverride, map);
                mPos[5] = new SquarePoint(pX + 1, pY + 1, pZ, pTarget.X, pTarget.Y, pMap.GetState(pX + 1, pY + 1), pUserOverride, map);
                mPos[7] = new SquarePoint(pX + 1, pY - 1, pZ, pTarget.X, pTarget.Y, pMap.GetState(pX + 1, pY - 1), pUserOverride, map);
            }

            mPos[0] = new SquarePoint(pX, pY - 1, pZ, pTarget.X, pTarget.Y, pMap.GetState(pX, pY - 1), pUserOverride, map);
            mPos[2] = new SquarePoint(pX - 1, pY, pZ, pTarget.X, pTarget.Y, pMap.GetState(pX - 1, pY), pUserOverride, map);
            mPos[4] = new SquarePoint(pX, pY + 1, pZ, pTarget.X, pTarget.Y, pMap.GetState(pX, pY + 1), pUserOverride, map);
            mPos[6] = new SquarePoint(pX + 1, pY, pZ, pTarget.X, pTarget.Y, pMap.GetState(pX + 1, pY), pUserOverride, map);

            ///456
            //Y3X7
            ///218
            ////X
        }

        //GetState( , pMap, pMaxX, pMaxY
        //private byte GetState(int pX, int pY, byte[,] pMap, int maxX, int maxY)
        //{
        //    if (pX >= maxX)
        //        return 0;
        //    if (pY >= maxY)
        //        return 0;

        //    return pMap[maxX, maxY];
        //}
        #endregion

        #region Return values
        //internal int X
        //{
        //    get
        //    {
        //        return mX;
        //    }
        //    set
        //    {
        //        mX = value;
        //    }
        //}

        //internal int Y
        //{
        //    get
        //    {
        //        return mY;
        //    }
        //    set
        //    {
        //        mY = value;
        //    }
        //}

        internal SquarePoint Pos(int val)
        {
            return mPos[val];
        }

        //internal SquarePoint Target
        //{
        //    get
        //    {
        //        return mTarget;
        //    }
        //}

        internal SquarePoint Point
        {
            get
            {
                return mPoint;
            }
        }
        #endregion
    }

    struct SquarePoint
    {
        #region Fields
        private int mX;
        private int mY;
        private double mZ;
        private double mDistance;
        private byte mSquareData;
        private bool mInUse;
        private bool mOverride;
        private bool mLastStep;
        private Stackmap map;
        #endregion

        #region Constructor
        internal SquarePoint(int pX, int pY, double pZ, int pTargetX, int pTargetY, byte SquareData, bool pOverride, Stackmap map)
        {
            mX = pX;
            mY = pY;
            mZ = pZ;
            //mSquareData = SquareData;
            mSquareData = SquareData;
            this.map = map;
            //map.TryGetSquareStatus(pX, pY, ref mSquareData);
                
            mInUse = true;
            mOverride = pOverride;

            mDistance = 0.0;
            mLastStep = (pX == pTargetX && pY == pTargetY);

            mDistance = DreamPathfinder.GetDistance(pX, pY, pTargetX, pTargetY);

        }
        #endregion

        #region Return values
        internal int X
        {
            get
            {
                return mX;
            }
        }

        internal int Y
        {
            get
            {
                return mY;
            }
        }

        internal double GetDistance
        {
            get
            {
                return mDistance;
            }
        }

        internal bool CanWalk(double h)
        {
            byte squareData = mSquareData;

            map.TryGetSquareStatus(X, Y, h, ref squareData);

            if (!mLastStep)
            {
                if (!mOverride) return (squareData == 1 || squareData == 4);
                else return true;
            }
            else
            {
                if (!mOverride)
                {
                    if (squareData == 3)
                        return true;
                    if (squareData == 1)
                        return true;
                }
                else return true;
            }
            return false;
        }

        internal bool InUse
        {
            get
            {
                return mInUse;
            }
        }
        #endregion
    }
}
