﻿using Butterfly.HabboHotel.Rooms;
using Butterfly.Messages;
using Butterfly.Messages.IOCodes;
using System;

namespace Butterfly.HabboHotel.Pets
{
    class Pet
    {
        internal uint petID;
        internal uint ownerID;
        internal int virtualID;

        internal uint petType;
        internal string name;
        internal string race;
        internal string color;

        internal int experience;
        internal int energy;
        internal int nutrition;

        internal uint roomID;
        internal int X;
        internal int Y;
        internal double Z;

        internal int respect;

        internal double creationStamp;
        internal bool isPlaced;
        internal uint currentRidingUser = 0;

        internal int[] experienceLevels = new int[] { 100, 200, 400, 600, 1000, 1300, 1800, 2400, 3200, 4300, 7200, 8500, 10100, 13300, 17500, 23000, 51900 };
        internal DatabaseUpdateState databaseState;

        internal int hairDye;
        internal int hairType;

        internal Room Room
        {
            get
            {
                return ButterflyEnvironment.Game.GetRoomManager().GetRoom(roomID);
            }
        }

        internal bool IsInRoom
        {
            get
            {
                return (roomID > 0);
            }
        }

        internal int Level
        {
            get
            {
                for (int level = 0; level < experienceLevels.Length; ++level)
                {
                    if (experience < experienceLevels[level])
                        return level + 1;
                }
                return experienceLevels.Length + 1;
            }
        }

        internal const int MAX_LEVEL = 20;

        internal int ExpirienceGoal
        {
            get
            {
                return experienceLevels[Level - 1];
            }
        }

        internal const int MAX_ENERGY = 100;

        internal const int MAX_NUTRITION = 150;

        internal int GetAge
        {
            get
            {
                return (int)Math.Floor((TextHandling.GetUnixTimestamp() - creationStamp) / 86400);
            }
        }

        internal string GetLook
        {
            get
            {
                return petType + " " + race + " " + color;
            }
        }

        internal string OwnerName
        {
            get
            {
                return ButterflyEnvironment.Game.GetClientManager().GetNameById(ownerID);
            }
        }

        internal Pet(uint PetId, uint OwnerId, uint RoomId, string Name, uint Type, string Race, string Color, int Expirience, int Energy, int Nutrition, int Respect, double CreationStamp, int X, int Y, double Z, int hairType, int hairDye)
        {
            this.petID = PetId;
            this.ownerID = OwnerId;
            this.roomID = RoomId;
            this.name = Name;
            this.petType = Type;
            this.race = Race;
            this.color = Color;
            this.experience = Expirience;
            this.energy = Energy;
            this.nutrition = Nutrition;
            this.respect = Respect;
            this.creationStamp = CreationStamp;
            this.X = X;
            this.Y = Y;
            this.Z = Z;
            this.hairType = hairType;
            this.hairDye = hairDye;

            this.isPlaced = false;
            this.databaseState = DatabaseUpdateState.Syncronized;
        }

        internal void OnRespect()
        {
            ServerMessage message = PacketHandelingManager.GetRecycledItem(OutputCode.PetRespectNotificationEvent);
            message.AddInt32(respect);
            message.AddUInt32(ownerID);
            message.AddUInt32(petID);
            message.AddString(name);
            message.AddBoolean(false);
            message.AddInt32(10);
            message.AddBoolean(false);
            message.AddInt32(-2);
            message.AddBoolean(true);
            message.AddString("281");
            Room.SendMessage(message);

            if (databaseState != DatabaseUpdateState.PendingInsert)
                databaseState = DatabaseUpdateState.PendingUpdate;

            if (experience <= 51900)
                AddExpirience(10);

            respect++;
        }

        internal void AddExpirience(int amount)
        {
            ServerMessage message;

            experience = experience + amount;

            if (experience >= 51900)
                return;

            if (databaseState != DatabaseUpdateState.PendingInsert)
                databaseState = DatabaseUpdateState.PendingUpdate;

            if (Room != null)
            {
                message = PacketHandelingManager.GetRecycledItem(OutputCode.PetExperienceEvent);
                message.AddUInt32(petID);
                message.AddInt32(virtualID);
                message.AddInt32(amount);
                Room.SendMessage(message);

                if (experience > ExpirienceGoal)
                {
                    message = PacketHandelingManager.GetRecycledItem(OutputCode.ChatMessageEvent);
                    message.AddInt32(virtualID);
                    message.AddString("*leveled up to level " + Level + " *");
                    message.AddInt32(0);
                    message.AddInt32(0);
                    message.AddInt32(-1);


                    Room.SendMessage(message);
                }
            }

        }

        internal void PetEnergy(bool addEnergy)
        {
            if (this.energy >= MAX_ENERGY) // If Energy is 100, no point.
                return;

            int randomUsage = TextHandling.GetRandomNumber(4, 15);

            if (!addEnergy)
            {
                this.energy -= randomUsage;

                if (this.energy < 0)
                {
                    this.energy = 1;
                    randomUsage = 1;
                }
            }
            else
            {
                this.energy = (this.energy + randomUsage) % MAX_ENERGY;
            }
            if (databaseState != DatabaseUpdateState.PendingInsert)
                databaseState = DatabaseUpdateState.PendingUpdate;
        }

        internal void SerializeInventory(ServerMessage message)
        {
            message.AddUInt32(petID);
            message.AddString(name);
            message.AddUInt32(petType);
            message.AddInt32(int.Parse(race));
            message.AddString(color);
            message.AddInt32(0);
            message.AddInt32(0);
            message.AddInt32(0);
        }

        internal ServerMessage SerializeInfo()
        {
            ServerMessage message = PacketHandelingManager.GetRecycledItem(OutputCode.PetInfoMessageEvent);
            message.AddInt32((int)petID);
            message.AddString(name);
            message.AddInt32(Level);
            message.AddInt32(MAX_LEVEL);
            message.AddInt32(experience);
            message.AddInt32(ExpirienceGoal);
            message.AddInt32(energy);
            message.AddInt32(MAX_ENERGY);
            message.AddInt32(nutrition);
            message.AddInt32(MAX_NUTRITION);
            message.AddInt32(respect);
            message.AddInt32((int)ownerID);
            message.AddInt32(GetAge);
            message.AddString(OwnerName);
            message.AddInt32(1);
            message.AddBoolean(GotSaddle); //Have saddle
            message.AddBoolean(GotUserRiding); //Got user riding
            message.AddInt32(0);
            message.AddInt32(0);
            message.AddInt32(0);
            message.AddInt32(0);
            message.AddInt32(0);
            message.AddInt32(0);
            message.AddInt32(0);
            message.AddInt32(0);
            message.AddString("");
            message.AddBoolean(false);
            message.AddInt32(-1);
            message.AddInt32(-1);
            message.AddInt32(-1);
            message.AddBoolean(false);

            return message;
        }

        internal bool GotSaddle
        {
            get
            {
                return Room.GetRoomUserManager().GetPet(petID).gotSaddle;
            }
        }

        internal bool GotUserRiding
        {
            get
            {
                return Room.GetRoomUserManager().GetPet(petID).PetGotUserRiding();
            }
        }

        internal void OnRoomUpdate()
        {
            ServerMessage message = PacketHandelingManager.GetRecycledItem(OutputCode.PetUpdateEvent); //1118
            message.AddInt32(this.virtualID);
            message.AddUInt32(petID);
            message.AddUInt32(petType);
            message.AddInt32(int.Parse(race));
            message.AddString(color.ToLower());
            if (GotSaddle)
            {
                message.AddInt32(2);
                message.AddInt32(3);
                message.AddInt32(4);
                message.AddInt32(9);
                message.AddInt32(0);
                message.AddInt32(3);
            }
            else
            {
                message.AddInt32(1);
                message.AddInt32(2);
                message.AddInt32(2);
            }

            message.AddInt32(hairType);
            message.AddInt32(hairDye);
            message.AddInt32(3);
            message.AddInt32(hairType);
            message.AddInt32(hairDye);

            message.AddBoolean(GotSaddle);
            message.AddBoolean(GotUserRiding);

            Room.SendMessage(message);
        }
    }

    internal enum DatabaseUpdateState
    {
        Syncronized,
        PendingUpdate,
        PendingInsert
    }
}
