﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using Database_Manager.Database.Session_Details.Interfaces;

namespace Butterfly.HabboHotel.RoomBots
{
    class BotManager
    {
        private List<RoomBot> Bots;
        private List<BotResponse> Responses;
        private List<RandomSpeech> Speeches;

        internal BotManager()
        {
            Bots = new List<RoomBot>();
        }

        internal void LoadBots(IQueryAdapter dbClient)
        {
            Bots = new List<RoomBot>();

            dbClient.setQuery("SELECT * FROM bots");
            DataTable Data = dbClient.getTable();

            dbClient.setQuery("SELECT id, bot_id, keywords, response_text, mode, serve_id FROM bots_responses");
            DataTable BotResponses = dbClient.getTable();

            dbClient.setQuery("SELECT text, shout, bot_id FROM bots_speech");
            DataTable BotSpeech = dbClient.getTable();

            Responses = new List<BotResponse>();
            Speeches = new List<RandomSpeech>();

            foreach (DataRow Row in BotResponses.Rows)
            {
                Responses.Add(new BotResponse(Convert.ToUInt32(Row["bot_id"]), (string)Row["keywords"], (string)Row["response_text"], Row["mode"].ToString(), (int)Row["serve_id"]));
            }

            foreach (DataRow Row in BotSpeech.Rows)
            {
                Speeches.Add(new RandomSpeech((string)Row["text"], TextHandling.EnumToBool(Row["shout"].ToString()), Convert.ToUInt32(Row["bot_id"])));
            }


            if (Data == null)
            {
                return;
            }

            foreach (DataRow Row in Data.Rows)
            {
                Bots.Add(FetchBot(Row));
            }
        }

        //internal bool RoomHasBots(UInt32 RoomId)
        //{
        //    return (GetBotsForRoom(RoomId).Count >= 1);
        //}

        internal RoomBot FetchBot(DataRow dRow)
        {
            string BotAI = (string)dRow["ai_type"];
            AIType BotAIType;
            switch (BotAI)
            {
                case "generic":
                    BotAIType = AIType.Generic;
                    break;
                case "guide":
                    BotAIType = AIType.Guide;
                    break;
                case "pet":
                    BotAIType = AIType.Pet;
                    break;
                default:
                    BotAIType = AIType.Generic;
                    break;
            } 

            return new RoomBot(Convert.ToUInt32(dRow["id"]), Convert.ToUInt32(dRow["room_id"]), BotAIType, (string)dRow["walk_mode"],
                        (String)dRow["name"], (string)dRow["motto"], (String)dRow["look"], (int)dRow["x"], (int)dRow["y"], (int)dRow["z"],
                        (int)dRow["rotation"], (int)dRow["min_x"], (int)dRow["min_y"], (int)dRow["max_x"], (int)dRow["max_y"], (uint)dRow["owner_id"], (int)dRow["speaking_interval"], (int)dRow["automatic_chat"] == 1 ? true : false, Speeches, Responses);
        }

        internal List<RoomBot> GetBotsForRoom(UInt32 RoomId)
        {
            return new List<RoomBot>(from p in Bots where p.RoomId == RoomId select p);
        }

        internal RoomBot GetBot(UInt32 BotId)
        {
            return (from p in Bots where p.BotId == BotId select p).SingleOrDefault();
        }
    }
}
