﻿using Butterfly.Concurrency;
using Butterfly.HabboHotel.GameClients;
using Butterfly.HabboHotel.Group;
using Butterfly.Messages;
using Butterfly.Messages.IOCodes;
using System.Collections.Generic;

namespace Butterfly.HabboHotel.Rooms.GroupHandling
{
    class GroupBadgeHandler
    {
        private IBroadcaster broadcaster;
        private Dictionary<GroupData, uint> currentUsers;
        private QueuedAction<GroupMember> listRequests;
        private QueuedAction<GroupMember> exitRequests;

        public GroupBadgeHandler()
        {
            this.currentUsers = new Dictionary<GroupData, uint>();
            this.exitRequests = new QueuedAction<GroupMember>(UserLeaving);
            this.listRequests = new QueuedAction<GroupMember>(ReceiveBadgeList);
        }

        internal void RegisterBroadcaster(IBroadcaster broadcaster)
        {
            this.broadcaster = broadcaster;
        }

        internal void RequestBroadcast(GroupMember member)
        {
            listRequests.RequestAction(member);
        }

        private void ReceiveBadgeList(GroupMember member)
        {
            uint count = 0;
            if (!currentUsers.TryGetValue(member.Group, out count))
            {
                currentUsers.Add(member.Group, 1);

                ServerMessage message = PacketHandelingManager.GetRecycledItem(Outgoing.SendGroup);
                message.AddInt32(1);
                message.AddUInt32(member.Group.ID);
                message.AddString(member.Group.image);
                broadcaster.SendMessage(message);
            }
            else
            {
                currentUsers[member.Group] = ++count;
            }

            GameClient client = ButterflyEnvironment.Game.GetClientManager().GetClient(member.data.Id);
            if (client != null)
                client.SendMessage(SerializeList());
        }

        internal void RequestExit(GroupMember member)
        {
            exitRequests.RequestAction(member);
        }

        private void UserLeaving(GroupMember member)
        {
            uint count = 0;
            if (currentUsers.TryGetValue(member.Group, out count))
            {
                if (count > 1)
                    currentUsers[member.Group] = --count;
                else
                    currentUsers.Remove(member.Group);
            }
        }

        private ServerMessage SerializeList()
        {
            ServerMessage message = PacketHandelingManager.GetRecycledItem(Outgoing.SendGroup);
            message.AddInt32(currentUsers.Count);

            foreach (KeyValuePair<GroupData, uint> pair in currentUsers)
            {
                message.AddUInt32(pair.Key.ID);
                message.AddString(pair.Key.image);
            }

            return message;
        }

        internal void ProcessRequests()
        {
            listRequests.HandleActions();
            exitRequests.HandleActions();
        }
    }
}
