﻿using System;
using System.Collections.Generic;
using System.Data;
using Butterfly.Core;
using Butterfly.Messages;
using Butterfly.HabboHotel.Group;

namespace Butterfly.HabboHotel.Rooms
{
    public class RoomData
    {
        internal UInt32 Id;
        internal uint ownerId;
        internal string Name;
        internal string Description;
        internal string Type;
        internal string Owner;
        internal string Password;
        internal int State;//byte
        internal int Category;//byte
        internal int UsersNow;//byte
        internal int UsersMax; //uint16
        internal string ModelName;
        internal string CCTs;
        internal int Score; //uint16
        internal List<string> Tags;
        internal bool AllowPets;
        internal bool AllowPetsEating;
        internal bool AllowWalkthrough;
        internal bool AllowRightsOverride;
        internal bool Hidewall;
        internal string Wallpaper;
        internal string Floor;
        internal string Landscape;
        private RoomModel mModel;
        internal int WallThickness;
        internal int FloorThickness;
        internal uint GroupID;

        internal int TagCount
        {
            get
            {
                return Tags.Count;
            }
        }

        internal RoomModel Model
        {
            get
            {
                if (mModel == null)
                    mModel = ButterflyEnvironment.Game.GetRoomManager().GetModel(ModelName, Id);
                return mModel;
            }
        }

        internal RoomData() { }

        internal void FillNull(UInt32 pId)
        {
            this.Id = pId;
            this.Name = "Unknown Room";
            this.ownerId = 0;
            this.Description = "-";
            this.Type = "private";
            this.Owner = "-";
            this.Category = 0;
            this.UsersNow = 0;
            this.UsersMax = 0;
            this.ModelName = "NO_MODEL";
            this.CCTs = "";
            this.Score = 0;
            this.Tags = new List<string>();
            this.AllowPets = true;
            this.AllowPetsEating = false;
            this.AllowWalkthrough = true;
            this.Hidewall = false;
            this.Password = "";
            this.Wallpaper = "0.0";
            this.Floor = "0.0";
            this.Landscape = "0.0";
            this.WallThickness = 0;
            this.FloorThickness = 0;
            //this.Event = null;
            this.AllowRightsOverride = false;

            mModel = ButterflyEnvironment.Game.GetRoomManager().GetModel(ModelName, pId);
        }

        internal void Fill(DataRow Row)
        {
            this.Id = Convert.ToUInt32(Row["id"]);
            this.Name = (string)Row["caption"];
            this.Description = (string)Row["description"];
            this.Type = "private";
            this.Owner = (string)Row["owner"];
            this.ownerId = (uint)Row["ownerid"];

            switch (Row["state"].ToString().ToLower())
            {
                case "open":

                    this.State = 0;
                    break;

                case "password":

                    this.State = 2;
                    break;

                case "locked":
                default:

                    this.State = 1;
                    break;
            }

            this.Category = (int)Row["category"];

            if (Row.Table.Columns.Contains("active_users"))
            {
                if (!string.IsNullOrEmpty(Row["active_users"].ToString()))
                    this.UsersNow = (int)Row["active_users"];
                else
                    this.UsersNow = 0;
            }
            else
            {
                this.UsersNow = 0;
            }
            this.UsersMax = (int)Row["users_max"];
            this.ModelName = (string)Row["model_name"];
            this.CCTs = string.Empty; //not in use any more
            this.Score = (int)Row["score"];
            this.Tags = new List<string>();
            this.AllowPets = TextHandling.EnumToBool(Row["allow_pets"].ToString());
            this.AllowPetsEating = TextHandling.EnumToBool(Row["allow_pets_eat"].ToString());
            this.AllowWalkthrough = TextHandling.EnumToBool(Row["allow_walkthrough"].ToString());
            this.AllowRightsOverride = TextHandling.EnumToBool(Row["allow_rightsoverride"].ToString());
            this.Hidewall = TextHandling.EnumToBool((string)Row["allow_hidewall"]);
            this.Password = (string)Row["password"];
            this.Wallpaper = (string)Row["wallpaper"];
            this.Floor = (string)Row["floor"];
            this.Landscape = (string)Row["landscape"];
            this.FloorThickness = (int)Row["floorthickness"];
            this.WallThickness = (int)Row["wallthickness"];
            this.GroupID = (uint)Row["group_id"];
            //this.Event = null;

            Dictionary<int, int> IconItems = new Dictionary<int,int>();

            if (!string.IsNullOrEmpty(Row["icon_items"].ToString()))
            {
                foreach (string Bit in Row["icon_items"].ToString().Split('|'))
                {
                    if (string.IsNullOrEmpty(Bit))
                        continue;

                    string[] tBit = Bit.Replace('.', ',').Split(',');

                    int a = 0;
                    int b = 0;

                    int.TryParse(tBit[0], out a);
                    if (tBit.Length > 1)
                        int.TryParse(tBit[1], out b);

                    try
                    {
                        if (!IconItems.ContainsKey(a))
                            IconItems.Add(a, b);
                    }
                    catch (Exception e)
                    {
                        Logging.LogException("Exception: " + e.ToString() + "[" + Bit + "]");
                    }
                }
            }

            //this.myIcon = new RoomIcon((int)Row["icon_bg"], (int)Row["icon_fg"], IconItems);

            foreach (string Tag in Row["tags"].ToString().Split(','))
            {
                this.Tags.Add(Tag);
            }

            mModel = ButterflyEnvironment.Game.GetRoomManager().GetModel(ModelName, Id);
        }

        internal void Serialize(ServerMessage Message, Boolean ShowEvents)
        {
            Message.AddInt32((int)Id);

            Message.AddString(Name);
            Message.AddBoolean(true);
            Message.AddUInt32(ButterflyEnvironment.Game.GetClientManager().GetIdByName(Owner)); //OwnerID
            Message.AddString(Owner);
            Message.AddInt32(State); // room state
            Message.AddInt32(UsersNow);
            Message.AddInt32(UsersMax);
            Message.AddString(Description);
            Message.AddInt32((Category == 3) ? 2 : 0); // dunno!
            Message.AddInt32(0); // can trade?
            Message.AddInt32(Score);
            Message.AddInt32(0);
            Message.AddInt32(Category);

            if (GroupID > 0)
            {
                GroupData group = ButterflyEnvironment.Game.GetGroupManager().GetGroup(GroupID);
                Message.AddUInt32(group.ID);
                Message.AddString(group.Name);
                Message.AddString(group.Badge);
            }
            else
            {
                //if no group id relationship
                Message.AddInt32(0);
                Message.AddInt32(0);
            }

            //if not
            // Group ID (int)
            // Group name (string)
            // Group badge (string)


            Message.AddString("");
            Message.AddInt32(TagCount);

            foreach (string Tag in Tags)
            {
                Message.AddString(Tag);
            }

            Message.AddInt32(0);
            Message.AddInt32(0);
            Message.AddInt32(0);
            Message.AddBoolean(true);
            Message.AddBoolean(true);
            Message.AddInt32(0);
            Message.AddInt32(0);
        }

        internal void SerializeGuestRoomResults(ServerMessage message, bool ario = false)
        {
            message.AddBoolean(ario);
            message.AddUInt32(Id);
            message.AddString(Name);
            message.AddBoolean(true);
            message.AddInt32((int)ButterflyEnvironment.Game.GetClientManager().GetIdByName(this.Owner));
            message.AddString(Owner);
            message.AddInt32(State); // room state
            message.AddInt32(UsersNow);
            message.AddInt32(UsersMax);
            message.AddString(Description);
            message.AddInt32(0); // dunno!
            message.AddInt32((Category == 9) ? 2 : 0); // can trade!
            message.AddInt32(Score);
            message.AddInt32(0);
            message.AddInt32(Category);

            if (GroupID > 0)
            {
                GroupData group = ButterflyEnvironment.Game.GetGroupManager().GetGroup(GroupID);
                message.AddUInt32(group.ID);
                message.AddString(group.Name);
                message.AddString(group.Badge);
            }
            else
            {
                //if no group id relationship
                message.AddInt32(0);
                message.AddInt32(0);
            }

            //Group
            //ID (int), string name, string badge

            message.AddString("");
            message.AddInt32(TagCount);

            foreach (string Tag in Tags)
            {
                message.AddString(Tag);
            }
            message.AddInt32(0);
            message.AddInt32(0);
            message.AddInt32(0);
            message.AddBoolean(true);
            message.AddBoolean(true);
            message.AddInt32(0);
            message.AddInt32(0);
            message.AddBoolean(false);
            message.AddBoolean(false);
            message.AddBoolean(false);
            message.AddInt32(0);
            message.AddInt32(0);
            message.AddInt32(0);
            message.AddBoolean(false);
            message.AddBoolean(true);
        }
    }
}
