﻿using Butterfly.Core;
using Butterfly.HabboHotel.Events;
using Butterfly.HabboHotel.GameClients;
using Butterfly.HabboHotel.Users;
using Butterfly.Messages;
using Collections;
using Database_Manager.Database.Session_Details.Interfaces;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using System.Diagnostics;
using System.Threading;


namespace Butterfly.HabboHotel.Rooms
{
    class RoomManager
    {
        #region Fields
        internal const int MAX_PETS_PER_ROOM = 15;

        private Dictionary<uint, Room> loadedRooms;

        private Queue roomsToAddQueue;
        private Queue roomsToRemoveQueue;

        private Queue votedRoomsAddQueue;
        private Queue votedRoomsRemoveQueue;

        private Queue activeRoomsUpdateQueue;
        private Queue activeRoomsAddQueue;
        private Queue activeRoomsRemoveQueue;
		private Queue roomLoadQueue;

        private Hashtable roomModels;
        //private Hashtable loadedRoomData;
        private Dictionary<uint, List<RoomLinkInformation>> roomLinks;
        private Dictionary<int, RoomCategory> roomCategories;

        private Dictionary<RoomData, int> votedRooms;
        //private IEnumerable<KeyValuePair<RoomData, int>> orderedVotedRooms;
        private OrderedArray<RoomData> orderedVotedRooms;

        private Dictionary<RoomData, int> activeRooms;
        //private IEnumerable<KeyValuePair<RoomData, int>> orderedActiveRooms;
        private OrderedArray<RoomData> orderedActiveRooms;

        private List<Room> mToRemove;

        private EventManager eventManager;

		internal delegate void OnRoomLoad();
        #endregion

        #region Return values

        internal int LoadedRoomsCount
        {
            get
            {
                return this.loadedRooms.Count;
            }
        }

        internal EventManager GetEventManager()
        {
            return eventManager;
        }

        internal ISortable<RoomData>[] GetRooms(int categoryID, out int count)
        {
            if (roomCategories.ContainsKey(categoryID))
            {
                return roomCategories[categoryID].GetOrderedRooms(out count);
            }
            else
            {
                count = 0;
                return new ISortable<RoomData>[] { };
            }
        }

        internal ISortable<RoomData>[] GetActiveRooms(out int count)
        {
            count = orderedActiveRooms.WritePosition;
            return orderedActiveRooms.StorageArray;
        }

        internal ISortable<RoomData>[] GetVotedRooms(out int count)
        {
            count = orderedActiveRooms.WritePosition;
            return orderedVotedRooms.StorageArray;
        }

        internal List<RoomLinkInformation> getLinkedRoomData(uint roomID)
        {
            if (roomLinks.ContainsKey(roomID))
                return roomLinks[roomID];
            else
                return new List<RoomLinkInformation>();
        }


        private static RoomModel GetCustomData(UInt32 roomID)
        {
            DataRow RoomData;
            using (IQueryAdapter dbClient = ButterflyEnvironment.DatabaseManager.getQueryreactor())
            {
                dbClient.setQuery("SELECT doorx,doory,modeldata FROM room_models_customs WHERE roomid = @roomid");
                dbClient.addParameter("roomid", roomID);
                RoomData = dbClient.getRow();
            }

            if (RoomData == null)
                throw new Exception("The custom room model for room " + roomID + " was not found");

            string Modeldata = (string)RoomData["modeldata"];
            RoomModel roomModelData = new RoomModel((int)RoomData["doorx"], (int)RoomData["doory"],0, 2, Modeldata, "", false, string.Empty);
            SquareState state = roomModelData.SqState[roomModelData.DoorX, roomModelData.DoorY];
            if (state != SquareState.BLOCKED)
            {
                short Z = roomModelData.SqFloorHeight[roomModelData.DoorX, roomModelData.DoorY];
                roomModelData.DoorZ = Z;
            }
            return roomModelData;
        }

        internal RoomModel GetModel(string Model, UInt32 RoomID)
        {
            if (Model == "custom")
                return GetCustomData(RoomID);

            if (roomModels.ContainsKey(Model))
                return (RoomModel)roomModels[Model];

            return null;
        }

        internal RoomData GenerateNullableRoomData(UInt32 RoomId)
        {
            if (GenerateRoomData(RoomId) != null)
            {
                return GenerateRoomData(RoomId);
            }

            RoomData Data = new RoomData();
            Data.FillNull(RoomId);
            return Data;
        }

        internal RoomData GenerateRoomData(UInt32 RoomId)
        {
			Room room = GetRoom(RoomId);
			if (room != null)
				return room.RoomData;

            RoomData Data = new RoomData();
            if (IsRoomLoaded(RoomId))
            {
                return GetRoom(RoomId).RoomData;
            }
            else
            {
                DataRow Row = null;

                using (IQueryAdapter dbClient = ButterflyEnvironment.DatabaseManager.getQueryreactor())
                {
                    dbClient.setQuery("SELECT rooms.*, room_active.active_users, users.id AS ownerid FROM rooms LEFT JOIN room_active ON (room_active.roomid = rooms.id) JOIN users ON (users.username = rooms.owner) WHERE rooms.id = " + RoomId);
                    Row = dbClient.getRow();
                }

                if (Row == null)
                {
                    return null;
                }

                Data.Fill(Row);
            }

            return Data;
        }

        internal ISortable<RoomData>[] GetEventRoomsForCategory(int categoryID, out int count)
        {
            return eventManager.GetRoomsForCategory(categoryID, out count);
        }

        internal Boolean IsRoomLoaded(UInt32 RoomId)
        {
            return loadedRooms.ContainsKey(RoomId);
        }

        internal Room LoadRoom(UInt32 Id)
        {
            if (IsRoomLoaded(Id))
            {
                return GetRoom(Id);
            }

            RoomData Data = GenerateRoomData(Id);

            if (Data == null)
                return null;

            Room Room = new Room(Data);

            //Room Room = new Room(Data.Id, Data.Name, Data.Description, Data.Type, Data.Owner, Data.Category, Data.State,
            //    Data.UsersMax, Data.ModelName, Data.CCTs, Data.Score, Data.Tags, Data.AllowPets, Data.AllowPetsEating,
            //    Data.AllowWalkthrough, Data.Hidewall, Data.Icon, Data.Password, Data.Wallpaper, Data.Floor, Data.Landscape, Data, Data.AllowRightsOverride);

            lock (roomsToAddQueue.SyncRoot)
            {
                roomsToAddQueue.Enqueue(Room);
            }


            Room.InitBots();
            Room.InitPets();

            //Logging.WriteLine("[RoomMgr] Loaded room: \"" + Room.Name + "\" (ID: " + Id + ")");

            return Room;
        }

		internal RoomData FetchRoomData(uint roomID, DataRow dRow)
		{
			Room room = GetRoom(roomID);
            if (room != null)
                return room.RoomData;
            else
            {
                RoomData data = new RoomData();
                data.Fill(dRow);
                return data;
            }
		}

        internal Room GetRoom(uint roomID)
        {
            Room room;
            if (loadedRooms.TryGetValue(roomID, out room))
                return room;

            return null;
            //throw new ObjectNotFoundException("No room for roomID " + RoomId + " found, script aborted.");
        }

		internal void RequestRoomLoad(OnRoomLoad loadDelegate)
		{
			lock (roomLoadQueue.SyncRoot)
			{
				roomLoadQueue.Enqueue(loadDelegate);
			}
		}

        internal RoomData CreateRoom(GameClient Session, string Name, string Model)
        {
            Name = TextHandling.FilterInjectionChars(Name);

            if (!roomModels.ContainsKey(Model))
            {
                Session.SendNotif(LanguageLocale.GetValue("room.modelmissing"));
                return null;
            }

            if (((RoomModel)roomModels[Model]).ClubOnly && !Session.GetHabboDataContainer().GetSubscriptionManager().HasSubscription())
            {
                Session.SendNotif(LanguageLocale.GetValue("room.missingclub"));
                return null;
            }

            if (Name.Length < 3)
            {
                Session.SendNotif(LanguageLocale.GetValue("room.namelengthshort"));
                return null;
            }

            UInt32 RoomId = 0;

            using (IQueryAdapter dbClient = ButterflyEnvironment.DatabaseManager.getQueryreactor())
            {
				//if (dbClient.dbType == Database_Manager.Database.DatabaseType.MSSQL)//description,public_ccts,tags,password
				//	dbClient.setQuery("INSERT INTO rooms (roomtype,caption,owner,model_name,description,public_ccts,tags,password) OUTPUT INSERTED.* VALUES ('private',@caption,@username,@model,'','','','')");
				//else
                    dbClient.setQuery("INSERT INTO rooms (roomtype,caption,owner,model_name) VALUES ('private',@caption,@username,@model)");
                dbClient.addParameter("caption", Name);
                dbClient.addParameter("model", Model);
                dbClient.addParameter("username", Session.GetHabbo().Username);

                RoomId = (UInt32)dbClient.insertQuery();


            }

            RoomData newRoomData = GenerateRoomData(RoomId);
            Session.GetHabbo().UsersRooms.Add(newRoomData);

            return newRoomData;
        }
        #endregion

        #region Boot

        internal RoomManager()
        {
			this.loadedRooms = new Dictionary<uint, Room>(5839);

            this.roomModels = new Hashtable();
            this.mToRemove = new List<Room>();
            this.roomCategories = new Dictionary<int, RoomCategory>();

			this.votedRooms = new Dictionary<RoomData, int>(5839);
			this.activeRooms = new Dictionary<RoomData, int>(5839);
            this.orderedActiveRooms = new OrderedArray<RoomData>(40);
            this.orderedVotedRooms = new OrderedArray<RoomData>(40);

            this.roomsToAddQueue = new Queue();
            this.roomsToRemoveQueue = new Queue();

            this.votedRoomsRemoveQueue = new Queue();
            this.votedRoomsAddQueue = new Queue();

            this.activeRoomsRemoveQueue = new Queue();
            this.activeRoomsUpdateQueue = new Queue();
            this.activeRoomsAddQueue = new Queue();
			this.roomLoadQueue = new Queue();

            this.eventManager = new EventManager();

            this.roomCycleStopwatch = new Stopwatch();
            this.removeTaskStopwatch = new Stopwatch();
            this.roomCycleStopwatch.Start();
            this.roomCycleStopwatch.Start();

        }

        internal void InitRoomLinks(IQueryAdapter dbClient)
        {
            this.roomLinks = new Dictionary<uint, List<RoomLinkInformation>>();

            //dbClient.setQuery("SELECT * FROM room_links"); NOOBODDYYYY CARREESSS because we dont got public rooms any more :D
            //DataTable roomLinkData = dbClient.getTable();

            //foreach (DataRow row in roomLinkData.Rows)
            //{
            //    RoomLinkInformation info = new RoomLinkInformation(row);

            //    if (roomLinks.ContainsKey(info.roomID))
            //    {
            //        roomLinks[info.roomID].Add(info);
            //    }
            //    else
            //    {
            //        List<RoomLinkInformation> newList = new List<RoomLinkInformation>();
            //        newList.Add(info);
            //        roomLinks.Add(info.roomID, newList);
            //    }
            //}


            //dbClient.setQuery("SELECT id,caption,min_rank FROM navigator_flatcats WHERE enabled = '1'");
            //DataTable dPrivCats = dbClient.getTable();

            //int categoryID;
            //string caption;
            //foreach (DataRow dRow in dPrivCats.Rows)
            //{
            //    categoryID = (int)dRow[0];
            //    caption = (string)dRow[1];

            //    RoomCategory category = new RoomCategory(categoryID, caption);
            //    roomCategories.Add(categoryID, category);
            //}
        }

        internal void InitVotedRooms(IQueryAdapter dbClient)
        {
            //if (dbClient.dbType == Database_Manager.Database.DatabaseType.MySQL)
            dbClient.setQuery("SELECT rooms.*, users.id AS ownerid FROM rooms LEFT JOIN room_active ON (room_active.roomid = rooms.id) JOIN users ON (users.username = rooms.owner) " + 
                "WHERE score > 0 AND roomtype = 'private' ORDER BY score DESC LIMIT 40");
			//else
			//	dbClient.setQuery("SELECT TOP 40 rooms.*, room_active.active_users FROM rooms LEFT JOIN room_active ON (room_active.roomid = rooms.id) WHERE score > 0 AND roomtype = 'private' ORDER BY score DESC");
            DataTable dTable = dbClient.getTable();

            foreach (DataRow dRow in dTable.Rows)
            {
                RoomData data = FetchRoomData(Convert.ToUInt32(dRow["id"]), dRow);
                QueueVoteAdd(data);
            }
        }

        internal void LoadModels(IQueryAdapter dbClient)
        {
            Hashtable models = new Hashtable();

            dbClient.setQuery("SELECT id,door_x,door_y,door_z,door_dir,heightmap,public_items,club_only,poolmap FROM room_models");
            DataTable Data = dbClient.getTable();

            if (Data == null)
                return;

            foreach (DataRow Row in Data.Rows)
            {
                string Modelname = (string)Row["id"];
                string staticFurniture = (string)Row["public_items"];

                models.Add(Modelname, new RoomModel((int)Row["door_x"],
                    (int)Row["door_y"], (Double)Row["door_z"], (int)Row["door_dir"], (string)Row["heightmap"],
                    staticFurniture, TextHandling.EnumToBool(Row["club_only"].ToString()), (string)Row["poolmap"]));
            }

            this.roomModels = models;
        }
        #endregion

        #region Threading
        internal void OnCycle(Stopwatch moduleWatch)
        {
            WorkRoomsToAddQueue();
			HandleFunctionReset(moduleWatch, "WorkRoomsToAddQueue");
            WorkRoomsToRemoveQueue();
			HandleFunctionReset(moduleWatch, "WorkRoomsToRemoveQueue");

            RoomCycleTask();
			HandleFunctionReset(moduleWatch, "RoomCycleTask");
            RemoveTask(); // Queries
			HandleFunctionReset(moduleWatch, "RemoveTask");

            WorkActiveRoomsAddQueue();
			HandleFunctionReset(moduleWatch, "WorkActiveRoomsAddQueue");
            WorkActiveRoomsRemoveQueue();
			HandleFunctionReset(moduleWatch, "WorkActiveRoomsRemoveQueue");
            WorkActiveRoomsUpdateQueue();
			HandleFunctionReset(moduleWatch, "WorkActiveRoomsUpdateQueue");

			HandleRoomLoadRequests();
			HandleFunctionReset(moduleWatch, "HandleRoomLoadRequests");
            WorkVotedRoomsAddQueue();
			HandleFunctionReset(moduleWatch, "WorkVotedRoomsAddQueue");
            WorkVotedRoomsRemoveQueue();
			HandleFunctionReset(moduleWatch, "WorkVotedRoomsRemoveQueue");

            WorkCategories();
			HandleFunctionReset(moduleWatch, "WorkCategories");
            eventManager.onCycle();
			HandleFunctionReset(moduleWatch, "eventManager.onCycle");

            orderedActiveRooms.Defrag();
			HandleFunctionReset(moduleWatch, "orderedActiveRooms.Defrag");
            orderedVotedRooms.Defrag();
			HandleFunctionReset(moduleWatch, "orderedVotedRooms.Defrag");
        }

		private void HandleFunctionReset(Stopwatch watch, string methodName)
		{
			if (watch.ElapsedMilliseconds > 10000)
				Console.WriteLine("High latency in {0}: {1}ms", methodName, watch.ElapsedMilliseconds);
			watch.Restart();
		}


        private void WorkCategories()
        {
            foreach (RoomCategory category in roomCategories.Values)
            {
                category.OnCycle();
            }
        }


        private bool WorkActiveRoomsUpdateQueue()
        {
            if (activeRoomsUpdateQueue.Count > 0)
            {
                lock (activeRoomsUpdateQueue.SyncRoot)
                {
                    while (activeRoomsUpdateQueue.Count > 0)
                    {
                        RoomData data = (RoomData)activeRoomsUpdateQueue.Dequeue();
                        if (!activeRooms.ContainsKey(data))
                            activeRooms.Add(data, data.UsersNow);
                        else
                            activeRooms[data] = data.UsersNow;
                        orderedActiveRooms.Insert(new ArrayElement<RoomData>(data, data.UsersNow, data.Id));
                    }
                }
                return true;
            }
            return false;
        }

        private bool WorkActiveRoomsAddQueue()
        {
            if (activeRoomsAddQueue.Count > 0)
            {
                lock (activeRoomsAddQueue.SyncRoot)
                {
                    while (activeRoomsAddQueue.Count > 0)
                    {
                        RoomData data = (RoomData)activeRoomsAddQueue.Dequeue();
                        if (!activeRooms.ContainsKey(data))
                            activeRooms.Add(data, data.UsersNow);
                        orderedActiveRooms.Insert(new ArrayElement<RoomData>(data, data.UsersNow, data.Id));
                    }
                }
                return true;
            }
            return false;
        }

        private bool WorkActiveRoomsRemoveQueue()
        {
            if (activeRoomsRemoveQueue.Count > 0)
            {
                lock (activeRoomsRemoveQueue.SyncRoot)
                {
                    while (activeRoomsRemoveQueue.Count > 0)
                    {
                        RoomData data = (RoomData)activeRoomsRemoveQueue.Dequeue();
                        activeRooms.Remove(data);
                        orderedActiveRooms.RemoveItem(data.Id);
                    }
                }
                return true;
            }
            return false;
        }

        private bool WorkVotedRoomsAddQueue()
        {
            if (votedRoomsAddQueue.Count > 0)
            {
                lock (votedRoomsAddQueue.SyncRoot)
                {
                    while (votedRoomsAddQueue.Count > 0)
                    {
                        RoomData data = (RoomData)votedRoomsAddQueue.Dequeue();
                        if (!votedRooms.ContainsKey(data))
                            votedRooms.Add(data, data.Score);
                        else
                            votedRooms[data] = data.Score;
                        orderedVotedRooms.Insert(new ArrayElement<RoomData>(data, data.Score, data.Id));
                    }
                }
                return true;
            }
            return false;
        }

		private List<OnRoomLoad> loadQueue = new List<OnRoomLoad>();

		private void HandleRoomLoadRequests()
		{
			if (this.roomLoadQueue.Count > 0)
			{
				lock (roomLoadQueue.SyncRoot)
				{
					while (roomLoadQueue.Count > 0)
					{
						OnRoomLoad _delegate = (OnRoomLoad)roomLoadQueue.Dequeue();
						loadQueue.Add(_delegate);
					}
				}
			}

			foreach (OnRoomLoad _delegate in loadQueue)
			{
				_delegate();
			}

			loadQueue.Clear();
		}

        private bool WorkVotedRoomsRemoveQueue()
        {
            if (votedRoomsRemoveQueue.Count > 0)
            {
                lock (votedRoomsRemoveQueue.SyncRoot)
                {
                    while (votedRoomsRemoveQueue.Count > 0)
                    {
                        RoomData data = (RoomData)votedRoomsRemoveQueue.Dequeue();
                        votedRooms.Remove(data);
                        orderedVotedRooms.RemoveItem(data.Id);
                    }
                }
                return true;
            }
            return false;
        }


        private void WorkRoomsToAddQueue()
        {
            if (roomsToAddQueue.Count > 0)
            {
                lock (roomsToAddQueue.SyncRoot)
                {
                    while (roomsToAddQueue.Count > 0)
                    {
                        Room room = (Room)roomsToAddQueue.Dequeue();
                        if (!loadedRooms.ContainsKey(room.RoomId))
                            loadedRooms.Add(room.RoomId, room);
                    }
                }
            }
        }

        private void WorkRoomsToRemoveQueue()
        {
            if (roomsToRemoveQueue.Count > 0)
            {
                lock (roomsToRemoveQueue.SyncRoot)
                {
                    while (roomsToRemoveQueue.Count > 0)
                    {
                        uint roomID = (uint)roomsToRemoveQueue.Dequeue();
                        loadedRooms.Remove(roomID);
                    }
                }
            }
        }

        internal static bool roomCyclingEnabled = true;

        private Stopwatch roomCycleStopwatch;
        internal void RoomCycleTask()
        {
            if (roomCycleStopwatch.ElapsedMilliseconds >= 500 && roomCyclingEnabled)
            {
                roomCycleStopwatch.Restart();

                foreach (Room Room in loadedRooms.Values)
                {
                    if (Room.mIsIdle)
                    {
                        loadedRooms.Remove(Room.RoomId);
                        mToRemove.Add(Room);
                    }
                    else if (!Room.isCycling)
                    {
                        ThreadPool.UnsafeQueueUserWorkItem(Room.ProcessRoom, null);
                    }
                }
            }
        }

        private Stopwatch removeTaskStopwatch;
        internal void RemoveTask()
        {
            if (removeTaskStopwatch.ElapsedMilliseconds >= 5000)
            {
                removeTaskStopwatch.Restart();
                using (IQueryAdapter dbClient = ButterflyEnvironment.DatabaseManager.getQueryreactor())
                {
                    foreach (Room room in mToRemove)
                    {
                        Logging.WriteLine("[RoomMgr] Requesting unload of idle room - ID#: " + room.RoomId);

                        room.GetRoomItemHandler().SaveFurniture(dbClient);
                        UnloadRoom(room);
                    }
                }
                mToRemove.Clear();
            }
        }
        #endregion

        #region Methods

        internal void QueueVoteAdd(RoomData data)
        {
            lock (votedRoomsAddQueue.SyncRoot)
            {
                votedRoomsAddQueue.Enqueue(data);
            }
        }

        internal void QueueVoteRemove(RoomData data)
        {
            lock (votedRoomsRemoveQueue.SyncRoot)
            {
                votedRoomsRemoveQueue.Enqueue(data);
            }
        }

        internal void QueueActiveRoomUpdate(RoomData data)
        {
            lock (activeRoomsUpdateQueue.SyncRoot)
            {
                activeRoomsUpdateQueue.Enqueue(data);
            }

            if (!roomCategories.ContainsKey(data.Category))
                roomCategories.Add(data.Category, new RoomCategory(data.Category, "MISSING CATEGORY"));
                roomCategories[data.Category].QueueUpdateActiveRooms(data);
        }

        internal void QueueActiveRoomAdd(RoomData data)
        {
            lock (activeRoomsAddQueue.SyncRoot)
            {
                activeRoomsAddQueue.Enqueue(data);
            }

            if (!roomCategories.ContainsKey(data.Category))
                roomCategories.Add(data.Category, new RoomCategory(data.Category, "MISSING CATEGORY"));
            roomCategories[data.Category].QueueAddActiveRooms(data);
        }

        internal void QueueActiveRoomRemove(RoomData data)
        {
            lock (activeRoomsRemoveQueue.SyncRoot)
            {
                activeRoomsRemoveQueue.Enqueue(data);
            }

            if (!roomCategories.ContainsKey(data.Category))
                roomCategories.Add(data.Category, new RoomCategory(data.Category, "MISSING CATEGORY"));
                roomCategories[data.Category].QueueRemoveActiveRooms(data);
        }

        internal void RemoveAllRooms()
        {
            int length = loadedRooms.Count;
            int i = 0;
            foreach (Room Room in loadedRooms.Values)
            {
                ButterflyEnvironment.Game.GetRoomManager().UnloadRoom(Room);
                Console.Clear();
                Console.WriteLine("<<- SERVER SHUTDOWN ->> ROOM ITEM SAVE: " + String.Format("{0:0.##}", ((double)i / length) * 100) + "%");
                i++;
            }
            Console.WriteLine("Done disposing rooms!");
        }

        internal void UnloadRoom(Room Room)
        {
            if (Room == null)
            {
                return;
            }

            lock (roomsToRemoveQueue.SyncRoot)
            {
                roomsToRemoveQueue.Enqueue(Room.RoomId);
            }

            Room.Destroy();

           Logging.WriteLine("[RoomMgr] Unloaded room: \"" + Room.Name + "\" (ID: " + Room.RoomId + ")");
        }

        #endregion

        internal string GetRoomName(uint roomID)
        {
            Room room;
            if (loadedRooms.TryGetValue(roomID, out room))
                return room.Name;

            string name = null;
            using (IQueryAdapter dbClient = ButterflyEnvironment.DatabaseManager.getQueryreactor())
            {
                dbClient.setQuery("SELECT caption FROM rooms WHERE id = " + roomID);
                name = dbClient.getString();
            }

            return name;
        }
    }
}
