﻿using Butterfly.Core;
using Butterfly.HabboHotel.ChatMessageStorage;
using Butterfly.HabboHotel.GameClients;
using Butterfly.HabboHotel.Group;
using Butterfly.HabboHotel.Items;
using Butterfly.HabboHotel.Misc;
using Butterfly.HabboHotel.Pets;
using Butterfly.HabboHotel.RoomBots;
using Butterfly.HabboHotel.Rooms.Games;
using Butterfly.Messages;
using Butterfly.Messages.IOCodes;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;

namespace Butterfly.HabboHotel.Rooms
{
    public class RoomUser : IEquatable<RoomUser>
    {
        internal UInt32 HabboId;
        internal Int32 VirtualId;
        internal UInt32 RoomId;
		internal string username;

        internal int IdleTime;

        internal int X;
        internal int Y;
        internal double Z;
        internal byte SqState;

        internal int CarryItemID;
        internal int CarryTimer;

        internal int RotHead;
        internal int RotBody;

        internal bool CanWalk;
        internal bool AllowOverride;
        internal bool TeleportEnabled;

        internal int GoalX;
        internal int GoalY;

        internal Boolean SetStep;
        internal int SetX;
        internal int SetY;
        internal double SetZ;
        internal uint userID;

        internal RoomBot BotData;
        internal BotAI BotAI;

        internal ItemEffectType CurrentItemEffect;
        internal bool Freezed;
        internal int FreezeCounter;
        internal Team team;
        internal FreezePowerUp banzaiPowerUp;
        internal int FreezeLives;

        internal bool shieldActive;
        internal int shieldCounter;

        internal bool throwBallAtGoal;
        internal bool moonwalkEnabled = false;
        internal bool isFlying = false;
        internal int flyk = 0;
        internal uint interactingBallID = 0;

        internal bool ignoreMap = false;
        internal uint currentRidingPet;
        internal bool IsHorseWalking = false;
        internal bool gotSaddle = false;
        internal int headRotDelay = 0;

        internal Point Coordinate
        {
            get
            {
                return new Point(X, Y);
            }
        }

		internal int GetDoubleCoordinate()
		{
			return TextHandling.CombineXYCoord(X, Y);
		}

        public bool Equals(RoomUser comparedUser)
        {
            return (comparedUser.HabboId == this.HabboId);
        }

        internal bool IsPet
        {
            get
            {
                return (IsBot && BotData.IsPet);
            }
        }

        internal string GetUsername()
        {
			return username;
        }

        internal Pet PetData;

        internal bool IsWalking;
        internal bool UpdateNeeded;
        internal bool IsAsleep;

        internal Dictionary<string, string> Statusses;

        internal int DanceId;

        private int FloodCount
        {
            get
            {
                if (mClient == null)
                    return 0;
                return mClient.GetSessionData().FloodCount;
            }
            set
            {
                if (mClient != null)
                {
                    mClient.GetSessionData().FloodCount = value;
                }
            }
        }

        private DateTime FloodProtectUntill
        {
            get
            {
                if (mClient == null)
                    return DateTime.MinValue;
                return mClient.GetSessionData().FloodProtectUntill;
            }
            set
            {
                if (mClient == null)
                    return;
                mClient.GetSessionData().FloodProtectUntill = value;
            }
        }
        private DateTime FloodTime
        {
            get
            {
                if (mClient == null)
                    return DateTime.MinValue;
                return mClient.GetSessionData().FloodTime;
            }
            set
            {
                if (mClient == null)
                    return;
                mClient.GetSessionData().FloodTime = value;
            }
        }

        internal Boolean IsDancing
        {
            get
            {
                if (DanceId >= 1)
                {
                    return true;
                }

                return false;
            }
        }

        internal Boolean NeedsAutokick
        {
            get
            {
                if (IsBot)
                    return false;
                if (GetClient() == null || GetClient().GetHabbo() == null)
                    return true;
                if (GetClient().GetHabbo().Rank > 1)
                    return false;
                if (IdleTime >= 1800)
                    return true;

                return false;
            }
        }

        internal bool IsTrading
        {
            get
            {
                if (IsBot)
                {
                    return false;
                }

                if (Statusses.ContainsKey("trd"))
                {
                    return true;
                }

                return false;
            }
        }

        internal bool IsOwner()
        {
            if (IsBot)
                return false;
            return (GetUsername() == GetRoom().Owner);
        }

        internal bool IsBot
        {
            get
            {
                if (this.BotData != null)
                {
                    return true;
                }

                return false;
            }
        }

        internal bool IsSpectator;

        internal int InternalRoomID;

        private List<uint> IgnoredPersons = new List<uint>();

        private Queue events;

        internal RoomUser(uint HabboId, string username, uint RoomId, int VirtualId, Room room, bool isSpectator)
        {
            this.Freezed = false;
            this.forcesit = false;
            this.HabboId = HabboId;
            this.RoomId = RoomId;
			this.username = username;
            this.VirtualId = VirtualId;
            this.IdleTime = 0;
            this.X = 0;
            this.Y = 0;
            this.Z = 0;
            this.RotHead = 0;
            this.RotBody = 0;
            this.UpdateNeeded = true;
            this.Statusses = new Dictionary<string, string>();
            this.mRoom = room;

            this.AllowOverride = false;
            this.CanWalk = true;

            this.IsSpectator = isSpectator;
            this.SqState = 3;

            this.InternalRoomID = 0;
            this.CurrentItemEffect = ItemEffectType.None;
            this.events = new Queue();
            this.FreezeLives = 0;
        }

        internal void IgnoreUser(uint id)
        {
            if (!IgnoredPersons.Contains(id))
                IgnoredPersons.Add(id);
        }

        internal void UnIgnoreUser(uint id)
        {
            IgnoredPersons.Remove(id);
        }

        internal void Unidle()
        {
            this.IdleTime = 0;

            if (this.IsAsleep)
            {
                this.IsAsleep = false;

                ServerMessage Message = PacketHandelingManager.GetRecycledItem(OutputCode.SleepMessageEvent);
                Message.AddInt32(VirtualId);
                Message.AddBoolean(false);

                GetRoom().SendMessage(Message);
            }
        }

        internal void OnFly()
        {
            if (flyk == 0)
            {
                flyk++;
                return;
            }

            double lastK = 0.5 * Math.Sin(0.7 * flyk);
            flyk++;
            double nextK = 0.5 * Math.Sin(0.7 * flyk);
            double differance = nextK - lastK;
			int coord = GetDoubleCoordinate();

            GetRoom().SendMessage(GetRoom().GetRoomItemHandler().UpdateUserOnRoller(this, X, Y, ref coord, 0, this.Z + differance));
        }

        internal void Dispose()
        {
            Statusses.Clear();
            mRoom = null;
            mClient = null;
        }

        internal bool UserIsRidingAHorse() //Wtf?
        {
            return (GetPedRiding() != null);
        }

        internal RoomUser GetPedRiding()
        {
            return GetRoom().GetRoomUserManager().GetPet(currentRidingPet);
        }

        internal RoomUser GetRoomUserRiding()
        {
            if (!IsPet)
                return null;
            return GetRoom().GetRoomUserManager().GetRoomUserByHabbo(PetData.currentRidingUser);
        }

        internal bool PetGotUserRiding()
        {
            if (IsPet && PetData.petType == 15)
            {
                RoomUser ridingUser = GetRoom().GetRoomUserManager().GetRoomUserByHabbo(PetData.currentRidingUser);
                if (ridingUser != null && ridingUser.currentRidingPet == PetData.petID)
                {
                    return true;
                }
            }
            return false;
        }

        internal void StopRide()
        {
            RoomUser pet = GetRoom().GetRoomUserManager().GetPet(currentRidingPet);
            if (pet != null)
                pet.PetData.currentRidingUser = 0;

            currentRidingPet = 0;
        }

		internal void PrepareTrade()
		{
			if (!Statusses.ContainsKey("trd"))
			{
				AddStatus("trd", "");
				UpdateNeeded = true;
			}
		}

        internal void Chat(GameClient Session, string Message, bool Shout, int textColor = 0)
        {
            if (Session != null)
            {
                if (Session.GetHabbo().Rank < 5)
                {
                    if (GetRoom().RoomMuted)
                        return;
                }

                if ((textColor == 1 || textColor == 2) && Session.GetHabbo().Rank < 5)
                    textColor = 0;

                if (textColor == 23 && Session.GetHabboDataContainer().data.Rank < 3)
                    textColor = 0;
            }

            if (textColor >= 24 || textColor < 0)
            {
                textColor = 0;
            }

            Unidle();

            if (!IsBot && GetClient().GetHabbo().Muted)
            {
                GetClient().SendNotif("You are muted.");
                return;
            }

            if (Message.StartsWith(":") && Session != null)
            {
                if (ButterflyEnvironment.CommandInvoker.TryCommand(Session, Message.Split(' ')))
                    return;
            }
            
            uint rank = 1;
            
            if (!IsBot && Session != null && Session.GetHabbo() != null)
                rank = Session.GetHabbo().Rank;

			if (rank < 3)
				Message = LanguageLocale.FilterSwearwords(Message);

            if (FloodProtectUntill != null)
            {
                if (FloodProtectUntill > DateTime.Now)
                {
                    ServerMessage Packet = PacketHandelingManager.GetRecycledItem(OutputCode.FloodControlMessageEvent);
                    Packet.AddInt32((int)(DateTime.Now-FloodProtectUntill).TotalSeconds); //Blocked for 30sec
                    GetClient().SendMessage(Packet);
                    return;
                }
            }

            TimeSpan SinceLastMessage = DateTime.Now - FloodTime;
            if (SinceLastMessage.Seconds > 4)
            {
                FloodCount = 0;
            }
            else if (SinceLastMessage.Seconds < 4 && FloodCount > 5 && !IsBot && rank < 5 && !GetClient().GetHabboDataContainer().HasFuse("ignore_flood_filter"))
            {
                ServerMessage Packet = PacketHandelingManager.GetRecycledItem(OutputCode.FloodControlMessageEvent);
                Packet.AddInt32(30); //Blocked for 30sec
                GetClient().SendMessage(Packet);
                FloodProtectUntill = DateTime.Now.AddSeconds(30);

                return;
            }
            FloodTime = DateTime.Now;
            FloodCount++;

            if (!IsBot)
            {
                ButterflyEnvironment.Game.GetQuestManager().ProgressUserQuest(Session, HabboHotel.Quests.QuestType.SOCIAL_CHAT);
            }
            
            InvokedChatMessage message = new InvokedChatMessage(this, Message, Shout, textColor);
            GetRoom().QueueChatMessage(message);
        }

        internal void OnChat(InvokedChatMessage message)
        {
            string Message = message.message;

            if (GetRoom() != null && !GetRoom().AllowsShous(this, Message))
                return;

            ushort ChatHeader = OutputCode.ChatMessageEvent;

            if (message.shout)
            {
                ChatHeader = OutputCode.ShoutMessageEvent;
            }

            //string Site = "";

            ServerMessage ChatMessage = PacketHandelingManager.GetRecycledItem(ChatHeader);
            ChatMessage.AddInt32(VirtualId);
            ChatMessage.AddString(Message);
            ChatMessage.AddInt32(GetSpeechEmotion(Message));
            ChatMessage.AddInt32(message.textColor);
            ChatMessage.AddInt32(0);
            ChatMessage.AddInt32(-1);

            GetRoom().GetRoomUserManager().TurnHeads(X, Y, HabboId);
            GetRoom().SendChatMessageToWithoutIgnore(ChatMessage, message.user);

            if (!IsBot)
            {
                GetRoom().OnUserSay(this, Message, message.shout);
                LogMessage(Message);
            }

            message.Dispose();
        }

        internal void OnStopWalk()
        {
            if (interactingBallID == 0)
                return;

            Room room = GetRoom();

            if (room == null)
                return;

            RoomItem item = room.GetRoomItemHandler().GetItem(interactingBallID);
            if (item == null)
                return;

            interactingBallID = 0;
            if (item.interactingBallUser == userID)
                return;
            item.interactingBallUser = 0;
        }

        private static void LogMessage(string message)
        {
        //    ChatMessage chatMessage = ChatMessageFactory.CreateMessage(message, GetClient(), GetRoom());

        //    foreach (RoomUser user in GetRoom().UserList.Values)
        //    {
        //        if (!user.IsBot && user.GetClient() != null && user.GetClient().GetHabbo() != null)
        //            user.GetClient().GetHabbo().GetChatMessageManager().AddMessage(chatMessage);
        //    }

        //    GetRoom().GetChatMessageManager().AddMessage(chatMessage);
        }

        internal static int GetSpeechEmotion(string Message)
        {
            Message = Message.ToLower();

            if (Message.Contains(":)") || Message.Contains(":d") || Message.Contains("=]") || 
                Message.Contains("=d") || Message.Contains(":>"))
            {
                return 1;
            }

            if (Message.Contains(">:(") || Message.Contains(":@"))
            {
                return 2;
            }

            if (Message.Contains(":o"))
            {
                return 3;
            }

            if (Message.Contains(":(") || Message.Contains("=[") || Message.Contains(":'(") || Message.Contains("='["))
            {
                return 4;
            }

            return 0;
        }

        internal void ClearMovement(bool Update)
        {
            IsWalking = false;
            OnStopWalk();
            Statusses.Remove("mv");
            GoalX = 0;
            GoalY = 0;
            SetStep = false;
            SetX = 0;
            SetY = 0;
            SetZ = 0;

            if (Update)
            {
                UpdateNeeded = true;
            }
        }

        internal void MoveTo(Point c)
        {
            MoveTo(c.X, c.Y);
        }

        internal void MoveTo(int pX, int pY, bool pOverride)
        {

            MoveTo(pX, pY, pOverride, GetRoom().GetGameMap().SqAbsoluteHeight(GoalX, GoalY));
        }

        internal void MoveTo(int pX, int pY, bool pOverride, double z)
        {
            Unidle();

            if (TeleportEnabled)
            {
                int coord = TextHandling.CombineXYCoord(pX, pY);
                if (GetRoom().GetGameMap().CanRollItemHere(pX, pY))
                {
                    GetRoom().SendMessage(GetRoom().GetRoomItemHandler().UpdateUserOnRoller(this, pX, pY, ref coord, 0, z));
                    GetRoom().GetRoomUserManager().UpdateUserStatus(this, false);
                }
            }
            else
            {
                forcesit = false;
                IsWalking = true;
                GoalX = pX;
                GoalY = pY;
                throwBallAtGoal = false;
            }
        }

        internal void MoveTo(int pX, int pY)
        {
            if (IsPet && PetGotUserRiding())
                return;
            MoveTo(pX, pY, false);
        }

        internal void UnlockWalking()
        {
            this.AllowOverride = false;
            this.CanWalk = true;
        }

        internal void SetPos(int pX, int pY, double pZ)
        {
            this.X = pX;
            this.Y = pY;
            this.Z = pZ;
            if (isFlying)
                Z += 4 + 0.5 * Math.Sin(0.7 * flyk);

            if (UserIsRidingAHorse())
                this.Z += 1;
            
        }

        internal void CarryItem(int Item)//TODO something was wrong here with giving hand items
        {
            this.CarryItemID = Item;

            if (Item > 0)
            {
                this.CarryTimer = 240;
            }
            else
            {
                this.CarryTimer = 0;
            }

            ServerMessage Message = PacketHandelingManager.GetRecycledItem(OutputCode.CarryObjectMessageEvent);
            Message.AddInt32(VirtualId);
            Message.AddInt32(Item);
            GetRoom().SendMessage(Message);
        }


        internal void SetRot(int Rotation)
        {
            SetRot(Rotation, false); //**************
        }

        internal void SetRot(int Rotation, bool HeadOnly)
        {
            if (Statusses.ContainsKey("lay") || IsWalking)
            {
                return;
            }

            int diff = this.RotBody - Rotation;

            this.RotHead = this.RotBody;

            if (Statusses.ContainsKey("sit") || HeadOnly)
            {
                if (RotBody == 2 || RotBody == 4)
                {
                    if (diff > 0)
                    {
                        RotHead = RotBody - 1;
                    }
                    else if (diff < 0)
                    {
                        RotHead = RotBody + 1;
                    }
                }
                else if (RotBody == 0 || RotBody == 6)
                {
                    if (diff > 0)
                    {
                        RotHead = RotBody - 1;
                    }
                    else if (diff < 0)
                    {
                        RotHead = RotBody + 1;
                    }
                }
            }
            else if (diff <= -2 || diff >= 2)
            {
                this.RotHead = Rotation;
                this.RotBody = Rotation;
            }
            else
            {
                this.RotHead = Rotation;
            }

            this.headRotDelay = 0;
            this.UpdateNeeded = true;
        }

        internal void AddStatus(string Key, string Value)
        {
            Statusses[Key] = Value;
        }

        internal void RemoveStatus(string Key)
        {
            if (Statusses.ContainsKey(Key))
            {
                Statusses.Remove(Key);
            }
        }

        internal void ApplyEffect(int effectID)
        {
            if (IsBot || GetClient() == null || GetClient().GetHabbo() == null || GetClient().GetHabboDataContainer().GetAvatarEffectsInventoryComponent() == null)
                return;

            GetClient().GetHabboDataContainer().GetAvatarEffectsInventoryComponent().ApplyCustomEffect(effectID);
        }

        internal void Serialize(ServerMessage Message)
        {
            if (IsBot)
            {
                Message.AddInt32(BotAI.BaseId);
                Message.AddString(BotData.Name);
                Message.AddString(BotData.Motto);
                if (PetData != null)
                {
                    Message.AddString(BotData.Look.ToLower() + ((gotSaddle) ? " 3 4 10 0 2 " + PetData.hairType + " " + PetData.hairDye + " 3 " + PetData.hairType + " " + PetData.hairDye + "" : " 2 2 " + PetData.hairType + " " + PetData.hairDye + " 3 " + PetData.hairType + " " + PetData.hairDye + ""));
                }
                else
                {
                    Message.AddString(BotData.Look.ToLower());
                }
                Message.AddInt32(VirtualId);
                Message.AddInt32(X);
                Message.AddInt32(Y);
                Message.AddString(TextHandling.GetString(Z));
                Message.AddInt32(0);
                if (BotData.ownerID > 0)
                    Message.AddInt32(4);
                else
                    Message.AddInt32((BotData.AiType == AIType.Pet) ? 2 : 3);
                if (BotData.AiType == AIType.Pet)
                {
                    Message.AddInt32((int)PetData.petType);
                    Message.AddInt32((int)PetData.ownerID); // userid
                    Message.AddString(PetData.OwnerName); // username
                    Message.AddInt32(1);

                    Message.AddBoolean(this.gotSaddle);
                    
                    Message.AddBoolean(false);
                    Message.AddInt32(0);
                    Message.AddInt32(0);
                    Message.AddString("");
                }
                else if (BotData.ownerID > 0)
                {
                    Message.AddString("m");
                    Message.AddUInt32(BotData.ownerID);
                    Message.AddString(ButterflyEnvironment.Game.GetClientManager().GetNameById(BotData.ownerID));
                    Message.AddInt32(4);
                    Message.AddInt16(1);
                    Message.AddInt16(2);
                    Message.AddInt16(5);
                    Message.AddInt16(4);
                }
            }
            else if (!IsBot && GetClient() != null && GetClient().GetHabbo() != null)
            {
                Users.Habbo User = GetClient().GetHabbo();
                Message.AddUInt32(User.Id);
                Message.AddString(User.Username);
                Message.AddString(User.Motto);
                Message.AddString(User.Look);
                Message.AddInt32(VirtualId);
                Message.AddInt32(X);
                Message.AddInt32(Y);
                Message.AddString(TextHandling.GetString(Z));
                Message.AddInt32(this.RotBody); // Rotation
                Message.AddInt32(1);
                Message.AddString(User.Gender.ToLower());

				if (User.EquippedGroup > 0)
                {
                    GroupData group = ButterflyEnvironment.Game.GetGroupManager().GetGroup(User.EquippedGroup);

					Message.AddUInt32(group.ID);
                    Message.AddInt32(2);
					Message.AddString(group.Name);
                    Message.AddString(string.Empty);
                }
                else
                {
                    Message.AddInt32(-1);
                    Message.AddInt32(-1);
                    Message.AddInt32(0);
                }

                Message.AddInt32(User.AchievementPoints);
            }
        }

        internal void SerializeStatus(ServerMessage Message)
        {
            StringBuilder userdata = new StringBuilder();
            if (IsSpectator)
            {
                return;
            }

            Message.AddInt32(VirtualId);
            Message.AddInt32(X);
            Message.AddInt32(Y);
            Message.AddString(TextHandling.GetString(Z));
            Message.AddInt32(RotHead);
            Message.AddInt32(RotBody);
            StringBuilder StatusComposer = new StringBuilder();
            StatusComposer.Append("/");

            foreach (KeyValuePair<string, string> Status in Statusses)
            {
                StatusComposer.Append(Status.Key);

                if (Status.Value != string.Empty)
                {
                    StatusComposer.Append(" ");
                    StatusComposer.Append(Status.Value);
                }

                StatusComposer.Append("/");
            }

            if (Statusses.ContainsKey("sign"))
            {
                Statusses.Remove("sign");
                UpdateNeeded = true;
            }

            StatusComposer.Append("/");
            Message.AddString(StatusComposer.ToString());
        }

		internal void SendMessage(byte[] message, int messageLength)
		{
			GameClient UsersClient = GetClient();
			if (!IsBot && UsersClient != null && UsersClient.GetConnection() != null && userID > 0)
				UsersClient.SendDataToConnection(message, messageLength);
		}

        private GameClient mClient;
		internal GameClient GetClient()
		{
			if (mClient == null && !IsBot)
				mClient = ButterflyEnvironment.Game.GetClientManager().GetClientByUserID(HabboId);
			return mClient;
		}

        private Room mRoom;
        
        private Room GetRoom()
        {
            if (mRoom == null)
                mRoom = ButterflyEnvironment.Game.GetRoomManager().GetRoom(RoomId);
            return mRoom;
        }

		internal void OnDestroy()
		{
			this.mClient = null;
			this.userID = 0;
		}

        internal bool isFacingDiagonal()
        {
            return this.RotBody == 1 || this.RotBody == 3 || this.RotBody == 5 || this.RotBody == 7;
        }

        internal bool HasIgnored(uint p)
        {
            return this.IgnoredPersons.Contains(p);
        }

        public bool forcesit { get; set; }
    }

    internal enum ItemEffectType
    {
        None,
        Swim,
        SwimLow,
        SwimHalloween,
        Iceskates,
        Normalskates,
        PublicPool
        //Skateboard?
    }

    internal static class ByteToItemEffectEnum
    {
        internal static ItemEffectType Parse(byte pByte)
        {
            switch (pByte)
            {
                case 0:
                    return ItemEffectType.None;
                case 1:
                    return ItemEffectType.Swim;
                case 2:
                    return ItemEffectType.Normalskates;
                case 3:
                    return ItemEffectType.Iceskates;
                case 4:
                    return ItemEffectType.SwimLow;
                case 5:
                    return ItemEffectType.SwimHalloween;
                case 6:
                    return ItemEffectType.PublicPool;
                default:
                    return ItemEffectType.None;
            }
        }
    }
    //0 = none
    //1 = pool
    //2 = normal skates
    //3 = ice skates
}
