﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Text;
using Butterfly.Collections;
using Butterfly.Core;
using Butterfly.HabboHotel.GameClients;
using Butterfly.HabboHotel.Items;
using Butterfly.HabboHotel.Pathfinding;
using Butterfly.HabboHotel.Pets;
using Butterfly.HabboHotel.RoomBots;
using Butterfly.HabboHotel.Rooms.Games;
using Butterfly.HabboHotel.Users.Inventory;
using Butterfly.Messages;
using Database_Manager.Database.Session_Details.Interfaces;
using Butterfly.Util;
using Butterfly.Messages.IOCodes;
using Butterfly.HabboHotel.Group;

namespace Butterfly.HabboHotel.Rooms
{
    class RoomUserManager
    {
        private Room room;

        internal Hashtable usersByUsername;
        internal Hashtable usersByUserID;
        private Hashtable bots;
        internal event RoomEventDelegate OnUserEnter;
        private List<int> cycledUsers;

        private Hashtable pets;

        private QueuedDictionary<int, RoomUser> userlist;
        //internal int RoomUserCounter;

        private int petCount;
        private int userCount;

        private int primaryPrivateUserID;
        private int secondaryPrivateUserID;

		private delegate void GlobalMessageBroadcast(byte[] message, int messageLength);
		private event GlobalMessageBroadcast onGlobalMessageBroadcast;


        internal int PetCount
        {
            get
            {
                return petCount;
            }
        }

        internal QueuedDictionary<int, RoomUser> UserList
        {
            get
            {
                return userlist;
            }
        }

        internal int GetRoomUserCount()
        {
            return userlist.Inner.Count;
        }

        public RoomUserManager(Room room)
        {
            //this.RoomUserCounter = 0;
            this.room = room;
            this.userlist = new QueuedDictionary<int, RoomUser>(new EventHandler(OnUserAdd), null, new EventHandler(onRemove), null);
            this.pets = new Hashtable();
            this.bots = new Hashtable();

            this.usersByUsername = new Hashtable();
            this.usersByUserID = new Hashtable();
            this.primaryPrivateUserID = 0;
            this.secondaryPrivateUserID = 0;
            this.ToRemove = new List<RoomUser>(room.UsersMax);
            this.cycledUsers = new List<int>();

            this.petCount = 0;
            this.userCount = 0;
        }

        internal RoomUser DeployBot(RoomBot Bot, Pet PetData)
        {

            RoomUser BotUser = new RoomUser(0, string.Empty, room.RoomId, primaryPrivateUserID++, room, false);
            BotUser.InternalRoomID = ++secondaryPrivateUserID;
            //this.UserList[PersonalID] = BotUser;
            userlist.Add(BotUser.InternalRoomID, BotUser);
            DynamicRoomModel Model = room.GetGameMap().Model;

            if ((Bot.X > 0 && Bot.Y > 0) && Bot.X < Model.MapSizeX && Bot.Y < Model.MapSizeY)
            {
                BotUser.SetPos(Bot.X, Bot.Y, Bot.Z);
                BotUser.SetRot(Bot.Rot, false);
            }
            else
            {
                Bot.X = Model.DoorX;
                Bot.Y = Model.DoorY;

                BotUser.SetPos(Model.DoorX, Model.DoorY, Model.DoorZ);
                BotUser.SetRot(Model.DoorOrientation, false);
            }

            BotUser.BotData = Bot;
            BotUser.BotAI = Bot.GenerateBotAI(BotUser.VirtualId);

            if (BotUser.IsPet)
            {
                BotUser.BotAI.Init((int)Bot.BotId, BotUser.VirtualId, room.RoomId, BotUser, room);
                BotUser.PetData = PetData;
                BotUser.PetData.virtualID = BotUser.VirtualId;
            }
            else
            {
                BotUser.BotAI.Init(Bot.ownerID > 0 ? (int)Bot.BotId : -1, BotUser.VirtualId, room.RoomId, BotUser, room);
            }

            UpdateUserStatus(BotUser, false);
            BotUser.UpdateNeeded = true;

            ServerMessage EnterMessage = PacketHandelingManager.GetRecycledItem(OutputCode.UsersMessageEvent);
            EnterMessage.AddInt32(1);
            BotUser.Serialize(EnterMessage);
            room.SendMessage(EnterMessage);

            BotUser.BotAI.OnSelfEnterRoom();

            if (BotUser.BotData.AiType == AIType.Guide)
                room.guideBotIsCalled = true;
            if (BotUser.IsPet)
            {
                if (pets.ContainsKey(BotUser.PetData.petID)) //Pet allready placed
                    pets[BotUser.PetData.petID] = BotUser;
                else
                    pets.Add(BotUser.PetData.petID, BotUser);

                petCount++;
            }

            bots.Add(Bot.BotId, BotUser);
            return BotUser;
        }

        internal void RemoveBot(int VirtualId, bool Kicked)
        {
            RoomUser User = GetRoomUserByVirtualId(VirtualId);

            if (User == null || !User.IsBot)
            {
                return;
            }

            if (User.IsPet)
            {
                pets.Remove(User.PetData.petID);
                petCount--;
            }

            bots.Remove(User.BotData.BotId);
            User.BotAI.OnSelfLeaveRoom(Kicked);
            RemoveRoomUser(User);
        }

        internal RoomUser GetRoomBot(uint botID)
        {
            return bots[botID] as RoomUser;
        }

        private void UpdateUserEffect(RoomUser User, int x, int y)
        {
            if (User.IsBot)
                return;
            byte NewCurrentUserItemEffect = room.GetGameMap().EffectMap[x, y];
            if (NewCurrentUserItemEffect > 0)
            {
                ItemEffectType Type = ByteToItemEffectEnum.Parse(NewCurrentUserItemEffect);
                if (Type != User.CurrentItemEffect)
                {
                    switch (Type)
                    {
                        case ItemEffectType.Iceskates:
                            {
                                if (User.GetClient().GetHabbo().Gender == "M")
                                    User.GetClient().GetHabboDataContainer().GetAvatarEffectsInventoryComponent().ApplyCustomEffect(38);
                                else
                                    User.GetClient().GetHabboDataContainer().GetAvatarEffectsInventoryComponent().ApplyCustomEffect(39);
                                User.CurrentItemEffect = ItemEffectType.Iceskates;
                                break;
                            }

                        case ItemEffectType.Normalskates:
                            {
                                if (User.GetClient().GetHabbo().Gender == "M")
                                {
                                    User.GetClient().GetHabboDataContainer().GetAvatarEffectsInventoryComponent().ApplyCustomEffect(55);
                                }
                                else
                                {
                                    User.GetClient().GetHabboDataContainer().GetAvatarEffectsInventoryComponent().ApplyCustomEffect(56);
                                }
                                //56=girls
                                //55=
                                User.CurrentItemEffect = Type;
                                break;
                            }
                        case ItemEffectType.Swim:
                            {
                                User.GetClient().GetHabboDataContainer().GetAvatarEffectsInventoryComponent().ApplyCustomEffect(29);
                                User.CurrentItemEffect = Type;
                                break;
                            }
                        case ItemEffectType.SwimLow:
                            {
                                User.GetClient().GetHabboDataContainer().GetAvatarEffectsInventoryComponent().ApplyCustomEffect(30);
                                User.CurrentItemEffect = Type;
                                break;
                            }
                        case ItemEffectType.SwimHalloween:
                            {
                                User.GetClient().GetHabboDataContainer().GetAvatarEffectsInventoryComponent().ApplyCustomEffect(37);
                                User.CurrentItemEffect = Type;
                                break;
                            }
                        case ItemEffectType.None:
                            {
                                User.GetClient().GetHabboDataContainer().GetAvatarEffectsInventoryComponent().ApplyCustomEffect(-1);
                                User.CurrentItemEffect = Type;
                                break;
                            }
                        case ItemEffectType.PublicPool:
                            {
                                User.AddStatus("swim", string.Empty);
                                User.CurrentItemEffect = Type;
                                break;
                            }

                    }
                }
            }
            else if (User.CurrentItemEffect != ItemEffectType.None && NewCurrentUserItemEffect == 0)
            {
                User.GetClient().GetHabboDataContainer().GetAvatarEffectsInventoryComponent().ApplyEffect(-1);
                User.CurrentItemEffect = ItemEffectType.None;
                User.RemoveStatus("swim");
            }
        }

        internal RoomUser GetUserForSquare(int x, int y)
        {
			int coord = TextHandling.CombineXYCoord(x, y);
            return room.GetGameMap().GetRoomUsers(ref coord).FirstOrDefault();
        }

		internal RoomUser GetUserForSquare(ref int coord)
		{
			return room.GetGameMap().GetRoomUsers(ref coord).FirstOrDefault();
		}

        internal void AddUserToRoom(GameClient Session, bool Spectator)
        {

            if (userCount >= room.UsersMax && Session.GetHabbo().Rank < 4)
            {
                ServerMessage message = PacketHandelingManager.GetRecycledItem(OutputCode.CantConnectMessageEvent);
                message.AddInt32(1);
                Session.SendMessage(message);

                message = PacketHandelingManager.GetRecycledItem(OutputCode.CloseConnectionMessageEvent);
                Session.SendMessage(message);
                return;
            }


            RoomUser user = new RoomUser(Session.GetHabbo().Id, Session.GetHabbo().Username, room.RoomId, primaryPrivateUserID++, room, Spectator);
            user.userID = Session.GetHabbo().Id;


            string username = Session.GetHabbo().Username;
            uint userID = user.userID;

            if (usersByUsername.ContainsKey(username.ToLower()))
                usersByUsername.Remove(username.ToLower());

            if (usersByUserID.ContainsKey(userID))
                usersByUserID.Remove(userID);

            usersByUsername.Add(Session.GetHabbo().Username.ToLower(), user);
            usersByUserID.Add(Session.GetHabbo().Id, user);

            user.InternalRoomID = AddRoomUser(user);

            Session.GetHabbo().CurrentRoomId = room.RoomId;
        }

        internal int AddRoomUser(RoomUser user)
        {
            UserList.Add(++secondaryPrivateUserID, user);
            return secondaryPrivateUserID;
        }

        private void OnUserAdd(object sender, EventArgs args)
        {
            try
            {
                KeyValuePair<int, RoomUser> userPair = (KeyValuePair<int, RoomUser>)sender;
                RoomUser user = userPair.Value;
                bool isBot = user.IsBot;
                GameClient session = null;

                if (user == null) 
                    return;

                if (!isBot)
                {
                    if (user.GetClient() == null || user.GetClient().GetHabbo() == null)
                        return;

                    session = user.GetClient();

                    if (session == null || session.GetHabbo() == null)
                        return;

                    if (userCount >= room.UsersMax && user.GetClient().GetHabbo().Rank < 4)
                    {
                        ServerMessage message = PacketHandelingManager.GetRecycledItem(OutputCode.CantConnectMessageEvent);
                        message.AddInt32(1);
                        session.SendMessage(message);

                        message = PacketHandelingManager.GetRecycledItem(OutputCode.CloseConnectionMessageEvent);
                        session.SendMessage(message);

                        return;
                    }

                    if (session.GetHabbo() != null && session.GetHabbo().IsGroupMember)
                    {
                        uint groupID = session.GetHabbo().EquippedGroup;
                        if (groupID > 0)
                        {
                            GroupData group = ButterflyEnvironment.Game.GetGroupManager().GetGroup(groupID);
                            GroupMember membership = group.GetMembership(session.GetHabbo().Id);
                            if (group != null && membership != null)
                            {
                                this.room.groupBadgeHandler.RequestBroadcast(membership);
                            }
                        }
                    }
                }
        

                if (!user.IsSpectator)
                {
                    DynamicRoomModel Model = room.GetGameMap().Model;
                    if (!user.IsBot)
                    {
                        

                        user.SetPos(Model.DoorX, Model.DoorY, Model.DoorZ);
                        user.SetRot(Model.DoorOrientation, false);

                        if (room.CheckRights(session, true))
                        {
                            user.AddStatus("flatctrl", "useradmin");
                        }
                        else if (room.CheckRights(session))
                        {
                            user.AddStatus("flatctrl", "");
                        }
                    }
                    else
                    {
                        user.SetPos(user.BotData.X, user.BotData.Y, user.BotData.Z);
                        user.SetRot(user.BotData.Rot);

                        if (user.BotData.X != Model.DoorX && user.BotData.Y != Model.DoorY)
                        {
                            int coord = user.GetDoubleCoordinate();
                            room.GetGameMap().AddUserToMap(user, ref coord);
                        }
                    }

                    user.CurrentItemEffect = ItemEffectType.None;

                    //UpdateUserEffect(User, User.X, User.Y);

                    if (!user.IsBot && user.GetClient().GetHabbo().IsTeleporting)
                    {
                        RoomItem Item = room.GetRoomItemHandler().GetItem(user.GetClient().GetHabbo().TeleporterId);

                        if (Item != null)
                        {
                            user.SetPos(Item.GetX, Item.GetY, Item.GetZ);
                            user.SetRot(Item.Rot, false);

                            Item.InteractingUser2 = session.GetHabbo().Id;
                            Item.ExtraData = "2";
                            Item.UpdateState(false, true);
                        }
                    }

                    if (!user.IsBot)
                    {
                        user.GetClient().GetHabbo().IsTeleporting = false;
                        user.GetClient().GetHabbo().TeleporterId = 0;
                    }

                    ServerMessage EnterMessage = PacketHandelingManager.GetRecycledItem(OutputCode.UsersMessageEvent);
                    EnterMessage.AddInt32(1);
                    user.Serialize(EnterMessage);
                    room.SendMessage(EnterMessage);
                }

                if (!isBot && session.GetHabboDataContainer().GetChatMessenger() != null)
                    session.GetHabboDataContainer().GetChatMessenger().OnStatusChanged(true);


                if (!user.IsSpectator && !isBot)
                {
                    foreach (RoomUser roomUser in UserList.Values)
                    {
                        if (roomUser.IsBot && roomUser.BotAI != null)
                            roomUser.BotAI.OnUserEnterRoom(user);
                    }
                }

				onGlobalMessageBroadcast += user.SendMessage;

                if (!isBot)
                { 
                    new OnRoomUserAddMessageEventHandler().HandleEvent(user.GetClient(), null);
                }

                if (OnUserEnter != null)
                    OnUserEnter(user, null);

                if (room.GotMusicController())
                    room.GetRoomMusicController().OnNewUserEnter(user);
            }
            catch (Exception e)
            {
                Logging.LogCriticalException(e.ToString());
            }

        }

		internal void BroadcastMessage(ServerMessage message)
		{
			if (onGlobalMessageBroadcast != null)
				onGlobalMessageBroadcast(message.Bytes(), message.GetLength());

			message.ReturnToOwner();
		}

        //internal void RequestRoomReload()
        //{
        //    userlist.QueueDelegate(new onCycleDoneDelegate(room.onReload));
        //}

        internal void UpdateUserStats(List<RoomUser> users, Hashtable userID, Hashtable userName, int primaryID, int secondaryID)
        {
            foreach (RoomUser user in users)
            {
                userlist.Inner.Add(user.InternalRoomID, user);
            }

            foreach (RoomUser user in userName.Values)
            {
                usersByUsername.Add(user.GetClient().GetHabbo().Username.ToLower(), user);
            }

            foreach (RoomUser user in userID.Values)
            {
                usersByUserID.Add(user.userID, user);
            }

            this.primaryPrivateUserID = primaryID;
            this.secondaryPrivateUserID = secondaryID;
            room.InitPets();
        }

        internal void RemoveUserFromRoom(GameClient Session, Boolean NotifyClient, Boolean NotifyKick)
        {
            try
            {
                if (Session == null)
                    return;

                if (Session.GetHabbo() == null)
                    return;


                Session.GetHabboDataContainer().GetAvatarEffectsInventoryComponent().OnRoomExit();

                if (NotifyClient)
                {
                    if (NotifyKick)
                    {
                        ServerMessage message = PacketHandelingManager.GetRecycledItem(OutputCode.GenericErrorEvent);
                        message.AddInt32(4008);
                        Session.SendMessage(message);
                    }

                    ServerMessage message2 = PacketHandelingManager.GetRecycledItem(OutputCode.CloseConnectionMessageEvent);
                    Session.SendMessage(message2);
                }

                RoomUser User = GetRoomUserByHabbo(Session.GetHabbo().Id);


                if (User != null)
                {
                    RemoveRoomUser(User);


                    if (Session.GetHabbo() != null)
                    {
                        if (!User.IsSpectator)
                        {
                            if (User.CurrentItemEffect != ItemEffectType.None)
                            {
                                User.GetClient().GetHabboDataContainer().GetAvatarEffectsInventoryComponent().CurrentEffect = -1;
                            }
                            //UserMatrix[User.X, User.Y] = false;

                            if (Session.GetHabbo() != null)
                            {
                                if (room.HasActiveTrade(Session.GetHabbo().Id))
                                    room.TryStopTrade(Session.GetHabbo().Id);

                                //if (Session.GetHabbo().Username == room.Owner)
                                //{
                                //    if (room.HasOngoingEvent)
                                //    {
                                //        ServerMessage Message = PacketHandelingManager.GetRecycledItem(OutputCode.RoomEventEvent);
                                //        Message.AddString("-1");
                                //        room.SendMessage(Message);

                                //        ButterflyEnvironment.Game.GetRoomManager().GetEventManager().QueueRemoveEvent(room.RoomData, room.Event.Category);
                                //    }
                                //}
                                Session.GetHabbo().CurrentRoomId = 0;

                                if (Session.GetHabboDataContainer().GetChatMessenger() != null)
                                    Session.GetHabboDataContainer().GetChatMessenger().OnStatusChanged(true);
                            }

                            //DateTime Start = DateTime.Now;
                            //using (IQueryAdapter dbClient = ButterflyEnvironment.DatabaseManager.getQueryreactor())
                            //{
                            //    //TimeSpan TimeUsed1 = DateTime.Now - Start;
                            //    //Logging.LogThreadException("Time used on sys part 2: " + TimeUsed1.Seconds + "s, " + TimeUsed1.Milliseconds + "ms", "");

                            //    //if (Session.GetHabbo() != null)
                            //    //    dbClient.runFastQuery("UPDATE user_roomvisits SET exit_timestamp = '" + TextHandling.GetUnixTimestamp() + "' WHERE room_id = '" + this.Id + "' AND user_id = '" + Id + "' ORDER BY exit_timestamp DESC LIMIT 1");
                            //    //dbClient.runFastQuery("UPDATE rooms SET users_now = " + UsersNow + " WHERE id = " + Id);
                            //    //dbClient.runFastQuery("REPLACE INTO room_active VALUES (" + RoomId + ", " + UsersNow + ")");
                            //    dbClient.runFastQuery("UPDATE room_active SET active_users = " + UsersNow);
                            //}
                        }
                    }

                    usersByUserID.Remove(User.userID);
                    if (Session.GetHabbo() != null)
                        usersByUsername.Remove(Session.GetHabbo().Username.ToLower());

					onGlobalMessageBroadcast -= User.SendMessage;
                    User.Dispose();
                }
                
            }
            catch (Exception e)
            {
                Logging.LogCriticalException("Error during removing user from room:" + e.ToString());
            }
        }

        private void onRemove(object sender, EventArgs args)
        {
            try
            {
                KeyValuePair<int, RoomUser> removedPair = (KeyValuePair<int, RoomUser>)sender;

                RoomUser user = removedPair.Value;
                GameClient session = user.GetClient();

				user.InternalRoomID = -1;
                int key = removedPair.Key;
                //freeIDs[key] = null;

                List<RoomUser> Bots = new List<RoomUser>();

                foreach (RoomUser roomUser in UserList.Values)
                {
                    if (roomUser.IsBot)
                        Bots.Add(roomUser);
                }

                List<RoomUser> PetsToRemove = new List<RoomUser>();
                foreach (RoomUser Bot in Bots)
                {
                    Bot.BotAI.OnUserLeaveRoom(session);

                    if (Bot.IsPet && Bot.PetData.ownerID == user.userID && !room.CheckRights(session, true))
                    {
                        PetsToRemove.Add(Bot);
                    }
                }

                foreach (RoomUser toRemove in PetsToRemove)
                {
                    if (user.GetClient() == null || user.GetClient().GetHabbo() == null || user.GetClient().GetHabboDataContainer().GetInventoryComponent() == null)
                        continue;

                    user.GetClient().GetHabboDataContainer().GetInventoryComponent().AddPet(toRemove.PetData);
                    RemoveBot(toRemove.VirtualId, false);
                }

                if (session != null && session.GetHabbo() != null && session.GetHabbo().IsGroupMember)
                {
                    GroupMember group = session.GetHabboDataContainer().GetEquippedGroup();
                    if (group != null)
                    {
                        this.room.groupBadgeHandler.RequestExit(group);
                    }
                }

				int userCoord = user.GetDoubleCoordinate();
                room.GetGameMap().RemoveUserFromMap(user, ref userCoord);
				user.OnDestroy();
            }
            catch (Exception e)
            {
                Logging.LogCriticalException(e.ToString());
            }
        }

        private void RemoveRoomUser(RoomUser user)
        {
			//if (!UserList.ContainsKey(user.InternalRoomID))
			//	Logging.LogCriticalException("Warning: The user " + user.userID + "in room " + room.RoomId + " is blocked");

            UserList.Remove(user.InternalRoomID);
            //user.InternalRoomID = -1;

            if (user.SetStep)
                room.GetGameMap().GameMap[user.SetX, user.SetY] = user.SqState;
            else
                room.GetGameMap().GameMap[user.X, user.Y] = user.SqState;

			int coordinate = user.GetDoubleCoordinate();
			room.GetGameMap().RemoveUserFromMap(user, ref coordinate);

			if (!user.IsBot)
			{
				usersByUsername.Remove(user.username);
				usersByUserID.Remove(user.userID);
			}

            ServerMessage LeaveMessage = PacketHandelingManager.GetRecycledItem(OutputCode.UserRemoveMessageEvent);
            LeaveMessage.AddString(user.VirtualId.ToString());
            room.SendMessage(LeaveMessage);
        }

        internal RoomUser GetPet(uint PetId)
        {
            if (pets.ContainsKey(PetId))
                return (RoomUser)pets[PetId];

            return null;
        }

        internal void UpdateUserCount(int count)
        {
            this.userCount = count;
            room.RoomData.UsersNow = count;

            using (IQueryAdapter dbClient = ButterflyEnvironment.DatabaseManager.getQueryreactor())
            {
                dbClient.runFastQuery("UPDATE room_active SET active_users = " + count + " WHERE roomid = " + room.RoomId);
            }

            if (room.HasOngoingEvent)
                ButterflyEnvironment.Game.GetRoomManager().GetEventManager().QueueUpdateEvent(room.RoomData, room.Event.Category);
            ButterflyEnvironment.Game.GetRoomManager().QueueActiveRoomUpdate(room.RoomData);
        }

        internal RoomUser GetRoomUserByVirtualId(int VirtualId)
        {
            return UserList.GetValue(VirtualId);
        }

        internal RoomUser GetRoomUserByHabbo(uint pId)
        {
            if (usersByUserID.ContainsKey(pId))
                return (RoomUser)usersByUserID[pId];

            return null;
        }

        internal List<RoomUser> GetRoomUsers()
        {
            List<KeyValuePair<int, RoomUser>> users = UserList.ToList();

            List<RoomUser> returnList = new List<RoomUser>();
            foreach (KeyValuePair<int, RoomUser> pair in users)
            {
                if (!pair.Value.IsBot)
                    returnList.Add(pair.Value);
            }

            return returnList;
        }

        internal List<RoomUser> GetRoomUserByRank(int minRank)
        {
            List<RoomUser> returnList = new List<RoomUser>();
            foreach (RoomUser user in UserList.Values)
            {
                if (!user.IsBot && user.GetClient() != null && user.GetClient().GetHabbo() != null && user.GetClient().GetHabbo().Rank > minRank)
                    returnList.Add(user);
            }

            return returnList;
        }

        internal RoomUser GetRoomUserByHabbo(string pName)
        {
            if (usersByUsername.ContainsKey(pName.ToLower()))
                return (RoomUser)usersByUsername[pName.ToLower()];

            return null;
        }


        internal void SavePets(IQueryAdapter dbClient)
        {
            try
            {
                if (GetPets().Count > 0)
                {
                    AppendPetsUpdateString(dbClient);
                }
            }
            catch (Exception e)
            {
                Logging.LogCriticalException("Error during saving furniture for room " + room.RoomId + ". Stack: " + e.ToString());
            }
        }

        internal void AppendPetsUpdateString(IQueryAdapter dbClient)
        {
            QueryChunk inserts = new QueryChunk("INSERT INTO user_pets (id,user_id,room_id,name,type,race,color,expirience,energy,createstamp,nutrition,respect,z,y,z) VALUES ");
            QueryChunk updates = new QueryChunk();

            List<uint> petsSaved = new List<uint>();
            foreach (Pet pet in GetPets())
            {
                if (petsSaved.Contains(pet.petID))
                    continue;

                petsSaved.Add(pet.petID);
                if (pet.databaseState == DatabaseUpdateState.PendingInsert)
                {
                    inserts.AddParameter(pet.petID + "name", pet.name);
                    inserts.AddParameter(pet.petID + "race", pet.race);
                    inserts.AddParameter(pet.petID + "color", pet.color);
                    inserts.AddQuery("(" + pet.petID + "," + pet.ownerID + "," + pet.roomID + ",@" + pet.petID + "name," + pet.petType + ",@" + pet.petID + "race,@" + pet.petID + "color,0,100,'" + pet.creationStamp + "',0,0,0,0,0)");
                }
                else if (pet.databaseState == DatabaseUpdateState.PendingUpdate)
                {
                    updates.AddParameter(pet.petID + "name", pet.name);
                    updates.AddParameter(pet.petID + "race", pet.race);
                    updates.AddParameter(pet.petID + "color", pet.color);
                    updates.AddQuery("UPDATE user_pets SET room_id = " + pet.roomID + ", name = @" + pet.petID + "name, race = @" + pet.petID + "race, color = @" + pet.petID + "color, type = " + pet.petType + ", expirience = " + pet.experience + ", " +
                        "energy = " + pet.energy + ", nutrition = " + pet.nutrition + ", respect = " + pet.respect + ", createstamp = '" + pet.creationStamp + "', x = " + pet.X + ", Y = " + pet.Y + ", Z = " + pet.Z + " WHERE id = " + pet.petID);
                }

                pet.databaseState = DatabaseUpdateState.Syncronized;
            }

            inserts.Execute(dbClient);
            updates.Execute(dbClient);

            inserts.Dispose();
            updates.Dispose();

            inserts = null;
            updates = null;
        }

        internal List<Pet> GetPets()
        {
            List<KeyValuePair<int, RoomUser>> users = UserList.ToList();

            List<Pet> results = new List<Pet>();
            foreach (KeyValuePair<int, RoomUser> pair in users)
            {
                RoomUser user = pair.Value;
                if (user.IsPet)
                    results.Add(user.PetData);
            }

            return results;
        }

        internal ServerMessage SerializeStatusUpdates(Boolean All)
        {
            List<RoomUser> Users = new List<RoomUser>();

            foreach (RoomUser User in UserList.Values)
            {
                if (!All)
                {
                    if (!User.UpdateNeeded)
                        continue;
                    User.UpdateNeeded = false;
                }

                Users.Add(User);
            }

            if (Users.Count == 0)
                return null;

            ServerMessage Message = PacketHandelingManager.GetRecycledItem(OutputCode.UserUpdateMessageEvent);
            Message.AddInt32(Users.Count);

            foreach (RoomUser User in Users)
                User.SerializeStatus(Message);

            return Message;
        }

        internal void UpdateUserStatusses()
        {
            onCycleDoneDelegate userUpdate = new onCycleDoneDelegate(onUserUpdateStatus);
            UserList.QueueDelegate(userUpdate);
        }

        private void onUserUpdateStatus()
        {
            foreach (RoomUser user in UserList.Values)
                UpdateUserStatus(user, false);
        }

        internal void backupCounters(ref int primaryCounter, ref int secondaryCounter)
        {
            primaryCounter = primaryPrivateUserID;
            secondaryCounter = secondaryPrivateUserID;
        }

        internal bool isValid(RoomUser user)
        {
            if (user.IsBot)
                return true;
			
            if (user.GetClient() == null)
                return false;
			if (user.GetClient().GetConnection() == null)
				return false;
            if (user.GetClient().GetHabbo() == null)
                return false;
            if (user.GetClient().GetHabbo().CurrentRoomId != room.RoomId)
                return false;
            if (user.InternalRoomID == -1)
                return false;
			if (user.VirtualId == -1)
				return false;

            return true;
        }

        internal void UpdateUserStatus(RoomUser User, bool cyclegameitems)
        {
            if (User == null)
                return;
            bool isBot = User.IsBot;
            if (isBot)
                cyclegameitems = false;

            if (User.Statusses.ContainsKey("lay") || User.Statusses.ContainsKey("sit"))
            {
                User.Statusses.Remove("lay");
                if (!User.forcesit)
                    User.Statusses.Remove("sit");
                User.UpdateNeeded = true;
            }

            //List<RoomItem> ItemsOnSquare = GetFurniObjects(User.X, User.Y);
            //CoordItemSearch ItemSearch = new CoordItemSearch(room.GetGameMap().CoordinatedItems);
			int userCoord = User.GetDoubleCoordinate();
			List<RoomItem> ItemsOnSquare = room.GetGameMap().GetCoordinatedItems(ref userCoord); //ItemSearch.GetAllRoomItemForSquare(User.X, User.Y);
            double newZ = room.GetGameMap().GetNewZForUser(User.X, User.Y, User.Z);
            
            if (newZ != User.Z)
            {
                if (User.isFlying)
                    newZ += 4 + (0.5 * Math.Sin(0.7 * User.flyk));
                if (User.UserIsRidingAHorse())
                    newZ += 1;

                User.Z = newZ;
                User.UpdateNeeded = true;
            }

            DynamicRoomModel Model = room.GetGameMap().Model;
            if (Model.SqState[User.X, User.Y] == SquareState.SEAT)
            {
                if (!User.Statusses.ContainsKey("sit"))
                {
                    User.Statusses.Add("sit", "1.0");
                }

                User.Z = Model.SqFloorHeight[User.X, User.Y];
                if (User.isFlying)
                    User.Z += 4 + (0.5 * Math.Sin(0.7 * User.flyk));
                User.RotHead = Model.SqSeatRot[User.X, User.Y];
                User.RotBody = Model.SqSeatRot[User.X, User.Y];

                User.UpdateNeeded = true;
			}

			#region Old crappy code
			//foreach (RoomItem Item in ItemsOnSquare)
			//{
			//	#region loooooop
			//	if (cyclegameitems)
			//	{
			//		Item.UserWalksOnFurni(User);
			//	}

			//	if (Item.GetBaseItem().IsSeat)
			//	{
			//		if (!User.Statusses.ContainsKey("sit"))
			//		{
			//			User.Statusses.Add("sit", TextHandling.GetString(Item.Height));
			//		}

			//		User.Z = Item.GetZ;
			//		if (User.isFlying)
			//			User.Z += 4 + (0.5 * Math.Sin(0.7 * User.flyk));
			//		User.RotHead = Item.Rot;
			//		User.RotBody = Item.Rot;

			//		User.UpdateNeeded = true;
			//	}


			//	switch (Item.GetBaseItem().InteractionType)
			//	{
			//		case InteractionType.bed:
			//			{
			//				if (!User.Statusses.ContainsKey("lay"))
			//				{
			//					User.Statusses.Add("lay", TextHandling.GetString(Item.Height) + " null");
			//				}

			//				User.Z = Item.GetZ;
			//				if (User.isFlying)
			//					User.Z += 4 + (0.2 * 0.5 * Math.Sin(0.7 * User.flyk));
			//				User.RotHead = Item.Rot;
			//				User.RotBody = Item.Rot;

			//				User.UpdateNeeded = true;
			//				break;
			//			}

			//		case InteractionType.fbgate:
			//			{
			//				if (cyclegameitems)
			//				{
			//					if (User.team != Item.team)
			//						User.team = Item.team;

			//					else if (User.team == Item.team)
			//						User.team = Team.none;

			//					if (!string.IsNullOrEmpty(Item.Figure))
			//					{
			//						//User = GetUserForSquare(Item.Coordinate.X, Item.Coordinate.Y);
			//						if (User != null && !User.IsBot)
			//						{
			//							if (User.Coordinate == Item.Coordinate)
			//							{
			//								if (User.GetClient().GetHabbo().Gender != Item.Gender && User.GetClient().GetHabbo().Look != Item.Figure 
			//									&& Item.Gender != null && Item.Figure != null)
			//								{

			//									User.GetClient().GetHabbo().tempGender = User.GetClient().GetHabbo().Gender;
			//									User.GetClient().GetHabbo().tempLook = User.GetClient().GetHabbo().Look;

			//									User.GetClient().GetHabbo().Gender = Item.Gender;
			//									User.GetClient().GetHabbo().Look = Item.Figure;
			//								}
			//								else if (User.GetClient().GetHabbo().tempGender != null && User.GetClient().GetHabbo().tempLook != null)
			//								{
			//									User.GetClient().GetHabbo().Gender = User.GetClient().GetHabbo().tempGender;
			//									User.GetClient().GetHabbo().Look = User.GetClient().GetHabbo().tempLook;
			//								}

			//								ServerMessage RoomUpdate = PacketHandelingManager.GetRecycledItem(OutputCode.UserChangeMessageEvent);
			//								RoomUpdate.AddInt32(User.VirtualId);
			//								RoomUpdate.AddString(User.GetClient().GetHabbo().Look);
			//								RoomUpdate.AddString(User.GetClient().GetHabbo().Gender.ToLower());
			//								RoomUpdate.AddString(User.GetClient().GetHabbo().Motto);
			//								RoomUpdate.AddInt32(User.GetClient().GetHabbo().AchievementPoints);
			//								room.SendMessage(RoomUpdate);
			//							}
			//						}
			//					}
			//				}

			//				break;
			//			}

			//		//33: Red
			//		//34: Green
			//		//35: Blue
			//		//36: Yellow

			//		case InteractionType.banzaigategreen:
			//		case InteractionType.banzaigateblue:
			//		case InteractionType.banzaigatered:
			//		case InteractionType.banzaigateyellow:
			//			{
			//				if (cyclegameitems)
			//				{
			//					int effectID = (int)Item.team + 32;
			//					AvatarEffectsInventoryComponent efectmanager = User.GetClient().GetHabboDataContainer().GetAvatarEffectsInventoryComponent();

			//					if (User.team != Item.team)
			//					{
			//						User.team = Item.team;

			//						if (efectmanager.CurrentEffect != effectID)
			//							efectmanager.ApplyCustomEffect(effectID);
			//					}
			//					else
			//					{
			//						if (efectmanager.CurrentEffect == effectID)
			//							efectmanager.ApplyCustomEffect(0);
			//						User.team = Team.none;
			//					}
			//				}
			//				break;
			//			}

			//		case InteractionType.freezeyellowgate:
			//		case InteractionType.freezeredgate:
			//		case InteractionType.freezegreengate:
			//		case InteractionType.freezebluegate:
			//			{
			//				if (cyclegameitems)
			//				{
			//					int effectID = (int)Item.team + 39;
			//					AvatarEffectsInventoryComponent efectmanager = User.GetClient().GetHabboDataContainer().GetAvatarEffectsInventoryComponent();

			//					if (User.team != Item.team)
			//					{
			//						User.team = Item.team;

			//						if (efectmanager.CurrentEffect != effectID)
			//							efectmanager.ApplyCustomEffect(effectID);
			//					}
			//					else
			//					{
			//						if (efectmanager.CurrentEffect == effectID)
			//							efectmanager.ApplyCustomEffect(0);
			//						User.team = Team.none;
			//					}

			//					//ServerMessage message = new ServerMessage(OutputCode.YouArePlayingGameMessageEvent);
			//					//message.AddBoolean((User.team != Team.none));

			//					//User.GetClient().SendMessage(message);
			//				}
			//				break;
			//			}
                    
			//		case InteractionType.banzaitele:
			//			{
			//				room.GetGameItemHandler().onTeleportRoomUserEnter(User, Item);
			//				break;
			//			}
			//	}
			//	#endregion
			//}
			#endregion

			
            //TODO get heighest items
            RoomItem[] items = room.GetGameMap().GetCoordinatedHeighestItems(ref userCoord).ToArray();
            foreach (RoomItem item in items)
            {
                item.OnUserWalk(User, cyclegameitems);
            }

			if (cyclegameitems)
            {
                if (room.GotSoccer())
                    room.GetSoccer().OnUserWalk(User);

                if (room.GotBanzai())
                    room.GetBanzai().OnUserWalk(User);

                //if (room.GotFreeze())
                    room.GetFreeze().OnUserWalk(User);
			}
		}

        internal void TurnHeads(int X, int Y, uint SenderId)
        {
            foreach (RoomUser user in UserList.Values)
            {
                if (user.HabboId == SenderId)
                {
                    if (user.headRotDelay > 0)
                    {
                        user.SetRot(user.RotBody, true);
                    }
                }
                else
                {
                    user.SetRot(Rotation.Calculate(user.X, user.Y, X, Y), true);
                    user.headRotDelay = 4;
                }
            }
        }

        private List<RoomUser> ToRemove;

        internal void OnCycle(ref int idleCount)
        {
            ToRemove.Clear();
            int userCounter = 0;
            this.cycledUsers.Clear();

            foreach (RoomUser User in UserList.Values)
            {
                if (!User.IsBot)
                    userCounter++;

                if (cycledUsers.Contains(User.InternalRoomID) )
                    continue;

                if (!isValid(User))
                {
                    if (User.GetClient() != null)
                        RemoveUserFromRoom(User.GetClient(), false, false);
                    RemoveRoomUser(User);
                }

                if (User.InternalRoomID == -1)
                    continue;

                bool updated = false;
                User.IdleTime++;

                if (!User.IsAsleep && User.IdleTime >= 600)
                {
                    User.IsAsleep = true;

                    ServerMessage FallAsleep = PacketHandelingManager.GetRecycledItem(OutputCode.SleepMessageEvent);
                    FallAsleep.AddInt32(User.VirtualId);
                    FallAsleep.AddBoolean(true);
                    room.SendMessage(FallAsleep);
                }

                if (User.headRotDelay > 0 && --User.headRotDelay == 0)
                {
                    User.SetRot(User.RotBody, true);
                }

                if (User.NeedsAutokick && !ToRemove.Contains(User))
                {
                    ToRemove.Add(User);
                    continue;
                }

                if (User.CarryItemID > 0)
                {
                    User.CarryTimer--;
                    if (User.CarryTimer <= 0)
                        User.CarryItem(0);
                }

                if (room.GotFreeze())
                {
                    room.GetFreeze().CycleUser(User);
                }

                if (User.isFlying)
                    User.OnFly();

                if (User.SetStep)
                {
                    if (SetStepForUser(User))
                    {
                        FinishUserCycle(User);
                        continue;
                    }
                }

                if (User.IsWalking && !User.Freezed)
                {
                    CalculatePath(User, ref updated);
                }
                else
                {
                    if (User.Statusses.ContainsKey("mv") && !User.IsHorseWalking)
                    {
                        User.RemoveStatus("mv");
                        User.UpdateNeeded = true;
                    }
                }

                if (User.IsBot)
                    User.BotAI.OnTimerTick();

                if (!updated)
                    UpdateUserEffect(User, User.X, User.Y);

                FinishUserCycle(User);
            }


            if (userCounter == 0)
                idleCount++;


            foreach (RoomUser toRemove in ToRemove)
            {
                if (toRemove.GetClient() != null)
                {
					RemoveUserFromRoom(toRemove.GetClient(), true, false);
                }
                else
                    RemoveRoomUser(toRemove);
            }

            if (userCount != userCounter)
            {
                UpdateUserCount(userCounter);
            }
        }


        private bool SetStepForUser(RoomUser User)
        {
            if (room.GetGameMap().CanWalk(User.SetX, User.SetY, User.SetZ, User.AllowOverride) || User.ignoreMap || User.PetGotUserRiding())
            {
				int currentCoord = User.GetDoubleCoordinate();
				int newCoord = TextHandling.CombineXYCoord(User.SetX, User.SetY);

				room.GetGameMap().UpdateUserMovement(ref currentCoord, ref newCoord, User);
				RoomItem[] items = room.GetGameMap().GetCoordinatedHeighestItems(ref currentCoord);

                User.X = User.SetX;
                User.Y = User.SetY;
                User.Z = User.SetZ;

                if (User.isFlying)
                {
                    User.Z += 4 + 0.5 * Math.Sin(0.7 * User.flyk);
                }
                else
                {
                    foreach (RoomItem item in items)
                    {
                        item.UserWalksOffFurni(User);
                    }
                   
                }

                if (User.X == room.GetGameMap().Model.DoorX && User.Y == room.GetGameMap().Model.DoorY && !ToRemove.Contains(User) && !User.IsBot)
                {
                    ToRemove.Add(User);
                    return true;
                }


                UpdateUserStatus(User, true);
            }
            User.SetStep = false;
            return false;
        }

        private void FinishUserCycle(RoomUser user)
        {
            if (!cycledUsers.Contains(user.InternalRoomID))
                cycledUsers.Add(user.InternalRoomID);
        }

        private void CalculatePath(RoomUser User, ref bool updated)
        {
            Gamemap map = room.GetGameMap();
            SquarePoint Point = DreamPathfinder.GetNextStep(User.X, User.Y, User.Z, User.GoalX, User.GoalY, map.GameMap, map.GetStackmap(), map.Model.SqFloorHeight,
                map.Model.MapSizeX, map.Model.MapSizeY, User.AllowOverride, map.DiagonalEnabled, map);

            if (Point.X == User.X && Point.Y == User.Y) //No path found, or reached goal (:
            {
                User.IsWalking = false;
                User.IsHorseWalking = false;
                User.OnStopWalk();
                User.RemoveStatus("mv");

                UpdateUserStatus(User, false);
            }
            else
            {
                HandleSetMovement(Point, User, ref updated, true);
                if (User.IsPet && User.PetGotUserRiding())
                {
                    RoomUser ridingPet = User.GetRoomUserRiding();
                    SetStepForUser(ridingPet);
                    HandleSetMovement(Point, ridingPet, ref updated, false);
                    UpdateUserEffect(ridingPet, ridingPet.X, ridingPet.Y);
                    ridingPet.UpdateNeeded = true;
                    FinishUserCycle(ridingPet);
                }
            }

            User.UpdateNeeded = true;
        }

        private void HandleSetMovement(SquarePoint Point, RoomUser User, ref bool updated, bool setSquareMap)
        {
            int nextX = Point.X;
            int nextY = Point.Y;

            User.RemoveStatus("mv");

            //double nextZ = room.GetGameMap().SqAbsoluteHeight(nextX, nextY);
            double nextZ = room.GetGameMap().GetNewZForUser(nextX, nextY, User.Z);

            //if (room.GetGameMap().SquareOccupied(nextX, nextY, nextZ))
            //    return;

            if (User.UserIsRidingAHorse())
                nextZ += 1;

            User.Statusses.Remove("lay");
            User.Statusses.Remove("sit");
            if (!User.isFlying)
                User.AddStatus("mv", nextX + "," + nextY + "," + TextHandling.GetString(nextZ));
            else
                User.AddStatus("mv", nextX + "," + nextY + "," + TextHandling.GetString(nextZ + 4 + (0.5 * Math.Sin(0.7 * User.flyk))));

            int newRot = Rotation.Calculate(User.X, User.Y, nextX, nextY, User.moonwalkEnabled);

            User.RotBody = newRot;
            User.RotHead = newRot;

            User.SetStep = true;
            User.SetX = nextX;
            User.SetY = nextY;
            User.SetZ = nextZ;

            UpdateUserEffect(User, User.SetX, User.SetY);
            updated = true;
            User.UpdateNeeded = true;


            if (setSquareMap)
            {
                room.GetGameMap().GameMap[User.X, User.Y] = User.SqState; // REstore the old one
                User.SqState = room.GetGameMap().GameMap[User.SetX, User.SetY];//Backup the new one
                
                if (!room.AllowWalkthrough)
                    room.GetGameMap().GameMap[nextX, nextY] = 0;
            }
        }

        internal void Destroy()
        {
            room = null;
            usersByUsername.Clear();
            usersByUsername = null;
            usersByUserID.Clear();
            usersByUserID = null;
            OnUserEnter = null;
            pets.Clear();
            pets = null;
            userlist.Destroy();
            userlist = null;
        }

        internal void RefreshUser(RoomUser user)
        {
            ServerMessage LeaveMessage = PacketHandelingManager.GetRecycledItem(OutputCode.UserRemoveMessageEvent);
            LeaveMessage.AddString(user.VirtualId.ToString());
            room.SendMessage(LeaveMessage);

            ServerMessage EnterMessage = PacketHandelingManager.GetRecycledItem(OutputCode.UsersMessageEvent);
            EnterMessage.AddInt32(1);
            user.Serialize(EnterMessage);
            room.SendMessage(EnterMessage);
        }
    }
}
