﻿using System;
using System.Collections.Generic;
using System.Threading.Tasks;
using Butterfly.Core;
using Butterfly.HabboHotel.GameClients;
using Butterfly.HabboHotel.Items;
using Butterfly.Messages;
using Butterfly.Messages.IOCodes;

namespace Butterfly.HabboHotel.Rooms
{
    class Trade
    {
        private TradeUser[] Users;
        private int TradeStage;
        private UInt32 RoomId;

        private UInt32 oneId;
        private UInt32 twoId;

        internal Trade(UInt32 UserOneId, UInt32 UserTwoId, UInt32 RoomId)
        {
            this.oneId = UserOneId;
            this.twoId = UserTwoId;

            this.Users = new TradeUser[2];
            this.Users[0] = new TradeUser(UserOneId, RoomId);
            this.Users[1] = new TradeUser(UserTwoId, RoomId);
            this.TradeStage = 1;
            this.RoomId = RoomId;
        }

        internal bool AllUsersAccepted
        {
            get
            {
                for (int i = 0; i < Users.Length; i++)
                {
                    if (Users[i] == null)
                        continue;
                    if (!Users[i].HasAccepted)
                        return false;
                }

                return true;
            }
        }

        internal bool ContainsUser(UInt32 Id)
        {
            for (int i = 0; i < Users.Length; i++)
            {
                if (Users[i] == null)
                    continue;
                if (Users[i].UserId == Id)
                    return true;
            }

            return false;
        }

        internal TradeUser GetTradeUser(UInt32 Id)
        {
            for (int i = 0; i < Users.Length; i++)
            {
                if (Users[i] == null)
                    continue;
                if (Users[i].UserId == Id)
                    return Users[i];
            }

            return null;
        }

        internal void OfferItem(UInt32 UserId, UserItem Item)
        {
            TradeUser User = GetTradeUser(UserId);

            if (User == null || Item == null || !Item.GetBaseItem().AllowTrade || User.HasAccepted || TradeStage != 1)
            {
                return;
            }

            ClearAccepted();
            if(!User.OfferedItems.Contains(Item))
                User.OfferedItems.Add(Item);
            UpdateTradeWindow();
        }

        internal void TakeBackItem(UInt32 UserId, UserItem Item)
        {
            TradeUser User = GetTradeUser(UserId);

            if (User == null || Item == null || User.HasAccepted || TradeStage != 1)
            {
                return;
            }

            ClearAccepted();

            User.OfferedItems.Remove(Item);
            UpdateTradeWindow();
        }

        internal void Accept(UInt32 UserId)
        {
            TradeUser User = GetTradeUser(UserId);

            if (User == null || TradeStage != 1)
            {
                return;
            }

            User.HasAccepted = true;

            ServerMessage Message = PacketHandelingManager.GetRecycledItem(OutputCode.TradingAcceptEvent);
            Message.AddUInt32(UserId);
            Message.AddInt32(1);
            SendMessageToUsers(Message);

            if (AllUsersAccepted)
            {
                SendMessageToUsers(PacketHandelingManager.GetRecycledItem(OutputCode.TradingConfirmationEvent));
                TradeStage++;
                ClearAccepted();
            }
        }

        internal void Unaccept(UInt32 UserId)
        {
            TradeUser User = GetTradeUser(UserId);

            if (User == null || TradeStage != 1 || AllUsersAccepted)
            {
                return;
            }

            User.HasAccepted = false;

            ServerMessage Message = PacketHandelingManager.GetRecycledItem(OutputCode.TradingAcceptEvent);
            Message.AddUInt32(UserId);
            Message.AddInt32(0);
            SendMessageToUsers(Message);
        }

        internal void CompleteTrade(UInt32 UserId)
        {
            TradeUser User = GetTradeUser(UserId);

            if (User == null || TradeStage != 2)
            {
                return;
            }

            User.HasAccepted = true;

            ServerMessage Message = PacketHandelingManager.GetRecycledItem(OutputCode.TradingAcceptEvent);
            Message.AddUInt32(UserId);
            Message.AddInt32(1);
            SendMessageToUsers(Message);

            if (AllUsersAccepted)
            {
                TradeStage = 999;
                Finnito();
                //Task pTask = new Task(Finnito);
                //pTask.Start();
            }
        }

        private void Finnito()
        {
            try
            {
                DeliverItems();
                CloseTradeClean();
            }
            catch (Exception e)
            {
                Logging.LogThreadException(e.ToString(), "Trade task");
            }
        }

        internal void ClearAccepted()
        {
            foreach (TradeUser User in Users)
            {
                User.HasAccepted = false;
            }
        }

        internal void UpdateTradeWindow()
        {
            ServerMessage Message = PacketHandelingManager.GetRecycledItem(OutputCode.TradingItemListEvent);

            for (int i = 0; i < Users.Length; i++)
            {
                TradeUser User = Users[i];
                if (User == null)
                    continue;

                Message.AddUInt32(User.UserId);
                Message.AddInt32(User.OfferedItems.Count);

                foreach (UserItem Item in User.OfferedItems)
                {
                    Message.AddUInt32(Item.itemID);
                    Message.AddChar(Item.GetBaseItem().Type);
                    Message.AddUInt32(Item.itemID);
                    Message.AddInt32(Item.GetBaseItem().SpriteId);
                    Message.AddInt32(0); // undef
                    Message.AddBoolean(true);
                    Message.AddInt32(0);
                    Message.AddString("");
                    Message.AddInt32(0); // xmas 09 furni had a special furni tag here, with wired day (wat?)
                    Message.AddInt32(0); // xmas 09 furni had a special furni tag here, wired month (wat?)
                    Message.AddInt32(0); // xmas 09 furni had a special furni tag here, wired year (wat?)

                    if (Item.GetBaseItem().Type == 's')
                    {
                        Message.AddInt32(0);
                    }
                }
            }


            SendMessageToUsers(Message);
        }

        internal void DeliverItems()
        {
            // List items
            List<UserItem> ItemsOne = GetTradeUser(oneId).OfferedItems;
            List<UserItem> ItemsTwo = GetTradeUser(twoId).OfferedItems;

            // Verify they are still in user inventory
            foreach (UserItem I in ItemsOne)
            {
                if (GetTradeUser(oneId).GetClient().GetHabboDataContainer().GetInventoryComponent().GetItem(I.itemID) == null)
                {
                    GetTradeUser(oneId).GetClient().SendNotif(LanguageLocale.GetValue("trade.failed"));
                    GetTradeUser(twoId).GetClient().SendNotif(LanguageLocale.GetValue("trade.failed"));

                    return;
                }
            }

            foreach (UserItem I in ItemsTwo)
            {
                if (GetTradeUser(twoId).GetClient().GetHabboDataContainer().GetInventoryComponent().GetItem(I.itemID) == null)
                {
                    GetTradeUser(oneId).GetClient().SendNotif(LanguageLocale.GetValue("trade.failed"));
                    GetTradeUser(twoId).GetClient().SendNotif(LanguageLocale.GetValue("trade.failed"));

                    return;
                }
            }

            GetTradeUser(twoId).GetClient().GetHabboDataContainer().GetInventoryComponent().RunDBUpdate();
            GetTradeUser(oneId).GetClient().GetHabboDataContainer().GetInventoryComponent().RunDBUpdate();
            // Deliver them
            foreach (UserItem I in ItemsOne)
            {
                GetTradeUser(oneId).GetClient().GetHabboDataContainer().GetInventoryComponent().RemoveItem(I.itemID, false);
                GetTradeUser(twoId).GetClient().GetHabboDataContainer().GetInventoryComponent().AddNewItem(I.itemID, I.BaseItemID, I.extraData, false, false, 0);

                GetTradeUser(oneId).GetClient().GetHabboDataContainer().GetInventoryComponent().RunDBUpdate();
                GetTradeUser(twoId).GetClient().GetHabboDataContainer().GetInventoryComponent().RunDBUpdate();
            }

            foreach (UserItem I in ItemsTwo)
            {
                GetTradeUser(twoId).GetClient().GetHabboDataContainer().GetInventoryComponent().RemoveItem(I.itemID, false);
                GetTradeUser(oneId).GetClient().GetHabboDataContainer().GetInventoryComponent().AddNewItem(I.itemID, I.BaseItemID, I.extraData, false, false, 0);

                GetTradeUser(twoId).GetClient().GetHabboDataContainer().GetInventoryComponent().RunDBUpdate();
                GetTradeUser(oneId).GetClient().GetHabboDataContainer().GetInventoryComponent().RunDBUpdate();
            }

            // Update inventories
            GetTradeUser(oneId).GetClient().GetHabboDataContainer().GetInventoryComponent().UpdateItems(false);
            GetTradeUser(twoId).GetClient().GetHabboDataContainer().GetInventoryComponent().UpdateItems(false);
        }

        internal void CloseTradeClean()
        {
            for (int i = 0; i < Users.Length; i++)
            {
                TradeUser User = Users[i];
                if (User == null)
                    continue;
                if (User.GetRoomUser() == null)
                    continue;

                User.GetRoomUser().RemoveStatus("trd");
                User.GetRoomUser().UpdateNeeded = true;
            }

            SendMessageToUsers(PacketHandelingManager.GetRecycledItem(OutputCode.TradingCompletedEvent));
            GetRoom().ActiveTrades.Remove(this);
        }

        internal void CloseTrade(UInt32 UserId)
        {
            for (int i = 0; i < Users.Length; i++)
            {
                TradeUser User = Users[i];
                if (User == null)
                    continue;
                if (User.GetRoomUser() == null)
                    continue;

                User.GetRoomUser().RemoveStatus("trd");
                User.GetRoomUser().UpdateNeeded = true;
            }

            ServerMessage Message = PacketHandelingManager.GetRecycledItem(OutputCode.TradingCloseEvent);
            Message.AddUInt32(UserId);
            Message.AddUInt32(2);
            
            SendMessageToUsers(Message);
        }

        internal void SendMessageToUsers(ServerMessage Message)
        {
            if (Users == null)
                return;
            for (int i = 0; i < Users.Length; i++)
            {
                TradeUser User = Users[i];
                if (User == null)
                    continue;
                if (User != null)
                    if (User.GetClient() != null)
                        User.GetClient().SendMessageWithoutReturning(Message);
            }
            Message.ReturnToOwner();
        }

        private Room GetRoom()
        {
            return ButterflyEnvironment.Game.GetRoomManager().GetRoom(RoomId);
        }
    }

    class TradeUser
    {
        internal UInt32 UserId;
        private UInt32 RoomId;
        private bool Accepted;

        internal List<UserItem> OfferedItems;

        internal bool HasAccepted
        {
            get
            {
                return Accepted;
            }

            set
            {
                Accepted = value;
            }
        }

        internal TradeUser(UInt32 UserId, UInt32 RoomId)
        {
            this.UserId = UserId;
            this.RoomId = RoomId;
            this.Accepted = false;
            this.OfferedItems = new List<UserItem>();
        }

        internal RoomUser GetRoomUser()
        {
            Room Room = ButterflyEnvironment.Game.GetRoomManager().GetRoom(RoomId);

            if (Room == null)
            {
                return null;
            }
            
            return Room.GetRoomUserManager().GetRoomUserByHabbo(UserId);
        }

        internal GameClient GetClient()
        {
            return ButterflyEnvironment.Game.GetClientManager().GetClientByUserID(UserId);
        }
    }
}
