﻿using Butterfly.HabboHotel.GameClients;
using System.Text;

namespace Butterfly.HabboHotel.Users.ChatMessageHandling
{
    class CommandMetaData
    {
        internal int RequiredRank { get; private set; }

        internal string Description { get; private set; }

        internal string[] Arguments { get; private set; }

        internal string[] Inputs { get; private set; }

        internal string ClubRequirement { get; private set; }

        public void SetMetaData(int rank, string desc, string[] args, string[] inputs, string clubRequirement)
        {
            this.RequiredRank = rank;
            this.Description = desc;
            this.Arguments = args;
            this.Inputs = inputs;
            this.ClubRequirement = clubRequirement;
        }

        public bool CanRun(GameClient session, int argLen = -1)
        {
            if (!string.IsNullOrEmpty(ClubRequirement) && !session.GetHabboDataContainer().GetSubscriptionManager().HasSubscription(ClubRequirement))
            {
                return false;
            }
            else if (RequiredRank == 0)
            {
                return true;
            }
            else if (RequiredRank > 0 && RequiredRank <= session.GetHabbo().Rank)
            {
                return true;
            }
            else if (RequiredRank < 0)
            {
                if (RequiredRank == -1)
                    return (session.GetHabbo().CurrentRoom.CheckRights(session, false));
                else if (RequiredRank == -2)
                    return (session.GetHabbo().CurrentRoom.CheckRights(session, true));
            }

            return false;
        }

        public override string ToString()
        {
            StringBuilder description = new StringBuilder();

            if (this.Inputs != null) 
                foreach (string input in this.Inputs)
                    description.Append(input + "/");

            if (description.Length > 0)
                description.Remove(description.Length - 1, 1); // remove last /

            if (this.Arguments != null)
                foreach (string arg in this.Arguments)  
                    description.Append(" (" + arg + ")");
            

            description.Append(" - ");
            description.Append(this.Description);

            return description.ToString();
        }
    }
}
