﻿using Butterfly.Core;
using Butterfly.HabboHotel.GameClients;
using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using System.Xml.Serialization;
using System.Runtime.Serialization.Json;

namespace Butterfly.HabboHotel.Users.ChatMessageHandling.Invoking
{
    class CommandInvoker
    {
        private Dictionary<string, IChatMessageHandler> register;
        private List<IChatMessageHandler> invokers;
        private const string NAMESPACE = "Butterfly.HabboHotel.Users.ChatMessageHandling.";

        public CommandInvoker()
        {
            this.register = new Dictionary<string,IChatMessageHandler>();
            this.invokers = new List<IChatMessageHandler>();

            CommandSet commands = JsonReader<CommandSet>.FromFile("Settings/Commands.json");
            foreach (Command command in commands.Commands)
            {
                Type type = Type.GetType(NAMESPACE + command.InvokerClass);

                if (type == null)
                    throw new Exception(string.Format("Attempted to register an unknown command handler: {0}", command.InvokerClass));

                IChatMessageHandler handler = Activator.CreateInstance(type) as IChatMessageHandler;
                handler.SetMetaData(command.RequiredRank, command.Description, command.Arguments, command.Inputs, command.ClubRequirement);
                
                this.invokers.Add(handler);

                foreach (string input in command.Inputs)
                {
                    register.Add(input, handler);
                }
            }
        }

        internal bool TryCommand(GameClient client, string[] args)
        {
            IChatMessageHandler handler;
            if (args[0][0] == ':' && this.register.TryGetValue(args[0].Substring(1), out handler))
            {
                if (handler.CanRun(client, args.Length))
                {
                    try
                    { 
                        handler.RunCommand(client, args);
                    }
                    catch (IndexOutOfRangeException)
                    {
                        client.SendNotif(LanguageLocale.GetValue("input.userparammissing"));
                    }
                    catch (FormatException)
                    {
                        client.SendNotif(LanguageLocale.GetValue("input.intonly"));
                    }

                    return true;
                }
                else
                {
                    client.SendNotif(LanguageLocale.GetValue("input.missingpriveliges"));
                }
            }
            else if (args[0] == ":" && args.Length == 1)
            {
                return true;
            }

            return false;
        }

        internal string GenerateCommandList(GameClient client)
        {
            StringBuilder list = new StringBuilder();
            
            foreach (IChatMessageHandler handler in this.invokers)
            {
                if (handler.CanRun(client))
                {
                    list.Append(handler.ToString() + '\n');
                }   
            }

            return list.ToString();
        }
    }
}
