﻿using Butterfly.HabboHotel.GameClients;
using Butterfly.HabboHotel.Group;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Butterfly.HabboHotel.Users.Group
{
    class UserGroupManager : IGroupMemberObserver
    {
        private Dictionary<uint, GroupMember> myGroups = new Dictionary<uint, GroupMember>();
        private GameClient mClient;
        private Habbo data;
        internal GroupMember currentFavorite { get; set; }

        public UserGroupManager(Habbo data, GameClient client)
        {
            this.mClient = client;
            this.data = data;
            BindClientToGroupMembers(data);
        }

        internal void BindClientToGroupMembers(Habbo data)
        {
            GroupMember m = null;
			List<uint> theMemberlist = new List<uint>(data.GroupList);
			foreach (uint i in theMemberlist)
            {
                m = data.GetGroupMember(i);
				if (m == null)
				{
					continue;
				}
                m.SetNewClient(this);
                this.myGroups.Add(m.Group.ID, m);
            }

            foreach (uint i in data.PendingGroupJoins)
            {
                m = ButterflyEnvironment.Game.GetGroupManager().GetGroup(i).GetPendingGroupUser(data.Id);
                if (m != null)
                {
                    m.SetNewClient(this);
                    this.myGroups.Add(m.Group.ID, m);
                }
            }
        }
        internal void SendMessage(Messages.ServerMessage message)
        {
            if (mClient != null)
                mClient.SendMessage(message);
        }

        public void RemovePendingGroupJoin(GroupMember m)
        {
            this.myGroups.Remove(m.Group.ID);
        }

        public void AddPendingGroupJoin(GroupMember m)
        {
            myGroups.Add(m.Group.ID, m);
        }

        public void AddNewMembership(GroupMember m)
        {
            if (myGroups.ContainsKey(m.Group.ID))
                return;
            myGroups.Add(m.Group.ID, m);
        }

        public void RemoveExistingGroupMembership(GroupMember m)
        {
            myGroups.Remove(m.Group.ID);
        }

        internal void Clean()
        {
            if (this.mClient != null)
            {
                this.mClient = null;
                this.data = null;
                lock (myGroups)
                {
                    foreach (GroupMember m in this.myGroups.Values)
                    {
                        m.SetNewClient(null);
                    }
                }
            }
        }

        internal GroupMember GetGroupMember(uint groupID)
        {
            GroupMember m = null;
            this.myGroups.TryGetValue(groupID, out m);
            return m;
        }

        internal MembershipStatus RequestToJoinGroup(uint groupID)
        {
            if(this.myGroups.ContainsKey(groupID))//Already in a group with given id, or awaiting approval
                return this.myGroups[groupID].status;
           GroupData d = ButterflyEnvironment.Game.GetGroupManager().GetGroup(groupID);
           if (d != null)
           {
              GroupMember m = d.RequestMembership(this.data, this);
              if (m != null)
                  return m.status;
           }

           return MembershipStatus.Unregistered;
        }
    }
}
