﻿using ConsoleWriter;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Butterfly.HabboHotel.Users
{
    class IUpdateable<E>
    {
        private ArrayList registeredListeners = new ArrayList();
        private Queue deleteQueue = new Queue();
        private Queue addQueue = new Queue();
        private bool looping;

        public int ActiveListenerCount { get { return registeredListeners.Count - deleteQueue.Count + addQueue.Count; } }

        internal void Updated(E item)
        {
            lock (registeredListeners.SyncRoot)
            {
                looping = true;
                foreach (IUpdateListener<E> listener in registeredListeners)
                {
                    try
                    {
                        listener.ItemUpdated(item);
                    }
                    catch (Exception e)
                    {
                        Writer.LogException("Exception in IUpdatable class: " + e.Message + "\n" + e.StackTrace.ToString());
                    }
                }
                while (addQueue.Count > 0)
                {
                    registeredListeners.Add(addQueue.Dequeue());
                }
                while (deleteQueue.Count > 0)
                {
                    registeredListeners.Remove(deleteQueue.Dequeue());
                }
                looping = false;
            }
        }

        internal virtual void RegisterUpdater(IUpdateListener<E> item)
        {
            if (looping)
                addQueue.Enqueue(item);
            else
            {
                lock (this.registeredListeners.SyncRoot)
                {
                    this.registeredListeners.Remove(item);
                }
            }
        }

        internal virtual void RemoveUpdater(IUpdateListener<E> item)
        {
            if (looping)
                deleteQueue.Enqueue(item);
            else
            {
                lock (registeredListeners.SyncRoot)
                {
                    registeredListeners.Remove(item);
                }
            }
        }
    }
}
