﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using System.Text;
using Butterfly.Core;
using Butterfly.HabboHotel.Catalogs;
using Butterfly.HabboHotel.GameClients;
using Butterfly.HabboHotel.Items;
using Butterfly.HabboHotel.Pets;
using Butterfly.Messages;
using System.Linq;
using Butterfly.Collections;
using Database_Manager.Database.Session_Details.Interfaces;
using Butterfly.HabboHotel.Users.UserDataManagement;
using Butterfly.Util;
using Butterfly.Messages.IOCodes;
using Butterfly.HabboHotel.RoomBots;  
namespace Butterfly.HabboHotel.Users.Inventory
{
    class InventoryComponent
    {
        private Hashtable floorItems;
        private Hashtable wallItems;
        //private Hashtable discs;
        
        internal Hashtable baseIDRegister;

        private Hashtable InventoryPets;
        private Hashtable mAddedItems;
        //private Hashtable InventoryBots;
        private ArrayList mRemovedItems;
        private GameClient mClient;

        private Hashtable bots;

        internal PlaceMode mode;
        internal int primaryPlaceParameter;
        internal uint placeAmount = 1;

        internal uint UserId;
        internal const int MAX_ITEMS_COUNT = 2500;

        internal InventoryComponent(uint UserId, GameClient Client, UserData UserData)
        {
            this.mClient = Client;
            this.UserId = UserId;
            this.floorItems = new Hashtable();
            this.wallItems = new Hashtable();
            this.bots = new Hashtable(UserData.bots);
            this.baseIDRegister = new Hashtable();


            foreach (UserItem item in UserData.inventory)
            {
                if (item.IsWallItem())
                    wallItems.Add(item.itemID, item);
                else
                    floorItems.Add(item.itemID, item);
                RegisterItem(item);
            }

            this.InventoryPets = new Hashtable(UserData.pets);
            this.mAddedItems = new Hashtable();
            this.mRemovedItems = new ArrayList();
            this.isUpdated = false;
        }

        internal void ClearItems()
        {
            using (IQueryAdapter dbClient = ButterflyEnvironment.DatabaseManager.getQueryreactor())
            {
                dbClient.runFastQuery("DELETE FROM items_users WHERE user_id = " + UserId); //Do join
                //dbClient.runFastQuery("DELETE FROM user_items_songs WHERE user_id = " + UserId);
            }

            mAddedItems.Clear();
            mRemovedItems.Clear();
            floorItems.Clear();
            wallItems.Clear();
            //discs.Clear();
            InventoryPets.Clear();
            bots.Clear();
            isUpdated = true;

            ServerMessage message = PacketHandelingManager.GetRecycledItem(OutputCode.FurniListUpdateEvent);
            GetClient().SendMessage(message);
        }

        internal void SetActiveState(GameClient client)
        {
            this.mClient = client;
            userAttatched = true;
        }

        internal void SetIdleState()
        {
            userAttatched = false;
            this.mClient = null;
        }

        private void RegisterItem(UserItem item)
        {
            uint baseID = item.BaseItemID; //Unique ID for the item type. 
            if (baseIDRegister.ContainsKey(baseID)) // If an item with the same base ID has been added to the baseIDRegister before...
            {
                Hashtable collection = (Hashtable)baseIDRegister[baseID]; //Then get the collection inside the collection
                collection.Add(item.itemID, item); //And add the item
            }
            else
            { // OR ELSE...
                Hashtable collection = new Hashtable(); //Create a bew collection
                collection.Add(item.itemID, item); //Put it in there
                baseIDRegister.Add(baseID, collection); //And put the collection in the collection *yodawg*
            }
        }

        private void UnregisterItem(UserItem item)
        {
            uint baseID = item.BaseItemID;
            if (baseIDRegister.ContainsKey(baseID))
            {
                Hashtable collection = (Hashtable)baseIDRegister[baseID];
                collection.Remove(item.itemID);
            }
        }

        #region PETS
        //internal void ClearPets()
        //{
        //    using (DatabaseClient dbClient = ButterflyEnvironment.GetDatabase().GetClient())
        //        dbClient.runFastQuery("DELETE FROM user_pets WHERE user_id = " + UserId + " AND room_id = 0;");

        //    UpdatePets(true);
        //}

        //internal void UpdatePets(bool FromDatabase)
        //{
        //    if (FromDatabase)
        //    {
        //        LoadInventory();
        //    }

        //    GetClient().SendMessage(SerializePetInventory());
        //}

        internal Pet GetPet(uint Id)
        {
            if (InventoryPets.ContainsKey(Id))
                return InventoryPets[Id] as Pet;
            return null;

            //{
            //    List<Pet>.Enumerator Pets = this.InventoryPets.GetEnumerator();

            //    while (Pets.MoveNext())
            //    {
            //        Pet Pet = Pets.Current;

            //        if (Pet.PetId == Id)
            //        {
            //            return Pet;
            //        }
            //    }
            //}

            //return null;
        }

        internal bool RemovePet(uint PetId)
        {
            isUpdated = false;
            //ServerMessage RemoveMessage = new ServerMessage(OutputCode.PetRemovedFromInventoryEvent);
            //RemoveMessage.AddUInt32(PetId);
            //GetClient().SendMessage(RemoveMessage);

            InventoryPets.Remove(PetId);
            return true;
        }

        internal void MovePetToRoom(UInt32 PetId)
        {
            isUpdated = false;
            RemovePet(PetId);
        }

        internal void AddPet(Pet Pet)
        {
            isUpdated = false;
            if (Pet == null || InventoryPets.ContainsKey(Pet.petID))
                return;

            Pet.isPlaced = false;
            Pet.roomID = 0;

            InventoryPets.Add(Pet.petID, Pet);

            //using (DatabaseClient dbClient = ButterflyEnvironment.GetDatabase().GetClient())
            //{
            //    dbClient.addParameter("botid", Pet.PetId);
            //    dbClient.runFastQuery("UPDATE user_pets SET room_id = 0, x = 0, y = 0, z = 0 WHERE id = @botid LIMIT 1");
            //}

            //ServerMessage AddMessage = new ServerMessage(OutputCode.PetAddedToInventoryEvent);
            //Pet.SerializeInventory(AddMessage);
            //GetClient().SendMessage(AddMessage);
            //UpdatePets(false);
        }

        #endregion

        internal int WallItemCount()
        {
            return wallItems.Count;
        }

        internal int FloorItemCount()
        {
            return floorItems.Count; // + songDisks.Count
        }

        internal void LoadInventory()
        {
            floorItems.Clear();
            wallItems.Clear();

            DataTable Data;

            using (IQueryAdapter dbClient = ButterflyEnvironment.DatabaseManager.getQueryreactor())
            {
                dbClient.setQuery("CALL getuseritems(@userid)");
                dbClient.addParameter("userid", (int)UserId);

                Data = dbClient.getTable();
            }

            uint id;
            ushort baseitem;
            string extradata;
            foreach (DataRow Row in Data.Rows)
            {
                id = (uint)Row[0];
                baseitem = (ushort)Row[1];

                if (!DBNull.Value.Equals(Row[2]))
                    extradata = (string)Row[2];
                else
                    extradata = string.Empty;

                UserItem item = new UserItem(id, baseitem, extradata);

                if (item.IsWallItem())
                    wallItems.Add(id, item);
                else
                    floorItems.Add(id, item);

                RegisterItem(item);
            }

            this.InventoryPets.Clear();
            DataTable Data2;
            using (IQueryAdapter dbClient = ButterflyEnvironment.DatabaseManager.getQueryreactor())
            {
                //dbClient.addParameter("userid", UserId);
                dbClient.setQuery("SELECT * FROM user_pets WHERE user_id = " + UserId + " AND room_id = 0");
                Data2 = dbClient.getTable();
            }

            if (Data2 != null)
            {
                foreach (DataRow Row in Data2.Rows)
                {
                    Pet newPet = Catalog.GeneratePetFromRow(Row);
                    InventoryPets.Add(newPet.petID, newPet);
                }
            }
        }

        internal void UpdateItems(bool FromDatabase)
        {
            if (FromDatabase)
            {
                RunDBUpdate();
                LoadInventory();
            }

            if (mClient == null)
                return;
                
            ServerMessage resp = PacketHandelingManager.GetRecycledItem(OutputCode.FurniListUpdateEvent);
            
            mClient.SendMessage(resp);
        }

        internal UserItem GetItem(uint Id)
        {
            isUpdated = false;
            if (floorItems.ContainsKey(Id))
                return (UserItem)floorItems[Id];
            else if (wallItems.ContainsKey(Id))
                return (UserItem)wallItems[Id];

            return null;
        }

        internal UserItem AddNewItem(UInt32 Id, ushort BaseItem, string ExtraData, bool insert, bool fromRoom, UInt32 songID = 0)
        {
            isUpdated = false;
            if (insert)
            {
                if (fromRoom)
                {
                    using (IQueryAdapter dbClient = ButterflyEnvironment.DatabaseManager.getQueryreactor())
                    {
                        dbClient.runFastQuery("REPLACE INTO items_users VALUES (" + Id + "," + UserId + ")");
                    }

                    Item baseItem = ButterflyEnvironment.Game.GetItemManager().GetItem(BaseItem);
                }
                else
                {
                    using (IQueryAdapter dbClient = ButterflyEnvironment.DatabaseManager.getQueryreactor())
                    {
                        dbClient.setQuery("INSERT INTO items (base_id) VALUES (" + BaseItem + ")");
                        Id = (uint)dbClient.insertQuery();

                        if (!string.IsNullOrEmpty(ExtraData))
                        {
                            dbClient.setQuery("INSERT INTO items_extradata VALUES (" + Id + ",@extradata)");
                            dbClient.addParameter("extradata", ExtraData);
                            dbClient.runQuery();
                        }

                        dbClient.runFastQuery("INSERT INTO items_users VALUES (" + Id + "," + UserId + ")");
                    }
                }
            }
            UserItem ItemToAdd = new UserItem(Id, BaseItem, ExtraData);

            if (UserHoldsItem(Id))
            {
                RemoveItem(Id, false);
            }

            if (ItemToAdd.IsWallItem())
                wallItems.Add(ItemToAdd.itemID, ItemToAdd);
            else
                floorItems.Add(ItemToAdd.itemID, ItemToAdd);

            if (mRemovedItems.Contains(Id))
                mRemovedItems.Remove(Id);

            if (!mAddedItems.ContainsKey(Id))
                mAddedItems.Add(Id, ItemToAdd);

            RegisterItem(ItemToAdd);

            return ItemToAdd;
        }

        private bool UserHoldsItem(uint itemID)
        {
            if (floorItems.ContainsKey(itemID))
                return true;
            if (wallItems.ContainsKey(itemID))
                return true;
            return false;
        }

        internal void RemoveItem(UInt32 Id, bool PlacedInroom)
        {
            isUpdated = false;
            ServerMessage message = PacketHandelingManager.GetRecycledItem(OutputCode.FurniListRemoveEvent);
            message.AddUInt32(Id);
            GetClient().SendMessage(message);

            if (wallItems.ContainsKey(Id))
            {
                UnregisterItem((UserItem)wallItems[Id]);
            }
            else if (floorItems.ContainsKey(Id))
            {
                UnregisterItem((UserItem)floorItems[Id]);
            }

            if (mAddedItems.ContainsKey(Id))
                mAddedItems.Remove(Id);
            if (mRemovedItems.Contains(Id))
                return;

            //discs.Remove(Id);
            floorItems.Remove(Id);
            wallItems.Remove(Id);
            if (PlacedInroom)
                mRemovedItems.Add(Id);
          }

        internal ServerMessage SerializeItemInventory()
        {
            ServerMessage Message = PacketHandelingManager.GetRecycledItem(OutputCode.FurniListEvent);
            //Message.AddString("S");
            //Message.AddInt32(1);
            Message.AddInt32(1);
            Message.AddInt32(0);

            //int itemCount = floorItems.Count; // + discs.Count
            int itemCount = floorItems.Count + wallItems.Count; // + discs.Count
            if (itemCount > MAX_ITEMS_COUNT)
                itemCount = MAX_ITEMS_COUNT;
            int counter = 0;

            Message.AddInt32(itemCount);
            foreach (UserItem item in wallItems.Values)
            {
                item.SerializeWall(Message, true);

                if (counter++ > itemCount)  
                {
                    GetClient().SendNotif(LanguageLocale.GetValue("inventory.load.max_reaced"));
                    return Message;
                }
            }

            foreach (UserItem item in floorItems.Values)
            {
                item.SerializeFloor(Message, true);
                if (counter++ > itemCount)
                {
                    GetClient().SendNotif(LanguageLocale.GetValue("inventory.load.max_reaced"));
                    return Message;
                }
            }

            //foreach (UserItem item in discs.Values)
            //{
            //    item.SerializeFloor(Message, true);
            //    if (counter++ > itemCount)
            //    {
            //        GetClient().SendNotif(LanguageLocale.GetValue("inventory.load.max_reaced"));
            //        return Message;
            //    }
            //}

            return Message;
        }

        internal bool InventoryFull()
        {
            return (floorItems.Count > MAX_ITEMS_COUNT || wallItems.Count > MAX_ITEMS_COUNT);
        }

        internal ServerMessage SerializeWallItemInventory()
        {
            ServerMessage Message = PacketHandelingManager.GetRecycledItem(OutputCode.FurniListEvent);
            Message.AddString("I");
            Message.AddInt32(1);
            Message.AddInt32(1);
            
        //internal ServerMessage SerializeWallItemInventory()
        //{
        //    ServerMessage Message = PacketHandelingManager.GetRecycledItem(OutputCode.FurniListEvent);
        //    Message.AddInt32(1);
        //    Message.AddInt32(1);

            int itemCount = wallItems.Count;
            if (itemCount > MAX_ITEMS_COUNT)
                itemCount = MAX_ITEMS_COUNT;
            int counter = 0;
        //    int itemCount = floorItems.Count;
        //    if (itemCount > MAX_ITEMS_COUNT)
        //        itemCount = MAX_ITEMS_COUNT;
        //    int counter = 0;

            Message.AddInt32(itemCount);
            foreach (UserItem item in wallItems.Values)
            {
                item.SerializeWall(Message, true);
        //    Message.AddInt32(itemCount);

                if (counter++ > itemCount)
                {
                    GetClient().SendNotif(LanguageLocale.GetValue("inventory.load.max_reaced"));
                    return Message;
                }
            }
        //    foreach (UserItem item in floorItems.Values)
        //    {
        //        item.SerializeFloor(Message, true);
        //        if (counter++ > itemCount)
        //        {
        //            GetClient().SendNotif(LanguageLocale.GetValue("inventory.load.max_reaced"));
        //            return Message;
        //        }
        //    }

            return Message;
        }
        //    return Message;
        //}

        internal ServerMessage SerializePetInventory()
        {
            int itemCount = InventoryPets.Count;
            if (itemCount > MAX_ITEMS_COUNT)
                itemCount = MAX_ITEMS_COUNT;
            int counter = 0;

            ServerMessage Message = PacketHandelingManager.GetRecycledItem(OutputCode.PetInventoryEvent);
            Message.AddInt32(1);
            Message.AddInt32(1);
            Message.AddInt32(itemCount);
            lock (InventoryPets)
            {
                foreach (Pet Pet in InventoryPets.Values)
                {
                    Pet.SerializeInventory(Message);

                    if (counter++ > itemCount)
                    {
                        GetClient().SendNotif(LanguageLocale.GetValue("inventory.load.max_reaced"));
                        return Message;
                    }
                }
            }
            return Message;
        }



        internal ServerMessage SerializeBotInventory()
        {
            ServerMessage message = PacketHandelingManager.GetRecycledItem(Outgoing.BotInventory);

            message.AddInt32(bots.Count);

            foreach (RoomBot Bot in bots.Values)
            {
                Bot.SerializeInventory(message);
            }

            return message;
        }




        private GameClient GetClient()
        {
            return ButterflyEnvironment.Game.GetClientManager().GetClientByUserID(UserId);
        }

        internal void AddItemArray(List<RoomItem> RoomItemList)
        {
            foreach (RoomItem Item in RoomItemList)
            {
                AddItem(Item);
            }
        }

        internal void AddItem(RoomItem item)
        {
            AddNewItem(item.Id, (ushort)item.BaseItem, item.ExtraData, true, true, 0);
        }


        private bool isUpdated;
        internal void RunCycleUpdate()
        {
            isUpdated = true;
            RunDBUpdate();
        }

        internal void RunDBUpdate()
        {
            //DateTime Start = DateTime.Now;
            try
            {
                if (mRemovedItems.Count > 0 || mAddedItems.Count > 0 || InventoryPets.Count > 0)
                {

                    QueryChunk queries = new QueryChunk();

                    if (mAddedItems.Count > 0) //This should be checked more carefully
                    {
                        foreach (UserItem Item in mAddedItems.Values)
                        {
                            queries.AddQuery("UPDATE items_users SET user_id = " + UserId + " WHERE item_id = " + Item.itemID);
                            //parameters.Add("extra_data" + Item.Id, Item.ExtraData);
                            //QueryBuilder.Append("UPDATE user_items SET user_id = " + UserId + " , base_item =" + Item.BaseItem + ", extra_data = @extra_data" + Item.Id + " WHERE id = " + Item.Id + "; ");
                        }

                        mAddedItems.Clear();
                    }
                    if (mRemovedItems.Count > 0)
                    {
                        foreach (UInt32 ItemID in mRemovedItems.ToArray())
                        {
                            queries.AddQuery("DELETE FROM items_users WHERE item_id=" + ItemID + " AND user_id=" + UserId);
                        }

                        mRemovedItems.Clear();
                    }
                    lock (InventoryPets)
                    {
                        foreach (Pet pet in InventoryPets.Values)
                        {
                            if (pet.databaseState == DatabaseUpdateState.PendingUpdate)
                            {

                                queries.AddParameter(pet.petID + "name", pet.name);
                                queries.AddParameter(pet.petID + "race", pet.race);
                                queries.AddParameter(pet.petID + "color", pet.color);
                                queries.AddQuery("UPDATE user_pets SET room_id = " + pet.roomID + ", name = @" + pet.petID + "name, race = @" + pet.petID + "race, color = @" + pet.petID + "color, type = " + pet.petType + ", expirience = " + pet.experience + ", " +
                                    "energy = " + pet.energy + ", nutrition = " + pet.nutrition + ", respect = " + pet.respect + ", createstamp = '" + pet.creationStamp + "', x = " + pet.X + ", Y = " + pet.Y + ", Z = " + pet.Z + " WHERE id = " + pet.petID);
                            }
                            pet.databaseState = DatabaseUpdateState.Syncronized;
                        }
                    }

                    using (IQueryAdapter dbClient = ButterflyEnvironment.DatabaseManager.getQueryreactor())
                    {
                        queries.Execute(dbClient);
                    }
                }
            }
            catch (Exception e)
            {
                Logging.LogCacheError("FATAL ERROR DURING USER INVENTORY DB UPDATE: " + e.ToString());
            }
        }

        private bool userAttatched = false;

        internal bool NeedsUpdate
        {
            get
            {
                return (!userAttatched && !isUpdated);
            }
        }

        //internal ServerMessage SerializeMusicDiscs()
        //{
        //    ServerMessage Message = new ServerMessage(OutputCode.UserSongDisksInventoryMessageEvent);
        //    Message.AddInt32(discs.Count);

        //    foreach (UserItem SongDisk in discs.Values)
        //    {
        //        uint SongId = 0;
        //        uint.TryParse(SongDisk.ExtraData, out SongId);

        //        Message.AddUInt32(SongDisk.Id);
        //        Message.AddUInt32(SongId);
        //    }

        //    return Message;
        //}

        internal List<Pet> GetPets()
        {
            List<Pet> toReturn = null;
            lock (InventoryPets)
            {
                toReturn = new List<Pet>(InventoryPets.Values.Count);
                foreach (Pet p in InventoryPets.Values)
                {
                    toReturn.Add(p);
                }
            }
            
            return toReturn;
        }

        public bool isInactive
        {
            get
            {
                return !userAttatched;
            }
        }

        internal void SendFloorInventoryUpdate()
        {
            this.mClient.SendMessage(SerializeItemInventory());
        }

        //internal Hashtable songDisks
        //{
        //    get
        //    {
        //        return this.discs;
        //    }
        //}

        internal void AddBot(RoomBot bot)
        {
            bots.Add(bot.BotId, bot);
            GetClient().SendMessage(SerializeBotInventory());
        }

        internal RoomBot GetBot(uint botID)
        {
            return bots[botID] as RoomBot;
        }

        internal void RemoveBot(uint botID)
        {
            bots.Remove(botID);
        }
    }

    enum PlaceMode
    {
        None,
        Square,
        Circle,
        Alligned,
        FillmodeAll,
        FillmodeEveryOther,
        ABVector
    }
}
