﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Butterfly.Messages;
using Butterfly.HabboHotel.GameClients;
using System.Data;
using Database_Manager.Database.Session_Details.Interfaces;
using Butterfly.Messages.IOCodes;
using Butterfly.Core;

namespace Butterfly.HabboHotel.Users.Messenger
{
    abstract class AbstractChatHandler
    {
        protected HabboMessenger messenger;


        public void setHandler(HabboMessenger messenger)
        {
            this.messenger = messenger;
        }


        public abstract ChatOnlineType getOnlineType();
        public abstract void cleanUp();
        public void Destroy()
        {
            this.messenger = null;
            this.cleanUp();
        }

        #region incomong
        //internal abstract void SendInstantMessage(uint ToId, string Message);
        internal abstract void OnNewFriendRequest(uint friendID, MessengerRequest request);
        internal bool RequestBuddy(string UserQuery)
        {
            uint userID;
            bool hasFQDisabled;

            GameClient client = ButterflyEnvironment.Game.GetClientManager().GetClientByUsername(UserQuery);

            if (client == null)
            {
                DataRow Row = null;
                using (IQueryAdapter dbClient = ButterflyEnvironment.DatabaseManager.getQueryreactor())
                {
                    dbClient.setQuery("SELECT id,block_newfriends FROM users WHERE username = @query");
                    dbClient.addParameter("query", UserQuery.ToLower());
                    Row = dbClient.getRow();
                }
                if (Row == null)
                    return false;
                userID = (uint)Row["id"];
                hasFQDisabled = TextHandling.EnumToBool(Row["block_newfriends"].ToString());
            }
            else
            {
                userID = client.GetHabbo().Id;
                hasFQDisabled = client.GetHabbo().HasFriendRequestsDisabled;
            }

            uint ToId = userID;

            if (messenger.RequestExists(ToId))
            {
                return false;
            }

            if (hasFQDisabled)
            {
                SendNotAcceptFriendRequests();

                return false;
            }

            if (messenger.friends.Count >= HabboMessenger.MAX_FRIENDS)
            {
                this.SendNotificationToUser(LanguageLocale.GetValue("max.friends.reached"));
                return false;
            }

            using (IQueryAdapter dbClient = ButterflyEnvironment.DatabaseManager.getQueryreactor())
            {
                dbClient.runFastQuery("REPLACE INTO messenger_requests (sender,receiver) VALUES (" + messenger.UserId + "," + ToId + ")");
            }

            MessengerRequest Request = new MessengerRequest(ToId, messenger.UserId, ButterflyEnvironment.Game.GetClientManager().GetNameById(messenger.UserId));

            MessengerManager.instance.GetUserMessenger(ToId).OnNewRequest(messenger.UserId, Request);
            messenger.requests.Add(ToId, Request);
            return true;
        }

        internal abstract void SendNotificationToUser(string p);
        internal abstract void SendNotAcceptFriendRequests();
        internal abstract IServerNotificationMessage SerializeFriends();
        internal abstract IServerNotificationMessage SerializeRequests();
        internal IServerNotificationMessage PerformFriendSearch(string query)
        {
            List<SearchResult> results = SearchResultFactory.GetSearchResult(query);

            List<SearchResult> existingFriends = new List<SearchResult>();
            List<SearchResult> othersUsers = new List<SearchResult>();

            foreach (SearchResult result in results)
            {
                if (this.messenger.FriendshipExists(result.userID))
                    existingFriends.Add(result);
                else
                    othersUsers.Add(result);
            }

            return this.SerializeSearchResult(existingFriends, othersUsers);
        }

        internal abstract IServerNotificationMessage SerializeSearchResult(List<SearchResult> existingFriends, List<SearchResult> othersUsers);

        #endregion
        internal abstract void DeliverInstantMessage(string message, uint convoID);
        internal abstract void DeliverInstantMessageError(int ErrorId, UInt32 ConversationId);
        internal abstract void OnRemoveFriend(uint Friend);
        internal abstract void SerializeAndSendUpdate(MessengerBuddy friend);


    }

    class PhoneChatHandler : AbstractChatHandler
    {
        public override ChatOnlineType getOnlineType()
        {
            return ChatOnlineType.PHONE;
        }


        internal override void OnNewFriendRequest(uint friendID, MessengerRequest request)
        {
            throw new NotImplementedException();
        }

        internal override void SendNotAcceptFriendRequests()
        {
            throw new NotImplementedException();
        }

        internal override IServerNotificationMessage SerializeFriends()
        {
            throw new NotImplementedException();
        }

        internal override IServerNotificationMessage SerializeRequests()
        {
            throw new NotImplementedException();
        }

        internal override IServerNotificationMessage SerializeSearchResult(List<SearchResult> existingFriends, List<SearchResult> othersUsers)
        {
            throw new NotImplementedException();
        }

        internal override void DeliverInstantMessage(string message, uint convoID)
        {
            throw new NotImplementedException();
        }

        internal override void DeliverInstantMessageError(int ErrorId, uint ConversationId)
        {
            throw new NotImplementedException();
        }

        internal override void OnRemoveFriend(uint Friend)
        {
            throw new NotImplementedException();
        }

        internal override void SerializeAndSendUpdate(MessengerBuddy friend)
        {
            throw new NotImplementedException();
        }

        public override void cleanUp()
        {
            throw new NotImplementedException();
        }

        internal override void SendNotificationToUser(string p)
        {
            throw new NotImplementedException();
        }
    }

    class HotelChatHandler : AbstractChatHandler
    {
        private GameClient gameClient;

        public HotelChatHandler(GameClient gameClient)
        {
            this.gameClient = gameClient;
        }

        public override ChatOnlineType getOnlineType()
        {
            return ChatOnlineType.HOTEL;
        }

        internal override void OnRemoveFriend(uint Friend)
        {
            ServerMessage msg = PacketHandelingManager.GetRecycledItem(OutputCode.FriendListUpdateEvent);

            msg.AddInt32(0);
            msg.AddInt32(1);
            msg.AddInt32(-1);
            msg.AddUInt32(Friend);

            GetClient().SendMessage(msg);
        }

        internal override IServerNotificationMessage SerializeSearchResult(List<SearchResult> existingFriends, List<SearchResult> othersUsers)
        {
            ServerMessage reply = PacketHandelingManager.GetRecycledItem(OutputCode.HabboSearchResultEvent);

            reply.AddInt32(existingFriends.Count);
            foreach (SearchResult result in existingFriends)
            {
                result.Searialize(reply);
            }

            reply.AddInt32(othersUsers.Count);
            foreach (SearchResult result in othersUsers)
            {
                result.Searialize(reply);
            }

            return reply;
        }
        internal GameClient GetClient()
        {
            return this.gameClient;
        }

        internal override void OnNewFriendRequest(uint friendID, MessengerRequest request)
        {
            ServerMessage NewFriendNotif = PacketHandelingManager.GetRecycledItem(OutputCode.NewBuddyRequestEvent);
            request.Serialize(NewFriendNotif);
            GetClient().SendMessage(NewFriendNotif);
        }

        internal override IServerNotificationMessage SerializeFriends()
        {
            ServerMessage reply = PacketHandelingManager.GetRecycledItem(OutputCode.MessengerInitEvent);
            reply.AddInt32(500);
            reply.AddInt32(300);
            reply.AddInt32(400);
            reply.AddInt32(500);
            reply.AddInt32(0);
            reply.AddInt32(messenger.friends.Count);

            foreach (MessengerBuddy friend in messenger.friends.Values)
            {
                friend.Serialize(reply);
            }

            return reply;
        }

        internal override IServerNotificationMessage SerializeRequests()
        {
            ServerMessage reply = PacketHandelingManager.GetRecycledItem(OutputCode.BuddyRequestsEvent);

            reply.AddInt32(messenger.requests.Count);
            reply.AddInt32(messenger.requests.Count);

            foreach (MessengerRequest request in messenger.requests.Values)
            {
                request.Serialize(reply);
            }

            return reply;
        }

        internal override void DeliverInstantMessage(string message, uint convoID)
        {
            if (GetClient() == null)
                return;
            ServerMessage InstantMessage = PacketHandelingManager.GetRecycledItem(OutputCode.NewConsoleMessageEvent);
            InstantMessage.AddUInt32(convoID);
            InstantMessage.AddString(message);
            InstantMessage.AddInt32(0);
            GetClient().SendMessage(InstantMessage);
        }

        internal override void DeliverInstantMessageError(int ErrorId, uint ConversationId)
        {
            /*
3                =     Your friend is muted and cannot reply.
4                =     Your message was not sent because you are muted.
5                =     Your friend is not online.
6                =     Receiver is not your friend anymore.
7                =     Your friend is busy.
8                =     Your friend is wanking*/
            if (GetClient() == null)
                return;
            ServerMessage reply = PacketHandelingManager.GetRecycledItem(OutputCode.InstantMessageErrorEvent);
            reply.AddInt32(ErrorId);
            reply.AddUInt32(ConversationId);
            reply.AddString("");
            GetClient().SendMessage(reply);
        }

        internal override void SerializeAndSendUpdate(MessengerBuddy friend)
        {
            if (GetClient() == null)
                return;
            ServerMessage reply = PacketHandelingManager.GetRecycledItem(OutputCode.FriendListUpdateEvent);
            reply.AddInt32(0);
            reply.AddInt32(1);
            reply.AddInt32(0);
            
            friend.Serialize(reply);
            reply.AddBoolean(false);

            GetClient().SendMessage(reply);
        }
        internal override void SendNotAcceptFriendRequests()
        {
            if (GetClient() == null)
                return;
            //TODO add locale to send CANNOT have friends
        }

        public override void cleanUp()
        {
            this.gameClient = null;
        }

        internal bool IsInRoom()
        {
            if (GetClient() == null || GetClient().GetHabbo() == null)
                return false;
            return GetClient().GetHabbo().InRoom;
        }

        internal override void SendNotificationToUser(string p)
        {
            if (GetClient() != null)
                GetClient().SendNotif(p);
        }

        internal ServerMessage SerializeRelationships()
        {
            return messenger.SerializeRelationships();
        }
    }


    public enum ChatOnlineType
    {
        PHONE,
        HOTEL,

    }
}
