﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using Butterfly.Collections;
using Butterfly.HabboHotel.GameClients;
using Butterfly.HabboHotel.Rooms;
using Butterfly.Messages;
using Butterfly.HabboHotel.Users.Messenger;
using Database_Manager.Database.Session_Details.Interfaces;
using Butterfly.Messages.IOCodes;
using Butterfly.Core;

namespace Butterfly.HabboHotel.Users.Messenger
{
    class HabboMessenger
    {
        public uint UserId { get; private set; }
        public Dictionary<uint, MessengerRequest> requests;
        public Dictionary<uint, MessengerBuddy> friends;
        internal const int MAX_FRIENDS = 500;
        private List<AbstractChatHandler> listeners = new List<AbstractChatHandler>();
        private Dictionary<uint, MessengerBuddy>[] relationships;

        private bool initialized;

        internal HabboMessenger(uint UserId)
        {
            this.requests = new Dictionary<uint, MessengerRequest>();
            this.friends = new Dictionary<uint, MessengerBuddy>();
            this.relationships = new Dictionary<uint, MessengerBuddy>[4]; // 0 = not in use
            this.UserId = UserId;

            for (int i = 1; i < relationships.Length; i++)
                relationships[i] = new Dictionary<uint, MessengerBuddy>();
        }

        internal void unload()
        {
            if (this.initialized)
            {
                this.requests = new Dictionary<uint, MessengerRequest>();
                this.friends = new Dictionary<uint, MessengerBuddy>();
                initialized = false;
            }
        }

        internal void Init(Dictionary<uint, MessengerBuddy> friends, Dictionary<uint, MessengerRequest> requests)
        {
            if (!this.initialized)
            {
                this.requests = new Dictionary<uint, MessengerRequest>();
                this.friends = new Dictionary<uint, MessengerBuddy>();

                for (int i = 1; i < relationships.Length; i++)
                    relationships[i] = new Dictionary<uint, MessengerBuddy>();

                foreach (KeyValuePair<uint, MessengerBuddy> pair in friends)
                {
                    if (this.friends.Count <= MAX_FRIENDS)
                    {
                        this.friends.Add(pair.Key, pair.Value);
                        if (pair.Value.relationship != RelationshipStatus.None)
                            relationships[(int)pair.Value.relationship].Add(pair.Key, pair.Value);
                    }
                    else break;
                }

                foreach (KeyValuePair<uint, MessengerRequest> pair in requests)
                {
                    if (this.requests.Count <= MAX_FRIENDS)
                    {
                        this.requests.Add(pair.Key, pair.Value);
                    }
                    else break;
                }
                initialized = true;
            }
        }

        internal void ClearRequests()
        {
            requests.Clear();
        }

        internal MessengerRequest GetRequest(uint senderID)
        {
            if (requests.ContainsKey(senderID))
                return requests[senderID];

            return null;
        }

        public bool isInRoom()
        {
            foreach (AbstractChatHandler chat in listeners)
            {
                if (chat is HotelChatHandler)
                {
                    return ((HotelChatHandler)chat).IsInRoom();
                }
            }
            return false;
        }

        public bool isOnline()
        {
            return listeners.Count > 0;
        }

        public void AddOnlineState(AbstractChatHandler handler)
        {
            if (!this.listeners.Contains(handler))
            {
                handler.setHandler(this);
                this.listeners.Add(handler);
            }
        }

        internal void OnStatusChanged(bool notification)
        {
            IEnumerable<GameClient> onlineUsers = ButterflyEnvironment.Game.GetClientManager().GetClientsById(friends.Keys);

            foreach (GameClient client in onlineUsers)
            {
                if (client == null || client.GetHabbo() == null)
                    continue;

                MessengerManager.instance.GetUserMessenger(client.GetHabbo().Id).UpdateFriend(this.UserId, true);
                UpdateFriend(client.GetHabbo().Id, notification);
            }
        }

        internal void UpdateFriend(uint userid, bool notification)
        {
            if (friends.ContainsKey(userid))
            {
                if (notification)
                {
                    foreach (AbstractChatHandler handler in listeners)
                    {
                        handler.SerializeAndSendUpdate((MessengerBuddy)friends[userid]);
                    }
                }
            }
        }

        internal void HandleAllRequests()
        {
            using (IQueryAdapter dbClient = ButterflyEnvironment.DatabaseManager.getQueryreactor())
            {
                dbClient.runFastQuery("DELETE FROM messenger_requests WHERE sender = " + UserId + " OR receiver = " + UserId);
            }

            ClearRequests();
        }

        internal void CreateFriendship(uint friendID)
        {
            using (IQueryAdapter dbClient = ButterflyEnvironment.DatabaseManager.getQueryreactor())
            {
				//if (dbClient.dbType == Database_Manager.Database.DatabaseType.MSSQL)
				//{
				//	dbClient.runFastQuery("DELETE FROM messenger_friendships WHERE sender = " + UserId + " AND receiver = " + friendID);
				//	dbClient.runFastQuery("INSERT INTO messenger_friendships (sender,receiver) VALUES (" + UserId + "," + friendID + ")");
				//}
				//else
                {
                    dbClient.runFastQuery("REPLACE INTO messenger_friendships (sender,receiver) VALUES (" + UserId + "," + friendID + ")");
                }
            }

            OnNewFriendship(friendID);

            MessengerManager.instance.GetUserMessenger(friendID).OnNewFriendship(UserId);
        }

        internal void DestroyFriendship(uint friendID)
        {
            using (IQueryAdapter dbClient = ButterflyEnvironment.DatabaseManager.getQueryreactor())
            {
                dbClient.runFastQuery("DELETE FROM messenger_friendships WHERE (sender = " + UserId + " AND receiver = " + friendID + ") OR (receiver = " + UserId + " AND sender = " + friendID + ")");
            }

            OnDestroyFriendship(friendID);
            MessengerManager.instance.GetUserMessenger(friendID).OnDestroyFriendship(UserId);
        }

        internal void OnNewFriendship(uint friendID)
        {
            if (friends.Count >= MAX_FRIENDS)
                return;

            GameClient friend = ButterflyEnvironment.Game.GetClientManager().GetClientByUserID(friendID);

            MessengerBuddy newFriend;
            if (friend == null || friend.GetHabbo() == null)
            {
                DataRow dRow;
                using (IQueryAdapter dbClient = ButterflyEnvironment.DatabaseManager.getQueryreactor())
                {
                    dbClient.setQuery("SELECT username,motto,look,last_online FROM users WHERE id = " + friendID);
                    dRow = dbClient.getRow();
                }

                newFriend = new MessengerBuddy(friendID, (string)dRow["username"], (string)dRow["look"], (string)dRow["motto"], (string)dRow["last_online"], 0);
            }
            else
            {
                Habbo user = friend.GetHabbo();
                newFriend = new MessengerBuddy(friendID, user.Username, user.Look, user.Motto, string.Empty, 0);
            }

            if (!friends.ContainsKey(friendID) && this.initialized)
                friends.Add(friendID, newFriend);

            foreach (AbstractChatHandler handlers in listeners)
            {
                handlers.SerializeAndSendUpdate(newFriend);
            }
        }

        internal bool RequestExists(uint requestID)
        {
            if (requests.ContainsKey(requestID))
                return true;

            using (IQueryAdapter dbClient = ButterflyEnvironment.DatabaseManager.getQueryreactor())
            {
                dbClient.setQuery("SELECT sender FROM messenger_friendships WHERE sender = @myID AND receiver = @friendID");
                dbClient.addParameter("myID", (int)this.UserId);
                dbClient.addParameter("friendID", (int)requestID);
                return dbClient.findsResult();
            }

        }

        internal bool FriendshipExists(uint friendID)
        {
            return friends.ContainsKey(friendID);
        }

        internal void OnDestroyFriendship(uint Friend)
        {
            for (int i = 1; i < relationships.Length; i++)
                relationships[i].Remove(Friend);
            friends.Remove(Friend);
            foreach (AbstractChatHandler handler in this.listeners)
            {
                handler.OnRemoveFriend(Friend);
            }
        }

        //internal bool RequestBuddy(string UserQuery)
        //{
        //    uint userID;
        //    bool hasFQDisabled;

        //    GameClient client = ButterflyEnvironment.Game.GetClientManager().GetClientByUsername(UserQuery);

        //    if (client == null)
        //    {
        //        DataRow Row = null;
        //        using (IQueryAdapter dbClient = ButterflyEnvironment.DatabaseManager.getQueryreactor())
        //        {
        //            dbClient.setQuery("SELECT id,block_newfriends FROM users WHERE username = @query");
        //            dbClient.addParameter("query", UserQuery.ToLower());
        //            Row = dbClient.getRow();
        //        }

        //        if (Row == null)
        //            return false;

        //        userID = Convert.ToUInt32(Row["id"]);
        //        hasFQDisabled = TextHandling.EnumToBool(Row["block_newfriends"].ToString());
        //    }
        //    else
        //    {
        //        userID = client.GetHabbo().Id;
        //        hasFQDisabled = client.GetHabbo().HasFriendRequestsDisabled;
        //    }

        //    if (hasFQDisabled)
        //    {
        //        //ServerMessage message = new ServerMessage(OutputCode.MessengerErrorEvent);
        //        //message.AddInt32(39);
        //        //message.AddInt32(3);
        //        //GetClient().SendMessage(message);
        //        return false;
        //    }

        //    if (friends.Count >= MAX_FRIENDS)
        //    {
        //        GetClient().SendNotif(LanguageLocale.GetValue("max.friends.reached"));
        //        return false;
        //    }

        //    uint ToId = userID;

        //    if (RequestExists(ToId))
        //    {
        //        return true;
        //    }

        //    using (IQueryAdapter dbClient = ButterflyEnvironment.DatabaseManager.getQueryreactor())
        //    {
        //        if (dbClient.dbType == Database_Manager.Database.DatabaseType.MSSQL)
        //        {
        //            dbClient.runFastQuery("DELETE FROM messenger_requests WHERE sender = " + userID + " AND receiver = " + ToId);
        //            dbClient.runFastQuery("INSERT INTO messenger_requests (sender,receiver) VALUES (" + this.UserId + "," + ToId + ")");
        //        }
        //        else
        //        {
        //            dbClient.runFastQuery("REPLACE INTO messenger_requests (sender,receiver) VALUES (" + this.UserId + "," + ToId + ")");
        //        }
        //    }

        //    GameClient ToUser = ButterflyEnvironment.Game.GetClientManager().GetClientByUserID(ToId);

        //    if (ToUser == null || ToUser.GetHabbo() == null)
        //    {
        //        return true;
        //    }

        //    MessengerRequest Request = new MessengerRequest(ToId, UserId, ButterflyEnvironment.Game.GetClientManager().GetNameById(UserId));

        //    ToUser.GetHabboDataContainer().GetMessenger().OnNewRequest(UserId);

        //    ServerMessage NewFriendNotif = PacketHandelingManager.GetRecycledItem(OutputCode.NewBuddyRequestEvent);
        //    Request.Serialize(NewFriendNotif);
        //    ToUser.SendMessage(NewFriendNotif);
        //    requests.Add(ToId, Request);
        //    return true;
        //}

        internal void SendInstantMessage(UInt32 ToId, string Message)
        {
            if (string.IsNullOrEmpty(Message))
                return;

            //TODO make phone compattible
            if (!FriendshipExists(ToId))
            {
                DeliverInstantMessageError(6, ToId);
                return;
            }

            GameClient Client = ButterflyEnvironment.Game.GetClientManager().GetClientByUserID(ToId);
            GameClient MyClient = ButterflyEnvironment.Game.GetClientManager().GetClientByUserID(this.UserId);
            
            HabboMessenger friendMessenger = MessengerManager.instance.GetUserMessenger(ToId);

            if (Client == null || Client.GetHabboDataContainer().GetMessenger() == null)
            {
                DeliverInstantMessageError(5, ToId);
                return;
            }

            if (MyClient.GetHabbo().Muted)
            {
                DeliverInstantMessageError(4, ToId);
                return;
            }

            if (Client.GetHabbo().Muted)
            {
                DeliverInstantMessageError(3, ToId); // No return, as this is just a warning.
            }

            friendMessenger.DeliverInstantMessage(Message, this.UserId);
            //foreach (AbstractChatHandler handler in listeners)
            //{
            //   handler.DeliverInstantMessage(Message, UserId);
            //}
        }

        internal void DeliverInstantMessage(string message, uint convoID)
        {
            foreach (AbstractChatHandler handler in listeners)
            {
                handler.DeliverInstantMessage(message, convoID);
            }
            //ServerMessage InstantMessage = new ServerMessage(134);
            //InstantMessage.AppendUInt(convoID);
            //InstantMessage.AddString(message);
            //GetClient().SendMessage(InstantMessage);
        }

        internal void DeliverInstantMessageError(int ErrorId, UInt32 ConversationId)
        {

            foreach (AbstractChatHandler handler in listeners)
            {
                handler.DeliverInstantMessageError(ErrorId, ConversationId);
            }
            /*
3                =     Your friend is muted and cannot reply.
4                =     Your message was not sent because you are muted.
5                =     Your friend is not online.
6                =     Receiver is not your friend anymore.
7                =     Your friend is busy.
8                =     Your friend is wanking*/

        }

        internal IEnumerable<RoomData> GetActiveFriendsRooms()
        {
            foreach (MessengerBuddy buddy in friends.Values.Where(p => p.InRoom && p.CurrentRoom != null))
                yield return buddy.CurrentRoom.RoomData;
        }

        internal void OnNewRequest(uint friendID, MessengerRequest request)
        {
            if (requests.Count >= MAX_FRIENDS)
                return;

            if (!requests.ContainsKey(friendID))
                requests.Add(friendID, new MessengerRequest(UserId, friendID, ButterflyEnvironment.Game.GetClientManager().GetNameById(friendID)));
            foreach (AbstractChatHandler handler in listeners)
            {
                handler.OnNewFriendRequest(friendID, request);
            }
        }
        
        internal void LogChatHandlerOut(AbstractChatHandler logout)
        {
            if (this.listeners.Remove(logout) )
            {
                if (this.listeners.Count == 0)
                {
                    OnStatusChanged(true);
                    this.unload();
                }
                logout.cleanUp();
            }
        }

        internal Room GetRoom()
        {
            foreach (AbstractChatHandler chat in listeners)
            {
                if (chat is HotelChatHandler)
                {
                    return ((HotelChatHandler)chat).GetClient().GetHabbo().CurrentRoom;
                }
            }
            return null;
        }

        internal void DeleteFriendRequestfromDatabase(uint sender)
        {
            using (IQueryAdapter dbClient = ButterflyEnvironment.DatabaseManager.getQueryreactor())
            {
                dbClient.runFastQuery("DELETE FROM messenger_requests WHERE (sender = " + UserId + " AND receiver = " + sender + ") OR (receiver = " + UserId + " AND sender = " + sender + ")");
            }
            requests.Remove(sender);
        }

        internal void UpdateRelationshipe(uint receiver, int status, bool dbUpdate = false)
        {
            MessengerBuddy buddy = null;
            if (!friends.TryGetValue(receiver, out buddy) || status < 0 || status > 3)
                return;
            
            if (buddy.relationship != RelationshipStatus.None)
                relationships[(int)buddy.relationship].Remove(receiver);

            buddy.relationship = (RelationshipStatus)status;

            if (dbUpdate)
            {
                using (IQueryAdapter dbClient = ButterflyEnvironment.DatabaseManager.getQueryreactor())
                {
                    dbClient.runFastQuery("UPDATE messenger_friendships SET relationship = " + status + " WHERE (receiver = " + receiver + " AND sender = " + UserId + ") "
                    + "OR (sender = " + receiver + " AND receiver = " + UserId + ")");
                }
            }

            relationships[status].Add(receiver, buddy);
            UpdateFriend(receiver, true);
        }

        internal ServerMessage SerializeRelationships()
        {
            int indexCounter = 0;
            KeyValuePair<uint, MessengerBuddy> relationship;

            for (int i = 1; i < 4; i++)
                if (relationships[i].Count > 0)
                    ++indexCounter;

            ServerMessage message = PacketHandelingManager.GetRecycledItem(Outgoing.ProfileRelationships);
            message.AddUInt32(this.UserId);
            message.AddInt32(indexCounter);

            if (indexCounter == 0) return message;

            for (int i = 1; i < 4; i++)
            {
                if (relationships[i].Count > 0)
                {
                    message.AddInt32(i);
                    message.AddInt32(relationships[i].Count);

                    if (relationships[i].Count > 0)
                    {
                        relationship = relationships[i].ElementAt(TextHandling.GetRandomNumber(0, relationships[i].Count));
                        message.AddUInt32(relationship.Key);
                        message.AddString(relationship.Value.Username);
                    }
                }
            }

            return message;
        }
    }
}