﻿using Butterfly.Messages.ClientMessages;
using System;
using System.Text;

namespace Butterfly.Messages
{
    public class ClientMessage : IDisposable
    {
        #region Fields
        private int position;
        private byte[] body;

        private static Encoding encoding = Encoding.UTF8;
        #endregion

        #region Constructor
        internal ClientMessage(byte[] body, int position, int packetLength)
        {
            this.Init(body, position, packetLength);
        }

        internal void Init(byte[] body, int position, int packetLength)
        {
            this.body = body;
            this.position = position;
        }
        #endregion

        #region Methods
        internal int ReadShort()
        {
            return EncodingUtillity.ToInt16(body, ref position);
        }

        internal string ReadString()
        {
            int stringLength = ReadShort();
            if (stringLength == 0 || position + stringLength > body.Length)
                return string.Empty;

            
            string value = encoding.GetString(body, position, stringLength);


            position += stringLength;

            return value;
        }

        internal int ReadInt()
        {
            return EncodingUtillity.ToInt32(body, ref position);
        }

        internal uint ReadUInt()
        {
            int value = ReadInt();
            return (value < 0 ? 0 : (uint)value);
        }

        internal bool ReadBool()
        {
            return (body[position++] == 1);
        }
        #endregion

        #region Return values
       
        internal int RemainingLength
        {
            get
            {
                return this.body.Length - position;
            }
        }

        //internal bool CanReadBytes(int bytes)
        //{
        //    return (position - bytes >= 0);
        //}

        #endregion

        #region IDisposable members
        public void Dispose()
        {
            ClientMessageFactory.ObjectCallback(this);
            GC.SuppressFinalize(this);
        }
        #endregion
    }
}
