﻿using Butterfly.Core;
using System.Collections;
using System.Collections.Generic;
using System.Diagnostics;

namespace Butterfly.Messages
{
    internal class PacketHandelingManager : IRecycleBin<ServerMessage>
    {
        private int createdPackets;
		private int anonymousPackets;
		private int recycledPackets;

        private SortedList<int, IRecycable<ServerMessage>> currentlyInBin;
		private Stack packetStorage;
        private static PacketHandelingManager ins;

		public static PacketHandelingManager instance
		{
			get
			{
				if (ins == null)
					ins = new PacketHandelingManager();
				return ins;
			}
		}

		private PacketHandelingManager()
		{
			this.createdPackets = 0;
			this.anonymousPackets = 0;
			this.recycledPackets = 0;
			this.packetStorage = new Stack(101);
			this.currentlyInBin = new SortedList<int, IRecycable<ServerMessage>>(1024);
		}

		public static ServerMessage GetRecycledItem(bool persistant)
		{
			if (persistant)
			{
				instance.anonymousPackets++;
				return new ServerMessage(-1);
			}
			else
				return instance.GetRecycledItem();
		}

        public ServerMessage GetRecycledItem()
        {
			lock (packetStorage.SyncRoot)
            {
				if (packetStorage.Count > 0)
                {
					ServerMessage item = packetStorage.Pop() as ServerMessage;
                    if (item != null)
                    {
                        this.currentlyInBin.Remove(item.getID());
                        return item;
                    }
                }
            }
            return CreateNewItem();
        }

		internal void PrepareComposers(int count)
		{
			for (int i = 0; i < count; i++)
			{
				PutInBin(CreateNewItem());
			}
		}

		private ServerMessage CreateNewItem()
        {
            ServerMessage item = new ServerMessage(createdPackets++);
			recycledPackets++;
            item.SetOwner(this);
			
            return item;
        }

        public void PutInBin(IRecycable<ServerMessage> recycledItem)
        {
			lock (packetStorage.SyncRoot)
			{
				if (this.currentlyInBin.ContainsKey(recycledItem.getID()))
				{
					StackTrace trace = new StackTrace();
					Logging.LogCacheError("Packet called serveral times:\n" + (recycledItem as ServerMessage).GetDebugMessage() + " \n" + trace.ToString() + "\n\n");
				}
                else
                { 
				    recycledItem.Clean();
				    packetStorage.Push(recycledItem);
				    currentlyInBin.Add(recycledItem.getID(), recycledItem);
                }
			}
        }

        internal static ServerMessage GetRecycledItem(ushort outputCode, bool keepAlive)
        {
			ServerMessage message = GetRecycledItem(keepAlive); 
            message.Init(outputCode, null);
            return message;
        }

        internal static ServerMessage GetRecycledItem(ushort outputCode)
		{
			return GetRecycledItem(outputCode, false);
		}
    }
}
