﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Xml.Linq;

namespace Butterfly.Messages.PacketShuffeling
{
    class PacketRegister
    {
        private static Dictionary<int, int> outgoing;
        private static Dictionary<int, int> incomming;

        internal static void Initialize()
        {
            outgoing = new Dictionary<int, int>();
            incomming = new Dictionary<int, int>();
        }

        internal static int GetOutgoingID(int ID)
        {
            int returnID;
            if (!outgoing.TryGetValue(ID, out returnID))
                throw new UnknownPacketException("No outgoing packet ID for ID " + ID);
            return returnID;
        }

        internal static int GetIncommingID(int ID)
        {
            int returnID;
            if (!incomming.TryGetValue(ID, out returnID))
                throw new UnknownPacketException("No incomming packet ID for ID " + ID);
            return returnID;
        }

        private static Dictionary<int, int> ParseFile(XDocument source, string elementValue)
        {
            int packetID;
            int shuffledID;

            Dictionary<int, int> colleciton = new Dictionary<int, int>();
            XElement incomming = source.Descendants(elementValue).FirstOrDefault();
            if (incomming == null)
                throw new ArgumentNullException("Nothing has been found in the " + elementValue);

            IEnumerable<XElement> descendants = incomming.Descendants("packet");

            foreach (XElement packetElement in descendants)
            {
                if (!int.TryParse(packetElement.Attribute("id").Value, out packetID))
                    throw new FormatException(string.Format("The packetID {0} is not a valid number", packetElement.Attribute("id").Value));
                if (!int.TryParse(packetElement.Element("shuffledid").Value, out shuffledID))
                    throw new FormatException(string.Format("The shuffled ID {0} is not a valid number", packetElement.Element("shuffledid").Value));
                if (colleciton.ContainsKey(packetID))
                    throw new InvalidOperationException("The packet ID " + packetID + " allready exists in the dictionary");

                colleciton.Add(packetID, shuffledID);
            }

            return colleciton;
        }
    }
}
