﻿using Butterfly.Core;
using Butterfly.Messages.ClientMessages;
using ConnectivityAsync;
using MemoryManagement;
using System;

namespace Butterfly.Messages.Parsing
{
    public class GamePacketParser : IDataParser
    {
        private static readonly int INT_SIZE = sizeof(int);
		private static MemoryContainer memoryContainer = new MemoryContainer(10, 2048);
		public delegate void HandlePacket(ClientMessage message, int packetID);

        private ConnectionInformation con;
        private byte[] bufferedData;
        private int bufferPos;
		private int currentPacketLength;
        public event HandlePacket onNewPacket;

		public GamePacketParser()
		{
			this.bufferPos = 0;
			this.currentPacketLength = -1;
			this.bufferedData = memoryContainer.TakeBuffer();
		}

        public void SetConnection(ConnectionInformation con)
        {
            this.con = con;
            this.onNewPacket = null;
        }

		public void handlePacketData(byte[] data, int bytes)
        {
            int pos = 0, messageID;

			try
			{
				while (pos < bytes && onNewPacket != null)
				{

					if (currentPacketLength == -1)
					{
						if (bytes < INT_SIZE)
						{
							this.bufferCopy(data, bytes); // store the bytes in the buffer for the next read and break operation
							break;
						}
						currentPacketLength = EncodingUtillity.ToInt32(data, ref pos);
					}
					if (currentPacketLength < 2 || currentPacketLength > 4096)
					{
						currentPacketLength = -1;
						break; //broken packet! might be better to disconnect EVERYTHING
					}
					if (currentPacketLength > ((bytes - pos) + bufferPos))
					{
						this.bufferCopy(data, bytes, pos);
						break;
					}

					if (currentPacketLength == ((bytes - pos) + bufferPos)) // if packet is exactly big enough (no more data needed, no excessive data)
					{

						if (bufferPos != 0) // do we have stuff in buffer
						{
							bufferCopy(data, bytes, pos);
							pos = 0;
							messageID = EncodingUtillity.ToInt16(bufferedData, ref pos);

							using (ClientMessage message = ClientMessageFactory.GetClientMessage(this.bufferedData, 2, currentPacketLength))//Not sure
							{
								onNewPacket.Invoke(message, messageID);
							}

						}
						else
						{
							messageID = EncodingUtillity.ToInt16(data, ref pos);
							//pos -= 2;// -2 cuz of toint16
							using (ClientMessage message = ClientMessageFactory.GetClientMessage(data, pos, currentPacketLength))//Not sure
							{
								onNewPacket.Invoke(message, messageID);
							}
						}
						pos = bytes;
						currentPacketLength = -1;
					}
					else //  we have remainder
					{
						int remainder = ((bytes - pos)) - (currentPacketLength - bufferPos);

						if (bufferPos != 0)
						{
							int toCopy = remainder - bufferPos;
							bufferCopy(data, toCopy, pos);
							int zero = 0;
							messageID = EncodingUtillity.ToInt16(bufferedData, ref zero); //small hack to preserve the POS value
							//Create packet with bufferData variable

							using (ClientMessage message = ClientMessageFactory.GetClientMessage(this.bufferedData, 2, currentPacketLength)) //Not sure
							{
								onNewPacket.Invoke(message, messageID);
							}
						}
						else
						{
							messageID = EncodingUtillity.ToInt16(data, ref pos);
							//create packet with data variable
							//pos -= 2;
							using (ClientMessage message = ClientMessageFactory.GetClientMessage(data, pos, currentPacketLength)) //Not sure
							{
								onNewPacket.Invoke(message, messageID);
							}
							pos -= 2; //because else the remainder will fail
						}
						currentPacketLength = -1;
						pos = (bytes - remainder); //set pos to max pos of remainder
					}
				}
			}
			catch (Exception e)
			{
				Logging.HandleException(e, "packet handling");
				con.Destroy();
			}
        }

        private void bufferCopy(byte[] data, int bytes, int offset = 0)
        {
            for (int i = 0; i < (bytes - offset); i++)
            {
                bufferedData[bufferPos++] = data[i + offset];
            }
        }

        private void bufferCopy(byte[] data, int bytes)
        {
            for (int i = 0; i < bytes; i++)
            {
                bufferedData[bufferPos++] = data[i];
            }
        }

        public void Dispose()
        {
            this.onNewPacket = null;
			memoryContainer.GiveBuffer(this.bufferedData);
        }

        public object Clone()
        {
            return new GamePacketParser();
        }

        internal void reset()
        {
            this.bufferPos = 0;
            this.currentPacketLength = -1;
        }
    }
}