using Butterfly.HabboHotel.GameClients;
using Butterfly.HabboHotel.Items;
using Butterfly.HabboHotel.Rooms;
using Butterfly.Messages.IOCodes;
using Database_Manager.Database.Session_Details.Interfaces;

namespace Butterfly.Messages
{
    class ApplyRoomEffectMessageEventHandler : IMessageEventHandler
    {
        public void HandleEvent(GameClient session, ClientMessage request)
        {
            Room Room = ButterflyEnvironment.Game.GetRoomManager().GetRoom(session.GetHabbo().CurrentRoomId);

            if (Room == null || !Room.CheckRights(session, true))
            {
                return;
            }

            UserItem Item = session.GetHabboDataContainer().GetInventoryComponent().GetItem(request.ReadUInt());

            if (Item == null)
            {
                return;
            }

            string type = "floor";

            if (Item.GetBaseItem().Name.ToLower().Contains("wallpaper"))
            {
                type = "wallpaper";
            }
            else if (Item.GetBaseItem().Name.ToLower().Contains("landscape"))
            {
                type = "landscape";
            }

            switch (type)
            {
                case "floor":

                    Room.Floor = Item.extraData;
                    Room.RoomData.Floor = Item.extraData;

                    ButterflyEnvironment.Game.GetQuestManager().ProgressUserQuest(session, HabboHotel.Quests.QuestType.FURNI_DECORATION_FLOOR);
                    break;

                case "wallpaper":

                    Room.Wallpaper = Item.extraData;
                    Room.RoomData.Wallpaper = Item.extraData;

                    ButterflyEnvironment.Game.GetQuestManager().ProgressUserQuest(session, HabboHotel.Quests.QuestType.FURNI_DECORATION_WALL);
                    break;

                case "landscape":

                    Room.Landscape = Item.extraData;
                    Room.RoomData.Landscape = Item.extraData;
                    break;
            }

            using (IQueryAdapter dbClient = ButterflyEnvironment.DatabaseManager.getQueryreactor())
            {
                dbClient.setQuery("UPDATE rooms SET " + type + " = @extradata WHERE id = " + Room.RoomId);
                dbClient.addParameter("extradata", Item.extraData);
                dbClient.runQuery();
            }

            session.GetHabboDataContainer().GetInventoryComponent().RemoveItem(Item.itemID, false);

            ServerMessage Message = PacketHandelingManager.GetRecycledItem(OutputCode.RoomPropertyMessageEvent);
            Message.AddString(type);
            Message.AddString(Item.extraData);
            Room.SendMessage(Message);
        }

    }
}