using Butterfly.HabboHotel.GameClients;
using Butterfly.HabboHotel.RoomBots;
using Butterfly.HabboHotel.Rooms;
using Butterfly.Messages.IOCodes;
using Database_Manager.Database.Session_Details.Interfaces;
using System;

namespace Butterfly.Messages
{
    class GetBotDataMessageEventHandler : IMessageEventHandler
    {
        public void HandleEvent(GameClient session, ClientMessage request)
        {
            uint botID = request.ReadUInt();
            int actionType = request.ReadInt();
            string data = request.ReadString();

            Room currentRoom = session.GetHabbo().CurrentRoom;
            if (currentRoom == null)
                return;

            RoomUser botUser = currentRoom.GetRoomUserManager().GetRoomBot(botID);
            if (botUser == null || botUser.BotData.ownerID != session.GetHabbo().Id)
                return;

            switch (actionType)
            {
                case 1:
                    using (IQueryAdapter dbClient = ButterflyEnvironment.DatabaseManager.getQueryreactor())
                    {
                        dbClient.setQuery("UPDATE bots SET look = @newlook WHERE id = @botid");
                        dbClient.addParameter("newlook", botUser.BotData.Look);
                        dbClient.addParameter("botid", botID);
                        dbClient.runQuery();
                    }

                    botUser.BotData.Look = session.GetHabbo().Look;
                    botUser.UpdateNeeded = true;

                    break;

                case 2:
                    string[] firstdata = data.Split(';');
                    string[] toinendata = firstdata[0].Split(new char[] { '\r', '\n' }, StringSplitOptions.RemoveEmptyEntries);
                    string automaticChat = firstdata[2];
                    string speakingInterval = firstdata[4];
                    if (String.IsNullOrEmpty(speakingInterval) || Convert.ToInt32(speakingInterval) <= 0)
                        speakingInterval = "7";

                    botUser.BotData.RandomSpeech.Clear();

                    using (IQueryAdapter dbClient = ButterflyEnvironment.DatabaseManager.getQueryreactor())
                    {
                        dbClient.runFastQuery("DELETE FROM bots_speech WHERE bot_id = '" + botID + "'");

                        for (int i = 0; i <= toinendata.Length - 1; i++)
                        {
                            dbClient.setQuery("INSERT INTO bots_speech (bot_id, text) VALUES ('" + botID + "', @data)");
                            dbClient.addParameter("data", toinendata[i]);
                            dbClient.runQuery();
                            botUser.BotData.RandomSpeech.Add(new RandomSpeech(toinendata[i], true, botID));
                        }

                        dbClient.setQuery("UPDATE bots SET automatic_chat = @autochat, speaking_interval = @speakinginteval WHERE id = " + botID);
                        dbClient.addParameter("autochat", (automaticChat == "true" ? 1 : 0));
                        dbClient.addParameter("speakinginteval", speakingInterval);
                        dbClient.runQuery();
                    }

                    botUser.BotData.AutomaticChat = automaticChat == "true";
                    botUser.BotData.SpeakingInterval = TextHandling.Parse(speakingInterval);
                    break;

                case 4:

                    if (botUser.DanceId > 0)
                        botUser.DanceId = 0;
                    else
                        botUser.DanceId = TextHandling.GetRandomNumber(1, 4);

                    ServerMessage message = PacketHandelingManager.GetRecycledItem(Outgoing.Dance);
                    message.AddInt32(botUser.VirtualId);
                    message.AddInt32(botUser.DanceId);
                    session.GetHabbo().CurrentRoom.SendMessage(message);

                    break;

                case 5:
                    using (IQueryAdapter dbClient = ButterflyEnvironment.DatabaseManager.getQueryreactor())
                    {
                        dbClient.setQuery("UPDATE bots SET name = @data WHERE id = '" + botID + "'");
                        dbClient.addParameter("data", data);
                        dbClient.runQuery();
                    }

                    botUser.BotData.Name = data;
                    botUser.UpdateNeeded = true;

                    break;

            }

            currentRoom.GetRoomUserManager().RefreshUser(botUser);
        }
    }
}