using Butterfly.Core;
using Butterfly.HabboHotel.GameClients;
using Butterfly.HabboHotel.Rooms;
using Butterfly.Messages.IOCodes;
using System;
using System.Collections;
using System.Collections.Generic;

namespace Butterfly.Messages
{
    class OnRoomUserAddMessageEventHandler : IMessageEventHandler
    {
        public void HandleEvent(GameClient session, ClientMessage request)
		{
			if (session == null || session.GetHabbo() == null)
				return;

			Room CurrentLoadingRoom = ButterflyEnvironment.Game.GetRoomManager().GetRoom(session.GetHabbo().LoadingRoom);
			if (CurrentLoadingRoom == null)
				return;

			List<RoomUser> UsersToDisplay = new List<RoomUser>(CurrentLoadingRoom.GetRoomUserManager().UserList.Values.Count);

			foreach (RoomUser User in CurrentLoadingRoom.GetRoomUserManager().UserList.Values)
			{
				if (User.IsSpectator || !CurrentLoadingRoom.GetRoomUserManager().isValid(User))
					continue;

				UsersToDisplay.Add(User);
			}

			ServerMessage message = PacketHandelingManager.GetRecycledItem(OutputCode.UsersMessageEvent);
			message.AddInt32(UsersToDisplay.Count);

			foreach (RoomUser User in UsersToDisplay)
			{
				User.Serialize(message);
			}

			session.SendMessage(message);
          
			message = PacketHandelingManager.GetRecycledItem(OutputCode.RoomVisualizationSettingsEvent);
			message.AddBoolean(CurrentLoadingRoom.Hidewall);
			message.AddInt32(CurrentLoadingRoom.WallThickness);
			message.AddInt32(CurrentLoadingRoom.FloorThickness);
			session.SendMessage(message);


			message = PacketHandelingManager.GetRecycledItem(OutputCode.RoomEntryInfoMessageEvent);
			message.AddBoolean(true);
			message.AddInt32((int)CurrentLoadingRoom.RoomId);

			if (CurrentLoadingRoom.CheckRights(session, true))
			{
				message.AddBoolean(true);
			}
			else
			{
				message.AddBoolean(false);
			}

			session.SendMessage(message);

			ServerMessage Message = PacketHandelingManager.GetRecycledItem(OutputCode.GetGuestRoomResultEvent);
            CurrentLoadingRoom.RoomData.SerializeGuestRoomResults(Message, true);
            Message.AddBoolean(true);
			session.SendMessage(Message);

			if (CurrentLoadingRoom.UsersWithRights.Count > 0 && CurrentLoadingRoom.CheckRights(session, true))
			{
				ServerMessage rightsMessage = PacketHandelingManager.GetRecycledItem(OutputCode.GetPowerList);
				rightsMessage.AddInt32((int)CurrentLoadingRoom.RoomData.Id);
				rightsMessage.AddInt32(CurrentLoadingRoom.UsersWithRights.Count);

				foreach (DictionaryEntry pair in CurrentLoadingRoom.UsersWithRights)
				{
					rightsMessage.AddUInt32((uint)pair.Key);
					rightsMessage.AddString((string)pair.Value);
				}

				session.SendMessage(rightsMessage);

			}

            ServerMessage statusUpdates = CurrentLoadingRoom.GetRoomUserManager().SerializeStatusUpdates(true);
            if (statusUpdates != null)
                session.SendMessage(statusUpdates);

			foreach (RoomUser User in CurrentLoadingRoom.GetRoomUserManager().UserList.Values)
			{
				if (User.IsSpectator)
					continue;

				if (User.IsDancing)
				{
					ServerMessage message2 = PacketHandelingManager.GetRecycledItem(OutputCode.DanceMessageEvent);
					message2.AddInt32(User.VirtualId);
					message2.AddInt32(User.DanceId);
					session.SendMessage(message2);
				}

				if (User.IsAsleep)
				{
					ServerMessage message3 = PacketHandelingManager.GetRecycledItem(OutputCode.SleepMessageEvent);
					message3.AddInt32(User.VirtualId);
					message3.AddBoolean(true);
					session.SendMessage(message3);
				}

				if (User.CarryItemID > 0 && User.CarryTimer > 0)
				{
					ServerMessage message4 = PacketHandelingManager.GetRecycledItem(OutputCode.CarryObjectMessageEvent);
					message4.AddInt32(User.VirtualId);
					message4.AddInt32(User.CarryTimer);
					session.SendMessage(message4);
				}

				if (!User.IsBot)
				{
					try
					{
						if (User.GetClient() != null && User.GetClient().GetHabbo() != null && User.GetClient().GetHabboDataContainer().GetAvatarEffectsInventoryComponent() != null && User.GetClient().GetHabboDataContainer().GetAvatarEffectsInventoryComponent().CurrentEffect >= 1)
						{
							message = PacketHandelingManager.GetRecycledItem(OutputCode.AvatarEffectMessageEvent);
							message.AddInt32(User.VirtualId);
							message.AddInt32(User.GetClient().GetHabboDataContainer().GetAvatarEffectsInventoryComponent().CurrentEffect);
							message.AddInt32(0);
							session.SendMessage(message);
						}
					}
					catch (Exception e) { Logging.HandleException(e, "Rooms.SendRoomData3"); }
				}
			}

			CurrentLoadingRoom = null;
		}

    }
}