using Butterfly.Core;
using Butterfly.HabboHotel.GameClients;
using Butterfly.HabboHotel.Items;
using Butterfly.HabboHotel.Rooms;
using System;
using System.Collections;

namespace Butterfly.Messages
{
    class PlaceItemMessageEventHandler : IMessageEventHandler
    {
        public void HandleEvent(GameClient session, ClientMessage request)
        {
            Room Room = ButterflyEnvironment.Game.GetRoomManager().GetRoom(session.GetHabbo().CurrentRoomId);

            if (Room == null || !Room.CheckRights(session))
                return;

            string PlacementData = request.ReadString();
            string[] dataBits = PlacementData.Split(' ');
            uint ItemId = uint.Parse(dataBits[0].Replace("-", ""));

            UserItem Item = session.GetHabboDataContainer().GetInventoryComponent().GetItem(ItemId);

            if (Item == null)
            {
                return;
            }

            if (Item.IsWallItem())
                Room.HandleWallItemPlace(Item, PlacementData, session);
            else
            {
                if (Room.GetRoomItemHandler().mFloorItems.Inner.Count >= RoomItem.MAX_ITEM_ALLOWED)
                {
                    session.SendNotif(LanguageLocale.GetValue("max.rooms.items.reached"));
                    return;
                }

                int X = int.Parse(dataBits[1]);
                int Y = int.Parse(dataBits[2]);
                int Rot = int.Parse(dataBits[3]);

                uint amount = session.GetHabboDataContainer().GetInventoryComponent().placeAmount;

                Hashtable registry = session.GetHabboDataContainer().GetInventoryComponent().baseIDRegister;

                for (int i = 0; i < amount; i++)
                {
                    switch (session.GetHabboDataContainer().GetInventoryComponent().mode)
                    {
                        case HabboHotel.Users.Inventory.PlaceMode.None:
                            {
                                Room.HandleFloorItemPlace(Item, session, X, Y, Rot);

                                if (amount > 1)
                                {
                                    Item = Gamemap.GetNextItemInStack(Item, registry);
                                    if (Item == null)
                                        return;
                                }
                                break;
                            }
                        case HabboHotel.Users.Inventory.PlaceMode.Circle:
                            {
                                double radius = session.GetHabboDataContainer().GetInventoryComponent().primaryPlaceParameter;

                                int startX = (int)(X - radius);
                                int startY = (int)(Y - radius);
                                double powRadius = Math.Pow(radius, 2);
                                radius -= 0.5;

                                for (int y = startY; y < startY + (2 * radius) + 8; y++)
                                {
                                    for (int x = startX; x < startX + (2 * radius) + 8; x++)
                                    {
                                        if (x < 0 || y < 0) //|| x > maxX || y > maxY)
                                            continue;

                                        //(x-a)^2+(y-b)^2=r^2, centrum is (a,b)
                                        if (Math.Pow(x - X, 2) + Math.Pow(y - Y, 2) < powRadius)
                                            Room.HandleFloorItemPlace(Item, session, x, y, Rot);

                                        Item = Gamemap.GetNextItemInStack(Item, registry);
                                        if (Item == null)
                                            return;
                                    }
                                }
                                break;
                            }
                    }
                }
            }
        }
    }
}