using Butterfly.Core;
using Butterfly.HabboHotel.GameClients;
using Butterfly.HabboHotel.Navigators;
using Butterfly.HabboHotel.Rooms;
using Butterfly.Messages.IOCodes;
using Database_Manager.Database.Session_Details.Interfaces;
using System.Collections.Generic;
using System.Text;

namespace Butterfly.Messages
{
    class SaveRoomDataMessageEventHandler : IMessageEventHandler
    {
        public void HandleEvent(GameClient session, ClientMessage request)
        {
            Room Room = ButterflyEnvironment.Game.GetRoomManager().GetRoom(session.GetHabbo().CurrentRoomId);

            if (Room == null || !Room.CheckRights(session, true))
            {
                return;
            }

            int Id = request.ReadInt();
            string Name = TextHandling.FilterInjectionChars(request.ReadString());
            string Description = TextHandling.FilterInjectionChars(request.ReadString());
            int State = request.ReadInt();
            string Password = TextHandling.FilterInjectionChars(request.ReadString());
            int MaxUsers = request.ReadInt();
            int CategoryId = request.ReadInt();
            int TagCount = request.ReadInt();

            List<string> Tags = new List<string>();
            StringBuilder formattedTags = new StringBuilder();

            for (int i = 0; i < TagCount; i++)
            {
                if (i > 0)
                {
                    formattedTags.Append(",");
                }

                string tag = TextHandling.FilterInjectionChars(request.ReadString().ToLower());

                Tags.Add(tag);
                formattedTags.Append(tag);
            }


            bool AllowPets = request.ReadBool();
            bool AllowPetsEat = request.ReadBool();
            bool AllowWalkthrough = request.ReadBool();
           
            request.ReadInt(); // ????
            bool Hidewall = request.ReadBool();
            int WallThickness = request.ReadInt();
            int FloorThickness = request.ReadInt();
            

            if (WallThickness < -2 || WallThickness > 1)
            {
                WallThickness = 0;
            }

            if (FloorThickness < -2 || FloorThickness > 1)
            {
                FloorThickness = 0;
            }

            if (Name.Length < 1)
            {
                return;
            }

            if (State < 0 || State > 2)
            {
                return;
            }

            if (State == 2 && string.IsNullOrEmpty(Password) && string.IsNullOrEmpty(Room.Password))
                return;
            else if (State == 2 && string.IsNullOrEmpty(Password) && !string.IsNullOrEmpty(Room.Password))
                Password = Room.Password;

            if (MaxUsers <= 2 || MaxUsers >= 510)
                return;

            FlatCat FlatCat = ButterflyEnvironment.Game.GetNavigator().GetFlatCat(CategoryId);

            if (FlatCat == null)
            {
                return;
            }

            if (FlatCat.MinRank > session.GetHabbo().Rank)
            {
                session.SendNotif(LanguageLocale.GetValue("user.roomdata.rightserror"));
                CategoryId = 0;
            }

            if (TagCount > 2)
            {
                return;
            }

            Room.AllowPets = AllowPets;
            Room.AllowPetsEating = AllowPetsEat;
            Room.AllowWalkthrough = AllowWalkthrough;
            Room.Hidewall = Hidewall;

            Room.Name = Name;
            Room.State = State;
            Room.Description = Description;
            Room.Category = CategoryId;
            Room.Password = Password;

            Room.RoomData.Name = Name;
            Room.RoomData.State = State;
            Room.RoomData.Description = Description;
            Room.RoomData.Category = CategoryId;
            Room.RoomData.Password = Password;

            Room.ClearTags();
            Room.AddTagRange(Tags);
            Room.UsersMax = MaxUsers;

            Room.RoomData.Tags.Clear();
            Room.RoomData.Tags.AddRange(Tags);
            Room.RoomData.UsersMax = MaxUsers;
            Room.RoomData.WallThickness = WallThickness;
            Room.RoomData.FloorThickness = FloorThickness;
            Room.RoomData.Hidewall = Hidewall;
            Room.WallThickness = WallThickness;
            Room.FloorThickness = FloorThickness;

            string formattedState = "open";

            if (Room.State == 1)
                formattedState = "locked";
            else if (Room.State > 1)
                formattedState = "password";

            using (IQueryAdapter dbClient = ButterflyEnvironment.DatabaseManager.getQueryreactor())
            {
                dbClient.setQuery("UPDATE rooms SET caption = @caption, description = @description, password = @password, category = " + CategoryId + ", state = '" + formattedState + "', tags = @tags, users_max = " + MaxUsers + ", allow_pets = " + TextHandling.BooleanToInt(AllowPets) + ", allow_pets_eat = " + TextHandling.BooleanToInt(AllowPetsEat) + ", allow_walkthrough = " + TextHandling.BooleanToInt(AllowWalkthrough) + ", allow_hidewall = " + TextHandling.BooleanToInt(Hidewall) + ", wallthickness = " + WallThickness + ", floorthickness = " + FloorThickness + " WHERE id = " + Room.RoomId);
                dbClient.addParameter("caption", Room.Name);
                dbClient.addParameter("description", Room.Description);
                dbClient.addParameter("password", Room.Password);
                dbClient.addParameter("tags", formattedTags.ToString());
                dbClient.runQuery();

				session.GetHabbo().UpdateRooms(dbClient);
            }


            ServerMessage response = PacketHandelingManager.GetRecycledItem(OutputCode.RoomSettingsSavedEvent);
            response.AddUInt32(Room.RoomId);
            session.SendMessage(response);

            //response = new ServerMessage(OutputCode.RoomInfoUpdatedEvent);
            //response.AddUInt32(Room.RoomId);
            //Session.SendMessage(response);


            response = PacketHandelingManager.GetRecycledItem(OutputCode.RoomVisualizationSettingsEvent);
            response.AddBoolean(Room.Hidewall);
            response.AddInt32(Room.WallThickness);
            response.AddInt32(Room.FloorThickness);

            Room.SendMessage(response);

            RoomData Data = Room.RoomData;

            response = PacketHandelingManager.GetRecycledItem(OutputCode.GetGuestRoomResultEvent);
            Data.SerializeGuestRoomResults(response);

            session.SendMessage(response);
        }

    }
}