using Butterfly.HabboHotel.GameClients;
using Butterfly.HabboHotel.Items;
using Butterfly.HabboHotel.Rooms;
using Butterfly.HabboHotel.Rooms.Wired;

namespace Butterfly.Messages
{
    class TriggerItemMessageEventHandler : IMessageEventHandler
    {
        public void HandleEvent(GameClient session, ClientMessage request)
        {
            Room Room = ButterflyEnvironment.Game.GetRoomManager().GetRoom(session.GetHabbo().CurrentRoomId);

            if (Room == null)
            {
                return;
            }

            uint itemID = request.ReadUInt();
            RoomItem Item = Room.GetRoomItemHandler().GetItem(itemID);

            if (Item == null)
            {
                return;
            }

            bool hasRights = false;

            if (Room.CheckRights(session))
            {
                hasRights = true;
            }

            string oldData = Item.ExtraData;
            int req = request.ReadInt();
            if (!Item.CanBeTriggered(session, req, hasRights))
            {
                return;
            }
            Item.Interactor.OnTrigger(session, Item, req, hasRights);
            Item.OnTrigger(Room.GetRoomUserManager().GetRoomUserByHabbo(session.GetHabbo().Id));

            if (Room.GotWired() && !WiredUtillity.TypeIsWired(Item.GetBaseItem().InteractionType))
            {
                bool shouldBeHandled = false;
                int x = Item.GetX;
                int y = Item.GetY;
                Point up = new Point(x, y + 1);
                Point down = new Point(x + 1, y);
                Point left = new Point(x, y - 1);
                Point right = new Point(x - 1, y);

                foreach (RoomItem item in Room.GetGameMap().GetCoordinatedItems(up))
                {
                    if (WiredHandler.TypeIsWire(item.GetBaseItem().InteractionType))
                        shouldBeHandled = true;
                }

                foreach (RoomItem item in Room.GetGameMap().GetCoordinatedItems(down))
                {
                    if (WiredHandler.TypeIsWire(item.GetBaseItem().InteractionType))
                        shouldBeHandled = true;
                }

                foreach (RoomItem item in Room.GetGameMap().GetCoordinatedItems(left))
                {
                    if (WiredHandler.TypeIsWire(item.GetBaseItem().InteractionType))
                        shouldBeHandled = true;
                }

                foreach (RoomItem item in Room.GetGameMap().GetCoordinatedItems(right))
                {
                    if (WiredHandler.TypeIsWire(item.GetBaseItem().InteractionType))
                        shouldBeHandled = true;
                }

                if (shouldBeHandled)
                    Room.GetWiredHandler().TriggerOnWire(Item.GetX, Item.GetY);
                else
                    Room.GetWiredHandler().RemoveWiredItem(Item.Coordinate);
            }
        }

    }
}