using Butterfly.Core;
using Butterfly.HabboHotel.GameClients;
using Butterfly.HabboHotel.Rooms;
using Butterfly.Messages.IOCodes;
using System;
using System.Collections.Generic;

namespace Butterfly.Messages
{
    class WhisperMessageEventHandler : IMessageEventHandler
    {
        public void HandleEvent(GameClient session, ClientMessage request)
        {

            if (session == null || session.GetHabbo() == null)
                return;

            Room Room = session.GetHabbo().CurrentRoom;
            
            if (Room == null)
            {
                return;
            }

            if (session.GetHabbo().Muted)
            {
                session.SendNotif(LanguageLocale.GetValue("user.ismuted"));
                return;
            }

            if (Room.RoomMuted)
                return;

            string Params = TextHandling.FilterInjectionChars(request.ReadString());
            string ToUser = Params.Split(' ')[0];

            if (ToUser.Length + 1 > Params.Length)
                return;

            string Message = Params.Substring(ToUser.Length + 1);
            Message = LanguageLocale.FilterSwearwords(Message);
            RoomUser User = Room.GetRoomUserManager().GetRoomUserByHabbo(session.GetHabbo().Id);
            RoomUser User2 = Room.GetRoomUserManager().GetRoomUserByHabbo(ToUser);

            if (User2 == null || User == null)
                return;

            ServerMessage TellMsg = PacketHandelingManager.GetRecycledItem(OutputCode.WhisperMessageEvent);
            TellMsg.AddInt32(User.VirtualId);
            TellMsg.AddString(Message);
            TellMsg.AddInt32(0);
            TellMsg.AddInt32(0);
            TellMsg.AddInt32(0);
            TellMsg.AddInt32(-1);

            if (User != null && !User.IsBot)
            {
				User.GetClient().SendMessageWithoutReturning(TellMsg);
            }

            User.Unidle();

            if (User2 != null && !User2.IsBot && !User2.HasIgnored(User.HabboId))
            {
				User2.GetClient().SendMessageWithoutReturning(TellMsg);
            }

			TellMsg.ReturnToOwner();

            TimeSpan SinceLastMessage = DateTime.Now - session.FloodTime;
            if (SinceLastMessage.Seconds > 4)
                session.FloodCount = 0;

            if (SinceLastMessage.Seconds < 4 && session.FloodCount > 5 && session.GetHabbo().Rank < 5)
            {
                ServerMessage Packet = PacketHandelingManager.GetRecycledItem(OutputCode.FloodControlMessageEvent);
                Packet.AddInt32(30); //Blocked for 30sec
                User.GetClient().SendMessage(Packet);
                return;
            }
            session.FloodTime = DateTime.Now;
            session.FloodCount++;


            List<RoomUser> ToNotify = Room.GetRoomUserManager().GetRoomUserByRank(4);
            if (ToNotify.Count > 0)
            {
                ServerMessage NotifyMessage = PacketHandelingManager.GetRecycledItem(OutputCode.WhisperMessageEvent);
                NotifyMessage.AddInt32(User.VirtualId);
                NotifyMessage.AddString(LanguageLocale.GetValue("moderation.whisper") + ToUser + ": " + Message);
                NotifyMessage.AddInt32(0);
                NotifyMessage.AddInt32(0);
                NotifyMessage.AddInt32(0);
                NotifyMessage.AddInt32(-1);

                foreach (RoomUser user in ToNotify)
                {
                    if (user != null && user.HabboId != User2.HabboId && user.HabboId != User.HabboId && user.GetClient() != null)
                    {
                        user.GetClient().SendMessageWithoutReturning(NotifyMessage);
                    }
                }
                NotifyMessage.ReturnToOwner();
            }
        }

    }
}