using Butterfly.HabboHotel.GameClients;
using Butterfly.HabboHotel.Misc;
using Butterfly.HabboHotel.Rooms;
using Butterfly.Messages.IOCodes;
using Database_Manager.Database.Session_Details.Interfaces;

namespace Butterfly.Messages
{
    class ChangeLookMessageEventHandler : IMessageEventHandler
    {
        public void HandleEvent(GameClient session, ClientMessage request)
		{
			if (session.GetHabbo().MutantPenalty)
			{
				session.SendNotif("Because of a penalty or restriction on your account, you are not allowed to change your look.");
				return;
			}

			string gender = request.ReadString().ToUpper();
			string look = TextHandling.FilterInjectionChars(request.ReadString());

			if (!AntiMutant.ValidateLook(look, gender))
				return;

			session.GetHabbo().Look = TextHandling.FilterFigure(look);
			session.GetHabbo().Gender = gender.ToLower();

			using (IQueryAdapter dbClient = ButterflyEnvironment.DatabaseManager.getQueryreactor())
			{
				dbClient.setQuery("UPDATE users SET look = @look, gender = @gender WHERE id = " + session.GetHabbo().Id);
				dbClient.addParameter("look", look);
				dbClient.addParameter("gender", gender);
				dbClient.runQuery();
			}

			ButterflyEnvironment.Game.GetQuestManager().ProgressUserQuest(session, HabboHotel.Quests.QuestType.PROFILE_CHANGE_LOOK);
			ButterflyEnvironment.Game.GetAchievementManager().ProgressUserAchievement(session, "ACH_AvatarLooks", 1);

			ServerMessage message = PacketHandelingManager.GetRecycledItem(OutputCode.UserChangeMessageEvent);
			message.AddInt32(-1);
			message.AddString(session.GetHabbo().Look);
			message.AddString(session.GetHabbo().Gender.ToLower());
			message.AddString(session.GetHabbo().Motto);
			message.AddInt32(session.GetHabbo().AchievementPoints);
			session.SendMessage(message);

			if (session.GetHabbo().InRoom)
			{
				Room room = session.GetHabbo().CurrentRoom;

				if (room == null)
					return;

				RoomUser user = room.GetRoomUserManager().GetRoomUserByHabbo(session.GetHabbo().Id);

				if (user == null)
					return;

				ServerMessage roomMessage = PacketHandelingManager.GetRecycledItem(OutputCode.UserChangeMessageEvent);
				roomMessage.AddInt32(user.VirtualId);
				roomMessage.AddString(session.GetHabbo().Look);
				roomMessage.AddString(session.GetHabbo().Gender.ToLower());
				roomMessage.AddString(session.GetHabbo().Motto);
				roomMessage.AddInt32(session.GetHabbo().AchievementPoints);
				room.SendMessage(roomMessage);
			}
		}

    }
}