﻿using Butterfly.Core;
using Butterfly.Messages.IOCodes;
using System;
using System.Collections;

namespace Butterfly.Messages.StaticMessageHandlers
{
    class StaticClientMessageHandler
    {
        private static IMessageEventHandler[] handlers;

		private const int MAX_PACKET_HEADER = 0xFFF;

        internal static void Initialize()
        {
            handlers = new IMessageEventHandler[MAX_PACKET_HEADER];
            RegisterPacketLibary();
        }

        internal static IMessageEventHandler GetEventHandler(int messageID)
        {
			if (messageID < 0 || messageID >= MAX_PACKET_HEADER)
				return null;

			if (handlers[messageID] != null)
            {
                if (ButterflyEnvironment.displayPacketActivity)
                    Logging.WriteLine(string.Format("Handling ID -> {0}", messageID));

                return handlers[messageID];
            }
            else 
            {
                if (ButterflyEnvironment.displayPacketActivity)
                    Logging.WriteLine(string.Format("Unknown ID -> {0}", messageID));
            }

            return null;
        }

        #region Register
        internal static void RegisterPacketLibary()
        {
            handlers[InputCode.InitCryptographyServiceComposer] = new InitCryptoMessageEventHandler();
            handlers[InputCode.InitCryptographyKeyComposer] = new InitSecretKeyMessageEventHandler();
            handlers[InputCode.ReadMachineIDComposer] = new SetMachineIDMessageEventHandler();
            handlers[InputCode.GetCatalogIndexComposer] = new GetCatalogIndexMessageEventHandler();
            handlers[InputCode.GetCatalogPageComposer] = new GetCatalogPageMessageEventHandler();
            handlers[InputCode.RedeemVoucherMessageComposer] = new RedeemVoucherMessageEventHandler();
            handlers[InputCode.PurchaseFromCatalogComposer] = new HandlePurchaseMessageEventHandler();
            handlers[InputCode.PurchaseFromCatalogAsGiftComposer] = new PurchaseGiftMessageEventHandler();
            //handlers[InputCode.GetRecyclerPrizesMessageComposer] = new GetRecyclerRewardsMessageEventHandler();  NS
            //handlers[InputCode.GetIsOfferGiftableComposer] = new CanGiftMessageEventHandler(); //*
            handlers[InputCode.GetMarketplaceConfigurationMessageComposer] = new GetCataData1MessageEventHandler();
            handlers[InputCode.GetGiftWrappingConfigurationComposer] = new GetCataData2MessageEventHandler();
            handlers[InputCode.GetMarketplaceCanMakeOfferComposer] = new MarketplaceCanSellMessageEventHandler(); //
            handlers[InputCode.MakeOfferMessageComposer] = new MarketplacePostItemMessageEventHandler(); //
            handlers[InputCode.GetOwnOffersMessageComposer] = new MarketplaceGetOwnOffersMessageEventHandler(); //
            handlers[InputCode.CancelOfferMessageComposer] = new MarketplaceTakeBackMessageEventHandler(); //
            handlers[InputCode.RedeemOfferCreditsMessageComposer] = new MarketplaceClaimCreditsMessageEventHandler(); //
            handlers[InputCode.GetOffersMessageComposer] = new MarketplaceGetOffersMessageEventHandler(); //
            handlers[InputCode.BuyOfferMessageComposer] = new MarketplacePurchaseMessageEventHandler(); //
            handlers[InputCode.ApproveNameMessageComposer] = new CheckPetNameMessageEventHandler();
            handlers[InputCode.GetSellablePetBreedsComposer] = new PetRacesMessageEventHandler();
            handlers[InputCode.PongMessageComposer] = new PongMessageEventHandler();
            //handlers[InputCode.GetHabboGroupDetailsMessageComposer] = new GetGroupdetailsMessageEventHandler(); //NF
            //handlers[InputCode.InitCryptoMessageComposer] = new SendSessionParamsMessageEventHandler();
            handlers[InputCode.SSOTicketMessageComposer] = new SSOLoginMessageEventHandler();
            handlers[InputCode.GetClientFaqsMessageComposer] = new InitHelpToolMessageEventHandler();
            //handlers[InputCode.GetFaqCategoriesMessageComposer] = new GetHelpCategoriesMessageEventHandler();
            //handlers[InputCode.GetFaqTextMessageComposer] = new ViewHelpTopicMessageEventHandler();
            //handlers[InputCode.SearchFaqsMessageComposer] = new SearchHelpTopicsMessageEventHandler();
            //handlers[InputCode.GetFaqCategoryMessageComposer] = new GetTopicsInCategoryMessageEventHandler();
            handlers[InputCode.CallForHelpMessageComposer] = new SubmitHelpTicketMessageEventHandler();
            handlers[Incoming.CallForHelp] = new SubmitHelpTicketMessageEventHandler();
            handlers[Incoming.SubmitHelpTicket] = new SubmitHelpTicketMessageEventHandler();
            //handlers[InputCode.DeletePendingCallsForHelpMessageComposer] = new DeletePendingCFHMessageEventHandler();
            //handlers[InputCode.CallGuideBotMessageComposer] = new CallGuideBotMessageEventHandler();
            handlers[InputCode.ModeratorActionMessageComposer] = new ModSendRoomAlertMessageEventHandler();
            handlers[InputCode.PickIssuesMessageComposer] = new ModPickTicketMessageEventHandler();
            handlers[InputCode.ReleaseIssuesMessageComposer] = new ModReleaseTicketMessageEventHandler();
            handlers[InputCode.CloseIssuesMessageComposer] = new ModCloseTicketMessageEventHandler();
            handlers[InputCode.GetModeratorUserInfoMessageComposer] = new ModGetUserInfoMessageEventHandler();
            handlers[InputCode.GetUserChatlogMessageComposer] = new ModGetUserChatlogMessageEventHandler();
            handlers[InputCode.GetRoomChatlogMessageComposer] = new ModGetRoomChatlogMessageEventHandler();
            handlers[InputCode.GetCfhChatlogMessageComposer] = new ModGetTicketChatlogMessageEventHandler();
            handlers[InputCode.GetRoomVisitsMessageComposer] = new ModGetRoomVisitsMessageEventHandler();
            handlers[InputCode.GetModeratorRoomInfoMessageComposer] = new ModGetRoomToolMessageEventHandler();
            handlers[InputCode.ModerateRoomMessageComposer] = new ModPerformRoomActionMessageEventHandler();
            handlers[InputCode.ModAlertMessageComposer] = new ModSendUserCautionMessageEventHandler();
            handlers[InputCode.ModMessageMessageComposer] = new ModSendUserMessageMessageEventHandler();
            handlers[InputCode.ModKickMessageComposer] = new ModKickUserMessageEventHandler();
            handlers[InputCode.ModBanMessageComposer] = new ModBanUserMessageEventHandler();
            //handlers[InputCode.MessengerInitMessageComposer] = new InitMessengerMessageEventHandler();
            handlers[InputCode.FriendListUpdateMessageComposer] = new FriendsListUpdateMessageEventHandler();
            handlers[InputCode.RemoveBuddyMessageComposer] = new RemoveBuddyMessageEventHandler();
            handlers[InputCode.HabboSearchMessageComposer] = new SearchHabboMessageEventHandler();
            handlers[InputCode.SendMsgMessageComposer] = new SendInstantMessengerMessageEventHandler();
            handlers[InputCode.AcceptBuddyMessageComposer] = new AcceptRequestMessageEventHandler();
            handlers[InputCode.DeclineBuddyMessageComposer] = new DeclineRequestMessageEventHandler();
            handlers[InputCode.RequestBuddyMessageComposer] = new RequestBuddyMessageEventHandler();
            handlers[InputCode.FollowFriendMessageComposer] = new FollowBuddyMessageEventHandler();
            handlers[InputCode.SendRoomInviteMessageComposer] = new SendInstantInviteMessageEventHandler();
            //handlers[InputCode.OpenFlatConnectionMessageComposer] = new OpenFlatMessageEventHandler();
            handlers[InputCode.AddFavouriteRoomMessageComposer] = new AddFavoriteMessageEventHandler();
            handlers[InputCode.DeleteFavouriteRoomMessageComposer] = new RemoveFavoriteMessageEventHandler();
            handlers[InputCode.QuitMessageComposer] = new GoToHotelViewMessageEventHandler();
            handlers[InputCode.GotoHotelviewComposer] = new GoToHotelViewMessageEventHandler();
            handlers[InputCode.GetUserFlatCatsMessageComposer] = new GetFlatCatsMessageEventHandler();
            //handlers[InputCode.GetBuddyRequestsMessageComposer] = new EnterInquiredRoomMessageEventHandler();
            handlers[InputCode.GetOfficialRoomsMessageComposer] = new GetPubsMessageEventHandler();
            //handlers[InputCode.ItemDoubleclickEvent] = new HandleDoubeclickMessageEventHandler(); ??
            handlers[InputCode.ItemInteractorSecondary] = new TriggerItemDiceSpecialMessageEventHandler();
            //handlers[InputCode.GetGuestRoomMessageComposer] = new GetRoomInfoMessageEventHandler();
            handlers[InputCode.PopularRoomsSearchMessageComposer] = new GetPopularRoomsMessageEventHandler();
            handlers[InputCode.RoomsWithHighestScoreSearchMessageComposer] = new GetHighRatedRoomsMessageEventHandler();
            handlers[InputCode.MyFriendsRoomsSearchMessageComposer] = new GetFriendsRoomsMessageEventHandler();
            handlers[InputCode.RoomsWhereMyFriendsAreSearchMessageComposer] = new GetRoomsWithFriendsMessageEventHandler();
            handlers[InputCode.MyRoomsSearchMessageComposer] = new GetOwnRoomsMessageEventHandler();
            handlers[InputCode.MyFavouriteRoomsSearchMessageComposer] = new GetFavoriteRoomsMessageEventHandler();
            handlers[InputCode.MyRoomHistorySearchMessageComposer] = new GetRecentRoomsMessageEventHandler();
            //handlers[InputCode.LatestEventsSearchMessageComposer] = new GetEventsMessageEventHandler();
            handlers[InputCode.GetPopularRoomTagsMessageComposer] = new GetPopularTagsMessageEventHandler();
            handlers[InputCode.RoomTextSearchMessageComposer] = new PerformSearchMessageEventHandler();
            /*handlers[InputCode.RoomTagSearchMessageComposer] = new PerformSearch2MessageEventHandler();
            handlers[InputCode.GetQuestsMessageComposer] = new OpenQuestsMessageEventHandler();
            handlers[InputCode.AcceptQuestMessageComposer] = new StartQuestMessageEventHandler();
            handlers[InputCode.RejectQuestMessageComposer] = new StopQuestMessageEventHandler();
            handlers[InputCode.OpenQuestTrackerMessageComposer] = new GetCurrentQuestMessageEventHandler();
            handlers[InputCode.GetInterstitialMessageComposer] = new GetAdvertisementMessageEventHandler();
            handlers[InputCode.GetPublicSpaceCastLibsMessageComposer] = new GetPubMessageEventHandler();
            handlers[InputCode.OpenConnectionMessageComposer] = new OpenPubMessageEventHandler();
            handlers[InputCode.GetHabboGroupBadgesMessageComposer] = new GetGroupBadgesMessageEventHandler(); */
            handlers[InputCode.ApplySignComposer] = new ApplySignMessageEventHandler();
            handlers[InputCode.SitActionComposer] = new SitMessageEventHandler();
            handlers[InputCode.GetFurnitureAliasesMessageComposer] = new GetRoomData1MessageEventHandler();
            handlers[InputCode.GetRoomEntryDataMessageComposer] = new GetRoomData2MessageEventHandler();
            handlers[InputCode.GetRoomAdMessageComposer] = new GetRoomData3MessageEventHandler();
            handlers[InputCode.ChatMessageComposer] = new TalkMessageEventHandler();
            handlers[InputCode.ShoutMessageComposer] = new ShoutMessageEventHandler();
            handlers[InputCode.WhisperMessageComposer] = new WhisperMessageEventHandler();
            handlers[InputCode.MoveAvatarMessageComposer] = new MoveMessageEventHandler();
            handlers[InputCode.CanCreateRoomMessageComposer] = new CanCreateRoomMessageEventHandler();
            handlers[InputCode.CreateFlatMessageComposer] = new CreateRoomMessageEventHandler();
            handlers[InputCode.GetRoomSettingsMessageComposer] = new GetRoomEditDataMessageEventHandler();
            //handlers[InputCode.UpdateRoomThumbnailMessageComposer] = new SaveRoomIconMessageEventHandler();
            handlers[InputCode.SaveRoomSettingsMessageComposer] = new SaveRoomDataMessageEventHandler();
            handlers[InputCode.AssignRightsMessageComposer] = new GiveRightsMessageEventHandler();
            handlers[InputCode.RemoveRightsMessageComposer] = new TakeRightsMessageEventHandler();
            handlers[InputCode.RemoveAllRightsMessageComposer] = new TakeAllRightsMessageEventHandler();
            handlers[InputCode.KickUserMessageComposer] = new KickUserMessageEventHandler();
            handlers[InputCode.BanUserMessageComposer] = new BanUserMessageEventHandler();
            handlers[InputCode.OpenTradingComposer] = new InitTradeMessageEventHandler();
            handlers[InputCode.UpdateNavigatorSettingsMessageComposer] = new SetHomeRoomMessageEventHandler();
            handlers[InputCode.DeleteRoomMessageComposer] = new DeleteRoomMessageEventHandler();
            handlers[InputCode.LookToMessageComposer] = new LookAtMessageEventHandler();
            handlers[InputCode.StartTypingMessageComposer] = new StartTypingMessageEventHandler();
            handlers[InputCode.CancelTypingMessageComposer] = new StopTypingMessageEventHandler();
            handlers[InputCode.IgnoreUserMessageComposer] = new IgnoreUserMessageEventHandler();
            handlers[InputCode.UnignoreUserMessageComposer] = new UnignoreUserMessageEventHandler();
            //handlers[InputCode.CanCreateEventMessageComposer] = new CanCreateRoomEventMessageEventHandler(); NS
            //handlers[InputCode.CreateEventMessageComposer] = new StartEventMessageEventHandler(); NS
            //handlers[InputCode.CancelEventMessageComposer] = new StopEventMessageEventHandler(); NS
            //handlers[InputCode.EditEventMessageComposer] = new EditEventMessageEventHandler(); NS
            handlers[InputCode.WaveMessageComposer] = new WaveMessageEventHandler();
            handlers[InputCode.GetUserTagsMessageComposer] = new GetUserTagsMessageEventHandler();
            handlers[InputCode.GetSelectedBadgesMessageComposer] = new GetUserBadgesMessageEventHandler();
            handlers[InputCode.RateFlatMessageComposer] = new RateRoomMessageEventHandler();
            handlers[InputCode.DanceMessageComposer] = new DanceMessageEventHandler();
            handlers[InputCode.LetUserInMessageComposer] = new AnswerDoorbellMessageEventHandler();
            handlers[InputCode.GoToFlatMessageComposer] = new ReqLoadRoomForUserMessageEventHandler();
            handlers[InputCode.RequestRoomPropertySet] = new ApplyRoomEffectMessageEventHandler();
            //handlers[InputCode.SaveBranding] = new SaveBrandingMessageEventHandler(); NS
            handlers[InputCode.PlaceObjectMessageComposer] = new PlaceItemMessageEventHandler();
            handlers[InputCode.PickupObjectMessageComposer] = new TakeItemMessageEventHandler();
            handlers[InputCode.PickupWallObjectMessageComposer] = new TakeItemMessageEventHandler();
            handlers[InputCode.MoveObjectMessageComposer] = new MoveItemMessageEventHandler();
            handlers[InputCode.MoveWallItemMessageComposer] = new MoveWallItemMessageEventHandler();
            handlers[InputCode.UseFurnitureMessageComposer] = new TriggerItemMessageEventHandler();
            handlers[InputCode.UseWallItemMessageComposer] = new TriggerItemMessageEventHandler();
            handlers[InputCode.UseWallItemSecondaryMessageComposer] = new TriggerItemMessageEventHandler();
            handlers[InputCode.GetItemDataMessageComposer] = new OpenPostitMessageEventHandler();
            handlers[InputCode.SetItemDataMessageComposer] = new SavePostitMessageEventHandler();
            handlers[InputCode.RemoveItemMessageComposer] = new DeletePostitMessageEventHandler();
            handlers[InputCode.PresentOpenMessageComposer] = new OpenPresentMessageEventHandler();
            handlers[InputCode.RoomDimmerGetPresetsMessageComposer] = new GetMoodlightMessageEventHandler();
            handlers[InputCode.RoomDimmerSavePresetMessageComposer] = new UpdateMoodlightMessageEventHandler();
            handlers[InputCode.RoomDimmerChangeStateMessageComposer] = new SwitchMoodlightStatusMessageEventHandler();
            handlers[InputCode.AddItemToTradeComposer] = new OfferTradeItemMessageEventHandler();
            handlers[InputCode.RemoveItemFromTradeComposer] = new TakeBackTradeItemMessageEventHandler();
            handlers[InputCode.CloseTradingComposer] = new StopTradeMessageEventHandler();
            handlers[InputCode.AcceptTradingComposer] = new AcceptTradeMessageEventHandler();
            handlers[InputCode.UnacceptTradingComposer] = new UnacceptTradeMessageEventHandler();
            handlers[InputCode.ConfirmAcceptTradingComposer] = new CompleteTradeMessageEventHandler();
            handlers[InputCode.RespectUserMessageComposer] = new GiveRespectMessageEventHandler();
            handlers[InputCode.AvatarEffectSelectedComposer] = new ApplyEffectMessageEventHandler();
            handlers[InputCode.AvatarEffectActivatedComposer] = new EnableEffectMessageEventHandler();
            handlers[InputCode.CreditFurniRedeemMessageComposer] = new RedeemExchangeFurniMessageEventHandler();
            handlers[InputCode.PlacePetMessageComposer] = new PlacePetMessageEventHandler();
            handlers[InputCode.GetPetInfoMessageComposer] = new GetPetInfoMessageEventHandler();
            handlers[InputCode.RemovePetFromFlatMessageComposer] = new PickUpPetMessageEventHandler();
            //handlers[InputCode.GetPetCommandsMessageComposer] = new CommandsPetMessageEventHandler();
            handlers[InputCode.RespectPetMessageComposer] = new RespectPetMessageEventHandler();
            handlers[InputCode.PlacePostItMessageComposer] = new PlacePostItMessageEventHandler();
            //handlers[InputCode.SetClothingChangeDataMessageComposer] = new SetLookTransferMessageEventHandler();
            handlers[InputCode.UpdateConditionMessageComposer] = new SaveWiredConditionsMessageEventHandler();
            handlers[InputCode.UpdateActionMessageComposer] = new SaveWiredMessageEventHandler();
            handlers[InputCode.UpdateTriggerMessageComposer] = new SaveWiredMessageEventHandler();
            handlers[InputCode.GiveUserHandItemComposer] = new GiveUserHandItemMessageEventHandler();
            handlers[InputCode.GetSongInfoMessageComposer] = new GetMusicDataMessageEventHandler();
            handlers[InputCode.AddJukeboxDiskComposer] = new AddPlaylistItemMessageEventHandler();
            handlers[InputCode.RemoveJukeboxDiskComposer] = new RemovePlaylistItemMessageEventHandler();
            handlers[InputCode.GetUserSongDisksMessageComposer] = new GetDisksMessageEventHandler();
            handlers[InputCode.GetJukeboxPlayListMessageComposer] = new GetPlaylistsMessageEventHandler();
            handlers[InputCode.InfoRetrieveMessageComposer] = new GetUserInfoMessageEventHandler();
            //handlers[InputCode.GetCreditsInfoComposer] = new GetBalanceMessageEventHandler();
            handlers[InputCode.ScrGetUserInfoMessageComposer] = new GetSubscriptionDataMessageEventHandler();
            handlers[InputCode.GetBadgesComposer] = new GetBadgesMessageEventHandler();
            handlers[InputCode.SetActivatedBadgesComposer] = new UpdateBadgesMessageEventHandler();
            handlers[InputCode.GetAchievementsComposer] = new GetAchievementsMessageEventHandler();
            handlers[InputCode.UpdateFigureDataMessageComposer] = new ChangeLookMessageEventHandler();
            handlers[InputCode.ChangeMottoMessageComposer] = new ChangeMottoMessageEventHandler();
            handlers[InputCode.GetWardrobeMessageComposer] = new GetWardrobeMessageEventHandler();
            handlers[InputCode.SaveWardrobeOutfitMessageComposer] = new SaveWardrobeMessageEventHandler();
            handlers[InputCode.RequestFurniInventoryComposer] = new GetInventoryMessageEventHandler();
            handlers[InputCode.GetPetInventoryComposer] = new GetPetsInventoryMessageEventHandler();
            //handlers[InputCode.GetBotInventoryComposer] = new GetBotsInventoryMessageEventHandler();
            handlers[InputCode.ReadSWFVariableComposer] = new setClientVarsMessageEventHandler();
            handlers[InputCode.UserInformation] = new GetOwnUserInformationMessageEventHandler();
            handlers[InputCode.AudioSettings] = new SetAudioSettingsMessageEventHandler();
            handlers[InputCode.ViewSWFRevisionComposer] = new ViewSWFRevisionMessageEventHandler();
            //handlers[InputCode.EnterPublicRoom] = new EnterPublicRoomMessageEventHandler(); NS
            handlers[InputCode.OnewayGateTrigger] = new TriggerItemMessageEventHandler();
            handlers[InputCode.AddSaddleToPetComposer] = new AddSaddleMessageEventHandler();
            handlers[InputCode.RemoveSaddleComposer] = new RemoveSaddleMessageEventHandler();
            handlers[InputCode.MountOnPetComposer] = new RideMessageEventHandler();
            //handlers[InputCode.ViewGroupsInfoComposer] = new GetGroupdetailsMessageEventHandler();
            //handlers[InputCode.EndGroupMembershipComposer] = new EndGroupMembershipMessageEventHandler();
            handlers[InputCode.PopularGuilds] = new PopularGuildsMessageEventHandler();
            handlers[InputCode.ExitGuild] = new ExitGuildMessageEventHandler();
            handlers[InputCode.SendRequestGuild] = new SendRequestGuildMessageEventHandler();
            handlers[InputCode.LoadMembersPetitions] = new LoadMembersPetitionsMessageEventHandler();
            handlers[InputCode.RejectGuildMember] = new RejectGuildMemberMessageEventHandler();
            handlers[InputCode.AcceptMember] = new AcceptMemberMessageEventHandler();
            handlers[InputCode.UpdateUserToRankGuild] = new UpdateUserToRankGuildMessageEventHandler();
            handlers[InputCode.UpdateUserFromRankGuild] = new UpdateUserToRankGuildMessageEventHandler();
            handlers[InputCode.SendFurniGuild] = new SendFurniGuildMessageEventHandler();
            handlers[InputCode.GetGuildFavorite] = new SetGuildFavoriteMessageEventHandler();
            handlers[InputCode.RemoveGuildFavorite] = new RemoveGuildFavoriteMessageEventHandler();
            handlers[InputCode.OpenGuildPage] = new OpenGuildPageMessageEventHandler();
            handlers[InputCode.EditGuild] = new EditGuildMessageEventHandler();
            handlers[InputCode.BuyGroup] = new BuyGroupMessageEventHandler();
            handlers[Incoming.EndConfirmBuy] = new DisplayGroupDataMessageEventHandler();
            handlers[Incoming.EditIdentidad] = new SaveGroupSettingsMessageEventHandler();
            handlers[Incoming.EditAjustes] = new SaveGroupSettingsMessageEventHandler();
            handlers[Incoming.TrainPet] = new GetTrainerPanelMessageEventHandler();
            handlers[Incoming.RoomMuteAll] = new ToggleRoomMuteMessageEventHandler();
            handlers[InputCode.OnUserToRoomEnterComposer] = new OnUserEnterRoomMessageEventHandler();
            handlers[InputCode.LoadUserComposer] = new LoadUserComposerMessageEventHandler();
            handlers[1215] = new GetUserSecondaryInfoMessageEventHandler();
            handlers[Incoming.SetRelationshipStatus] = new SetRelationshipeStatusMessageEventHandler();
            handlers[Incoming.GetRelationshipsProfile] = new GetRelationshipsProfileMessageEventHandler();
            handlers[1311] = new SaveMannequinMessageEventHandler();
            handlers[Incoming.BotsInventary] = new GetBotsInventoryMessageEventHandler();
            handlers[Incoming.BotComposer] = new PlaceBotMessageEventHandler();
            handlers[Incoming.BotSpeechList] = new BotSpeechListMessageEventHandler();
            handlers[Incoming.SerializeBotInformation] = new GetBotDataMessageEventHandler();
            handlers[Incoming.BotDisposer] = new TakeBotMessageEventHandler();
            handlers[Incoming.BuyGroup] = new BuyGroupMessageEventHandler();
            handlers[Incoming.EditPlaca] = new SaveGroupBadgeMessageEventHandler();
            handlers[Incoming.EditColores] = new SaveGroupColorsMessageEventHandler();
        }

        #endregion
    }
}
