﻿using Butterfly.Core;
using Butterfly.HabboHotel.GameClients;
using Butterfly.Messages;
using Butterfly.Messages.IOCodes;
using ConnectivityAsync;
using Database_Manager.Database.Session_Details.Interfaces;
using System;
using System.Data;

namespace Butterfly.Net
{
    class MusSocket
    {
		private ConnectionManager connectionManager;
        private string[] allowedIps;

        internal MusSocket(uint musPort, String[] allowedIps)
        {
			this.allowedIps = allowedIps;
			this.connectionManager = new ConnectionManager((int)musPort, 500, new MusParser(), false);
        }

		internal void WaitForConnections()
		{
			connectionManager.StartListening();
		}
    }

	class MusParser : IDataParser
	{
		public void handlePacketData(byte[] data, int bytesReceived)
		{
			int i = 0;
			int index;
			string rawData = ButterflyEnvironment.DefaultEncoding.GetString(data, 0, bytesReceived);

			string[] packets = rawData.Split(Convert.ToChar(1));
			index = packets.Length;

			string header;
			string command;
			while (index > 0)
			{
				header = packets[i++];
				command = packets[i++];

				MusConnection.processCommand(header, command);
			}
		}

		public object Clone()
		{
			return new MusParser();
		}

		public void Dispose()
		{ }
	}

    class MusConnection
    {
        internal static void processCommand(string header, string param)
        {
			Logging.WriteLine("[MUSConnection.ProcessCommand]: " + header);

            GameClient Client = null;
            switch (header.ToLower())
            {
                //case "settoken":
                //    {
                //        //String primeandgenerator = ButterflyEnvironment.globalCrypto.Prime + ":" + ButterflyEnvironment.globalCrypto.Generator;
                //        //String primeandgenerator = ButterflyEnvironment.globalCrypto.Prime + ":" + ButterflyEnvironment.globalCrypto.Generator;
                //        //this.socket.Send(ButterflyEnvironment.DefaultEncoding.GetBytes(primeandgenerator));

                //        break;
                //    }
                case "updatecredits":
                    {
                        if (param == "ALL")
                        {
                         //   ButterflyEnvironment.Game.GetClientManager().GetClient()
                        }
                        else
                        {
                            Client = ButterflyEnvironment.Game.GetClientManager().GetClientByUserID(uint.Parse(param));

                            if (Client == null)
                            {
                                return;
                            }

                            DataRow newCredits;

                            using (IQueryAdapter dbClient = ButterflyEnvironment.DatabaseManager.getQueryreactor())
                            {
                                dbClient.setQuery("SELECT credits FROM users WHERE id = @userid");
                                dbClient.addParameter("userid", (int)Client.GetHabbo().Id);
                                newCredits = dbClient.getRow();
                            }
                            
                            Client.GetHabbo().Credits = (int)newCredits["credits"];
                            Client.GetHabboDataContainer().UpdateCreditsBalance();
                        }

                        break;
                    }
                case "signout":
                    {
                        ButterflyEnvironment.Game.GetClientManager().GetClientByUserID(uint.Parse(param)).Disconnect();
                        break;
                    }
                
                case "ha":
                    {
                        //String extradata = data.Split(Convert.ToChar(1))[2];

                        ServerMessage HotelAlert = PacketHandelingManager.GetRecycledItem(OutputCode.HabboBroadcastMessageEvent);
                        HotelAlert.AddString(LanguageLocale.GetValue("hotelallert.notice") + "\r\n" + 
                        param + "\r\n");

                        /*if (extradata.Contains("://"))
                        {
                            Console.WriteLine("TEST");
                            HotelAlert.AddString(extradata);
                        }*/
                        

                        ButterflyEnvironment.Game.GetClientManager().QueueBroadcaseMessage(HotelAlert);
                        break;
                    }
                case "useralert":
                    {
                        String extradata = param.Split(Convert.ToChar(1))[1];
                        String url = extradata.Split(Convert.ToChar(1))[0];
                        GameClient TargetClient = null;
                        TargetClient = ButterflyEnvironment.Game.GetClientManager().GetClientByUserID(uint.Parse(param));

                        if (TargetClient == null)
                        {
                            return;
                        }
                        if (url.Contains("://"))
                        {
                            extradata = extradata + Convert.ToChar(2) + url;
                        }
                        TargetClient.SendNotif(extradata);
                        break;
                    }

                case "ban":
                    {

                        String extradata = param.Split(Convert.ToChar(1))[1];
                        String reason = extradata.Split(Convert.ToChar(1))[0];

                        GameClient TargetClient = null;

                        TargetClient = ButterflyEnvironment.Game.GetClientManager().GetClientByUsername(param);

                        if (TargetClient == null)
                        {
                            return;
                        }

                        int BanTime = 0;

                        try
                        {
                            BanTime = int.Parse(extradata);
                        }
                        catch (FormatException) { return; }

                        ButterflyEnvironment.Game.GetBanManager().BanUser(TargetClient, "Housekeeping Ban", BanTime, reason, false);
                        break;
                    }
                
                default:
                    {
						Logging.WriteLine("Unrecognized MUS packet: " + header);
                        break;
                    }
            }
        }
    }
}
