﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Butterfly.HabboHotel.Items;
using Butterfly.HabboHotel.Items.FurniTriggers.Triggers.Wired;
using Butterfly.HabboHotel.Rooms.Wired.WiredHandlers.Interfaces;
using Butterfly.Messages;
using Butterfly.Messages.IOCodes;

namespace Butterfly.PacketBuilders.Wired
{
    public static class ConditionPacketBuilder
    {
        internal static ServerMessage GenerateMessage(RoomItem wiredItem)
        {
            ServerMessage message = PacketHandelingManager.GetRecycledItem(OutputCode.WiredFurniConditionEvent);
            InteractionType type = wiredItem.GetBaseItem().InteractionType;
            if (NeedsFurni(type))
            {
                message.AddBoolean(true);
                message.AddInt32(BaseWiredInteractor.MAX_FURNI_SELECTION);
                List<RoomItem> list;
                if (wiredItem.wiredCondition != null)
                {
                    list = wiredItem.wiredCondition.getAffectedItemList();
                }
                else
                {
                    list = new List<RoomItem>();
                }
                message.AddInt32(list.Count);
                foreach (RoomItem item2 in list)
                {
                    message.AddInt32((int)item2.Id);
                }
            }
            else
            {
                message.AddBoolean(false);
                message.AddInt32(0);
                message.AddInt32(0);
            }

            message.AddInt32(wiredItem.GetBaseItem().SpriteId);
            message.AddInt32((int)wiredItem.Id);
            message.AddString("");
            if (wiredItem.wiredCondition != null && NeedsSettings(type))
            {
                int max = 1;
                if(type == InteractionType.conditionhasfurnionfurni)
                {
                    message.AddInt32(1);
                }
                else
                {
                    message.AddInt32(3);
                    max = 3;
                }
                string[] settings = (wiredItem.wiredCondition as IWiredSettingsTrigger).GetSettings();
                for (int i = 0; i < max; i++)
                {
                    message.AddInt32(settings[i].ToLower() == "true" ? 1 : 0);
                }
            }
            else
            {
                message.AddInt32(0);
            }
            message.AddInt32(0); // does nothing, seriously
            message.AddInt32(GetDialogtype(type));
            return (message);
        }


        internal static bool NeedsSettings(InteractionType type)
        {
            switch (type)
            {
                case InteractionType.conditionstatepos:
                case InteractionType.conditionhasfurnionfurni:
                    return true;
            }
            return false;
        }

        internal static bool NeedsFurni(InteractionType type)
        {
            switch (type)
            {
                case InteractionType.conditionfurnishaveusers:
                case InteractionType.conditionstatepos:
                case InteractionType.conditiontriggeronfurni:
                case InteractionType.conditionhasfurnionfurni:
                    return true;
            }
            return false;
        }
        private static int GetDialogtype(InteractionType type)
        {
            switch (type)
            {
                case InteractionType.conditionstatepos:
                    return 0;
                case InteractionType.conditionfurnishaveusers:
                case InteractionType.conditiontriggeronfurni:
                    return 1;
                case InteractionType.conditiontimelessthan:
                case InteractionType.conditiontimemorethan:
                    return 3;
                case InteractionType.conditionhasfurnionfurni:
                    return 7;
            }
            return 0;
        }
    }
}
