﻿using Butterfly.Core;
using ConnectivityAsync;
using Database_Manager.Database;

namespace Butterfly.ServerManager
{
	class HeartbeatParser : IDataParser
	{
		public delegate void PacketsReceived(string d);
		public event PacketsReceived OnDataReceived;

		public void handlePacketData(byte[] data, int bytesReceived)
		{
			string[] packets = System.Text.Encoding.Default.GetString(data, 0, bytesReceived).Split('|');
	        foreach (string packet in packets)
			{
				OnDataReceived.Invoke(packet);
			}
		}

		public object Clone()
		{
			return new HeartbeatParser();
		}

		public void Dispose()
		{
			this.OnDataReceived = null;
		}
	}

    class Session
    {
        private ConnectionInformation mSock;
        private bool Authorizated;

        private int mDisconnections;
        internal int Disconnection
        {
            get
            {
                return mDisconnections;
            }
            set
            {
                mDisconnections = value;
            }
        }

        private int mDisconnectionErrors;
        internal int DisconnectionError
        {
            get
            {
                return mDisconnectionErrors;
            }
            set
            {
                mDisconnectionErrors = value;
            }
        }

        public Session(ConnectionInformation pSock)
        {
            mSock = pSock;
            Authorizated = false;
			(pSock.Parser as HeartbeatParser).OnDataReceived += BytesReceived;
            Logging.WriteLine("Received connection");

            SendData("authreq");
			pSock.StartPacketProcessing();
        }

        private int count = 0;
        private int queryspersecond = 0;

		private void BytesReceived(string Packet)
		{
			try
			{
				if (string.IsNullOrEmpty(Packet))
					return;
				string[] Data = Packet.Split(':');
				if (Data[0].Length == 0)
				{
					Close();
					return;
				}

				switch (Data[0])
				{
					/*
					 * 

					 */
					case "auth":
						{
							if (Data[1] == "bitch")
							{
								Logging.WriteLine(GetLongIP + " -> Monitoring");
								SendData("authok");
								SessionManagement.RegisterSession(this);
								Authorizated = true;

								DatabaseStats.totalQueries = 0;
								DatabaseStats.totalQueryTime = 0;
							}
							else
							{
								Close();
							}
							break;
						}
					case "upda":
						{
							if (!Authorizated)
								Close();
							else
							{
								int Rate;
								if (mDisconnections == 0)
									Rate = 0;
								else if (mDisconnections == 0 && mDisconnectionErrors > 0)
									Rate = mDisconnectionErrors;
								else
									Rate = mDisconnectionErrors / mDisconnections;
								mDisconnections = 0;
								mDisconnectionErrors = 0;

								int totalQueries = DatabaseStats.totalQueries;
								double totalQueryTime = DatabaseStats.totalQueryTime;

								DatabaseStats.totalQueries = 0;
								DatabaseStats.totalQueryTime = 0;

								int rate = 0;

								if (totalQueryTime != 0 && totalQueries != 0)
									rate = (int)(totalQueryTime / totalQueries);

								if (count == 0)
								{
									queryspersecond = totalQueries;
									count++;
								}
								else
								{
									count = 0;
								}

								//
								SendData("data:onlinecount=" + (ButterflyEnvironment.Game.GetClientManager().ClientCount) + ",roomloadcount=" + ButterflyEnvironment.Game.GetRoomManager().LoadedRoomsCount + ",disconnectionrate=" + Rate + ",qurate=" + rate + ",qps=" + queryspersecond);
							}
							break;
						}
				}
			}
			catch
			{
				Close();
			}
		}

        private void SendData(string pData)
        {
            try
            {
                byte[] bytesToSend = System.Text.Encoding.Default.GetBytes(pData + "|");
				mSock.SendData(bytesToSend, bytesToSend.Length);
            }
            catch
            {
                Close();
            }
        }

        internal void SendMessage(string data)
        {
            SendData(data);
        }

        internal void Close()
        {
            Authorizated = false;
			mSock.Destroy();

            SessionManagement.RemoveSession(this);
            Logging.WriteLine("Reached end of session");
        }

        internal string GetLongIP
        {
            get
            {
				return mSock.GetIP();
            }
        }
    }
}
