﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Globalization;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;

namespace Butterfly
{
    class TextHandling
    {
        private static Random rnd;
        private static List<char> allowedchars;
        private static DateTime UNIX_Start;
        internal static CultureInfo cultureInfo;
        
        static TextHandling()
        {
            UNIX_Start = new DateTime(1970, 1, 1, 0, 0, 0);
            rnd = new Random();
            allowedchars = new List<char>(new char[]{ 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 
                                                'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 
                                                'y', 'z', '1', '2', '3', '4', '5', '6', '7', '8', '9', '0', '-', '.' });
            cultureInfo = CultureInfo.CreateSpecificCulture("en-GB");
        }

        internal static void Split(double k, out int a, out int b)
        {
            b = (int)Math.Round(k % 1 * 100);
            a = ((int)Math.Round(k * 100) - b) / 100;
        }

		internal static double CombineDBVal(int a, int b)
        {
            return a + ((float)b / 100);
        }

		internal static int CombineXYCoord(int a, int b)
		{
			return (a << 16) | b;
		}

        internal static int Parse(string input)
        {
            int sum = 0, i = 0, length = input.Length, k;

            while (i < length)
            {
                k = input[i++];
                if (k < 48 || k > 59)
                    return 0;
                sum = 10 * sum + k - 48;
            }

            return sum;
        }

        internal static uint ParseUInt32(string input)
        {
            uint sum = 0, k; 
            int i = 0, length = input.Length;

            while (i < length)
            {
                k = input[i++];
                if (k < 48 || k > 59)
                    return 0;
                sum = 10 * sum + k - 48;
            }

            return sum;
        }

		internal static int ParseInt32(string input)
		{
			int sum = 0, i = 0, length = input.Length, k;
			bool isNegative = false;

			while (i < length)
			{
				k = input[i++];
				if (k == 45)
					isNegative = true;
				if (k < 48 || k > 59)
					return 0;
				sum = 10 * sum + k - 48;
			}

			if (isNegative) sum *= -1;
			return sum;
		}

        internal static int GetFirstSiffer(double k)
        {
            return (int)Math.Round(k % 1 * 100);
        }

        internal static string GetString(double k)
        {
            return k.ToString(cultureInfo);
        }

        internal static int BooleanToInt(bool k)
        {
            return k ? 1 : 0;
        }
        
        internal static bool EnumToBool(string val)
        {
            return (val == "1");
        }

        internal static string BoolToNumber(bool val)
        {
            return val ? "1" : "0";
        }

        internal static int GetRandomNumber(int min, int max)
        {
            return rnd.Next(min, max);
        }

        internal static int GetUnixTimestamp()
        {
            TimeSpan ts = (DateTime.UtcNow - UNIX_Start);
            return (int)ts.TotalSeconds;
        }

        internal static string FilterInjectionChars(string Input)
        {
            return FilterInjectionChars(Input, false);
        }

        internal static string FilterFigure(string figure)
        {
            foreach (char character in figure)
            {
                if (!isValid(character))
                    return "lg-3023-1335.hr-828-45.sh-295-1332.hd-180-4.ea-3168-89.ca-1813-62.ch-235-1332";
            }

            return figure;
        }

        private static bool isValid(char character)
        {
            return allowedchars.Contains(character);
        }

        internal static string FilterInjectionChars(string input, bool allowLinebreaks)
        {
            return input;
        }

        internal static bool IsValidAlphaNumeric(string inputStr)
        {
            inputStr = inputStr.ToLower();
            if (string.IsNullOrEmpty(inputStr))
            {
                return false;
            }

            for (int i = 0; i < inputStr.Length; i++)
            {
                if (!isValid(inputStr[i]))
                {
                    return false;
                }
            }

            return true;
        }
        
        internal static string TimeSpanToString(TimeSpan span)
        {
            return string.Format("{0} s, {1} ms", span.Seconds, span.Milliseconds);
        }

        internal static void AppendTimeStampWithComment(ref StringBuilder builder, DateTime time, string text)
        {
            builder.AppendLine(text + " =>[" + TimeSpanToString(DateTime.Now - time) + "]");
        }

        internal static string GetHair(string figure)
        {
            string hair = Regex.Split(figure, "hr")[1];
            string body = Regex.Split(figure, "hd")[1];

            return Convert.ToString("hr" + hair.Split('.')[0] + "." + "hd" + body.Split('.')[0] + ".");
        }

        internal static string ConcatArray(string[] arr, int index = 0, char splitter = ' ')
        {
            StringBuilder MergedParams = new StringBuilder();

            for (int i = index; i < arr.Length; i++)
            {
                if (i > index)
                    MergedParams.Append(splitter);

                MergedParams.Append(arr[i]);
            }

            return MergedParams.ToString();
        }

    }

	class Formatting
	{
		internal static int PointToInt(Point p)
		{
			return TextHandling.CombineXYCoord(p.X, p.Y);
		}
	}
	
}
