﻿using System;
using System.Collections;

namespace Collections.ModernCollections
{
    public class OrderedArray<T, V> where T : IComparable
    {
        private ISortable<T, V>[] storage;
        private int capcity;
        private Hashtable identificationStorage;
        private int writePos;
        private SortType sortType;
        private int count;

        public int Count
        {
            get
            {
                return this.writePos;
            }
        }

        public ISortable<T, V>[] StorageArray
        {
            get
            {
                return this.storage;
            }
        }

        public int WritePosition
        {
            get
            {
                return this.writePos;
            }
        }

        public OrderedArray(int capacity, SortType type)
        {
            this.capcity = capacity;
            this.storage = new ISortable<T, V>[capacity];
            this.writePos = 0;
            this.sortType = type;
            this.identificationStorage = new Hashtable();
        }

        public void Insert(T key, V value)
        {
            if (++count == storage.Length)
            {
                capcity = (int)Math.Ceiling(storage.Length * 1.5);

                ISortable<T, V>[] newArray = new ISortable<T, V>[capcity];
                Array.Copy(storage, newArray, count - 1);
                storage = newArray;
            }

            Insert(new ArrayElement<T, V>(key, value));

        }

        private void Insert(ISortable<T, V> value)
        {
            if (identificationStorage.ContainsKey(value.Value))
                RemoveItem(value.Value);
            ISortable<T, V> storedObject;
            T orderKey = value.Key;
            T storedKey;

            for (int i = 0; i < capcity; i++)
            {
                storedObject = storage[i];

                if (storedObject == null) //Empty or reached bottom
                {
                    Insert(value, i);
                    i = capcity;
                }
                else
                {
                    storedKey = storedObject.Key;
                    //if (orderKey > storedKey)// 1 = Is more than, 0 equals, -1 less than
                    if (orderKey.CompareTo(storedKey) == (int)sortType)
                    {
                        Insert(value, i);
                        i = capcity;
                    }
                }
            }

            Defrag();
        }

        private T GetOrderKey(int position)
        {
            if (position > storage.Length - 1)
                return default(T);
            else
                return storage[position].Key;
        }

        private void Insert(ISortable<T, V> value, int position)
        {

            ISortable<T, V> tempVar = storage[0];
            for (int i = writePos; i > position; i--) //We start at where last data was inserted, end at where we are gonna insert data
            {
                if (i > capcity - 1) //If we are outside the array
                    continue;

                if (i == capcity - 1 && storage[i] != null) //If there is data at the end of the array, kick it out
                {
                    RemoveItem(storage[i].Value);
                }

                storage[i] = storage[i - 1];
                if (storage[i] != null)
                    storage[i].Index++;
            }

            storage[position] = value;
            identificationStorage.Add(value.Value, value);
            value.Index = position;
            writePos++;
        }

        public void RemoveItem(V value)
        {
            count--;
            if (!identificationStorage.ContainsKey(value))
            {
                return;
            }

            ISortable<T, V> entryData = (ISortable<T, V>)identificationStorage[value];

            for (int i = entryData.Index; i < writePos; i++)
            {
                if (i + 1 > capcity - 1) //If we are outside the array
                {
                    storage[i] = null;
                    continue;
                }

                storage[i] = storage[i + 1];
                if (storage[i] != null)
                    storage[i].Index--;
            }

            identificationStorage.Remove(value);

            writePos--;
            Defrag();
        }

        public void Defrag()
        {
            int counter = 0;
            int lastFreePosition = 0;
            bool freePos = false;
            ISortable<T, V> item;

            for (int i = 0; i < this.storage.Length; i++)
            {
                item = storage[i];
                if (item == null)
                {
                    if (freePos == false)
                    {
                        freePos = true;
                        lastFreePosition = i;
                    }
                }
                else
                {
                    counter++;
                    if (freePos)
                    {
                        storage[item.Index] = null;
                        storage[lastFreePosition] = item;
                        item.Index = lastFreePosition;


                        lastFreePosition++;
                    }
                }
            }

            this.writePos = counter;
        }

        public void Clear()
        {
            Array.Clear(storage, 0, storage.Length);
            identificationStorage.Clear();
            writePos = 0;
            count = 0;
        }
    }

    public enum SortType : int
    {
        Ascending = -1,
        Descending = 1
    }
}
