﻿using System;
using System.Collections;

namespace Collections
{
    public class OrderedArray<T>
    {
        private ISortable<T>[] storage;
        private readonly int capcity;
        private Hashtable identificationStorage;
        private int writePos;

        public int Count
        {
            get
            {
                return this.writePos;
            }
        }

        public ISortable<T>[] StorageArray
        {
            get
            {
                return this.storage;
            }
        }

        public int WritePosition
        {
            get
            {
                return this.writePos;
            }
        }

        public OrderedArray(int capacity)
        {
            this.capcity = capacity;
            this.storage = new ISortable<T>[capacity];
            this.writePos = 0;
            this.identificationStorage = new Hashtable();
        }

        public void Insert(ISortable<T> value)
        {
            if (identificationStorage.ContainsKey(value.Identificator))
                RemoveItem(value.Identificator);
            ISortable<T> storedObject;
            int orderKey = value.OrderKey;
            int storedKey;

            for (int i = 0; i < capcity; i++)
            {
                storedObject = storage[i];

                if (storedObject == null) //Empty or reached bottom
                {
                    Insert(value, i);
                    i = capcity;
                }
                else
                {
                    storedKey = storedObject.OrderKey;
                    if (orderKey > storedKey)
                    {
                        Insert(value, i);
                        i = capcity;
                    }
                }
            }

            Defrag();
        }

        private int GetOrderKey(int position)
        {
            if (position > storage.Length - 1)
                return 0;
            else
                return storage[position].OrderKey;
        }

        private void Insert(ISortable<T> value, int position)
        {

            ISortable<T> tempVar = storage[0];
            for (int i = writePos; i > position; i--) //We start at where last data was inserted, end at where we are gonna insert data
            {
                if (i > capcity - 1) //If we are outside the array
                    continue;

                if (i == capcity - 1 && storage[i] != null) //If there is data at the end of the array, kick it out
                {
                    RemoveItem(storage[i].Identificator);
                }

                storage[i] = storage[i - 1];
                if (storage[i] != null)
                    storage[i].ArrayPosition++;
            }

            storage[position] = value;
            identificationStorage.Add(value.Identificator, value);
            value.ArrayPosition = position;
            writePos++;
        }

        public void RemoveItem(uint value)
        {
            if (!identificationStorage.ContainsKey(value))
            {
                return;
            }

            ISortable<T> entryData = (ISortable<T>)identificationStorage[value];

            for (int i = entryData.ArrayPosition; i < writePos; i++)
            {
                if (i + 1 > capcity - 1) //If we are outside the array
                {
                    storage[i] = null;
                    continue;
                }

                storage[i] = storage[i + 1];
                if (storage[i] != null)
                    storage[i].ArrayPosition--;
            }

            identificationStorage.Remove(value);

            writePos--;
            Defrag();
        }

        public void Defrag()
        {
            int counter = 0;
            int lastFreePosition = 0;
            bool freePos = false;
            ISortable<T> item;

            for (int i = 0; i < this.storage.Length; i++)
            {
                item = storage[i];
                if (item == null)
                {
                    if (freePos == false)
                    {
                        freePos = true;
                        lastFreePosition = i;
                    }
                }
                else
                {
                    counter++;
                    if (freePos)
                    {
                        storage[item.ArrayPosition] = null;
                        storage[lastFreePosition] = item;
                        item.ArrayPosition = lastFreePosition;


                        lastFreePosition++;
                    }
                }
            }

            this.writePos = counter;
        }

        //internal void Print()
        //{
        //    ISortable<T> item;
        //    for (int i = 0; i < this.storage.Length; i++)
        //    {
        //        item = storage[i];
        //        if (item == null)
        //        {
        //            Console.Write("[NULL]");
        //        }
        //        else
        //        {
        //            Console.Write("[" + item.OrderKey + "]");
        //        }
        //    }
        //    Console.WriteLine();
        //}
    }
}
