﻿using System;
using System.Net.Sockets;

namespace ConnectivityAsync
{
    public class ConnectionInformation
    {
        private ConnectionManager manager;
        private Socket socket;
        private byte[] buffer;
        private bool isClosed = false;
        private uint connectionID;
        public IDataParser Parser { get; set; }

        public ConnectionInformation(uint connectionID, ConnectionManager manager, Socket socket, IDataParser parser, byte[] buffer)
        {
            this.connectionID = connectionID;
            this.manager = manager;
            this.socket = socket;
            this.Parser = parser;
            this.buffer = buffer;

            if (manager.WriteDebugLines)
                Console.WriteLine("Connection {0} accepted", connectionID);
        }

        public uint GetConnectionID()
        {
            return connectionID;
        }

        public event ConnectionEvent OnConnectionClosed;

        public string GetIP()
        {
            return socket.RemoteEndPoint.ToString();
        }

        public void SendData(byte[] data, int length)
        {
            try
            {
                SocketAsyncEventArgs args = new SocketAsyncEventArgs();
                args.SetBuffer(data, 0, length);
                socket.SendAsync(args);
                // Fire and forget
            }
            catch
            {
                Destroy();
            }
        }

        public void StartPacketProcessing()
        {
            SocketAsyncEventArgs args = new SocketAsyncEventArgs();
            args.Completed += DataReceivedAsynchronously;
            args.SetBuffer(buffer, 0, buffer.Length);

            // If data was received immediately
            if (!socket.ReceiveAsync(args))
            {
                DataReceived(args);
            }
        }

        private void DataReceivedAsynchronously(object sender, SocketAsyncEventArgs e)
        {
            DataReceived(e);
        }

        private void DataReceived(SocketAsyncEventArgs args)
        {
            try
            {
                if (args.SocketError == SocketError.Success && args.BytesTransferred > 0)
                {
                    Parser.handlePacketData(buffer, args.BytesTransferred);
                    StartPacketProcessing();
                }
                else
                {
                    Destroy();
                }
            }
            catch
            {
                Destroy();
            }
        }

        public void Destroy()
        {
            if (!isClosed)
            {
                if (manager.WriteDebugLines)
                {
                    Console.WriteLine("Connection {0} closed", connectionID);
                }
                isClosed = true;
                try
                {
                    socket.Close();
                }
                catch { }
                OnConnectionClosed(this);
            }
        }
    }
}
