﻿using System;
using System.Net;
using System.Net.Sockets;

namespace ConnectivityAsync
{
    public delegate void ConnectionEvent(ConnectionInformation t);

    public class ConnectionManager
    {
        private int port;
        private int maxConnections;
        private IDataParser parser;
        private bool disableNaglesAlgorithm;
        private Socket listener;
        private uint connectionCounter = 0;

        public bool WriteDebugLines { get; set; }

        public ConnectionManager(int port, int maxConnections, IDataParser dataParser, bool disableNaglesAlgorithm)
        {
            this.port = port;
            this.maxConnections = maxConnections;
            this.parser = dataParser;
            this.disableNaglesAlgorithm = disableNaglesAlgorithm;
            this.listener = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
        }

        public void StartListening()
        {
            listener.Bind(new IPEndPoint(IPAddress.Any, port));
            listener.Listen(maxConnections);

            AcceptConnection();
        }

        private void AcceptConnection()
        {
            SocketAsyncEventArgs args = new SocketAsyncEventArgs();
            args.Completed += ConnectionAsynchronouslyAccepted;

            // If a connection was immediately accepted, accept the connection
            if (!listener.AcceptAsync(args))
            {
                ConnectionAccepted(args);
            }
        }

        private void ConnectionAsynchronouslyAccepted(object sender, SocketAsyncEventArgs e)
        {
            ConnectionAccepted(e);
        }

        private void ConnectionAccepted(SocketAsyncEventArgs args)
        {
            if (args.SocketError == SocketError.Success)
            {
                ConnectionInformation connection = new ConnectionInformation(++connectionCounter, this, args.AcceptSocket, (IDataParser)parser.Clone(), new byte[1024]);
                connectionEvent(connection);
                AcceptConnection();
            }
        }

        public event ConnectionEvent connectionEvent;

        public void Destroy()
        {
            this.listener.Close();
        }

        public bool IsConnected()
        {
            try
            {
                return this.listener.Connected;
            }
            catch
            {
                return false;
            }
        }
    }
}
