﻿using System;
using System.Collections;
using System.IO;
using System.Text;

namespace ConsoleWriter
{
    public class Writer
    {
        private static bool disabled = false;

        public static bool Disabled
        {
            get
            {
                return disabled;
            }
            set
            {
                disabled = value;
            }
        }

        public static void WriteLine(string Line)
        {
            if (!disabled)
                Console.WriteLine(Line);
        }

        public static void LogException(string logText)
        {
            WriteToFile("Logs/exceptions.txt", logText + "\r\n\r\n");
            WriteLine("Exception has been logged");
        }

        public static void LogCriticalException(string logText)
        {
            WriteToFile("Logs/criticalexceptions.txt", logText + "\r\n\r\n");
            WriteLine("Runtime exception logged");
        }

        public static void LogCacheError(string logText)
        {
            WriteToFile("Logs/cacheerror.txt", logText + "\r\n\r\n");
            WriteLine("Cache exception logged");
        }

        public static void LogMessage(string logText)
        {
            WriteToFile("Logs/logg.txt", logText + "\r\n\r\n");
            WriteLine(logText);
        }

        public static void LogDDOS(string logText)
        {
            WriteToFile("Logs/ddos.txt", logText + "\r\n\r\n");
            WriteLine(logText);
        }

        public static void LogThreadException(string Exception, string Threadname)
        {
            WriteToFile("Logs/threaderror.txt", "Error in thread " + Threadname + ": \r\n" + Exception + "\r\n\r\n");
            WriteLine("Error in " + Threadname + " logged");
        }

        public static void LogQueryError(Exception Exception, string query)
        {
            WriteToFile("Logs/MySQLerrors.txt", "Error in query: \r\n" + query + "\r\n" + Exception + "\r\n\r\n");
            WriteLine("Query error logged");
        }

        public static void LogPacketException(string Packet, string Exception)
        {
            WriteToFile("Logs/packeterror.txt", "Error in packet " + Packet + ": \r\n" + Exception + "\r\n\r\n");
            WriteLine("User exception logged");
        }

        public static void HandleException(Exception pException, string pLocation)
        {
            StringBuilder ExceptionData = new StringBuilder();
            ExceptionData.AppendLine("Exception logged " + DateTime.Now.ToString() + " in " + pLocation + ":");
            ExceptionData.AppendLine(pException.ToString());

            if (pException.InnerException != null)
            {
                ExceptionData.AppendLine("Inner exception:");
                ExceptionData.AppendLine(pException.InnerException.ToString());
            }

            if (pException.HelpLink != null)
            {
                ExceptionData.AppendLine("Help link:");
                ExceptionData.AppendLine(pException.HelpLink);
            }

            if (pException.Source != null)
            {
                ExceptionData.AppendLine("Source:");
                ExceptionData.AppendLine(pException.Source);
            }

            if (pException.Data != null)
            {
                ExceptionData.AppendLine("Data:");
                foreach (DictionaryEntry Entry in pException.Data)
                {
                    ExceptionData.AppendLine("  Key: " + Entry.Key + "Value: " + Entry.Value);
                }
            }

            if (pException.Message != null)
            {
                ExceptionData.AppendLine("Message:");
                ExceptionData.AppendLine(pException.Message);
            }

            if (pException.StackTrace != null)
            {
                ExceptionData.AppendLine("Stack trace:");
                ExceptionData.AppendLine(pException.StackTrace);
            }

            ExceptionData.AppendLine();
            ExceptionData.AppendLine();
            LogException(ExceptionData.ToString());
        }

        public static void DisablePrimaryWriting(bool clearConsole)
        {
            disabled = true;
            if (clearConsole)
                Console.Clear();
        }

        public static void LogShutdown(StringBuilder builder)
        {
            WriteToFile("Logs/shutdownlog.txt", builder.ToString());
        }

        private static void WriteToFile(string path, string content)
        {
            try
            {
                FileStream errWriter = new System.IO.FileStream(AppDomain.CurrentDomain.BaseDirectory + path, System.IO.FileMode.Append, System.IO.FileAccess.Write);
                byte[] Msg = ASCIIEncoding.ASCII.GetBytes(Environment.NewLine + content);
                errWriter.Write(Msg, 0, Msg.Length);
                errWriter.Dispose();
            }
            catch (Exception e)
            {
                WriteLine("Exception occurred wile writing exception to file: " + e.ToString() + Environment.NewLine + "Content:" + Environment.NewLine + content);
            }
        }

        private static void WriteCallback(IAsyncResult callback)
        {
            FileStream stream = (FileStream)callback.AsyncState;
            stream.EndWrite(callback);
            stream.Dispose();
        }
    }
}
