﻿using Database_Manager.Database.Session_Details.Interfaces;
using MySql.Data.MySqlClient;
using System;
using System.Collections;
using System.Collections.Generic;

namespace Database_Manager.Database
{
    public class DatabaseManager
    {
        private List<MySqlClient> databaseClients;
        private Queue connections;
        private string host;
        private uint port;
        private string user;
        private string password;
        private string databaseName;
        private uint minPoolSize;
        private uint maxPoolSize;
        private string connectionString;

        public DatabaseManager(string host, uint port, string user, string password, string databaseName, uint minPoolSize, uint maxPoolSize)
        {
            this.connections = new Queue();
            this.host = host;
            this.port = port;
            this.user = user;
            this.password = password;
            this.databaseName = databaseName;
            this.minPoolSize = minPoolSize;
            this.maxPoolSize = maxPoolSize;
        }
        
        private void createNewConnectionString()
        {
            MySqlConnectionStringBuilder builder = new MySqlConnectionStringBuilder
            {
                Server = host,
                Port = port,
                UserID = user,
                Password = password,
                Database = databaseName,
                MinimumPoolSize = minPoolSize,
                MaximumPoolSize = maxPoolSize,
                AllowZeroDateTime = true,
                ConvertZeroDateTime = true,
                DefaultCommandTimeout = 300,
                ConnectionTimeout = 10,
                SslMode = MySqlSslMode.None
            };

            this.connectionString = builder.ToString();
        }
        
        public void destroy()
        {
            lock (this)
            {
                if (this.databaseClients != null)
                {
                    foreach (MySqlClient client in this.databaseClients)
                    {
                        if (!client.isAvailable())
                        {
                            client.Dispose();
                        }
                        client.disconnect();
                    }
                    this.databaseClients.Clear();
                }
            }
        }

        private void disconnectUnusedClients()
        {
            lock (this)
            {
                foreach (MySqlClient client in this.databaseClients)
                {
                    if (client.isAvailable())
                    {
                        client.disconnect();
                    }
                }
            }
        }

        public IQueryAdapter getQueryreactor()
        {
            IDatabaseClient dbClient = null;
            lock (connections.SyncRoot)
            {
                if (connections.Count > 0)
                {
                    dbClient = (IDatabaseClient)connections.Dequeue();
                }
            }

            if (dbClient != null)
            {
                dbClient.connect();  
                dbClient.prepare();
                return dbClient.getQueryReactor();
            }
            else
            {
                IDatabaseClient connection = new MySqlClient(this, 0, connectionString);
                connection.connect();
                connection.prepare();
                return connection.getQueryReactor();
            }
        }

        internal void FreeConnection(IDatabaseClient dbClient)
        {
            lock (connections.SyncRoot)
            {
                connections.Enqueue(dbClient);
            }
        }

        public void init()
        {
            try
            {
                this.createNewConnectionString();
                this.databaseClients = new List<MySqlClient>((int)maxPoolSize);
            }
            catch (MySqlException exception)
            {
                throw new Exception("Could not connect the clients to the database: " + exception.Message);
            }

            getQueryreactor().Dispose(); //Test connection
        }
    }
}

